@@ -16,9 +16,10 @@
 
 import os
 import sys
-import ConfigParser
 import ssl
 
+from six.moves import configparser
+
 
 def ssl_settings(host, config_file, env=os.environ):
     """
@@ -38,15 +39,27 @@ def ssl_settings(host, config_file, env=os.environ):
     either in the config file or as an environment variable.
     Environment variables override any options set in cqlsh config file.
     """
-    configs = ConfigParser.SafeConfigParser()
+    configs = configparser.SafeConfigParser()
     configs.read(config_file)
 
     def get_option(section, option):
         try:
             return configs.get(section, option)
-        except ConfigParser.Error:
+        except configparser.Error:
             return None
 
+    def get_best_tls_protocol(ssl_ver_str):
+        # newer python versions suggest to use PROTOCOL_TLS to negotiate the highest TLS version.
+        # older protocol versions have been deprecated:
+        # https://docs.python.org/2/library/ssl.html#ssl.PROTOCOL_TLS
+        # https://docs.python.org/3/library/ssl.html#ssl.PROTOCOL_TLS
+        if ssl_ver_str:
+            return getattr(ssl, "PROTOCOL_%s" % ssl_ver_str, None)
+        for protocol in ['PROTOCOL_TLS', 'PROTOCOL_TLSv1_2', 'PROTOCOL_TLSv1_1', 'PROTOCOL_TLSv1']:
+            if hasattr(ssl, protocol):
+                return getattr(ssl, protocol)
+        return None
+
     ssl_validate = env.get('SSL_VALIDATE')
     if ssl_validate is None:
         ssl_validate = get_option('ssl', 'validate')
@@ -55,13 +68,11 @@ def ssl_settings(host, config_file, env=os.environ):
     ssl_version_str = env.get('SSL_VERSION')
     if ssl_version_str is None:
         ssl_version_str = get_option('ssl', 'version')
-    if ssl_version_str is None:
-        ssl_version_str = "TLSv1"
 
-    ssl_version = getattr(ssl, "PROTOCOL_%s" % ssl_version_str, None)
+    ssl_version = get_best_tls_protocol(ssl_version_str)
     if ssl_version is None:
-        sys.exit("%s is not a valid SSL protocol, please use one of SSLv23, "
-                 "TLSv1, TLSv1.1, or TLSv1.2" % (ssl_version_str,))
+        sys.exit("%s is not a valid SSL protocol, please use one of "
+                 "TLS, TLSv1_2, TLSv1_1, or TLSv1" % (ssl_version_str,))
 
     ssl_certfile = env.get('SSL_CERTFILE')
     if ssl_certfile is None:
