@@ -14,25 +14,18 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import os
-import sys
 import logging
-from itertools import izip
+import os
 from os.path import dirname, join, normpath
+import sys
+import unittest
 
 cqlshlog = logging.getLogger('test_cqlsh')
 
-try:
-    # a backport of python2.7 unittest features, so we can test against older
-    # pythons as necessary. python2.7 users who don't care about testing older
-    # versions need not install.
-    import unittest2 as unittest
-except ImportError:
-    import unittest
-
 test_dir = dirname(__file__)
 cassandra_dir = normpath(join(test_dir, '..', '..', '..'))
 cqlsh_dir = join(cassandra_dir, 'bin')
+path_to_cqlsh = join(cqlsh_dir, 'cqlsh.py')
 
 sys.path.append(cqlsh_dir)
 
@@ -48,13 +41,27 @@ TEST_PORT = int(os.environ.get('CQL_TEST_PORT', 9042))
 
 class BaseTestCase(unittest.TestCase):
     def assertNicelyFormattedTableHeader(self, line, msg=None):
-        return self.assertRegexpMatches(line, r'^ +\w+( +\| \w+)*\s*$', msg=msg)
+        return self.assertRegex(line, r'^ +\w+( +\| \w+)*\s*$', msg=msg)
 
     def assertNicelyFormattedTableRule(self, line, msg=None):
-        return self.assertRegexpMatches(line, r'^-+(\+-+)*\s*$', msg=msg)
+        return self.assertRegex(line, r'^-+(\+-+)*\s*$', msg=msg)
 
     def assertNicelyFormattedTableData(self, line, msg=None):
-        return self.assertRegexpMatches(line, r'^ .* \| ', msg=msg)
+        return self.assertRegex(line, r'^ .* \| ', msg=msg)
+
+    def assertRegex(self, text, regex, msg=None):
+        """Call assertRegexpMatches() if in Python 2"""
+        if hasattr(unittest.TestCase, 'assertRegex'):
+            return super().assertRegex(text, regex, msg)
+        else:
+            return self.assertRegexpMatches(text, regex, msg)
+
+    def assertNotRegex(self, text, regex, msg=None):
+        """Call assertNotRegexpMatches() if in Python 2"""
+        if hasattr(unittest.TestCase, 'assertNotRegex'):
+            return super().assertNotRegex(text, regex, msg)
+        else:
+            return self.assertNotRegexpMatches(text, regex, msg)
 
 
 def dedent(s):
@@ -67,4 +74,4 @@ def dedent(s):
 
 
 def at_a_time(i, num):
-    return izip(*([iter(i)] * num))
+    return zip(*([iter(i)] * num))
