@@ -15,28 +15,40 @@
 # limitations under the License.
 
 # to configure behavior, define $CQL_TEST_HOST to the destination address
-# for Thrift connections, and $CQL_TEST_PORT to the associated port.
+# and $CQL_TEST_PORT to the associated port.
 
-from __future__ import with_statement
+from __future__ import unicode_literals, with_statement
 
 import locale
 import os
 import re
-from itertools import izip
-from .basecase import (BaseTestCase, cqlshlog, dedent, at_a_time, cqlsh,
-                       TEST_HOST, TEST_PORT)
-from .cassconnect import (get_keyspace, testrun_cqlsh, testcall_cqlsh,
-                          cassandra_cursor, split_cql_commands, quote_name)
-from .ansi_colors import (ColoredText, lookup_colorcode, lookup_colorname,
-                          lookup_colorletter, ansi_seq)
-import unittest
+import subprocess
 import sys
+import six
+import unittest
+
+from .basecase import (BaseTestCase, TEST_HOST, TEST_PORT,
+                       at_a_time, cqlsh, cqlshlog, dedent)
+from .cassconnect import (cassandra_cursor, create_db, get_keyspace,
+                          quote_name, remove_db, split_cql_commands,
+                          testcall_cqlsh, testrun_cqlsh)
+from .ansi_colors import (ColoredText, ansi_seq, lookup_colorcode,
+                          lookup_colorname, lookup_colorletter)
 
 CONTROL_C = '\x03'
 CONTROL_D = '\x04'
 
+
 class TestCqlshOutput(BaseTestCase):
 
+    @classmethod
+    def setUpClass(cls):
+        create_db()
+
+    @classmethod
+    def tearDownClass(cls):
+        remove_db()
+
     def setUp(self):
         env = os.environ.copy()
         env['COLUMNS'] = '100000'
@@ -51,10 +63,10 @@ class TestCqlshOutput(BaseTestCase):
         pass
 
     def assertNoHasColors(self, text, msg=None):
-        self.assertNotRegexpMatches(text, ansi_seq, msg='ANSI CSI sequence found in %r' % text)
+        self.assertNotRegex(text, ansi_seq, msg='ANSI CSI sequence found in %r' % text)
 
     def assertHasColors(self, text, msg=None):
-        self.assertRegexpMatches(text, ansi_seq, msg=msg)
+        self.assertRegex(text, ansi_seq, msg=msg)
 
     def assertColored(self, coloredtext, colorname):
         wanted_colorcode = lookup_colorcode(colorname)
@@ -66,7 +78,7 @@ class TestCqlshOutput(BaseTestCase):
                                      % (coloredtext, num, lookup_colorname(ccolor), colorname))
 
     def assertColorFromTags(self, coloredtext, tags):
-        for (char, tag) in izip(coloredtext, tags):
+        for (char, tag) in zip(coloredtext, tags):
             if char.isspace():
                 continue
             if tag.isspace():
@@ -94,6 +106,19 @@ class TestCqlshOutput(BaseTestCase):
                     self.assertEqual(outputline.plain().rstrip(), plain)
                     self.assertColorFromTags(outputline, colorcodes)
 
+    def strip_read_repair_chance(self, describe_statement):
+        """
+        Remove read_repair_chance and dclocal_read_repair_chance options
+        from output of DESCRIBE statements. The resulting string may be
+        reused as a CREATE statement.
+        Useful after CASSANDRA-13910, which removed read_repair_chance
+        options from CREATE statements but did not remove them completely
+        from the system.
+        """
+        describe_statement = re.sub(r"( AND)? (dclocal_)?read_repair_chance = [\d\.]+", "", describe_statement)
+        describe_statement = re.sub(r"WITH[\s]*;", "", describe_statement)
+        return describe_statement
+
     def test_no_color_output(self):
         env = self.default_env
         for termname in ('', 'dumb', 'vt100'):
@@ -119,7 +144,7 @@ class TestCqlshOutput(BaseTestCase):
             output = output.splitlines()
             for line in output:
                 self.assertNoHasColors(line)
-                self.assertNotRegexpMatches(line, r'^cqlsh\S*>')
+                self.assertNotRegex(line, r'^cqlsh\S*>')
             self.assertEqual(len(output), 6,
                              msg='output: %r' % '\n'.join(output))
             self.assertEqual(output[0], '')
@@ -288,9 +313,9 @@ class TestCqlshOutput(BaseTestCase):
         # same query should show up as empty in cql 3
         self.assertQueriesGiveColoredOutput((
             (q, """
-             num | asciicol | bigintcol | blobcol | booleancol | decimalcol | doublecol | floatcol | intcol | smallintcol | textcol | timestampcol | tinyintcol | uuidcol | varcharcol | varintcol
-             RRR   MMMMMMMM   MMMMMMMMM   MMMMMMM   MMMMMMMMMM   MMMMMMMMMM   MMMMMMMMM   MMMMMMMM   MMMMMM   MMMMMMMMMMM   MMMMMMM   MMMMMMMMMMMM   MMMMMMMMMM   MMMMMMM   MMMMMMMMMM   MMMMMMMMM
-            -----+----------+-----------+---------+------------+------------+-----------+----------+--------+-------------+---------+--------------+------------+---------+------------+-----------
+             num | asciicol | bigintcol | blobcol | booleancol | decimalcol | doublecol | durationcol | floatcol | intcol | smallintcol | textcol | timestampcol | tinyintcol | uuidcol | varcharcol | varintcol
+             RRR   MMMMMMMM   MMMMMMMMM   MMMMMMM   MMMMMMMMMM   MMMMMMMMMM   MMMMMMMMM   MMMMMMMMMMM   MMMMMMMM   MMMMMM   MMMMMMMMMMM   MMMMMMM   MMMMMMMMMMMM   MMMMMMMMMM   MMMMMMM   MMMMMMMMMM   MMMMMMMMM
+            -----+----------+-----------+---------+------------+------------+-----------+-------------+----------+--------+-------------+---------+--------------+------------+---------+------------+-----------
 
 
             (0 rows)
@@ -494,7 +519,7 @@ class TestCqlshOutput(BaseTestCase):
         env = self.default_env
         env['LANG'] = 'en_US.UTF-8'
         self.assertQueriesGiveColoredOutput((
-            ("select * from utf8_with_special_chars where k in (0, 1, 2, 3, 4, 5, 6);", u"""
+            ("select * from utf8_with_special_chars where k in (0, 1, 2, 3, 4, 5, 6);", """
              k | val
              R   MMM
             ---+-------------------------------
@@ -517,7 +542,7 @@ class TestCqlshOutput(BaseTestCase):
 
             (7 rows)
             nnnnnnnn
-            """.encode('utf-8')),
+            """),
         ), env=env)
 
     def test_blob_output(self):
@@ -539,7 +564,29 @@ class TestCqlshOutput(BaseTestCase):
 
             (4 rows)
             nnnnnnnn
-            """),
+            """, ),
+        ))
+
+    def test_duration_output(self):
+        self.assertQueriesGiveColoredOutput((
+            ("select num, durationcol from has_all_types where num in (0, 1, 2, 3);", r"""
+             num | durationcol
+             RRR   MMMMMMMMMMM
+            -----+----------------
+
+               0 |            12h
+               G   GGGGGGGGGGGGGG
+               1 |         12h30m
+               G   GGGGGGGGGGGGGG
+               2 |      12h30m30s
+               G   GGGGGGGGGGGGGG
+               3 | 12h30m30s250ms
+               G   GGGGGGGGGGGGGG
+
+
+            (4 rows)
+            nnnnnnnn
+            """, ),
         ))
 
     def test_prompt(self):
@@ -595,19 +642,20 @@ class TestCqlshOutput(BaseTestCase):
             with cassandra_cursor() as curs:
                 try:
                     for stmt in statements:
+                        stmt = self.strip_read_repair_chance(stmt)
                         cqlshlog.debug('TEST EXEC: %s' % stmt)
                         curs.execute(stmt)
                 finally:
                     curs.execute('use system')
                     if do_drop:
-                        curs.execute('drop keyspace %s' % quote_name(new_ks_name))
+                        curs.execute('drop keyspace {}'.format(new_ks_name))
 
     def check_describe_keyspace_output(self, output, qksname):
         expected_bits = [r'(?im)^CREATE KEYSPACE %s WITH\b' % re.escape(qksname),
                          r';\s*$',
                          r'\breplication = {\'class\':']
         for expr in expected_bits:
-            self.assertRegexpMatches(output, expr)
+            self.assertRegex(output, expr)
 
     def test_describe_columnfamily_output(self):
         # we can change these to regular expressions if/when it makes sense
@@ -615,7 +663,6 @@ class TestCqlshOutput(BaseTestCase):
 
         # note columns are now comparator-ordered instead of original-order.
         table_desc3 = dedent("""
-
             CREATE TABLE %s.has_all_types (
                 num int PRIMARY KEY,
                 asciicol ascii,
@@ -624,6 +671,7 @@ class TestCqlshOutput(BaseTestCase):
                 booleancol boolean,
                 decimalcol decimal,
                 doublecol double,
+                durationcol duration,
                 floatcol float,
                 intcol int,
                 smallintcol smallint,
@@ -633,39 +681,36 @@ class TestCqlshOutput(BaseTestCase):
                 uuidcol uuid,
                 varcharcol text,
                 varintcol varint
-            ) WITH bloom_filter_fp_chance = 0.01
+            ) WITH additional_write_policy = '99p'
+                AND bloom_filter_fp_chance = 0.01
                 AND caching = {'keys': 'ALL', 'rows_per_partition': 'NONE'}
                 AND cdc = false
                 AND comment = ''
                 AND compaction = {'class': 'org.apache.cassandra.db.compaction.SizeTieredCompactionStrategy', 'max_threshold': '32', 'min_threshold': '4'}
-                AND compression = {'chunk_length_in_kb': '64', 'class': 'org.apache.cassandra.io.compress.LZ4Compressor'}
+                AND compression = {'chunk_length_in_kb': '16', 'class': 'org.apache.cassandra.io.compress.LZ4Compressor'}
                 AND crc_check_chance = 1.0
-                AND dclocal_read_repair_chance = 0.1
                 AND default_time_to_live = 0
+                AND extensions = {}
                 AND gc_grace_seconds = 864000
                 AND max_index_interval = 2048
                 AND memtable_flush_period_in_ms = 0
                 AND min_index_interval = 128
-                AND read_repair_chance = 0.0
-                AND speculative_retry = '99PERCENTILE';
-
-        """ % quote_name(get_keyspace()))
+                AND read_repair = 'BLOCKING'
+                AND speculative_retry = '99p';""" % quote_name(get_keyspace()))
 
         with testrun_cqlsh(tty=True, env=self.default_env) as c:
             for cmdword in ('describe table', 'desc columnfamily'):
                 for semicolon in (';', ''):
                     output = c.cmd_and_response('%s has_all_types%s' % (cmdword, semicolon))
                     self.assertNoHasColors(output)
-                    self.assertSequenceEqual(output.split('\n'), table_desc3.split('\n'))
+                    self.assertSequenceEqual(dedent(output).split('\n'), table_desc3.split('\n'))
 
     def test_describe_columnfamilies_output(self):
         output_re = r'''
-            \n
-            Keyspace [ ] (?P<ksname> \S+ ) \n
+            \n? Keyspace [ ] (?P<ksname> \S+ ) \n
             -----------* \n
-            (?P<cfnames> .*? )
-            \n
-        '''
+            (?P<cfnames> ( ( ["']?\w+["']? [^\w\n]* )+ \n )* )
+            '''
 
         ks = get_keyspace()
 
@@ -677,7 +722,7 @@ class TestCqlshOutput(BaseTestCase):
                     ksnames = []
                     output = c.cmd_and_response(cmdword + semicolon)
                     self.assertNoHasColors(output)
-                    self.assertRegexpMatches(output, '(?xs) ^ ( %s )+ $' % output_re)
+                    self.assertRegex(output, '(?xs) ^ ( %s )+ $' % output_re)
 
                     for section in re.finditer('(?xs)' + output_re, output):
                         ksname = section.group('ksname')
@@ -709,13 +754,14 @@ class TestCqlshOutput(BaseTestCase):
             \n
             Cluster: [ ] (?P<clustername> .* ) \n
             Partitioner: [ ] (?P<partitionername> .* ) \n
+            Snitch: [ ] (?P<snitchname> .* ) \n
             \n
         '''
 
         ringinfo_re = r'''
             Range[ ]ownership: \n
             (
-              [ ] .*? [ ][ ] \[ ( \d+ \. ){3} \d+ \] \n
+              [ ] .*? [ ][ ] \[ .*? / ( \d+ \. ){3} \d+ : \d+ \] \n
             )+
             \n
         '''
@@ -726,7 +772,7 @@ class TestCqlshOutput(BaseTestCase):
             for semicolon in ('', ';'):
                 output = c.cmd_and_response('describe cluster' + semicolon)
                 self.assertNoHasColors(output)
-                self.assertRegexpMatches(output, output_re + '$')
+                self.assertRegex(output, output_re + '$')
 
             c.send('USE %s;\n' % quote_name(get_keyspace()))
             c.read_to_next_prompt()
@@ -734,29 +780,47 @@ class TestCqlshOutput(BaseTestCase):
             for semicolon in ('', ';'):
                 output = c.cmd_and_response('describe cluster' + semicolon)
                 self.assertNoHasColors(output)
-                self.assertRegexpMatches(output, output_re + ringinfo_re + '$')
+                self.assertRegex(output, output_re + ringinfo_re + '$')
 
     def test_describe_schema_output(self):
-        with testrun_cqlsh(tty=True) as c:
+        with testrun_cqlsh(tty=True, env=self.default_env) as c:
             for semicolon in ('', ';'):
                 output = c.cmd_and_response('desc full schema' + semicolon)
                 self.assertNoHasColors(output)
-                self.assertRegexpMatches(output, '^\nCREATE KEYSPACE')
-                self.assertIn("\nCREATE KEYSPACE system WITH replication = {'class': 'LocalStrategy'}  AND durable_writes = true;\n",
+                # Since CASSANDRA-7622 'DESC FULL SCHEMA' also shows all VIRTUAL keyspaces
+                self.assertIn('VIRTUAL KEYSPACE system_virtual_schema', output)
+                self.assertIn("\nCREATE KEYSPACE system_auth WITH replication = {'class': 'SimpleStrategy', 'replication_factor': '1'}  AND durable_writes = true;\n",
                               output)
-                self.assertRegexpMatches(output, ';\s*$')
+                self.assertRegex(output, '.*\s*$')
 
     def test_show_output(self):
         with testrun_cqlsh(tty=True, env=self.default_env) as c:
             output = c.cmd_and_response('show version;')
-            self.assertRegexpMatches(output,
+            self.assertRegex(output,
                     '^\[cqlsh \S+ \| Cassandra \S+ \| CQL spec \S+ \| Native protocol \S+\]$')
 
             output = c.cmd_and_response('show host;')
             self.assertHasColors(output)
-            self.assertRegexpMatches(output, '^Connected to .* at %s:%d\.$'
+            self.assertRegex(output, '^Connected to .* at %s:%d$'
                                              % (re.escape(TEST_HOST), TEST_PORT))
 
+    @unittest.skipIf(six.PY3, 'Will not emit warning when running Python 3')
+    def test_warn_py2(self):
+        # has the warning
+        with testrun_cqlsh(tty=True, env=self.default_env) as c:
+            self.assertIn('Python 2.7 support is deprecated.', c.output_header, 'cqlsh did not output expected warning.')
+
+        # can suppress
+        env = self.default_env.copy()
+        env['CQLSH_NO_WARN_PY2'] = '1'
+        with testrun_cqlsh(tty=True, env=env) as c:
+            self.assertNotIn('Python 2.7 support is deprecated.', c.output_header, 'cqlsh did not output expected warning.')
+
+    @unittest.skipIf(six.PY2, 'Warning will be emitted when running Python 2.7')
+    def test_no_warn_py3(self):
+        with testrun_cqlsh(tty=True, env=self.default_env) as c:
+            self.assertNotIn('Python 2.7 support is deprecated.', c.output_header, 'cqlsh did not output expected warning.')
+
     @unittest.skipIf(sys.platform == "win32", 'EOF signaling not supported on Windows')
     def test_eof_prints_newline(self):
         with testrun_cqlsh(tty=True, env=self.default_env) as c:
@@ -824,6 +888,7 @@ class TestCqlshOutput(BaseTestCase):
             nnnnnnnn
             """),
         ))
+
         self.assertQueriesGiveColoredOutput((
             ("select phone_numbers from users;", r"""
              phone_numbers
@@ -856,6 +921,7 @@ class TestCqlshOutput(BaseTestCase):
             nnnnnnnn
             """),
         ))
+
         self.assertQueriesGiveColoredOutput((
             ("select tags from songs;", r"""
              tags
@@ -885,3 +951,13 @@ class TestCqlshOutput(BaseTestCase):
             nnnnnnnn
             """),
         ))
+
+    def test_expanded_output_counts_past_page(self):
+        query = "PAGING 5; EXPAND ON; SELECT * FROM twenty_rows_table;"
+        output, result = testcall_cqlsh(prompt=None, env=self.default_env,
+                                        tty=False, input=query)
+        self.assertEqual(0, result)
+        # format is "@ Row 1"
+        row_headers = [s for s in output.splitlines() if "@ Row" in s]
+        row_ids = [int(s.split(' ')[2]) for s in row_headers]
+        self.assertEqual([i for i in range(1, 21)], row_ids)
