@@ -29,15 +29,14 @@ import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
 import org.apache.cassandra.config.DatabaseDescriptor;
-import org.apache.cassandra.config.Schema;
-import org.apache.cassandra.config.SchemaConstants;
+import org.apache.cassandra.exceptions.RequestExecutionException;
+import org.apache.cassandra.schema.SchemaConstants;
 import org.apache.cassandra.cql3.QueryOptions;
 import org.apache.cassandra.cql3.QueryProcessor;
 import org.apache.cassandra.cql3.UntypedResultSet;
 import org.apache.cassandra.cql3.statements.SelectStatement;
 import org.apache.cassandra.exceptions.AuthenticationException;
 import org.apache.cassandra.exceptions.ConfigurationException;
-import org.apache.cassandra.exceptions.RequestExecutionException;
 import org.apache.cassandra.service.ClientState;
 import org.apache.cassandra.service.QueryState;
 import org.apache.cassandra.transport.messages.ResultMessage;
@@ -69,9 +68,6 @@ public class PasswordAuthenticator implements IAuthenticator
     static final byte NUL = 0;
     private SelectStatement authenticateStatement;
 
-    public static final String LEGACY_CREDENTIALS_TABLE = "credentials";
-    private SelectStatement legacyAuthenticateStatement;
-
     private CredentialsCache cache;
 
     // No anonymous access.
@@ -103,17 +99,15 @@ public class PasswordAuthenticator implements IAuthenticator
         return new AuthenticatedUser(username);
     }
 
-    private String queryHashedPassword(String username) throws AuthenticationException
+    private String queryHashedPassword(String username)
     {
         try
         {
-            SelectStatement authenticationStatement = authenticationStatement();
-
             ResultMessage.Rows rows =
-                authenticationStatement.execute(QueryState.forInternalCalls(),
-                                                QueryOptions.forInternalCalls(consistencyForRole(username),
-                                                                              Lists.newArrayList(ByteBufferUtil.bytes(username))),
-                                                System.nanoTime());
+            authenticateStatement.execute(QueryState.forInternalCalls(),
+                                            QueryOptions.forInternalCalls(consistencyForRole(username),
+                                                                          Lists.newArrayList(ByteBufferUtil.bytes(username))),
+                                            System.nanoTime());
 
             // If either a non-existent role name was supplied, or no credentials
             // were found for that role we don't want to cache the result so we throw
@@ -133,25 +127,6 @@ public class PasswordAuthenticator implements IAuthenticator
         }
     }
 
-    /**
-     * If the legacy users table exists try to verify credentials there. This is to handle the case
-     * where the cluster is being upgraded and so is running with mixed versions of the authn tables
-     */
-    private SelectStatement authenticationStatement()
-    {
-        if (Schema.instance.getCFMetaData(SchemaConstants.AUTH_KEYSPACE_NAME, LEGACY_CREDENTIALS_TABLE) == null)
-            return authenticateStatement;
-        else
-        {
-            // the statement got prepared, we to try preparing it again.
-            // If the credentials was initialised only after statement got prepared, re-prepare (CASSANDRA-12813).
-            if (legacyAuthenticateStatement == null)
-                prepareLegacyAuthenticateStatement();
-            return legacyAuthenticateStatement;
-        }
-    }
-
-
     public Set<DataResource> protectedResources()
     {
         // Also protected by CassandraRoleManager, but the duplication doesn't hurt and is more explicit
@@ -170,21 +145,9 @@ public class PasswordAuthenticator implements IAuthenticator
                                      AuthKeyspace.ROLES);
         authenticateStatement = prepare(query);
 
-        if (Schema.instance.getCFMetaData(SchemaConstants.AUTH_KEYSPACE_NAME, LEGACY_CREDENTIALS_TABLE) != null)
-            prepareLegacyAuthenticateStatement();
-
         cache = new CredentialsCache(this);
     }
 
-    private void prepareLegacyAuthenticateStatement()
-    {
-        String query = String.format("SELECT %s from %s.%s WHERE username = ?",
-                                     SALTED_HASH,
-                                     SchemaConstants.AUTH_KEYSPACE_NAME,
-                                     LEGACY_CREDENTIALS_TABLE);
-        legacyAuthenticateStatement = prepare(query);
-    }
-
     public AuthenticatedUser legacyAuthenticate(Map<String, String> credentials) throws AuthenticationException
     {
         String username = credentials.get(USERNAME_KEY);
@@ -205,7 +168,7 @@ public class PasswordAuthenticator implements IAuthenticator
 
     private static SelectStatement prepare(String query)
     {
-        return (SelectStatement) QueryProcessor.getStatement(query, ClientState.forInternalCalls()).statement;
+        return (SelectStatement) QueryProcessor.getStatement(query, ClientState.forInternalCalls());
     }
 
     private class PlainTextSaslAuthenticator implements SaslNegotiator
