@@ -24,6 +24,7 @@ import com.google.common.annotations.VisibleForTesting;
 
 import io.netty.util.concurrent.FastThreadLocal;
 import io.netty.util.concurrent.FastThreadLocalThread;
+import org.apache.cassandra.utils.memory.BufferPool;
 
 /**
  * This class is an implementation of the <i>ThreadFactory</i> interface. This
@@ -35,6 +36,10 @@ public class NamedThreadFactory implements ThreadFactory
 {
     private static volatile String globalPrefix;
     public static void setGlobalPrefix(String prefix) { globalPrefix = prefix; }
+    public static String globalPrefix() {
+        String prefix = globalPrefix;
+        return prefix == null ? "" : prefix;
+    }
 
     public final String id;
     private final int priority;
@@ -70,26 +75,6 @@ public class NamedThreadFactory implements ThreadFactory
         return thread;
     }
 
-    /**
-     * Ensures that {@link FastThreadLocal#remove() FastThreadLocal.remove()} is called when the {@link Runnable#run()}
-     * method of the given {@link Runnable} instance completes to ensure cleanup of {@link FastThreadLocal} instances.
-     * This is especially important for direct byte buffers allocated locally for a thread.
-     */
-    public static Runnable threadLocalDeallocator(Runnable r)
-    {
-        return () ->
-        {
-            try
-            {
-                r.run();
-            }
-            finally
-            {
-                FastThreadLocal.removeAll();
-            }
-        };
-    }
-
     private static final AtomicInteger threadCounter = new AtomicInteger();
 
     @VisibleForTesting
@@ -116,7 +101,7 @@ public class NamedThreadFactory implements ThreadFactory
     public static Thread createThread(ThreadGroup threadGroup, Runnable runnable, String name, boolean daemon)
     {
         String prefix = globalPrefix;
-        Thread thread = new FastThreadLocalThread(threadGroup, threadLocalDeallocator(runnable), prefix != null ? prefix + name : name);
+        Thread thread = new FastThreadLocalThread(threadGroup, runnable, prefix != null ? prefix + name : name);
         thread.setDaemon(daemon);
         return thread;
     }
