@@ -21,24 +21,35 @@ import java.util.ArrayList;
 import java.util.List;
 import java.util.concurrent.ConcurrentLinkedQueue;
 import java.util.concurrent.TimeUnit;
+import java.util.concurrent.atomic.AtomicInteger;
 import java.util.concurrent.atomic.AtomicLong;
 
-import org.apache.cassandra.metrics.SEPMetrics;
+import com.google.common.annotations.VisibleForTesting;
+
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
+
+import org.apache.cassandra.metrics.ThreadPoolMetrics;
+import org.apache.cassandra.utils.MBeanWrapper;
 import org.apache.cassandra.utils.concurrent.SimpleCondition;
 
 import static org.apache.cassandra.concurrent.SEPWorker.Work;
 
-public class SEPExecutor extends AbstractLocalAwareExecutorService
+public class SEPExecutor extends AbstractLocalAwareExecutorService implements SEPExecutorMBean
 {
+    private static final Logger logger = LoggerFactory.getLogger(SEPExecutor.class);
     private final SharedExecutorPool pool;
 
-    public final int maxWorkers;
+    private final AtomicInteger maximumPoolSize;
+    private final MaximumPoolSizeListener maximumPoolSizeListener;
     public final String name;
-    private final SEPMetrics metrics;
+    private final String mbeanName;
+    @VisibleForTesting
+    public final ThreadPoolMetrics metrics;
 
     // stores both a set of work permits and task permits:
     //  bottom 32 bits are number of queued tasks, in the range [0..maxTasksQueued]   (initially 0)
-    //  top 32 bits are number of work permits available in the range [0..maxWorkers]   (initially maxWorkers)
+    //  top 32 bits are number of work permits available in the range [-resizeDelta..maximumPoolSize]   (initially maximumPoolSize)
     private final AtomicLong permits = new AtomicLong();
 
     private final AtomicLong completedTasks = new AtomicLong();
@@ -49,13 +60,16 @@ public class SEPExecutor extends AbstractLocalAwareExecutorService
     // TODO: see if other queue implementations might improve throughput
     protected final ConcurrentLinkedQueue<FutureTask<?>> tasks = new ConcurrentLinkedQueue<>();
 
-    SEPExecutor(SharedExecutorPool pool, int maxWorkers, String jmxPath, String name)
+    SEPExecutor(SharedExecutorPool pool, int maximumPoolSize, MaximumPoolSizeListener maximumPoolSizeListener, String jmxPath, String name)
     {
         this.pool = pool;
         this.name = name;
-        this.maxWorkers = maxWorkers;
-        this.permits.set(combine(0, maxWorkers));
-        this.metrics = new SEPMetrics(this, jmxPath, name);
+        this.mbeanName = "org.apache.cassandra." + jmxPath + ":type=" + name;
+        this.maximumPoolSize = new AtomicInteger(maximumPoolSize);
+        this.maximumPoolSizeListener = maximumPoolSizeListener;
+        this.permits.set(combine(0, maximumPoolSize));
+        this.metrics = new ThreadPoolMetrics(this, jmxPath, name).register();
+        MBeanWrapper.instance.registerMBean(this, mbeanName);
     }
 
     protected void onCompletion()
@@ -63,6 +77,12 @@ public class SEPExecutor extends AbstractLocalAwareExecutorService
         completedTasks.incrementAndGet();
     }
 
+    @Override
+    public int getMaxTasksQueued()
+    {
+        return Integer.MAX_VALUE;
+    }
+
     // schedules another worker for this pool if there is work outstanding and there are no spinning threads that
     // will self-assign to it in the immediate future
     boolean maybeSchedule()
@@ -101,19 +121,42 @@ public class SEPExecutor extends AbstractLocalAwareExecutorService
         }
     }
 
+    public enum TakeTaskPermitResult
+    {
+        NONE_AVAILABLE,        // No task permits available
+        TOOK_PERMIT,           // Took a permit and reduced task permits
+        RETURNED_WORK_PERMIT   // Detected pool shrinking and returned work permit ahead of SEPWorker exit.
+    };
+
     // takes permission to perform a task, if any are available; once taken it is guaranteed
     // that a proceeding call to tasks.poll() will return some work
-    boolean takeTaskPermit()
+    TakeTaskPermitResult takeTaskPermit(boolean checkForWorkPermitOvercommit)
     {
+        TakeTaskPermitResult result;
         while (true)
         {
             long current = permits.get();
+            long updated;
+            int workPermits = workPermits(current);
             int taskPermits = taskPermits(current);
-            if (taskPermits == 0)
-                return false;
-            if (permits.compareAndSet(current, updateTaskPermits(current, taskPermits - 1)))
+            if (workPermits < 0 && checkForWorkPermitOvercommit)
             {
-                return true;
+                // Work permits are negative when the pool is reducing in size.  Atomically
+                // adjust the number of work permits so there is no race of multiple SEPWorkers
+                // exiting.  On conflicting update, recheck.
+                result = TakeTaskPermitResult.RETURNED_WORK_PERMIT;
+                updated = updateWorkPermits(current, workPermits + 1);
+            }
+            else
+            {
+                if (taskPermits == 0)
+                    return TakeTaskPermitResult.NONE_AVAILABLE;
+                result = TakeTaskPermitResult.TOOK_PERMIT;
+                updated = updateTaskPermits(current, taskPermits - 1);
+            }
+            if (permits.compareAndSet(current, updated))
+            {
+                return result;
             }
         }
     }
@@ -127,7 +170,7 @@ public class SEPExecutor extends AbstractLocalAwareExecutorService
             long current = permits.get();
             int workPermits = workPermits(current);
             int taskPermits = taskPermits(current);
-            if (workPermits == 0 || taskPermits == 0)
+            if (workPermits <= 0 || taskPermits == 0)
                 return false;
             if (permits.compareAndSet(current, combine(taskPermits - taskDelta, workPermits - 1)))
             {
@@ -144,14 +187,11 @@ public class SEPExecutor extends AbstractLocalAwareExecutorService
             long current = permits.get();
             int workPermits = workPermits(current);
             if (permits.compareAndSet(current, updateWorkPermits(current, workPermits + 1)))
-            {
-                if (shuttingDown && workPermits + 1 == maxWorkers)
-                    shutdown.signalAll();
-                break;
-            }
+                return;
         }
     }
 
+    @Override
     public void maybeExecuteImmediately(Runnable command)
     {
         FutureTask<?> ft = newTaskFor(command, null);
@@ -178,23 +218,24 @@ public class SEPExecutor extends AbstractLocalAwareExecutorService
 
     public synchronized void shutdown()
     {
+        if (shuttingDown)
+            return;
         shuttingDown = true;
         pool.executors.remove(this);
-        if (getActiveCount() == 0 && getPendingTasks() == 0)
+        if (getActiveTaskCount() == 0)
             shutdown.signalAll();
 
         // release metrics
         metrics.release();
+        MBeanWrapper.instance.unregisterMBean(mbeanName);
     }
 
     public synchronized List<Runnable> shutdownNow()
     {
         shutdown();
         List<Runnable> aborted = new ArrayList<>();
-        while (takeTaskPermit())
+        while (takeTaskPermit(false) == TakeTaskPermitResult.TOOK_PERMIT)
             aborted.add(tasks.poll());
-        if (getActiveCount() == 0)
-            shutdown.signalAll();
         return aborted;
     }
 
@@ -214,19 +255,67 @@ public class SEPExecutor extends AbstractLocalAwareExecutorService
         return isTerminated();
     }
 
-    public long getPendingTasks()
+    @Override
+    public int getPendingTaskCount()
     {
         return taskPermits(permits.get());
     }
 
-    public long getCompletedTasks()
+    @Override
+    public long getCompletedTaskCount()
     {
         return completedTasks.get();
     }
 
-    public int getActiveCount()
+    public int getActiveTaskCount()
     {
-        return maxWorkers - workPermits(permits.get());
+        return maximumPoolSize.get() - workPermits(permits.get());
+    }
+
+    public int getCorePoolSize()
+    {
+        return 0;
+    }
+
+    public void setCorePoolSize(int newCorePoolSize)
+    {
+        throw new IllegalArgumentException("Cannot resize core pool size of SEPExecutor");
+    }
+
+    @Override
+    public int getMaximumPoolSize()
+    {
+        return maximumPoolSize.get();
+    }
+
+    @Override
+    public synchronized void setMaximumPoolSize(int newMaximumPoolSize)
+    {
+        final int oldMaximumPoolSize = maximumPoolSize.get();
+
+        if (newMaximumPoolSize < 0)
+        {
+            throw new IllegalArgumentException("Maximum number of workers must not be negative");
+        }
+
+        int deltaWorkPermits = newMaximumPoolSize - oldMaximumPoolSize;
+        if (!maximumPoolSize.compareAndSet(oldMaximumPoolSize, newMaximumPoolSize))
+        {
+            throw new IllegalStateException("Maximum pool size has been changed while resizing");
+        }
+
+        if (deltaWorkPermits == 0)
+            return;
+
+        permits.updateAndGet(cur -> updateWorkPermits(cur, workPermits(cur) + deltaWorkPermits));
+        logger.info("Resized {} maximum pool size from {} to {}", name, oldMaximumPoolSize, newMaximumPoolSize);
+
+        // If we we have more work permits than before we should spin up a worker now rather than waiting
+        // until either a new task is enqueued (if all workers are descheduled) or a spinning worker calls
+        // maybeSchedule().
+        pool.maybeStartSpinningWorker();
+
+        maximumPoolSizeListener.onUpdateMaximumPoolSize(newMaximumPoolSize);
     }
 
     private static int taskPermits(long both)
@@ -234,9 +323,9 @@ public class SEPExecutor extends AbstractLocalAwareExecutorService
         return (int) both;
     }
 
-    private static int workPermits(long both)
+    private static int workPermits(long both) // may be negative if resizing
     {
-        return (int) (both >>> 32);
+        return (int) (both >> 32); // sign extending right shift
     }
 
     private static long updateTaskPermits(long prev, int taskPermits)
