@@ -17,6 +17,7 @@
  */
 package org.apache.cassandra.concurrent;
 
+import java.util.concurrent.ThreadLocalRandom;
 import java.util.concurrent.TimeUnit;
 import java.util.concurrent.atomic.AtomicReference;
 import java.util.concurrent.locks.LockSupport;
@@ -27,6 +28,9 @@ import org.slf4j.LoggerFactory;
 import io.netty.util.concurrent.FastThreadLocalThread;
 import org.apache.cassandra.utils.JVMStabilityInspector;
 
+import static org.apache.cassandra.concurrent.SEPExecutor.TakeTaskPermitResult.RETURNED_WORK_PERMIT;
+import static org.apache.cassandra.concurrent.SEPExecutor.TakeTaskPermitResult.TOOK_PERMIT;
+
 final class SEPWorker extends AtomicReference<SEPWorker.Work> implements Runnable
 {
     private static final Logger logger = LoggerFactory.getLogger(SEPWorker.class);
@@ -102,6 +106,8 @@ final class SEPWorker extends AtomicReference<SEPWorker.Work> implements Runnabl
                 // if we do have tasks assigned, nobody will change our state so we can simply set it to WORKING
                 // (which is also a state that will never be interrupted externally)
                 set(Work.WORKING);
+                boolean shutdown;
+                SEPExecutor.TakeTaskPermitResult status = null; // make sure set if shutdown check short circuits
                 while (true)
                 {
                     // before we process any task, we maybe schedule a new worker _to our executor only_; this
@@ -113,16 +119,28 @@ final class SEPWorker extends AtomicReference<SEPWorker.Work> implements Runnabl
                     task.run();
                     task = null;
 
-                    // if we're shutting down, or we fail to take a permit, we don't perform any more work
-                    if (!assigned.takeTaskPermit())
+                    if (shutdown = assigned.shuttingDown)
                         break;
+
+                    if (TOOK_PERMIT != (status = assigned.takeTaskPermit(true)))
+                        break;
+
                     task = assigned.tasks.poll();
                 }
 
                 // return our work permit, and maybe signal shutdown
-                assigned.returnWorkPermit();
+                if (status != RETURNED_WORK_PERMIT)
+                    assigned.returnWorkPermit();
+
+                if (shutdown)
+                {
+                    if (assigned.getActiveTaskCount() == 0)
+                        assigned.shutdown.signalAll();
+                    return;
+                }
                 assigned = null;
 
+
                 // try to immediately reassign ourselves some work; if we fail, start spinning
                 if (!selfAssign())
                     startSpinning();
@@ -131,24 +149,22 @@ final class SEPWorker extends AtomicReference<SEPWorker.Work> implements Runnabl
         catch (Throwable t)
         {
             JVMStabilityInspector.inspectThrowable(t);
-            if (task != null)
-                logger.error("Failed to execute task, unexpected exception killed worker: {}", t);
-            else
-                logger.error("Unexpected exception killed worker: {}", t);
-        }
-        finally
-        {
-            if (assigned != null)
-                assigned.returnWorkPermit();
-
-            do
+            while (true)
             {
                 if (get().assigned != null)
                 {
-                    get().assigned.returnWorkPermit();
+                    assigned = get().assigned;
                     set(Work.WORKING);
                 }
-            } while (!assign(Work.STOPPED, true));
+                if (assign(Work.STOPPED, true))
+                    break;
+            }
+            if (assigned != null)
+                assigned.returnWorkPermit();
+            if (task != null)
+                logger.error("Failed to execute task, unexpected exception killed worker", t);
+            else
+                logger.error("Unexpected exception killed worker", t);
         }
     }
 
@@ -238,7 +254,7 @@ final class SEPWorker extends AtomicReference<SEPWorker.Work> implements Runnabl
         // we should always have a thread about to wake up, but most threads are sleeping
         long sleep = 10000L * pool.spinningCount.get();
         sleep = Math.min(1000000, sleep);
-        sleep *= Math.random();
+        sleep *= ThreadLocalRandom.current().nextDouble();
         sleep = Math.max(10000, sleep);
 
         long start = System.nanoTime();
