@@ -20,12 +20,13 @@ package org.apache.cassandra.cql3;
 import java.util.ArrayList;
 import java.util.Collections;
 import java.util.List;
+import java.util.Objects;
 import java.util.stream.Collectors;
 
 import com.google.common.base.Joiner;
 
-import org.apache.cassandra.config.CFMetaData;
-import org.apache.cassandra.config.ColumnDefinition;
+import org.apache.cassandra.schema.ColumnMetadata;
+import org.apache.cassandra.schema.TableMetadata;
 import org.apache.cassandra.cql3.Term.Raw;
 import org.apache.cassandra.cql3.restrictions.Restriction;
 import org.apache.cassandra.cql3.restrictions.TokenRestriction;
@@ -47,11 +48,11 @@ import static org.apache.cassandra.cql3.statements.RequestValidations.invalidReq
  */
 public final class TokenRelation extends Relation
 {
-    private final List<ColumnDefinition.Raw> entities;
+    private final List<ColumnIdentifier> entities;
 
     private final Term.Raw value;
 
-    public TokenRelation(List<ColumnDefinition.Raw> entities, Operator type, Term.Raw value)
+    public TokenRelation(List<ColumnIdentifier> entities, Operator type, Term.Raw value)
     {
         this.entities = entities;
         this.relationType = type;
@@ -75,44 +76,44 @@ public final class TokenRelation extends Relation
     }
 
     @Override
-    protected Restriction newEQRestriction(CFMetaData cfm, VariableSpecifications boundNames) throws InvalidRequestException
+    protected Restriction newEQRestriction(TableMetadata table, VariableSpecifications boundNames)
     {
-        List<ColumnDefinition> columnDefs = getColumnDefinitions(cfm);
-        Term term = toTerm(toReceivers(cfm, columnDefs), value, cfm.ksName, boundNames);
-        return new TokenRestriction.EQRestriction(cfm, columnDefs, term);
+        List<ColumnMetadata> columnDefs = getColumnDefinitions(table);
+        Term term = toTerm(toReceivers(table, columnDefs), value, table.keyspace, boundNames);
+        return new TokenRestriction.EQRestriction(table, columnDefs, term);
     }
 
     @Override
-    protected Restriction newINRestriction(CFMetaData cfm, VariableSpecifications boundNames) throws InvalidRequestException
+    protected Restriction newINRestriction(TableMetadata table, VariableSpecifications boundNames)
     {
         throw invalidRequest("%s cannot be used with the token function", operator());
     }
 
     @Override
-    protected Restriction newSliceRestriction(CFMetaData cfm,
+    protected Restriction newSliceRestriction(TableMetadata table,
                                               VariableSpecifications boundNames,
                                               Bound bound,
-                                              boolean inclusive) throws InvalidRequestException
+                                              boolean inclusive)
     {
-        List<ColumnDefinition> columnDefs = getColumnDefinitions(cfm);
-        Term term = toTerm(toReceivers(cfm, columnDefs), value, cfm.ksName, boundNames);
-        return new TokenRestriction.SliceRestriction(cfm, columnDefs, bound, inclusive, term);
+        List<ColumnMetadata> columnDefs = getColumnDefinitions(table);
+        Term term = toTerm(toReceivers(table, columnDefs), value, table.keyspace, boundNames);
+        return new TokenRestriction.SliceRestriction(table, columnDefs, bound, inclusive, term);
     }
 
     @Override
-    protected Restriction newContainsRestriction(CFMetaData cfm, VariableSpecifications boundNames, boolean isKey) throws InvalidRequestException
+    protected Restriction newContainsRestriction(TableMetadata table, VariableSpecifications boundNames, boolean isKey)
     {
         throw invalidRequest("%s cannot be used with the token function", operator());
     }
 
     @Override
-    protected Restriction newIsNotRestriction(CFMetaData cfm, VariableSpecifications boundNames) throws InvalidRequestException
+    protected Restriction newIsNotRestriction(TableMetadata table, VariableSpecifications boundNames)
     {
         throw invalidRequest("%s cannot be used with the token function", operator());
     }
 
     @Override
-    protected Restriction newLikeRestriction(CFMetaData cfm, VariableSpecifications boundNames, Operator operator) throws InvalidRequestException
+    protected Restriction newLikeRestriction(TableMetadata table, VariableSpecifications boundNames, Operator operator)
     {
         throw invalidRequest("%s cannot be used with the token function", operator);
     }
@@ -128,66 +129,86 @@ public final class TokenRelation extends Relation
         return term;
     }
 
-    public Relation renameIdentifier(ColumnDefinition.Raw from, ColumnDefinition.Raw to)
+    @Override
+    public Relation renameIdentifier(ColumnIdentifier from, ColumnIdentifier to)
     {
         if (!entities.contains(from))
             return this;
 
-        List<ColumnDefinition.Raw> newEntities = entities.stream().map(e -> e.equals(from) ? to : e).collect(Collectors.toList());
+        List<ColumnIdentifier> newEntities = entities.stream().map(e -> e.equals(from) ? to : e).collect(Collectors.toList());
         return new TokenRelation(newEntities, operator(), value);
     }
 
     @Override
-    public String toString()
+    public String toCQLString()
     {
-        return String.format("token%s %s %s", Tuples.tupleToString(entities), relationType, value);
+        return String.format("token%s %s %s", Tuples.tupleToString(entities, ColumnIdentifier::toCQLString), relationType, value);
+    }
+
+    @Override
+    public int hashCode()
+    {
+        return Objects.hash(relationType, entities, value);
+    }
+
+    @Override
+    public boolean equals(Object o)
+    {
+        if (this == o)
+            return true;
+
+        if (!(o instanceof TokenRelation))
+            return false;
+
+        TokenRelation tr = (TokenRelation) o;
+        return relationType.equals(tr.relationType) && entities.equals(tr.entities) && value.equals(tr.value);
     }
 
     /**
      * Returns the definition of the columns to which apply the token restriction.
      *
-     * @param cfm the column family metadata
+     * @param table the table metadata
      * @return the definition of the columns to which apply the token restriction.
      * @throws InvalidRequestException if the entity cannot be resolved
      */
-    private List<ColumnDefinition> getColumnDefinitions(CFMetaData cfm) throws InvalidRequestException
+    private List<ColumnMetadata> getColumnDefinitions(TableMetadata table)
     {
-        List<ColumnDefinition> columnDefs = new ArrayList<>(entities.size());
-        for ( ColumnDefinition.Raw raw : entities)
-            columnDefs.add(raw.prepare(cfm));
+        List<ColumnMetadata> columnDefs = new ArrayList<>(entities.size());
+        for (ColumnIdentifier id : entities)
+            columnDefs.add(table.getExistingColumn(id));
         return columnDefs;
     }
 
     /**
      * Returns the receivers for this relation.
      *
-     * @param cfm the Column Family meta data
+     * @param table the table meta data
      * @param columnDefs the column definitions
      * @return the receivers for the specified relation.
      * @throws InvalidRequestException if the relation is invalid
      */
-    private static List<? extends ColumnSpecification> toReceivers(CFMetaData cfm,
-                                                                   List<ColumnDefinition> columnDefs)
+    private static List<? extends ColumnSpecification> toReceivers(TableMetadata table,
+                                                                   List<ColumnMetadata> columnDefs)
                                                                    throws InvalidRequestException
     {
 
-        if (!columnDefs.equals(cfm.partitionKeyColumns()))
+        if (!columnDefs.equals(table.partitionKeyColumns()))
         {
-            checkTrue(columnDefs.containsAll(cfm.partitionKeyColumns()),
+            checkTrue(columnDefs.containsAll(table.partitionKeyColumns()),
                       "The token() function must be applied to all partition key components or none of them");
 
             checkContainsNoDuplicates(columnDefs, "The token() function contains duplicate partition key components");
 
-            checkContainsOnly(columnDefs, cfm.partitionKeyColumns(), "The token() function must contains only partition key components");
+            checkContainsOnly(columnDefs, table.partitionKeyColumns(), "The token() function must contains only partition key components");
 
             throw invalidRequest("The token function arguments must be in the partition key order: %s",
-                                 Joiner.on(", ").join(ColumnDefinition.toIdentifiers(cfm.partitionKeyColumns())));
+                                 Joiner.on(", ").join(ColumnMetadata.toIdentifiers(table.partitionKeyColumns())));
         }
 
-        ColumnDefinition firstColumn = columnDefs.get(0);
+        ColumnMetadata firstColumn = columnDefs.get(0);
         return Collections.singletonList(new ColumnSpecification(firstColumn.ksName,
                                                                  firstColumn.cfName,
                                                                  new ColumnIdentifier("partition key token", true),
-                                                                 cfm.partitioner.getTokenValidator()));
+                                                                 table.partitioner.getTokenValidator()));
     }
 }
