@@ -24,18 +24,19 @@ import java.util.*;
 
 import com.google.common.annotations.VisibleForTesting;
 
-import org.apache.cassandra.service.ClientState;
-import org.apache.cassandra.transport.ProtocolVersion;
-import org.apache.cassandra.utils.AbstractIterator;
-
-import org.apache.cassandra.config.CFMetaData;
-import org.apache.cassandra.config.ColumnDefinition;
+import com.datastax.driver.core.CodecUtils;
+import org.apache.cassandra.cql3.functions.types.LocalDate;
+import org.apache.cassandra.schema.ColumnMetadata;
 import org.apache.cassandra.cql3.statements.SelectStatement;
 import org.apache.cassandra.db.*;
-import org.apache.cassandra.db.rows.*;
-import org.apache.cassandra.db.partitions.PartitionIterator;
 import org.apache.cassandra.db.marshal.*;
+import org.apache.cassandra.db.partitions.PartitionIterator;
+import org.apache.cassandra.db.rows.*;
+import org.apache.cassandra.schema.TableMetadata;
+import org.apache.cassandra.service.ClientState;
 import org.apache.cassandra.service.pager.QueryPager;
+import org.apache.cassandra.transport.ProtocolVersion;
+import org.apache.cassandra.utils.AbstractIterator;
 import org.apache.cassandra.utils.FBUtilities;
 
 /** a utility for doing internal cql-based queries */
@@ -123,71 +124,6 @@ public abstract class UntypedResultSet implements Iterable<UntypedResultSet.Row>
         }
     }
 
-    /**
-     * Pager that calls `execute` rather than `executeInternal`
-     */
-    private static class FromDistributedPager extends UntypedResultSet
-    {
-        private final SelectStatement select;
-        private final ConsistencyLevel cl;
-        private final ClientState clientState;
-        private final QueryPager pager;
-        private final int pageSize;
-        private final List<ColumnSpecification> metadata;
-
-        private FromDistributedPager(SelectStatement select,
-                                     ConsistencyLevel cl,
-                                     ClientState clientState,
-                                     QueryPager pager, int pageSize)
-        {
-            this.select = select;
-            this.cl = cl;
-            this.clientState = clientState;
-            this.pager = pager;
-            this.pageSize = pageSize;
-            this.metadata = select.getResultMetadata().requestNames();
-        }
-
-        public int size()
-        {
-            throw new UnsupportedOperationException();
-        }
-
-        public Row one()
-        {
-            throw new UnsupportedOperationException();
-        }
-
-        public Iterator<Row> iterator()
-        {
-            return new AbstractIterator<Row>()
-            {
-                private Iterator<List<ByteBuffer>> currentPage;
-
-                protected Row computeNext()
-                {
-                    int nowInSec = FBUtilities.nowInSeconds();
-                    while (currentPage == null || !currentPage.hasNext())
-                    {
-                        if (pager.isExhausted())
-                            return endOfData();
-
-                        try (PartitionIterator iter = pager.fetchPage(pageSize, cl, clientState, System.nanoTime()))
-                        {
-                            currentPage = select.process(iter, nowInSec).rows.iterator();
-                        }
-                    }
-                    return new Row(metadata, currentPage.next());
-                }
-            };
-        }
-
-        public List<ColumnSpecification> metadata()
-        {
-            return metadata;
-        }
-    }
-
     private static class FromResultList extends UntypedResultSet
     {
         private final List<Map<String, ByteBuffer>> cqlRows;
@@ -286,6 +222,71 @@ public abstract class UntypedResultSet implements Iterable<UntypedResultSet.Row>
         }
     }
 
+    /**
+     * Pager that calls `execute` rather than `executeInternal`
+     */
+    private static class FromDistributedPager extends UntypedResultSet
+    {
+        private final SelectStatement select;
+        private final ConsistencyLevel cl;
+        private final ClientState clientState;
+        private final QueryPager pager;
+        private final int pageSize;
+        private final List<ColumnSpecification> metadata;
+
+        private FromDistributedPager(SelectStatement select,
+                                     ConsistencyLevel cl,
+                                     ClientState clientState,
+                                     QueryPager pager, int pageSize)
+        {
+            this.select = select;
+            this.cl = cl;
+            this.clientState = clientState;
+            this.pager = pager;
+            this.pageSize = pageSize;
+            this.metadata = select.getResultMetadata().requestNames();
+        }
+
+        public int size()
+        {
+            throw new UnsupportedOperationException();
+        }
+
+        public Row one()
+        {
+            throw new UnsupportedOperationException();
+        }
+
+        public Iterator<Row> iterator()
+        {
+            return new AbstractIterator<Row>()
+            {
+                private Iterator<List<ByteBuffer>> currentPage;
+
+                protected Row computeNext()
+                {
+                    int nowInSec = FBUtilities.nowInSeconds();
+                    while (currentPage == null || !currentPage.hasNext())
+                    {
+                        if (pager.isExhausted())
+                            return endOfData();
+
+                        try (PartitionIterator iter = pager.fetchPage(pageSize, cl, clientState, System.nanoTime()))
+                        {
+                            currentPage = select.process(iter, nowInSec).rows.iterator();
+                        }
+                    }
+                    return new Row(metadata, currentPage.next());
+                }
+            };
+        }
+
+        public List<ColumnSpecification> metadata()
+        {
+            return metadata;
+        }
+    }
+
     public static class Row
     {
         private final Map<String, ByteBuffer> data = new HashMap<>();
@@ -303,25 +304,25 @@ public abstract class UntypedResultSet implements Iterable<UntypedResultSet.Row>
                 data.put(names.get(i).name.toString(), columns.get(i));
         }
 
-        public static Row fromInternalRow(CFMetaData metadata, DecoratedKey key, org.apache.cassandra.db.rows.Row row)
+        public static Row fromInternalRow(TableMetadata metadata, DecoratedKey key, org.apache.cassandra.db.rows.Row row)
         {
             Map<String, ByteBuffer> data = new HashMap<>();
 
             ByteBuffer[] keyComponents = SelectStatement.getComponents(metadata, key);
-            for (ColumnDefinition def : metadata.partitionKeyColumns())
+            for (ColumnMetadata def : metadata.partitionKeyColumns())
                 data.put(def.name.toString(), keyComponents[def.position()]);
 
-            Clustering clustering = row.clustering();
-            for (ColumnDefinition def : metadata.clusteringColumns())
-                data.put(def.name.toString(), clustering.get(def.position()));
+            Clustering<?> clustering = row.clustering();
+            for (ColumnMetadata def : metadata.clusteringColumns())
+                data.put(def.name.toString(), clustering.bufferAt(def.position()));
 
-            for (ColumnDefinition def : metadata.partitionColumns())
+            for (ColumnMetadata def : metadata.regularAndStaticColumns())
             {
                 if (def.isSimple())
                 {
-                    Cell cell = row.getCell(def);
+                    Cell<?> cell = row.getCell(def);
                     if (cell != null)
-                        data.put(def.name.toString(), cell.value());
+                        data.put(def.name.toString(), cell.buffer());
                 }
                 else
                 {
@@ -405,6 +406,8 @@ public abstract class UntypedResultSet implements Iterable<UntypedResultSet.Row>
             return TimestampType.instance.compose(data.get(column));
         }
 
+        public LocalDate getDate(String column) { return LocalDate.fromDaysSinceEpoch(CodecUtils.fromUnsignedToSignedInt(data.get(column).getInt()));}
+
         public long getLong(String column)
         {
             return LongType.instance.compose(data.get(column));
