@@ -15,44 +15,73 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-
 package org.apache.cassandra.cql3.restrictions;
 
-import org.apache.cassandra.config.CFMetaData;
+import java.util.Objects;
+
 import org.apache.cassandra.cql3.*;
 import org.apache.cassandra.db.filter.RowFilter;
 import org.apache.cassandra.db.marshal.AbstractType;
+import org.apache.cassandra.schema.TableMetadata;
 
 public class CustomIndexExpression
 {
     private final ColumnIdentifier valueColId = new ColumnIdentifier("custom index expression", false);
 
-    public final IndexName targetIndex;
+    public final QualifiedName targetIndex;
     public final Term.Raw valueRaw;
 
     private Term value;
 
-    public CustomIndexExpression(IndexName targetIndex, Term.Raw value)
+    public CustomIndexExpression(QualifiedName targetIndex, Term.Raw value)
     {
         this.targetIndex = targetIndex;
         this.valueRaw = value;
     }
 
-    public void prepareValue(CFMetaData cfm, AbstractType<?> expressionType, VariableSpecifications boundNames)
+    public void prepareValue(TableMetadata table, AbstractType<?> expressionType, VariableSpecifications boundNames)
     {
-        ColumnSpecification spec = new ColumnSpecification(cfm.ksName, cfm.ksName, valueColId, expressionType);
-        value = valueRaw.prepare(cfm.ksName, spec);
+        ColumnSpecification spec = new ColumnSpecification(table.keyspace, table.keyspace, valueColId, expressionType);
+        value = valueRaw.prepare(table.keyspace, spec);
         value.collectMarkerSpecification(boundNames);
     }
 
-    public void addToRowFilter(RowFilter filter,
-                               CFMetaData cfm,
-                               QueryOptions options)
+    public void addToRowFilter(RowFilter filter, TableMetadata table, QueryOptions options)
     {
-        filter.addCustomIndexExpression(cfm,
-                                        cfm.getIndexes()
-                                           .get(targetIndex.getIdx())
-                                           .orElseThrow(() -> IndexRestrictions.indexNotFound(targetIndex, cfm)),
+        filter.addCustomIndexExpression(table,
+                                        table.indexes
+                                             .get(targetIndex.getName())
+                                             .orElseThrow(() -> IndexRestrictions.indexNotFound(targetIndex, table)),
                                         value.bindAndGet(options));
     }
+
+    public String toCQLString()
+    {
+        return String.format("expr(%s,%s)", targetIndex.toCQLString(), valueRaw.getText());
+    }
+
+    @Override
+    public String toString()
+    {
+        return toCQLString();
+    }
+
+    @Override
+    public int hashCode()
+    {
+        return Objects.hash(targetIndex, valueRaw);
+    }
+
+    @Override
+    public boolean equals(Object o)
+    {
+        if (this == o)
+            return true;
+
+        if (!(o instanceof CustomIndexExpression))
+            return false;
+
+        CustomIndexExpression cie = (CustomIndexExpression) o;
+        return targetIndex.equals(cie.targetIndex) && valueRaw.equals(cie.valueRaw);
+    }
 }
