@@ -17,24 +17,28 @@
  */
 package org.apache.cassandra.cql3.statements;
 
+import java.util.Collection;
 import java.util.Collections;
 import java.util.List;
 
-import org.apache.cassandra.config.CFMetaData;
-import org.apache.cassandra.config.ColumnDefinition;
+import org.apache.cassandra.audit.AuditLogContext;
+import org.apache.cassandra.audit.AuditLogEntryType;
 import org.apache.cassandra.cql3.*;
+import org.apache.cassandra.cql3.conditions.ColumnCondition;
+import org.apache.cassandra.cql3.conditions.Conditions;
 import org.apache.cassandra.cql3.restrictions.StatementRestrictions;
 import org.apache.cassandra.db.Clustering;
-import org.apache.cassandra.db.CompactTables;
 import org.apache.cassandra.db.Slice;
 import org.apache.cassandra.db.partitions.PartitionUpdate;
+import org.apache.cassandra.schema.ColumnMetadata;
+import org.apache.cassandra.schema.TableMetadata;
 import org.apache.cassandra.utils.ByteBufferUtil;
 import org.apache.cassandra.utils.Pair;
+import org.apache.commons.lang3.builder.ToStringBuilder;
+import org.apache.commons.lang3.builder.ToStringStyle;
 
-import static com.google.common.collect.Lists.newArrayList;
 import static org.apache.cassandra.cql3.statements.RequestValidations.checkContainsNoDuplicates;
 import static org.apache.cassandra.cql3.statements.RequestValidations.checkFalse;
-import static org.apache.cassandra.cql3.statements.RequestValidations.checkTrue;
 
 /**
  * An <code>UPDATE</code> statement parsed from a CQL query statement.
@@ -45,55 +49,48 @@ public class UpdateStatement extends ModificationStatement
     private static final Constants.Value EMPTY = new Constants.Value(ByteBufferUtil.EMPTY_BYTE_BUFFER);
 
     private UpdateStatement(StatementType type,
-                            int boundTerms,
-                            CFMetaData cfm,
+                            VariableSpecifications bindVariables,
+                            TableMetadata metadata,
                             Operations operations,
                             StatementRestrictions restrictions,
                             Conditions conditions,
                             Attributes attrs)
     {
-        super(type, boundTerms, cfm, operations, restrictions, conditions, attrs);
-    }
-
-    public boolean requireFullClusteringKey()
-    {
-        return true;
+        super(type, bindVariables, metadata, operations, restrictions, conditions, attrs);
     }
 
     @Override
-    public void addUpdateForKey(PartitionUpdate update, Clustering clustering, UpdateParameters params)
+    public void addUpdateForKey(PartitionUpdate.Builder updateBuilder, Clustering<?> clustering, UpdateParameters params)
     {
         if (updatesRegularRows())
         {
             params.newRow(clustering);
 
-            // We update the row timestamp (ex-row marker) only on INSERT (#6782)
+            // We update the row timestamp only on INSERT (#6782)
             // Further, COMPACT tables semantic differs from "CQL3" ones in that a row exists only if it has
             // a non-null column, so we don't want to set the row timestamp for them.
-            if (type.isInsert() && cfm.isCQLTable())
+            if (type.isInsert() && !metadata.isCompactTable())
                 params.addPrimaryKeyLivenessInfo();
 
             List<Operation> updates = getRegularOperations();
 
-            // For compact table, when we translate it to thrift, we don't have a row marker. So we don't accept an insert/update
-            // that only sets the PK unless the is no declared non-PK columns (in the latter we just set the value empty).
-
-            // For a dense layout, when we translate it to thrift, we don't have a row marker. So we don't accept an insert/update
-            // that only sets the PK unless the is no declared non-PK columns (which we recognize because in that case the compact
-            // value is of type "EmptyType").
-            if ((cfm.isCompactTable() && !cfm.isSuper()) && updates.isEmpty())
+            // For compact table, we don't accept an insert/update that only sets the PK unless the is no
+            // declared non-PK columns (which we recognize because in that case
+            // the compact value is of type "EmptyType").
+            if (metadata().isCompactTable() && updates.isEmpty())
             {
-                checkTrue(CompactTables.hasEmptyCompactValue(cfm),
-                          "Column %s is mandatory for this COMPACT STORAGE table",
-                          cfm.compactValueColumn().name);
+                TableMetadata.CompactTableMetadata metadata = (TableMetadata.CompactTableMetadata) metadata();
+                RequestValidations.checkTrue(metadata.hasEmptyCompactValue(),
+                                             "Column %s is mandatory for this COMPACT STORAGE table",
+                                             metadata.compactValueColumn);
 
-                updates = Collections.<Operation>singletonList(new Constants.Setter(cfm.compactValueColumn(), EMPTY));
+                updates = Collections.singletonList(new Constants.Setter(metadata.compactValueColumn, EMPTY));
             }
 
             for (int i = 0, isize = updates.size(); i < isize; i++)
-                updates.get(i).execute(update.partitionKey(), params);
+                updates.get(i).execute(updateBuilder.partitionKey(), params);
 
-            update.add(params.buildRow());
+            updateBuilder.add(params.buildRow());
         }
 
         if (updatesStaticRow())
@@ -101,20 +98,20 @@ public class UpdateStatement extends ModificationStatement
             params.newRow(Clustering.STATIC_CLUSTERING);
             List<Operation> staticOps = getStaticOperations();
             for (int i = 0, isize = staticOps.size(); i < isize; i++)
-                staticOps.get(i).execute(update.partitionKey(), params);
-            update.add(params.buildRow());
+                staticOps.get(i).execute(updateBuilder.partitionKey(), params);
+            updateBuilder.add(params.buildRow());
         }
     }
 
     @Override
-    public void addUpdateForKey(PartitionUpdate update, Slice slice, UpdateParameters params)
+    public void addUpdateForKey(PartitionUpdate.Builder update, Slice slice, UpdateParameters params)
     {
         throw new UnsupportedOperationException();
     }
 
     public static class ParsedInsert extends ModificationStatement.Parsed
     {
-        private final List<ColumnDefinition.Raw> columnNames;
+        private final List<ColumnIdentifier> columnNames;
         private final List<Term.Raw> columnValues;
 
         /**
@@ -126,9 +123,9 @@ public class UpdateStatement extends ModificationStatement
          * @param columnValues list of column values (corresponds to names)
          * @param ifNotExists true if an IF NOT EXISTS condition was specified, false otherwise
          */
-        public ParsedInsert(CFName name,
+        public ParsedInsert(QualifiedName name,
                             Attributes.Raw attrs,
-                            List<ColumnDefinition.Raw> columnNames,
+                            List<ColumnIdentifier> columnNames,
                             List<Term.Raw> columnValues,
                             boolean ifNotExists)
         {
@@ -138,14 +135,14 @@ public class UpdateStatement extends ModificationStatement
         }
 
         @Override
-        protected ModificationStatement prepareInternal(CFMetaData cfm,
-                                                        VariableSpecifications boundNames,
+        protected ModificationStatement prepareInternal(TableMetadata metadata,
+                                                        VariableSpecifications bindVariables,
                                                         Conditions conditions,
                                                         Attributes attrs)
         {
 
             // Created from an INSERT
-            checkFalse(cfm.isCounter(), "INSERT statements are not allowed on counter tables, use UPDATE instead");
+            checkFalse(metadata.isCounter(), "INSERT statements are not allowed on counter tables, use UPDATE instead");
 
             checkFalse(columnNames == null, "Column names for INSERT must be provided when using VALUES");
             checkFalse(columnNames.isEmpty(), "No columns provided to INSERT");
@@ -156,50 +153,40 @@ public class UpdateStatement extends ModificationStatement
             Operations operations = new Operations(type);
             boolean hasClusteringColumnsSet = false;
 
-            if (cfm.isSuper() && cfm.isDense())
+            for (int i = 0; i < columnNames.size(); i++)
             {
-                // SuperColumn familiy updates are always row-level
-                hasClusteringColumnsSet = true;
-                SuperColumnCompatibility.prepareInsertOperations(cfm, columnNames, whereClause, columnValues, boundNames, operations);
-            }
-            else
-            {
-                for (int i = 0; i < columnNames.size(); i++)
+                ColumnMetadata def = metadata.getExistingColumn(columnNames.get(i));
+
+                if (def.isClusteringColumn())
+                    hasClusteringColumnsSet = true;
+
+                Term.Raw value = columnValues.get(i);
+
+                if (def.isPrimaryKeyColumn())
                 {
-                    ColumnDefinition def = getColumnDefinition(cfm, columnNames.get(i));
-
-                    if (def.isClusteringColumn())
-                        hasClusteringColumnsSet = true;
-
-                    Term.Raw value = columnValues.get(i);
-
-                    if (def.isPrimaryKeyColumn())
-                    {
-                        whereClause.add(new SingleColumnRelation(columnNames.get(i), Operator.EQ, value));
-                    }
-                    else
-                    {
-                        Operation operation = new Operation.SetValue(value).prepare(cfm, def);
-                        operation.collectMarkerSpecification(boundNames);
-                        operations.add(operation);
-                    }
+                    whereClause.add(new SingleColumnRelation(columnNames.get(i), Operator.EQ, value));
+                }
+                else
+                {
+                    Operation operation = new Operation.SetValue(value).prepare(metadata, def);
+                    operation.collectMarkerSpecification(bindVariables);
+                    operations.add(operation);
                 }
             }
 
             boolean applyOnlyToStaticColumns = !hasClusteringColumnsSet && appliesOnlyToStaticColumns(operations, conditions);
 
             StatementRestrictions restrictions = new StatementRestrictions(type,
-                                                                           cfm,
+                                                                           metadata,
                                                                            whereClause.build(),
-                                                                           boundNames,
+                                                                           bindVariables,
                                                                            applyOnlyToStaticColumns,
                                                                            false,
-                                                                           false,
                                                                            false);
 
             return new UpdateStatement(type,
-                                       boundNames.size(),
-                                       cfm,
+                                       bindVariables,
+                                       metadata,
                                        operations,
                                        restrictions,
                                        conditions,
@@ -215,7 +202,7 @@ public class UpdateStatement extends ModificationStatement
         private final Json.Raw jsonValue;
         private final boolean defaultUnset;
 
-        public ParsedInsertJson(CFName name, Attributes.Raw attrs, Json.Raw jsonValue, boolean defaultUnset, boolean ifNotExists)
+        public ParsedInsertJson(QualifiedName name, Attributes.Raw attrs, Json.Raw jsonValue, boolean defaultUnset, boolean ifNotExists)
         {
             super(name, StatementType.INSERT, attrs, null, ifNotExists, false);
             this.jsonValue = jsonValue;
@@ -223,33 +210,21 @@ public class UpdateStatement extends ModificationStatement
         }
 
         @Override
-        protected ModificationStatement prepareInternal(CFMetaData cfm,
-                                                        VariableSpecifications boundNames,
+        protected ModificationStatement prepareInternal(TableMetadata metadata,
+                                                        VariableSpecifications bindVariables,
                                                         Conditions conditions,
                                                         Attributes attrs)
         {
-            checkFalse(cfm.isCounter(), "INSERT statements are not allowed on counter tables, use UPDATE instead");
+            checkFalse(metadata.isCounter(), "INSERT statements are not allowed on counter tables, use UPDATE instead");
 
-            List<ColumnDefinition> defs = newArrayList(cfm.allColumnsInSelectOrder());
-            Json.Prepared prepared = jsonValue.prepareAndCollectMarkers(cfm, defs, boundNames);
+            Collection<ColumnMetadata> defs = metadata.columns();
+            Json.Prepared prepared = jsonValue.prepareAndCollectMarkers(metadata, defs, bindVariables);
 
             WhereClause.Builder whereClause = new WhereClause.Builder();
             Operations operations = new Operations(type);
             boolean hasClusteringColumnsSet = false;
 
-            if (cfm.isSuper() && cfm.isDense())
-            {
-                hasClusteringColumnsSet = true;
-                SuperColumnCompatibility.prepareInsertJSONOperations(cfm, defs, boundNames, prepared, whereClause, operations);
-            }
-            else
-            {
-
-
-
-// TODO: indent
-
-            for (ColumnDefinition def : defs)
+            for (ColumnMetadata def : defs)
             {
                 if (def.isClusteringColumn())
                     hasClusteringColumnsSet = true;
@@ -257,35 +232,29 @@ public class UpdateStatement extends ModificationStatement
                 Term.Raw raw = prepared.getRawTermForColumn(def, defaultUnset);
                 if (def.isPrimaryKeyColumn())
                 {
-                    whereClause.add(new SingleColumnRelation(ColumnDefinition.Raw.forColumn(def), Operator.EQ, raw));
+                    whereClause.add(new SingleColumnRelation(def.name, Operator.EQ, raw));
                 }
                 else
                 {
-                    Operation operation = new Operation.SetValue(raw).prepare(cfm, def);
-                    operation.collectMarkerSpecification(boundNames);
+                    Operation operation = new Operation.SetValue(raw).prepare(metadata, def);
+                    operation.collectMarkerSpecification(bindVariables);
                     operations.add(operation);
                 }
             }
 
-
-
-
-            }
-
             boolean applyOnlyToStaticColumns = !hasClusteringColumnsSet && appliesOnlyToStaticColumns(operations, conditions);
 
             StatementRestrictions restrictions = new StatementRestrictions(type,
-                                                                           cfm,
+                                                                           metadata,
                                                                            whereClause.build(),
-                                                                           boundNames,
+                                                                           bindVariables,
                                                                            applyOnlyToStaticColumns,
                                                                            false,
-                                                                           false,
                                                                            false);
 
             return new UpdateStatement(type,
-                                       boundNames.size(),
-                                       cfm,
+                                       bindVariables,
+                                       metadata,
                                        operations,
                                        restrictions,
                                        conditions,
@@ -296,8 +265,8 @@ public class UpdateStatement extends ModificationStatement
     public static class ParsedUpdate extends ModificationStatement.Parsed
     {
         // Provided for an UPDATE
-        private final List<Pair<ColumnDefinition.Raw, Operation.RawUpdate>> updates;
-        private WhereClause whereClause;
+        private final List<Pair<ColumnIdentifier, Operation.RawUpdate>> updates;
+        private final WhereClause whereClause;
 
         /**
          * Creates a new UpdateStatement from a column family name, columns map, consistency
@@ -309,11 +278,11 @@ public class UpdateStatement extends ModificationStatement
          * @param whereClause the where clause
          * @param ifExists flag to check if row exists
          * */
-        public ParsedUpdate(CFName name,
+        public ParsedUpdate(QualifiedName name,
                             Attributes.Raw attrs,
-                            List<Pair<ColumnDefinition.Raw, Operation.RawUpdate>> updates,
+                            List<Pair<ColumnIdentifier, Operation.RawUpdate>> updates,
                             WhereClause whereClause,
-                            List<Pair<ColumnDefinition.Raw, ColumnCondition.Raw>> conditions,
+                            List<Pair<ColumnIdentifier, ColumnCondition.Raw>> conditions,
                             boolean ifExists)
         {
             super(name, StatementType.UPDATE, attrs, conditions, false, ifExists);
@@ -322,45 +291,49 @@ public class UpdateStatement extends ModificationStatement
         }
 
         @Override
-        protected ModificationStatement prepareInternal(CFMetaData cfm,
-                                                        VariableSpecifications boundNames,
+        protected ModificationStatement prepareInternal(TableMetadata metadata,
+                                                        VariableSpecifications bindVariables,
                                                         Conditions conditions,
                                                         Attributes attrs)
         {
             Operations operations = new Operations(type);
 
-            if (cfm.isSuper() && cfm.isDense())
+            for (Pair<ColumnIdentifier, Operation.RawUpdate> entry : updates)
             {
-                conditions = SuperColumnCompatibility.rebuildLWTColumnConditions(conditions, cfm, whereClause);
-                whereClause = SuperColumnCompatibility.prepareUpdateOperations(cfm, whereClause, updates, boundNames, operations);
-            }
-            else
-            {
-                for (Pair<ColumnDefinition.Raw, Operation.RawUpdate> entry : updates)
-                {
-                    ColumnDefinition def = getColumnDefinition(cfm, entry.left);
+                ColumnMetadata def = metadata.getExistingColumn(entry.left);
 
-                    checkFalse(def.isPrimaryKeyColumn(), "PRIMARY KEY part %s found in SET part", def.name);
+                checkFalse(def.isPrimaryKeyColumn(), "PRIMARY KEY part %s found in SET part", def.name);
 
-                    Operation operation = entry.right.prepare(cfm, def);
-                    operation.collectMarkerSpecification(boundNames);
-                    operations.add(operation);
-                }
+                Operation operation = entry.right.prepare(metadata, def);
+                operation.collectMarkerSpecification(bindVariables);
+                operations.add(operation);
             }
-            
-            StatementRestrictions restrictions = newRestrictions(cfm,
-                                                                 boundNames,
+
+            StatementRestrictions restrictions = newRestrictions(metadata,
+                                                                 bindVariables,
                                                                  operations,
                                                                  whereClause,
                                                                  conditions);
 
             return new UpdateStatement(type,
-                                       boundNames.size(),
-                                       cfm,
+                                       bindVariables,
+                                       metadata,
                                        operations,
                                        restrictions,
                                        conditions,
                                        attrs);
         }
     }
+    
+    @Override
+    public String toString()
+    {
+        return ToStringBuilder.reflectionToString(this, ToStringStyle.SHORT_PREFIX_STYLE);
+    }
+
+    @Override
+    public AuditLogContext getAuditLogContext()
+    {
+        return new AuditLogContext(AuditLogEntryType.UPDATE, keyspace(), columnFamily());
+    }
 }
