@@ -281,7 +281,8 @@ public final class CreateViewStatement extends AlterSchemaStatement
 
         attrs.validate();
 
-        if (attrs.hasOption(TableParams.Option.DEFAULT_TIME_TO_LIVE))
+        if (attrs.hasOption(TableParams.Option.DEFAULT_TIME_TO_LIVE)
+            && attrs.getInt(TableParams.Option.DEFAULT_TIME_TO_LIVE.toString(), 0) != 0)
         {
             throw ire("Cannot set default_time_to_live for a materialized view. " +
                       "Data in a materialized view always expire at the same time than " +
