@@ -19,15 +19,20 @@ package org.apache.cassandra.db;
 
 import java.io.IOException;
 import java.nio.ByteBuffer;
-import java.security.MessageDigest;
 import java.util.*;
 
 import org.apache.cassandra.cache.IMeasurableMemory;
 import org.apache.cassandra.config.*;
+import org.apache.cassandra.db.marshal.ByteArrayAccessor;
+import org.apache.cassandra.db.marshal.ByteBufferAccessor;
+import org.apache.cassandra.db.marshal.CompositeType;
+import org.apache.cassandra.db.marshal.ValueAccessor;
 import org.apache.cassandra.db.rows.*;
 import org.apache.cassandra.db.marshal.AbstractType;
 import org.apache.cassandra.io.util.DataInputPlus;
 import org.apache.cassandra.io.util.DataOutputPlus;
+import org.apache.cassandra.schema.TableMetadata;
+import org.apache.cassandra.utils.ByteArrayUtil;
 import org.apache.cassandra.utils.ByteBufferUtil;
 
 /**
@@ -37,13 +42,13 @@ import org.apache.cassandra.utils.ByteBufferUtil;
  * a "kind" that allows us to implement slices with inclusive and exclusive bounds.
  * <p>
  * In practice, {@code ClusteringPrefix} is just the common parts to its 3 main subtype: {@link Clustering} and
- * {@link Slice.Bound}/{@link RangeTombstone.Bound}, where:
+ * {@link ClusteringBound}/{@link ClusteringBoundary}, where:
  *   1) {@code Clustering} represents the clustering values for a row, i.e. the values for it's clustering columns.
- *   2) {@code Slice.Bound} represents a bound (start or end) of a slice (of rows).
- *   3) {@code RangeTombstoneBoundMarker.Bound} represents a range tombstone marker "bound".
+ *   2) {@code ClusteringBound} represents a bound (start or end) of a slice (of rows) or a range tombstone.
+ *   3) {@code ClusteringBoundary} represents the threshold between two adjacent range tombstones.
  * See those classes for more details.
  */
-public interface ClusteringPrefix extends IMeasurableMemory, Clusterable
+public interface ClusteringPrefix<V> extends IMeasurableMemory, Clusterable<V>
 {
     public static final Serializer serializer = new Serializer();
 
@@ -51,7 +56,7 @@ public interface ClusteringPrefix extends IMeasurableMemory, Clusterable
      * The kind of clustering prefix this actually is.
      *
      * The kind {@code STATIC_CLUSTERING} is only implemented by {@link Clustering#STATIC_CLUSTERING} and {@code CLUSTERING} is
-     * implemented by the {@link Clustering} class. The rest is used by {@link Slice.Bound} and {@link RangeTombstone.Bound}.
+     * implemented by the {@link Clustering} class. The rest is used by {@link ClusteringBound} and {@link ClusteringBoundary}.
      */
     public enum Kind
     {
@@ -74,7 +79,7 @@ public interface ClusteringPrefix extends IMeasurableMemory, Clusterable
          */
         public final int comparedToClustering;
 
-        private Kind(int comparison, int comparedToClustering)
+        Kind(int comparison, int comparedToClustering)
         {
             this.comparison = comparison;
             this.comparedToClustering = comparedToClustering;
@@ -122,8 +127,9 @@ public interface ClusteringPrefix extends IMeasurableMemory, Clusterable
                 case EXCL_START_BOUND:
                 case EXCL_END_BOUND:
                     return true;
+                default:
+                    return false;
             }
-            return false;
         }
 
         public boolean isBoundary()
@@ -133,8 +139,9 @@ public interface ClusteringPrefix extends IMeasurableMemory, Clusterable
                 case INCL_END_EXCL_START_BOUNDARY:
                 case EXCL_END_INCL_START_BOUNDARY:
                     return true;
+                default:
+                    return false;
             }
-            return false;
         }
 
         public boolean isStart()
@@ -192,6 +199,16 @@ public interface ClusteringPrefix extends IMeasurableMemory, Clusterable
         }
     }
 
+    default boolean isBottom()
+    {
+        return kind() == Kind.INCL_START_BOUND && size() == 0;
+    }
+
+    default boolean isTop()
+    {
+        return kind() == Kind.INCL_END_BOUND && size() == 0;
+    }
+
     public Kind kind();
 
     /**
@@ -203,6 +220,11 @@ public interface ClusteringPrefix extends IMeasurableMemory, Clusterable
      */
     public int size();
 
+    default boolean isEmpty()
+    {
+        return size() == 0;
+    }
+
     /**
      * Retrieves the ith value of this prefix.
      *
@@ -210,14 +232,40 @@ public interface ClusteringPrefix extends IMeasurableMemory, Clusterable
      *
      * @return the ith value of this prefix. Note that a value can be {@code null}.
      */
-    public ByteBuffer get(int i);
+    public V get(int i);
+
+    public ValueAccessor<V> accessor();
+
+    default ByteBuffer bufferAt(int i)
+    {
+        return accessor().toBuffer(get(i));
+    }
+
+    default String stringAt(int i, ClusteringComparator comparator)
+    {
+        return comparator.subtype(i).getString(get(i), accessor());
+    }
+
+    default void validate(int i, ClusteringComparator comparator)
+    {
+        comparator.subtype(i).validate(get(i), accessor());
+    }
 
     /**
-     * Adds the data of this clustering prefix to the provided digest.
+     * Adds the data of this clustering prefix to the provided Digest instance.
      *
-     * @param digest the digest to which to add this prefix.
+     * @param digest the Digest instance to which to add this prefix.
      */
-    public void digest(MessageDigest digest);
+    default void digest(Digest digest)
+    {
+        for (int i = 0; i < size(); i++)
+        {
+            V value = get(i);
+            if (value != null)
+                digest.update(value, accessor());
+        }
+        digest.updateWithByte(kind().ordinal());
+    }
 
     /**
      * The size of the data hold by this prefix.
@@ -225,7 +273,16 @@ public interface ClusteringPrefix extends IMeasurableMemory, Clusterable
      * @return the size of the data hold by this prefix (this is not the size of the object in memory, just
      * the size of the data it stores).
      */
-    public int dataSize();
+    default int dataSize()
+    {
+        int size = 0;
+        for (int i = 0; i < size(); i++)
+        {
+            V v = get(i);
+            size += v == null ? 0 : accessor().size(v);
+        }
+        return size;
+    }
 
     /**
      * Generates a proper string representation of the prefix.
@@ -233,8 +290,24 @@ public interface ClusteringPrefix extends IMeasurableMemory, Clusterable
      * @param metadata the metadata for the table the clustering prefix is of.
      * @return a human-readable string representation fo this prefix.
      */
-    public String toString(CFMetaData metadata);
+    public String toString(TableMetadata metadata);
 
+    /*
+     * TODO: we should stop using Clustering for partition keys. Maybe we can add
+     * a few methods to DecoratedKey so we don't have to (note that while using a Clustering
+     * allows to use buildBound(), it's actually used for partition keys only when every restriction
+     * is an equal, so we could easily create a specific method for keys for that.
+     */
+    default ByteBuffer serializeAsPartitionKey()
+    {
+        if (size() == 1)
+            return accessor().toBuffer(get(0));
+
+        ByteBuffer[] values = new ByteBuffer[size()];
+        for (int i = 0; i < size(); i++)
+            values[i] = accessor().toBuffer(get(i));
+        return CompositeType.build(ByteBufferAccessor.instance, values);
+    }
     /**
      * The values of this prefix as an array.
      * <p>
@@ -244,32 +317,45 @@ public interface ClusteringPrefix extends IMeasurableMemory, Clusterable
      *
      * @return the values for this prefix as an array.
      */
-    public ByteBuffer[] getRawValues();
+    public V[] getRawValues();
+
+    public ByteBuffer[] getBufferArray();
 
     /**
      * If the prefix contains byte buffers that can be minimized (see {@link ByteBufferUtil#minimalBufferFor(ByteBuffer)}),
      * this will return a copy of the prefix with minimized values, otherwise it returns itself.
      */
-    public ClusteringPrefix minimize();
+    public ClusteringPrefix<V> minimize();
 
     public static class Serializer
     {
-        public void serialize(ClusteringPrefix clustering, DataOutputPlus out, int version, List<AbstractType<?>> types) throws IOException
+        public void serialize(ClusteringPrefix<?> clustering, DataOutputPlus out, int version, List<AbstractType<?>> types) throws IOException
         {
             // We shouldn't serialize static clusterings
             assert clustering.kind() != Kind.STATIC_CLUSTERING;
             if (clustering.kind() == Kind.CLUSTERING)
             {
                 out.writeByte(clustering.kind().ordinal());
-                Clustering.serializer.serialize((Clustering)clustering, out, version, types);
+                Clustering.serializer.serialize((Clustering<?>)clustering, out, version, types);
             }
             else
             {
-                RangeTombstone.Bound.serializer.serialize((RangeTombstone.Bound)clustering, out, version, types);
+                ClusteringBoundOrBoundary.serializer.serialize((ClusteringBoundOrBoundary<?>)clustering, out, version, types);
             }
         }
 
-        public ClusteringPrefix deserialize(DataInputPlus in, int version, List<AbstractType<?>> types) throws IOException
+        public void skip(DataInputPlus in, int version, List<AbstractType<?>> types) throws IOException
+        {
+            Kind kind = Kind.values()[in.readByte()];
+            // We shouldn't serialize static clusterings
+            assert kind != Kind.STATIC_CLUSTERING;
+            if (kind == Kind.CLUSTERING)
+                Clustering.serializer.skip(in, version, types);
+            else
+                ClusteringBoundOrBoundary.serializer.skipValues(in, kind, version, types);
+        }
+
+        public ClusteringPrefix<byte[]> deserialize(DataInputPlus in, int version, List<AbstractType<?>> types) throws IOException
         {
             Kind kind = Kind.values()[in.readByte()];
             // We shouldn't serialize static clusterings
@@ -277,23 +363,24 @@ public interface ClusteringPrefix extends IMeasurableMemory, Clusterable
             if (kind == Kind.CLUSTERING)
                 return Clustering.serializer.deserialize(in, version, types);
             else
-                return RangeTombstone.Bound.serializer.deserializeValues(in, kind, version, types);
+                return ClusteringBoundOrBoundary.serializer.deserializeValues(in, kind, version, types);
         }
 
-        public long serializedSize(ClusteringPrefix clustering, int version, List<AbstractType<?>> types)
+        public long serializedSize(ClusteringPrefix<?> clustering, int version, List<AbstractType<?>> types)
         {
             // We shouldn't serialize static clusterings
             assert clustering.kind() != Kind.STATIC_CLUSTERING;
             if (clustering.kind() == Kind.CLUSTERING)
-                return 1 + Clustering.serializer.serializedSize((Clustering)clustering, version, types);
+                return 1 + Clustering.serializer.serializedSize((Clustering<?>)clustering, version, types);
             else
-                return RangeTombstone.Bound.serializer.serializedSize((RangeTombstone.Bound)clustering, version, types);
+                return ClusteringBoundOrBoundary.serializer.serializedSize((ClusteringBoundOrBoundary<?>)clustering, version, types);
         }
 
-        void serializeValuesWithoutSize(ClusteringPrefix clustering, DataOutputPlus out, int version, List<AbstractType<?>> types) throws IOException
+        <V> void serializeValuesWithoutSize(ClusteringPrefix<V> clustering, DataOutputPlus out, int version, List<AbstractType<?>> types) throws IOException
         {
             int offset = 0;
             int clusteringSize = clustering.size();
+            ValueAccessor<V> accessor = clustering.accessor();
             // serialize in batches of 32, to avoid garbage when deserializing headers
             while (offset < clusteringSize)
             {
@@ -305,15 +392,15 @@ public interface ClusteringPrefix extends IMeasurableMemory, Clusterable
                 out.writeUnsignedVInt(makeHeader(clustering, offset, limit));
                 while (offset < limit)
                 {
-                    ByteBuffer v = clustering.get(offset);
-                    if (v != null && v.hasRemaining())
-                        types.get(offset).writeValue(v, out);
+                    V v = clustering.get(offset);
+                    if (v != null && !accessor.isEmpty(v))
+                        types.get(offset).writeValue(v, accessor, out);
                     offset++;
                 }
             }
         }
 
-        long valuesWithoutSizeSerializedSize(ClusteringPrefix clustering, int version, List<AbstractType<?>> types)
+        <V> long valuesWithoutSizeSerializedSize(ClusteringPrefix<V> clustering, int version, List<AbstractType<?>> types)
         {
             long result = 0;
             int offset = 0;
@@ -324,22 +411,23 @@ public interface ClusteringPrefix extends IMeasurableMemory, Clusterable
                 result += TypeSizes.sizeofUnsignedVInt(makeHeader(clustering, offset, limit));
                 offset = limit;
             }
+            ValueAccessor<V> accessor = clustering.accessor();
             for (int i = 0; i < clusteringSize; i++)
             {
-                ByteBuffer v = clustering.get(i);
-                if (v == null || !v.hasRemaining())
+                V v = clustering.get(i);
+                if (v == null || accessor.isEmpty(v))
                     continue; // handled in the header
 
-                result += types.get(i).writtenLength(v);
+                result += types.get(i).writtenLength(v, accessor);
             }
             return result;
         }
 
-        ByteBuffer[] deserializeValuesWithoutSize(DataInputPlus in, int size, int version, List<AbstractType<?>> types) throws IOException
+        byte[][] deserializeValuesWithoutSize(DataInputPlus in, int size, int version, List<AbstractType<?>> types) throws IOException
         {
             // Callers of this method should handle the case where size = 0 (in all case we want to return a special value anyway).
             assert size > 0;
-            ByteBuffer[] values = new ByteBuffer[size];
+            byte[][] values = new byte[size][];
             int offset = 0;
             while (offset < size)
             {
@@ -348,30 +436,50 @@ public interface ClusteringPrefix extends IMeasurableMemory, Clusterable
                 while (offset < limit)
                 {
                     values[offset] = isNull(header, offset)
-                                ? null
-                                : (isEmpty(header, offset) ? ByteBufferUtil.EMPTY_BYTE_BUFFER : types.get(offset).readValue(in, DatabaseDescriptor.getMaxValueSize()));
+                                     ? null
+                                     : (isEmpty(header, offset) ? ByteArrayUtil.EMPTY_BYTE_ARRAY
+                                                                : types.get(offset).readArray(in, DatabaseDescriptor.getMaxValueSize()));
                     offset++;
                 }
             }
             return values;
         }
 
+        void skipValuesWithoutSize(DataInputPlus in, int size, int version, List<AbstractType<?>> types) throws IOException
+        {
+            // Callers of this method should handle the case where size = 0 (in all case we want to return a special value anyway).
+            assert size > 0;
+            int offset = 0;
+            while (offset < size)
+            {
+                long header = in.readUnsignedVInt();
+                int limit = Math.min(size, offset + 32);
+                while (offset < limit)
+                {
+                    if (!isNull(header, offset) && !isEmpty(header, offset))
+                         types.get(offset).skipValue(in);
+                    offset++;
+                }
+            }
+        }
+
         /**
          * Whatever the type of a given clustering column is, its value can always be either empty or null. So we at least need to distinguish those
          * 2 values, and because we want to be able to store fixed width values without appending their (fixed) size first, we need a way to encode
          * empty values too. So for that, every clustering prefix includes a "header" that contains 2 bits per element in the prefix. For each element,
          * those 2 bits encode whether the element is null, empty, or none of those.
          */
-        private static long makeHeader(ClusteringPrefix clustering, int offset, int limit)
+        private static <V> long makeHeader(ClusteringPrefix<V> clustering, int offset, int limit)
         {
             long header = 0;
+            ValueAccessor<V> accessor = clustering.accessor();
             for (int i = offset ; i < limit ; i++)
             {
-                ByteBuffer v = clustering.get(i);
+                V v = clustering.get(i);
                 // no need to do modulo arithmetic for i, since the left-shift execute on the modulus of RH operand by definition
                 if (v == null)
                     header |= (1L << (i * 2) + 1);
-                else if (!v.hasRemaining())
+                else if (accessor.isEmpty(v))
                     header |= (1L << (i * 2));
             }
             return header;
@@ -413,7 +521,8 @@ public interface ClusteringPrefix extends IMeasurableMemory, Clusterable
         private int nextSize;
         private ClusteringPrefix.Kind nextKind;
         private int deserializedSize;
-        private ByteBuffer[] nextValues;
+        private byte[][] nextValues;
+        private final ValueAccessor<byte[]> accessor = ByteArrayAccessor.instance;
 
         public Deserializer(ClusteringComparator comparator, DataInputPlus in, SerializationHeader header)
         {
@@ -438,12 +547,12 @@ public interface ClusteringPrefix extends IMeasurableMemory, Clusterable
             // nextValues is of the proper size. Note that the 2nd condition may not hold for range tombstone bounds, but all
             // rows have a fixed size clustering, so we'll still save in the common case.
             if (nextValues == null || nextValues.length != nextSize)
-                this.nextValues = new ByteBuffer[nextSize];
+                this.nextValues = new byte[nextSize][];
         }
 
-        public int compareNextTo(Slice.Bound bound) throws IOException
+        public <T> int compareNextTo(ClusteringBoundOrBoundary<T> bound) throws IOException
         {
-            if (bound == Slice.Bound.TOP)
+            if (bound.isTop())
                 return -1;
 
             for (int i = 0; i < bound.size(); i++)
@@ -451,7 +560,7 @@ public interface ClusteringPrefix extends IMeasurableMemory, Clusterable
                 if (!hasComponent(i))
                     return nextKind.comparedToClustering;
 
-                int cmp = comparator.compareComponent(i, nextValues[i], bound.get(i));
+                int cmp = comparator.compareComponent(i, nextValues[i], accessor, bound.get(i), bound.accessor());
                 if (cmp != 0)
                     return cmp;
             }
@@ -485,7 +594,8 @@ public interface ClusteringPrefix extends IMeasurableMemory, Clusterable
             int i = deserializedSize++;
             nextValues[i] = Serializer.isNull(nextHeader, i)
                           ? null
-                          : (Serializer.isEmpty(nextHeader, i) ? ByteBufferUtil.EMPTY_BYTE_BUFFER : serializationHeader.clusteringTypes().get(i).readValue(in, DatabaseDescriptor.getMaxValueSize()));
+                          : (Serializer.isEmpty(nextHeader, i) ? ByteArrayUtil.EMPTY_BYTE_ARRAY
+                                                               : serializationHeader.clusteringTypes().get(i).readArray(in, DatabaseDescriptor.getMaxValueSize()));
             return true;
         }
 
@@ -495,20 +605,20 @@ public interface ClusteringPrefix extends IMeasurableMemory, Clusterable
                 continue;
         }
 
-        public RangeTombstone.Bound deserializeNextBound() throws IOException
+        public ClusteringBoundOrBoundary<byte[]> deserializeNextBound() throws IOException
         {
             assert !nextIsRow;
             deserializeAll();
-            RangeTombstone.Bound bound = new RangeTombstone.Bound(nextKind, nextValues);
+            ClusteringBoundOrBoundary<byte[]> bound = accessor.factory().boundOrBoundary(nextKind, nextValues);
             nextValues = null;
             return bound;
         }
 
-        public Clustering deserializeNextClustering() throws IOException
+        public Clustering<byte[]> deserializeNextClustering() throws IOException
         {
             assert nextIsRow;
             deserializeAll();
-            Clustering clustering = new Clustering(nextValues);
+            Clustering<byte[]> clustering = accessor.factory().clustering(nextValues);
             nextValues = null;
             return clustering;
         }
@@ -526,4 +636,44 @@ public interface ClusteringPrefix extends IMeasurableMemory, Clusterable
             return nextKind;
         }
     }
+
+    public static <V> int hashCode(ClusteringPrefix<V> prefix)
+    {
+        int result = 31;
+        for (int i = 0; i < prefix.size(); i++)
+            result += 31 * prefix.accessor().hashCode(prefix.get(i));
+        return 31 * result + Objects.hashCode(prefix.kind());
+    }
+
+    static <V1, V2> boolean equals(ClusteringPrefix<V1> left, ClusteringPrefix<V2> right)
+    {
+        if (left.kind() != right.kind() || left.size() != right.size())
+            return false;
+
+        for (int i = 0; i < left.size(); i++)
+        {
+            V1 lVal = left.get(i);
+            V2 rVal = right.get(i);
+
+            if (lVal == null && rVal == null)
+                continue;
+
+            if (lVal == null || rVal == null)
+                return false;
+
+            if (!ValueAccessor.equals(lVal, left.accessor(), rVal, right.accessor()))
+                return false;
+        }
+
+        return true;
+    }
+
+    public static boolean equals(ClusteringPrefix<?> prefix, Object o)
+    {
+        if(!(o instanceof ClusteringPrefix))
+            return false;
+
+        return equals(prefix, (ClusteringPrefix<?>) o);
+    }
+
 }
