@@ -147,6 +147,11 @@ public class Mutation implements IMutation
         return table == null ? null : modifications.get(table.id);
     }
 
+    public long getApproxCreatedAtNanos()
+    {
+        return approxCreatedAtNanos;
+    }
+
     public boolean isEmpty()
     {
         return modifications.isEmpty();
