@@ -21,11 +21,10 @@ import java.io.IOException;
 import java.nio.ByteBuffer;
 import java.util.*;
 
-import org.apache.cassandra.config.CFMetaData;
 import org.apache.cassandra.db.marshal.AbstractType;
+import org.apache.cassandra.db.marshal.ByteArrayAccessor;
 import org.apache.cassandra.io.util.DataInputPlus;
 import org.apache.cassandra.io.util.DataOutputPlus;
-import org.apache.cassandra.utils.ByteBufferUtil;
 
 /**
  * A slice represents the selection of a range of rows.
@@ -39,10 +38,10 @@ public class Slice
     public static final Serializer serializer = new Serializer();
 
     /** The slice selecting all rows (of a given partition) */
-    public static final Slice ALL = new Slice(ClusteringBound.BOTTOM, ClusteringBound.TOP)
+    public static final Slice ALL = new Slice(BufferClusteringBound.BOTTOM, BufferClusteringBound.TOP)
     {
         @Override
-        public boolean includes(ClusteringComparator comparator, ClusteringPrefix clustering)
+        public boolean includes(ClusteringComparator comparator, ClusteringPrefix<?> clustering)
         {
             return true;
         }
@@ -60,19 +59,21 @@ public class Slice
         }
     };
 
-    private final ClusteringBound start;
-    private final ClusteringBound end;
+    private final ClusteringBound<?> start;
+    private final ClusteringBound<?> end;
 
-    private Slice(ClusteringBound start, ClusteringBound end)
+    private Slice(ClusteringBound<?> start, ClusteringBound<?> end)
     {
         assert start.isStart() && end.isEnd();
         this.start = start;
         this.end = end;
     }
 
-    public static Slice make(ClusteringBound start, ClusteringBound end)
+    public static Slice make(ClusteringBound<?> start, ClusteringBound<?> end)
     {
-        if (start == ClusteringBound.BOTTOM && end == ClusteringBound.TOP)
+        assert start != null && end != null;
+
+        if (start.isBottom() && end.isTop())
             return ALL;
 
         return new Slice(start, end);
@@ -91,49 +92,37 @@ public class Slice
         return new Slice(builder.buildBound(true, true), builder.buildBound(false, true));
     }
 
-    public static Slice make(Clustering clustering)
+    public static Slice make(Clustering<?> clustering)
     {
         // This doesn't give us what we want with the clustering prefix
         assert clustering != Clustering.STATIC_CLUSTERING;
-        ByteBuffer[] values = extractValues(clustering);
-        return new Slice(ClusteringBound.inclusiveStartOf(values), ClusteringBound.inclusiveEndOf(values));
+        return new Slice(ClusteringBound.inclusiveStartOf(clustering), ClusteringBound.inclusiveEndOf(clustering));
     }
 
-    public static Slice make(Clustering start, Clustering end)
+    public static Slice make(Clustering<?> start, Clustering<?> end)
     {
         // This doesn't give us what we want with the clustering prefix
         assert start != Clustering.STATIC_CLUSTERING && end != Clustering.STATIC_CLUSTERING;
 
-        ByteBuffer[] startValues = extractValues(start);
-        ByteBuffer[] endValues = extractValues(end);
-
-        return new Slice(ClusteringBound.inclusiveStartOf(startValues), ClusteringBound.inclusiveEndOf(endValues));
+        return new Slice(ClusteringBound.inclusiveStartOf(start), ClusteringBound.inclusiveEndOf(end));
     }
 
-    private static ByteBuffer[] extractValues(ClusteringPrefix clustering)
-    {
-        ByteBuffer[] values = new ByteBuffer[clustering.size()];
-        for (int i = 0; i < clustering.size(); i++)
-            values[i] = clustering.get(i);
-        return values;
-    }
-
-    public ClusteringBound start()
+    public ClusteringBound<?> start()
     {
         return start;
     }
 
-    public ClusteringBound end()
+    public ClusteringBound<?> end()
     {
         return end;
     }
 
-    public ClusteringBound open(boolean reversed)
+    public ClusteringBound<?> open(boolean reversed)
     {
         return reversed ? end : start;
     }
 
-    public ClusteringBound close(boolean reversed)
+    public ClusteringBound<?> close(boolean reversed)
     {
         return reversed ? start : end;
     }
@@ -158,10 +147,18 @@ public class Slice
      * @return whether the slice formed by {@code start} and {@code end} is
      * empty or not.
      */
-    public static boolean isEmpty(ClusteringComparator comparator, ClusteringBound start, ClusteringBound end)
+    public static boolean isEmpty(ClusteringComparator comparator, ClusteringBound<?> start, ClusteringBound<?> end)
     {
         assert start.isStart() && end.isEnd();
-        return comparator.compare(end, start) <= 0;
+
+        int cmp = comparator.compare(start, end);
+
+        if (cmp < 0)
+            return false;
+        else if (cmp > 0)
+            return true;
+        else
+            return start.isExclusive() || end.isExclusive();
     }
 
     /**
@@ -172,7 +169,7 @@ public class Slice
      *
      * @return whether {@code bound} is within the bounds of this slice.
      */
-    public boolean includes(ClusteringComparator comparator, ClusteringPrefix bound)
+    public boolean includes(ClusteringComparator comparator, ClusteringPrefix<?> bound)
     {
         return comparator.compare(start, bound) <= 0 && comparator.compare(bound, end) <= 0;
     }
@@ -190,7 +187,7 @@ public class Slice
      * @return a new slice that selects results coming after {@code lastReturned}, or {@code null} if paging
      * the resulting slice selects nothing (i.e. if it originally selects nothing coming after {@code lastReturned}).
      */
-    public Slice forPaging(ClusteringComparator comparator, Clustering lastReturned, boolean inclusive, boolean reversed)
+    public Slice forPaging(ClusteringComparator comparator, Clustering<?> lastReturned, boolean inclusive, boolean reversed)
     {
         if (lastReturned == null)
             return this;
@@ -198,28 +195,37 @@ public class Slice
         if (reversed)
         {
             int cmp = comparator.compare(lastReturned, start);
-            if (cmp < 0 || (!inclusive && cmp == 0))
+            assert cmp != 0;
+            // start is > than lastReturned; got nothing to return no more
+            if (cmp < 0)
                 return null;
 
             cmp = comparator.compare(end, lastReturned);
-            if (cmp < 0 || (inclusive && cmp == 0))
+            assert cmp != 0;
+            if (cmp < 0)
                 return this;
 
-            ByteBuffer[] values = extractValues(lastReturned);
-            return new Slice(start, inclusive ? ClusteringBound.inclusiveEndOf(values) : ClusteringBound.exclusiveEndOf(values));
+            Slice slice = new Slice(start, inclusive ? ClusteringBound.inclusiveEndOf(lastReturned) : ClusteringBound.exclusiveEndOf(lastReturned));
+            if (slice.isEmpty(comparator))
+                return null;
+            return slice;
         }
         else
         {
             int cmp = comparator.compare(end, lastReturned);
-            if (cmp < 0 || (!inclusive && cmp == 0))
+            assert cmp != 0;
+            if (cmp < 0)
                 return null;
 
             cmp = comparator.compare(lastReturned, start);
-            if (cmp < 0 || (inclusive && cmp == 0))
+            assert cmp != 0;
+            if (cmp < 0)
                 return this;
 
-            ByteBuffer[] values = extractValues(lastReturned);
-            return new Slice(inclusive ? ClusteringBound.inclusiveStartOf(values) : ClusteringBound.exclusiveStartOf(values), end);
+            Slice slice = new Slice(inclusive ? ClusteringBound.inclusiveStartOf(lastReturned) : ClusteringBound.exclusiveStartOf(lastReturned), end);
+            if (slice.isEmpty(comparator))
+                return null;
+            return slice;
         }
     }
 
@@ -240,11 +246,6 @@ public class Slice
         return start.compareTo(comparator, maxClusteringValues) <= 0 && end.compareTo(comparator, minClusteringValues) >= 0;
     }
 
-    public String toString(CFMetaData metadata)
-    {
-        return toString(metadata.comparator);
-    }
-
     public String toString(ClusteringComparator comparator)
     {
         StringBuilder sb = new StringBuilder();
@@ -253,14 +254,14 @@ public class Slice
         {
             if (i > 0)
                 sb.append(':');
-            sb.append(comparator.subtype(i).getString(start.get(i)));
+            sb.append(start.stringAt(i, comparator));
         }
         sb.append(", ");
         for (int i = 0; i < end.size(); i++)
         {
             if (i > 0)
                 sb.append(':');
-            sb.append(comparator.subtype(i).getString(end.get(i)));
+            sb.append(end.stringAt(i, comparator));
         }
         sb.append(end.isInclusive() ? "]" : ")");
         return sb.toString();
@@ -299,8 +300,8 @@ public class Slice
 
         public Slice deserialize(DataInputPlus in, int version, List<AbstractType<?>> types) throws IOException
         {
-            ClusteringBound start = (ClusteringBound) ClusteringBound.serializer.deserialize(in, version, types);
-            ClusteringBound end = (ClusteringBound) ClusteringBound.serializer.deserialize(in, version, types);
+            ClusteringBound<byte[]> start = (ClusteringBound<byte[]>) ClusteringBound.serializer.deserialize(in, version, types);
+            ClusteringBound<byte[]> end = (ClusteringBound<byte[]>) ClusteringBound.serializer.deserialize(in, version, types);
             return new Slice(start, end);
         }
     }
