@@ -56,7 +56,7 @@ public class LeveledCompactionTask extends CompactionTask
     @Override
     protected boolean partialCompactionsAcceptable()
     {
-        throw new UnsupportedOperationException("This is now handled in reduceScopeForLimitedSpace");
+        return level == 0;
     }
 
     protected int getLevel()
@@ -65,7 +65,7 @@ public class LeveledCompactionTask extends CompactionTask
     }
 
     @Override
-    public boolean reduceScopeForLimitedSpace(long expectedSize)
+    public boolean reduceScopeForLimitedSpace(Set<SSTableReader> nonExpiredSSTables, long expectedSize)
     {
         if (transaction.originals().size() > 1 && level <= 1)
         {
@@ -82,7 +82,7 @@ public class LeveledCompactionTask extends CompactionTask
             // This suboptimal but ok since the caller will unmark all the sstables at the end.
             int l0SSTableCount = 0;
             SSTableReader largestL0SSTable = null;
-            for (SSTableReader sstable : transaction.originals())
+            for (SSTableReader sstable : nonExpiredSSTables)
             {
                 if (sstable.getSSTableLevel() == 0)
                 {
