@@ -21,15 +21,16 @@ import java.io.IOException;
 import java.nio.ByteBuffer;
 import java.util.Comparator;
 
-import org.apache.cassandra.config.CFMetaData;
-import org.apache.cassandra.config.ColumnDefinition;
 import org.apache.cassandra.db.TypeSizes;
-import org.apache.cassandra.db.rows.CellPath;
 import org.apache.cassandra.db.marshal.AbstractType;
 import org.apache.cassandra.db.marshal.CollectionType;
 import org.apache.cassandra.db.marshal.UTF8Type;
+import org.apache.cassandra.db.rows.CellPath;
+import org.apache.cassandra.exceptions.UnknownColumnException;
 import org.apache.cassandra.io.util.DataInputPlus;
 import org.apache.cassandra.io.util.DataOutputPlus;
+import org.apache.cassandra.schema.ColumnMetadata;
+import org.apache.cassandra.schema.TableMetadata;
 import org.apache.cassandra.utils.ByteBufferUtil;
 
 /**
@@ -45,28 +46,28 @@ public abstract class ColumnSubselection implements Comparable<ColumnSubselectio
     /* this enum is used in serialization; preserve order for compatibility */
     private enum Kind { SLICE, ELEMENT }
 
-    protected final ColumnDefinition column;
+    protected final ColumnMetadata column;
 
-    protected ColumnSubselection(ColumnDefinition column)
+    protected ColumnSubselection(ColumnMetadata column)
     {
         this.column = column;
     }
 
-    public static ColumnSubselection slice(ColumnDefinition column, CellPath from, CellPath to)
+    public static ColumnSubselection slice(ColumnMetadata column, CellPath from, CellPath to)
     {
         assert column.isComplex() && column.type instanceof CollectionType;
         assert from.size() <= 1 && to.size() <= 1;
         return new Slice(column, from, to);
     }
 
-    public static ColumnSubselection element(ColumnDefinition column, CellPath elt)
+    public static ColumnSubselection element(ColumnMetadata column, CellPath elt)
     {
         assert column.isComplex() && column.type instanceof CollectionType;
         assert elt.size() == 1;
         return new Element(column, elt);
     }
 
-    public ColumnDefinition column()
+    public ColumnMetadata column()
     {
         return column;
     }
@@ -92,7 +93,7 @@ public abstract class ColumnSubselection implements Comparable<ColumnSubselectio
         private final CellPath from;
         private final CellPath to;
 
-        private Slice(ColumnDefinition column, CellPath from, CellPath to)
+        private Slice(ColumnMetadata column, CellPath from, CellPath to)
         {
             super(column);
             this.from = from;
@@ -133,7 +134,7 @@ public abstract class ColumnSubselection implements Comparable<ColumnSubselectio
     {
         private final CellPath element;
 
-        private Element(ColumnDefinition column, CellPath elt)
+        private Element(ColumnMetadata column, CellPath elt)
         {
             super(column);
             this.element = elt;
@@ -167,7 +168,7 @@ public abstract class ColumnSubselection implements Comparable<ColumnSubselectio
     {
         public void serialize(ColumnSubselection subSel, DataOutputPlus out, int version) throws IOException
         {
-            ColumnDefinition column = subSel.column();
+            ColumnMetadata column = subSel.column();
             ByteBufferUtil.writeWithShortLength(column.name.bytes, out);
             out.writeByte(subSel.kind().ordinal());
             switch (subSel.kind())
@@ -186,21 +187,21 @@ public abstract class ColumnSubselection implements Comparable<ColumnSubselectio
             }
         }
 
-        public ColumnSubselection deserialize(DataInputPlus in, int version, CFMetaData metadata) throws IOException
+        public ColumnSubselection deserialize(DataInputPlus in, int version, TableMetadata metadata) throws IOException
         {
             ByteBuffer name = ByteBufferUtil.readWithShortLength(in);
-            ColumnDefinition column = metadata.getColumnDefinition(name);
+            ColumnMetadata column = metadata.getColumn(name);
             if (column == null)
             {
                 // If we don't find the definition, it could be we have data for a dropped column, and we shouldn't
-                // fail deserialization because of that. So we grab a "fake" ColumnDefinition that ensure proper
+                // fail deserialization because of that. So we grab a "fake" ColumnMetadata that ensure proper
                 // deserialization. The column will be ignored later on anyway.
-                column = metadata.getDroppedColumnDefinition(name);
+                column = metadata.getDroppedColumn(name);
                 if (column == null)
                 {
                     String errorMsg = String.format("Unknown column %s in table %s.%s during deserialization",
-                                                    UTF8Type.instance.getString(name), metadata.ksName, metadata.cfName);
-                    throw new RuntimeException(errorMsg);
+                                                    UTF8Type.instance.getString(name), metadata.keyspace, metadata.name);
+                    throw new UnknownColumnException(errorMsg);
                 }
             }
 
@@ -222,7 +223,7 @@ public abstract class ColumnSubselection implements Comparable<ColumnSubselectio
         {
             long size = 0;
 
-            ColumnDefinition column = subSel.column();
+            ColumnMetadata column = subSel.column();
             size += TypeSizes.sizeofWithShortLength(column.name.bytes);
             size += 1; // kind
             switch (subSel.kind())
