@@ -18,44 +18,49 @@
 */
 package org.apache.cassandra.db.partitions;
 
+import java.util.Collections;
 import java.util.Iterator;
+import java.util.NavigableSet;
 
-import org.apache.cassandra.config.CFMetaData;
+import com.google.common.annotations.VisibleForTesting;
+import com.google.common.collect.Iterators;
+
+import org.apache.cassandra.schema.TableMetadata;
 import org.apache.cassandra.db.*;
 import org.apache.cassandra.db.filter.ColumnFilter;
 import org.apache.cassandra.db.rows.*;
+import org.apache.cassandra.utils.ObjectSizes;
 import org.apache.cassandra.utils.SearchIterator;
 import org.apache.cassandra.utils.btree.BTree;
-import org.apache.cassandra.utils.btree.BTreeSearchIterator;
 
 import static org.apache.cassandra.utils.btree.BTree.Dir.desc;
 
 public abstract class AbstractBTreePartition implements Partition, Iterable<Row>
 {
-    protected static final Holder EMPTY = new Holder(PartitionColumns.NONE, BTree.empty(), DeletionInfo.LIVE, Rows.EMPTY_STATIC_ROW, EncodingStats.NO_STATS);
+    protected static final Holder EMPTY = new Holder(RegularAndStaticColumns.NONE, BTree.empty(), DeletionInfo.LIVE, Rows.EMPTY_STATIC_ROW, EncodingStats.NO_STATS);
+    public static final long HOLDER_UNSHARED_HEAP_SIZE = ObjectSizes.measure(EMPTY);
 
-    protected final CFMetaData metadata;
     protected final DecoratedKey partitionKey;
 
     protected abstract Holder holder();
     protected abstract boolean canHaveShadowedData();
 
-    protected AbstractBTreePartition(CFMetaData metadata, DecoratedKey partitionKey)
+    protected AbstractBTreePartition(DecoratedKey partitionKey)
     {
-        this.metadata = metadata;
         this.partitionKey = partitionKey;
     }
 
-    protected static final class Holder
+    @VisibleForTesting
+    public static final class Holder
     {
-        final PartitionColumns columns;
-        final DeletionInfo deletionInfo;
+        public final RegularAndStaticColumns columns;
+        public final DeletionInfo deletionInfo;
         // the btree of rows
-        final Object[] tree;
-        final Row staticRow;
-        final EncodingStats stats;
+        public final Object[] tree;
+        public final Row staticRow;
+        public final EncodingStats stats;
 
-        Holder(PartitionColumns columns, Object[] tree, DeletionInfo deletionInfo, Row staticRow, EncodingStats stats)
+        Holder(RegularAndStaticColumns columns, Object[] tree, DeletionInfo deletionInfo, Row staticRow, EncodingStats stats)
         {
             this.columns = columns;
             this.tree = tree;
@@ -87,10 +92,7 @@ public abstract class AbstractBTreePartition implements Partition, Iterable<Row>
         return !BTree.isEmpty(holder.tree);
     }
 
-    public CFMetaData metadata()
-    {
-        return metadata;
-    }
+    public abstract TableMetadata metadata();
 
     public DecoratedKey partitionKey()
     {
@@ -102,7 +104,7 @@ public abstract class AbstractBTreePartition implements Partition, Iterable<Row>
         return deletionInfo().getPartitionDeletion();
     }
 
-    public PartitionColumns columns()
+    public RegularAndStaticColumns columns()
     {
         return holder().columns;
     }
@@ -112,12 +114,37 @@ public abstract class AbstractBTreePartition implements Partition, Iterable<Row>
         return holder().stats;
     }
 
-    public Row getRow(Clustering clustering)
+    public Row getRow(Clustering<?> clustering)
     {
-        Row row = searchIterator(ColumnFilter.selection(columns()), false).next(clustering);
-        // Note that for statics, this will never return null, this will return an empty row. However,
-        // it's more consistent for this method to return null if we don't really have a static row.
-        return row == null || (clustering == Clustering.STATIC_CLUSTERING && row.isEmpty()) ? null : row;
+        ColumnFilter columns = ColumnFilter.selection(columns());
+        Holder holder = holder();
+
+        if (clustering == Clustering.STATIC_CLUSTERING)
+        {
+            // Note that for statics, this will never return null, this will return an empty row. However,
+            // it's more consistent for this method to return null if we don't really have a static row.
+            Row staticRow = staticRow(holder, columns, true);
+            return staticRow.isEmpty() ? null : staticRow;
+        }
+
+        final Row row = (Row) BTree.find(holder.tree, metadata().comparator, clustering);
+        DeletionTime activeDeletion = holder.deletionInfo.getPartitionDeletion();
+        RangeTombstone rt = holder.deletionInfo.rangeCovering(clustering);
+
+        if (rt != null && rt.deletionTime().supersedes(activeDeletion))
+            activeDeletion = rt.deletionTime();
+
+
+        if (row == null)
+        {
+            // this means our partition level deletion supersedes all other deletions and we don't have to keep the row deletions
+            if (activeDeletion == holder.deletionInfo.getPartitionDeletion())
+                return null;
+            // no need to check activeDeletion.isLive here - if anything superseedes the partitionDeletion
+            // it must be non-live
+            return BTreeRow.emptyDeletedRow(clustering, Row.Deletion.regular(activeDeletion));
+        }
+        return row.filter(columns, activeDeletion, true, metadata());
     }
 
     private Row staticRow(Holder current, ColumnFilter columns, boolean setActiveDeletionToRow)
@@ -126,47 +153,21 @@ public abstract class AbstractBTreePartition implements Partition, Iterable<Row>
         if (columns.fetchedColumns().statics.isEmpty() || (current.staticRow.isEmpty() && partitionDeletion.isLive()))
             return Rows.EMPTY_STATIC_ROW;
 
-        Row row = current.staticRow.filter(columns, partitionDeletion, setActiveDeletionToRow, metadata);
+        Row row = current.staticRow.filter(columns, partitionDeletion, setActiveDeletionToRow, metadata());
         return row == null ? Rows.EMPTY_STATIC_ROW : row;
     }
 
-    public SearchIterator<Clustering, Row> searchIterator(final ColumnFilter columns, final boolean reversed)
+    @Override
+    public UnfilteredRowIterator unfilteredIterator(ColumnFilter selection, NavigableSet<Clustering<?>> clusteringsInQueryOrder, boolean reversed)
     {
-        // TODO: we could optimize comparison for "NativeRow" à la #6755
-        final Holder current = holder();
-        return new SearchIterator<Clustering, Row>()
+        Row staticRow = staticRow(holder(), selection, false);
+        if (clusteringsInQueryOrder.isEmpty())
         {
-            private final SearchIterator<Clustering, Row> rawIter = new BTreeSearchIterator<>(current.tree, metadata.comparator, desc(reversed));
-            private final DeletionTime partitionDeletion = current.deletionInfo.getPartitionDeletion();
+            DeletionTime partitionDeletion = holder().deletionInfo.getPartitionDeletion();
+            return UnfilteredRowIterators.noRowsIterator(metadata(), partitionKey(), staticRow, partitionDeletion, reversed);
+        }
 
-            public Row next(Clustering clustering)
-            {
-                if (clustering == Clustering.STATIC_CLUSTERING)
-                    return staticRow(current, columns, true);
-
-                Row row = rawIter.next(clustering);
-                RangeTombstone rt = current.deletionInfo.rangeCovering(clustering);
-
-                // A search iterator only return a row, so it doesn't allow to directly account for deletion that should apply to to row
-                // (the partition deletion or the deletion of a range tombstone that covers it). So if needs be, reuse the row deletion
-                // to carry the proper deletion on the row.
-                DeletionTime activeDeletion = partitionDeletion;
-                if (rt != null && rt.deletionTime().supersedes(activeDeletion))
-                    activeDeletion = rt.deletionTime();
-
-                if (row == null)
-                {
-                    // this means our partition level deletion superseedes all other deletions and we don't have to keep the row deletions
-                    if (activeDeletion == partitionDeletion)
-                        return null;
-                    // no need to check activeDeletion.isLive here - if anything superseedes the partitionDeletion
-                    // it must be non-live
-                    return BTreeRow.emptyDeletedRow(clustering, Row.Deletion.regular(activeDeletion));
-                }
-
-                return row.filter(columns, activeDeletion, true, metadata);
-            }
-        };
+        return new ClusteringsIterator(selection, clusteringsInQueryOrder, reversed, holder(), staticRow);
     }
 
     public UnfilteredRowIterator unfilteredIterator()
@@ -185,7 +186,7 @@ public abstract class AbstractBTreePartition implements Partition, Iterable<Row>
         if (slices.size() == 0)
         {
             DeletionTime partitionDeletion = current.deletionInfo.getPartitionDeletion();
-            return UnfilteredRowIterators.noRowsIterator(metadata, partitionKey(), staticRow, partitionDeletion, reversed);
+            return UnfilteredRowIterators.noRowsIterator(metadata(), partitionKey(), staticRow, partitionDeletion, reversed);
         }
 
         return slices.size() == 1
@@ -195,9 +196,9 @@ public abstract class AbstractBTreePartition implements Partition, Iterable<Row>
 
     private UnfilteredRowIterator sliceIterator(ColumnFilter selection, Slice slice, boolean reversed, Holder current, Row staticRow)
     {
-        ClusteringBound start = slice.start() == ClusteringBound.BOTTOM ? null : slice.start();
-        ClusteringBound end = slice.end() == ClusteringBound.TOP ? null : slice.end();
-        Iterator<Row> rowIter = BTree.slice(current.tree, metadata.comparator, start, true, end, true, desc(reversed));
+        ClusteringBound<?> start = slice.start().isBottom() ? null : slice.start();
+        ClusteringBound<?> end = slice.end().isTop() ? null : slice.end();
+        Iterator<Row> rowIter = BTree.slice(current.tree, metadata().comparator, start, true, end, true, desc(reversed));
         Iterator<RangeTombstone> deleteIter = current.deletionInfo.rangeIterator(slice, reversed);
         return merge(rowIter, deleteIter, selection, reversed, current, staticRow);
     }
@@ -205,7 +206,7 @@ public abstract class AbstractBTreePartition implements Partition, Iterable<Row>
     private RowAndDeletionMergeIterator merge(Iterator<Row> rowIter, Iterator<RangeTombstone> deleteIter,
                                               ColumnFilter selection, boolean reversed, Holder current, Row staticRow)
     {
-        return new RowAndDeletionMergeIterator(metadata, partitionKey(), current.deletionInfo.getPartitionDeletion(),
+        return new RowAndDeletionMergeIterator(metadata(), partitionKey(), current.deletionInfo.getPartitionDeletion(),
                                                selection, staticRow, reversed, current.stats,
                                                rowIter, deleteIter,
                                                canHaveShadowedData());
@@ -218,12 +219,12 @@ public abstract class AbstractBTreePartition implements Partition, Iterable<Row>
 
         private AbstractIterator(Holder current, Row staticRow, ColumnFilter selection, boolean isReversed)
         {
-            super(AbstractBTreePartition.this.metadata,
+            super(AbstractBTreePartition.this.metadata(),
                   AbstractBTreePartition.this.partitionKey(),
                   current.deletionInfo.getPartitionDeletion(),
                   selection.fetchedColumns(), // non-selected columns will be filtered in subclasses by RowAndDeletionMergeIterator
-                                              // it would also be more precise to return the intersection of the selection and current.columns,
-                                              // but its probably not worth spending time on computing that.
+                  // it would also be more precise to return the intersection of the selection and current.columns,
+                  // but its probably not worth spending time on computing that.
                   staticRow,
                   isReversed,
                   current.stats);
@@ -232,18 +233,14 @@ public abstract class AbstractBTreePartition implements Partition, Iterable<Row>
         }
     }
 
-    public class SlicesIterator extends AbstractIterator
+    private class SlicesIterator extends AbstractIterator
     {
         private final Slices slices;
 
         private int idx;
         private Iterator<Unfiltered> currentSlice;
 
-        private SlicesIterator(ColumnFilter selection,
-                               Slices slices,
-                               boolean isReversed,
-                               Holder current,
-                               Row staticRow)
+        private SlicesIterator(ColumnFilter selection, Slices slices, boolean isReversed, Holder current, Row staticRow)
         {
             super(current, staticRow, selection, isReversed);
             this.slices = slices;
@@ -271,24 +268,72 @@ public abstract class AbstractBTreePartition implements Partition, Iterable<Row>
         }
     }
 
+    private class ClusteringsIterator extends AbstractIterator
+    {
+        private final Iterator<Clustering<?>> clusteringsInQueryOrder;
+        private final SearchIterator<Clustering<?>, Row> rowSearcher;
+
+        private Iterator<Unfiltered> currentIterator;
+
+        private ClusteringsIterator(ColumnFilter selection,
+                                    NavigableSet<Clustering<?>> clusteringsInQueryOrder,
+                                    boolean isReversed,
+                                    Holder current,
+                                    Row staticRow)
+        {
+            super(current, staticRow, selection, isReversed);
+
+            this.clusteringsInQueryOrder = clusteringsInQueryOrder.iterator();
+            this.rowSearcher = BTree.slice(current.tree, metadata().comparator, desc(isReversed));
+        }
+
+        protected Unfiltered computeNext()
+        {
+            while (true)
+            {
+                if (currentIterator == null)
+                {
+                    if (!clusteringsInQueryOrder.hasNext())
+                        return endOfData();
+
+                    currentIterator = nextIterator(clusteringsInQueryOrder.next());
+                }
+
+                if (currentIterator != null && currentIterator.hasNext())
+                    return currentIterator.next();
+
+                currentIterator = null;
+            }
+        }
+
+        private Iterator<Unfiltered> nextIterator(Clustering<?> next)
+        {
+            Row nextRow = rowSearcher.next(next);
+            // rangeCovering() will return original RT covering clustering key, but we want to generate fake RT with
+            // given clustering bound to be consistent with fake RT generated from sstable read.
+            Iterator<RangeTombstone> deleteIter = current.deletionInfo.rangeIterator(Slice.make(next), isReverseOrder());
+
+            if (nextRow == null && !deleteIter.hasNext())
+                return null;
+
+            Iterator<Row> rowIterator = nextRow == null ? Collections.emptyIterator() : Iterators.singletonIterator(nextRow);
+            return merge(rowIterator, deleteIter, selection, isReverseOrder, current, staticRow);
+        }
+    }
+
     protected static Holder build(UnfilteredRowIterator iterator, int initialRowCapacity)
     {
-        return build(iterator, initialRowCapacity, true, null);
+        return build(iterator, initialRowCapacity, true);
     }
 
-    protected static Holder build(UnfilteredRowIterator iterator, int initialRowCapacity, boolean ordered, BTree.Builder.QuickResolver<Row> quickResolver)
+    protected static Holder build(UnfilteredRowIterator iterator, int initialRowCapacity, boolean ordered)
     {
-        return build(iterator, initialRowCapacity, ordered, quickResolver, iterator.columns());
-    }
-
-    protected static Holder build(UnfilteredRowIterator iterator, int initialRowCapacity, boolean ordered, BTree.Builder.QuickResolver<Row> quickResolver, PartitionColumns columns)
-    {
-        CFMetaData metadata = iterator.metadata();
+        TableMetadata metadata = iterator.metadata();
+        RegularAndStaticColumns columns = iterator.columns();
         boolean reversed = iterator.isReverseOrder();
 
         BTree.Builder<Row> builder = BTree.builder(metadata.comparator, initialRowCapacity);
         builder.auto(!ordered);
-        builder.setQuickResolver(quickResolver);
         MutableDeletionInfo.Builder deletionBuilder = MutableDeletionInfo.builder(iterator.partitionLevelDeletion(), metadata.comparator, reversed);
 
         while (iterator.hasNext())
@@ -308,25 +353,25 @@ public abstract class AbstractBTreePartition implements Partition, Iterable<Row>
 
     // Note that when building with a RowIterator, deletion will generally be LIVE, but we allow to pass it nonetheless because PartitionUpdate
     // passes a MutableDeletionInfo that it mutates later.
-    protected static Holder build(RowIterator rows, DeletionInfo deletion, boolean buildEncodingStats, int initialRowCapacity)
+    protected static Holder build(RowIterator rows, DeletionInfo deletion, boolean buildEncodingStats)
     {
-        CFMetaData metadata = rows.metadata();
-        PartitionColumns columns = rows.columns();
+        RegularAndStaticColumns columns = rows.columns();
         boolean reversed = rows.isReverseOrder();
 
-        BTree.Builder<Row> builder = BTree.builder(metadata.comparator, initialRowCapacity);
-        builder.auto(false);
-        while (rows.hasNext())
-            builder.add(rows.next());
+        try (BTree.FastBuilder<Row> builder = BTree.fastBuilder())
+        {
+            while (rows.hasNext())
+                builder.add(rows.next());
 
-        if (reversed)
-            builder.reverse();
 
-        Row staticRow = rows.staticRow();
-        Object[] tree = builder.build();
-        EncodingStats stats = buildEncodingStats ? EncodingStats.Collector.collect(staticRow, BTree.iterator(tree), deletion)
-                                                 : EncodingStats.NO_STATS;
-        return new Holder(columns, tree, deletion, staticRow, stats);
+            Object[] tree = reversed ? builder.buildReverse()
+                                     : builder.build();
+
+            Row staticRow = rows.staticRow();
+            EncodingStats stats = buildEncodingStats ? EncodingStats.Collector.collect(staticRow, BTree.iterator(tree), deletion)
+                                                     : EncodingStats.NO_STATS;
+            return new Holder(columns, tree, deletion, staticRow, stats);
+        }
     }
 
     @Override
@@ -334,20 +379,19 @@ public abstract class AbstractBTreePartition implements Partition, Iterable<Row>
     {
         StringBuilder sb = new StringBuilder();
 
-        sb.append(String.format("[%s.%s] key=%s partition_deletion=%s columns=%s",
-                                metadata.ksName,
-                                metadata.cfName,
-                                metadata.getKeyValidator().getString(partitionKey().getKey()),
+        sb.append(String.format("[%s] key=%s partition_deletion=%s columns=%s",
+                                metadata(),
+                                metadata().partitionKeyType.getString(partitionKey().getKey()),
                                 partitionLevelDeletion(),
                                 columns()));
 
         if (staticRow() != Rows.EMPTY_STATIC_ROW)
-            sb.append("\n    ").append(staticRow().toString(metadata, true));
+            sb.append("\n    ").append(staticRow().toString(metadata(), true));
 
         try (UnfilteredRowIterator iter = unfilteredIterator())
         {
             while (iter.hasNext())
-                sb.append("\n    ").append(iter.next().toString(metadata, true));
+                sb.append("\n    ").append(iter.next().toString(metadata(), true));
         }
 
         return sb.toString();
@@ -371,4 +415,16 @@ public abstract class AbstractBTreePartition implements Partition, Iterable<Row>
 
         return BTree.findByIndex(tree, BTree.size(tree) - 1);
     }
+
+    @VisibleForTesting
+    public static Holder unsafeGetEmptyHolder()
+    {
+        return EMPTY;
+    }
+
+    @VisibleForTesting
+    public static Holder unsafeConstructHolder(RegularAndStaticColumns columns, Object[] tree, DeletionInfo deletionInfo, Row staticRow, EncodingStats stats)
+    {
+        return new Holder(columns, tree, deletionInfo, staticRow, stats);
+    }
 }
