@@ -18,19 +18,19 @@
 package org.apache.cassandra.db.rows;
 
 import java.nio.ByteBuffer;
-import java.security.MessageDigest;
 import java.util.Objects;
 
-import org.apache.cassandra.config.ColumnDefinition;
+import org.apache.cassandra.db.Digest;
 import org.apache.cassandra.db.DeletionPurger;
 import org.apache.cassandra.db.TypeSizes;
 import org.apache.cassandra.db.context.CounterContext;
 import org.apache.cassandra.db.marshal.AbstractType;
 import org.apache.cassandra.db.marshal.CollectionType;
+import org.apache.cassandra.db.marshal.ValueAccessor;
+import org.apache.cassandra.schema.ColumnMetadata;
 import org.apache.cassandra.serializers.MarshalException;
 import org.apache.cassandra.utils.ByteBufferUtil;
-import org.apache.cassandra.utils.FBUtilities;
-import org.apache.cassandra.utils.memory.AbstractAllocator;
+import org.apache.cassandra.utils.memory.ByteBufferCloner;
 
 /**
  * Base abstract class for {@code Cell} implementations.
@@ -38,9 +38,9 @@ import org.apache.cassandra.utils.memory.AbstractAllocator;
  * Unless you have a very good reason not to, every cell implementation
  * should probably extend this class.
  */
-public abstract class AbstractCell extends Cell
+public abstract class AbstractCell<V> extends Cell<V>
 {
-    protected AbstractCell(ColumnDefinition column)
+    protected AbstractCell(ColumnMetadata column)
     {
         super(column);
     }
@@ -65,17 +65,17 @@ public abstract class AbstractCell extends Cell
         return ttl() != NO_TTL;
     }
 
-    public Cell markCounterLocalToBeCleared()
+    public Cell<?> markCounterLocalToBeCleared()
     {
         if (!isCounterCell())
             return this;
 
-        ByteBuffer value = value();
+        ByteBuffer value = buffer();
         ByteBuffer marked = CounterContext.instance().markLocalToBeCleared(value);
         return marked == value ? this : new BufferCell(column, timestamp(), ttl(), localDeletionTime(), marked, path());
     }
 
-    public Cell purge(DeletionPurger purger, int nowInSec)
+    public Cell<?> purge(DeletionPurger purger, int nowInSec)
     {
         if (!isLive(nowInSec))
         {
@@ -98,16 +98,17 @@ public abstract class AbstractCell extends Cell
         return this;
     }
 
-    public Cell copy(AbstractAllocator allocator)
+    @Override
+    public Cell<?> clone(ByteBufferCloner cloner)
     {
         CellPath path = path();
-        return new BufferCell(column, timestamp(), ttl(), localDeletionTime(), allocator.clone(value()), path == null ? null : path.copy(allocator));
+        return new BufferCell(column, timestamp(), ttl(), localDeletionTime(), cloner.clone(buffer()), path == null ? null : path.clone(cloner));
     }
 
     // note: while the cell returned may be different, the value is the same, so if the value is offheap it must be referenced inside a guarded context (or copied)
-    public Cell updateAllTimestamp(long newTimestamp)
+    public Cell<?> updateAllTimestamp(long newTimestamp)
     {
-        return new BufferCell(column, isTombstone() ? newTimestamp - 1 : newTimestamp, ttl(), localDeletionTime(), value(), path());
+        return new BufferCell(column, isTombstone() ? newTimestamp - 1 : newTimestamp, ttl(), localDeletionTime(), buffer(), path());
     }
 
     public int dataSize()
@@ -116,24 +117,20 @@ public abstract class AbstractCell extends Cell
         return TypeSizes.sizeof(timestamp())
                + TypeSizes.sizeof(ttl())
                + TypeSizes.sizeof(localDeletionTime())
-               + value().remaining()
+               + valueSize()
                + (path == null ? 0 : path.dataSize());
     }
 
-    public void digest(MessageDigest digest)
+    public void digest(Digest digest)
     {
         if (isCounterCell())
-        {
-            CounterContext.instance().updateDigest(digest, value());
-        }
+            digest.updateWithCounterContext(value(), accessor());
         else
-        {
-            digest.update(value().duplicate());
-        }
+            digest.update(value(), accessor());
 
-        FBUtilities.updateWithLong(digest, timestamp());
-        FBUtilities.updateWithInt(digest, ttl());
-        FBUtilities.updateWithBoolean(digest, isCounterCell());
+        digest.updateWithLong(timestamp())
+              .updateWithInt(ttl())
+              .updateWithBoolean(isCounterCell());
         if (path() != null)
             path().digest(digest);
     }
@@ -148,17 +145,35 @@ public abstract class AbstractCell extends Cell
             throw new MarshalException("Shoud not have a TTL without an associated local deletion time");
 
         // non-frozen UDTs require both the cell path & value to validate,
-        // so that logic is pushed down into ColumnDefinition. Tombstone
+        // so that logic is pushed down into ColumnMetadata. Tombstone
         // validation is done there too as it also involves the cell path
         // for complex columns
         column().validateCell(this);
     }
 
+    public boolean hasInvalidDeletions()
+    {
+        if (ttl() < 0 || localDeletionTime() < 0 || (isExpiring() && localDeletionTime() == NO_DELETION_TIME))
+            return true;
+        return false;
+    }
+
     public long maxTimestamp()
     {
         return timestamp();
     }
 
+    public static <V1, V2> boolean equals(Cell<V1> left, Cell<V2> right)
+    {
+        return left.column().equals(right.column())
+               && left.isCounterCell() == right.isCounterCell()
+               && left.timestamp() == right.timestamp()
+               && left.ttl() == right.ttl()
+               && left.localDeletionTime() == right.localDeletionTime()
+               && ValueAccessor.equals(left.value(), left.accessor(), right.value(), right.accessor())
+               && Objects.equals(left.path(), right.path());
+    }
+
     @Override
     public boolean equals(Object other)
     {
@@ -168,27 +183,20 @@ public abstract class AbstractCell extends Cell
         if(!(other instanceof Cell))
             return false;
 
-        Cell that = (Cell)other;
-        return this.column().equals(that.column())
-            && this.isCounterCell() == that.isCounterCell()
-            && this.timestamp() == that.timestamp()
-            && this.ttl() == that.ttl()
-            && this.localDeletionTime() == that.localDeletionTime()
-            && Objects.equals(this.value(), that.value())
-            && Objects.equals(this.path(), that.path());
+        return equals(this, (Cell<?>) other);
     }
 
     @Override
     public int hashCode()
     {
-        return Objects.hash(column(), isCounterCell(), timestamp(), ttl(), localDeletionTime(), value(), path());
+        return Objects.hash(column(), isCounterCell(), timestamp(), ttl(), localDeletionTime(), accessor().hashCode(value()), path());
     }
 
     @Override
     public String toString()
     {
         if (isCounterCell())
-            return String.format("[%s=%d ts=%d]", column().name, CounterContext.instance().total(value()), timestamp());
+            return String.format("[%s=%d ts=%d]", column().name, CounterContext.instance().total(value(), accessor()), timestamp());
 
         AbstractType<?> type = column().type;
         if (type instanceof CollectionType && type.isMultiCell())
@@ -196,25 +204,25 @@ public abstract class AbstractCell extends Cell
             CollectionType ct = (CollectionType)type;
             return String.format("[%s[%s]=%s %s]",
                                  column().name,
-                                 safeToString(ct.nameComparator(), path().get(0)),
-                                 safeToString(ct.valueComparator(), value()),
+                                 ct.nameComparator().getString(path().get(0)),
+                                 ct.valueComparator().getString(value(), accessor()),
                                  livenessInfoString());
         }
         if (isTombstone())
             return String.format("[%s=<tombstone> %s]", column().name, livenessInfoString());
         else
-            return String.format("[%s=%s %s]", column().name, safeToString(type, value()), livenessInfoString());
+            return String.format("[%s=%s %s]", column().name, safeToString(type), livenessInfoString());
     }
 
-    private static String safeToString(AbstractType<?> type, ByteBuffer data)
+    private String safeToString(AbstractType<?> type)
     {
         try
         {
-            return type.getString(data);
+            return type.getString(value(), accessor());
         }
         catch (Exception e)
         {
-            return "0x" + ByteBufferUtil.bytesToHex(data);
+            return "0x" + ByteBufferUtil.bytesToHex(buffer());
         }
     }
 
