@@ -20,16 +20,19 @@ package org.apache.cassandra.db.rows;
 import java.nio.ByteBuffer;
 import java.util.Iterator;
 import java.util.Objects;
-import java.util.function.BiFunction;
 
+import com.google.common.annotations.VisibleForTesting;
 import com.google.common.base.Function;
 
 import org.apache.cassandra.db.DeletionPurger;
 import org.apache.cassandra.db.DeletionTime;
 import org.apache.cassandra.db.Digest;
 import org.apache.cassandra.db.LivenessInfo;
+import org.apache.cassandra.db.context.CounterContext;
 import org.apache.cassandra.db.filter.ColumnFilter;
+import org.apache.cassandra.db.marshal.AbstractType;
 import org.apache.cassandra.db.marshal.ByteType;
+import org.apache.cassandra.db.marshal.CollectionType;
 import org.apache.cassandra.db.marshal.SetType;
 import org.apache.cassandra.schema.ColumnMetadata;
 import org.apache.cassandra.schema.DroppedColumn;
@@ -38,6 +41,7 @@ import org.apache.cassandra.utils.LongAccumulator;
 import org.apache.cassandra.utils.ObjectSizes;
 import org.apache.cassandra.utils.SearchIterator;
 import org.apache.cassandra.utils.btree.BTree;
+import org.apache.cassandra.utils.memory.Cloner;
 
 /**
  * The data for a complex column, that is it's cells and potential complex
@@ -54,7 +58,6 @@ public class ComplexColumnData extends ColumnData implements Iterable<Cell<?>>
 
     private final DeletionTime complexDeletion;
 
-    // Only ArrayBackedRow should call this.
     ComplexColumnData(ColumnMetadata column, Object[] cells, DeletionTime complexDeletion)
     {
         super(column);
@@ -95,6 +98,11 @@ public class ComplexColumnData extends ColumnData implements Iterable<Cell<?>>
         return complexDeletion;
     }
 
+    Object[] tree()
+    {
+        return cells;
+    }
+
     public Iterator<Cell<?>> iterator()
     {
         return BTree.iterator(cells);
@@ -130,7 +138,7 @@ public class ComplexColumnData extends ColumnData implements Iterable<Cell<?>>
 
     public long unsharedHeapSizeExcludingData()
     {
-        long heapSize = EMPTY_SIZE + ObjectSizes.sizeOfArray(cells);
+        long heapSize = EMPTY_SIZE + BTree.sizeOnHeapOf(cells);
         // TODO: this can be turned into a simple multiplication, at least while we have only one Cell implementation
         for (Cell<?> cell : this)
             heapSize += cell.unsharedHeapSizeExcludingData();
@@ -213,14 +221,25 @@ public class ComplexColumnData extends ColumnData implements Iterable<Cell<?>>
         return new ComplexColumnData(column, newCells, newDeletion);
     }
 
+    public ComplexColumnData transformAndFilter(Function<? super Cell<?>, ? extends Cell<?>> function)
+    {
+        return update(complexDeletion, BTree.transformAndFilter(cells, function));
+    }
+
     public ComplexColumnData transformAndFilter(DeletionTime newDeletion, Function<? super Cell, ? extends Cell> function)
     {
         return update(newDeletion, BTree.transformAndFilter(cells, function));
     }
 
-    public <V> ComplexColumnData transformAndFilter(BiFunction<? super Cell, ? super V, ? extends Cell> function, V param)
+    public <V> ComplexColumnData transform(Function<? super Cell<?>, ? extends Cell<?>> function)
     {
-        return update(complexDeletion, BTree.transformAndFilter(cells, function, param));
+        return update(complexDeletion, BTree.transform(cells, function));
+    }
+
+    @Override
+    public ColumnData clone(Cloner cloner)
+    {
+        return transform(c -> cloner.clone(c));
     }
 
     public ComplexColumnData updateAllTimestamp(long newTimestamp)
@@ -266,6 +285,21 @@ public class ComplexColumnData extends ColumnData implements Iterable<Cell<?>>
         return Objects.hash(column(), complexDeletion(), BTree.hashCode(cells));
     }
 
+    @Override
+    public String toString()
+    {
+        return String.format("[%s=%s %s]",
+                             column().name,
+                             complexDeletion.toString(),
+                             BTree.toString(cells));
+    }
+
+    @VisibleForTesting
+    public static ComplexColumnData unsafeConstruct(ColumnMetadata column, Object[] cells, DeletionTime complexDeletion)
+    {
+        return new ComplexColumnData(column, cells, complexDeletion);
+    }
+
     public static Builder builder()
     {
         return new Builder();
