@@ -17,17 +17,13 @@
  */
 package org.apache.cassandra.db.rows;
 
-import java.nio.ByteBuffer;
-import java.security.MessageDigest;
-import java.util.Set;
-
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
-import org.apache.cassandra.config.CFMetaData;
+import org.apache.cassandra.db.Digest;
 import org.apache.cassandra.db.filter.ColumnFilter;
 import org.apache.cassandra.db.transform.Transformation;
-import org.apache.cassandra.utils.FBUtilities;
+import org.apache.cassandra.schema.TableMetadata;
 
 /**
  * Static methods to work with row iterators.
@@ -38,35 +34,20 @@ public abstract class RowIterators
 
     private RowIterators() {}
 
-    public static void digest(RowIterator iterator, MessageDigest digest, MessageDigest altDigest, Set<ByteBuffer> columnsToExclude)
+    public static void digest(RowIterator iterator, Digest digest)
     {
         // TODO: we're not computing digest the same way that old nodes. This is
         // currently ok as this is only used for schema digest and the is no exchange
         // of schema digest between different versions. If this changes however,
         // we'll need to agree on a version.
-        digest.update(iterator.partitionKey().getKey().duplicate());
+        digest.update(iterator.partitionKey().getKey());
         iterator.columns().regulars.digest(digest);
         iterator.columns().statics.digest(digest);
-        FBUtilities.updateWithBoolean(digest, iterator.isReverseOrder());
+        digest.updateWithBoolean(iterator.isReverseOrder());
         iterator.staticRow().digest(digest);
 
-        if (altDigest != null)
-        {
-            // Compute the "alternative digest" here.
-            altDigest.update(iterator.partitionKey().getKey().duplicate());
-            iterator.columns().regulars.digest(altDigest, columnsToExclude);
-            iterator.columns().statics.digest(altDigest, columnsToExclude);
-            FBUtilities.updateWithBoolean(altDigest, iterator.isReverseOrder());
-            iterator.staticRow().digest(altDigest, columnsToExclude);
-        }
-
         while (iterator.hasNext())
-        {
-            Row row = iterator.next();
-            row.digest(digest);
-            if (altDigest != null)
-                row.digest(altDigest, columnsToExclude);
-        }
+            iterator.next().digest(digest);
     }
 
     /**
@@ -93,12 +74,12 @@ public abstract class RowIterators
      */
     public static RowIterator loggingIterator(RowIterator iterator, final String id)
     {
-        CFMetaData metadata = iterator.metadata();
+        TableMetadata metadata = iterator.metadata();
         logger.info("[{}] Logging iterator on {}.{}, partition key={}, reversed={}",
                     id,
-                    metadata.ksName,
-                    metadata.cfName,
-                    metadata.getKeyValidator().getString(iterator.partitionKey().getKey()),
+                    metadata.keyspace,
+                    metadata.name,
+                    metadata.partitionKeyType.getString(iterator.partitionKey().getKey()),
                     iterator.isReverseOrder());
 
         class Log extends Transformation
