@@ -21,14 +21,14 @@
 package org.apache.cassandra.db.transform;
 
 import org.apache.cassandra.db.DeletionTime;
-import org.apache.cassandra.db.PartitionColumns;
+import org.apache.cassandra.db.RegularAndStaticColumns;
 import org.apache.cassandra.db.rows.EncodingStats;
 import org.apache.cassandra.db.rows.Unfiltered;
 import org.apache.cassandra.db.rows.UnfilteredRowIterator;
 
 final class UnfilteredRows extends BaseRows<Unfiltered, UnfilteredRowIterator> implements UnfilteredRowIterator
 {
-    private PartitionColumns partitionColumns;
+    private RegularAndStaticColumns regularAndStaticColumns;
     private DeletionTime partitionLevelDeletion;
 
     public UnfilteredRows(UnfilteredRowIterator input)
@@ -36,10 +36,10 @@ final class UnfilteredRows extends BaseRows<Unfiltered, UnfilteredRowIterator> i
         this(input, input.columns());
     }
 
-    public UnfilteredRows(UnfilteredRowIterator input, PartitionColumns columns)
+    public UnfilteredRows(UnfilteredRowIterator input, RegularAndStaticColumns columns)
     {
         super(input);
-        partitionColumns = columns;
+        regularAndStaticColumns = columns;
         partitionLevelDeletion = input.partitionLevelDeletion();
     }
 
@@ -47,14 +47,14 @@ final class UnfilteredRows extends BaseRows<Unfiltered, UnfilteredRowIterator> i
     void add(Transformation add)
     {
         super.add(add);
-        partitionColumns = add.applyToPartitionColumns(partitionColumns);
+        regularAndStaticColumns = add.applyToPartitionColumns(regularAndStaticColumns);
         partitionLevelDeletion = add.applyToDeletion(partitionLevelDeletion);
     }
 
     @Override
-    public PartitionColumns columns()
+    public RegularAndStaticColumns columns()
     {
-        return partitionColumns;
+        return regularAndStaticColumns;
     }
 
     @Override
