@@ -18,6 +18,8 @@
 package org.apache.cassandra.gms;
 
 import java.net.UnknownHostException;
+import java.util.List;
+import java.util.Map;
 
 public interface GossiperMBean
 {
@@ -29,4 +31,11 @@ public interface GossiperMBean
 
     public void assassinateEndpoint(String address) throws UnknownHostException;
 
+    public List<String> reloadSeeds();
+
+    public List<String> getSeeds();
+
+    /** Returns each node's database release version */
+    public Map<String, List<String>> getReleaseVersionsWithPort();
+
 }
\ No newline at end of file
