@@ -812,6 +812,14 @@ public class TokenMetadata
         return builder.build();
     }
 
+    @VisibleForTesting
+    public void setPendingRangesUnsafe(String keyspaceName, Multimap<Range<Token>, Replica> rangeMap)
+    {
+        PendingRangeMaps prm = new PendingRangeMaps();
+        rangeMap.entries().forEach(entry -> prm.addPendingRange(entry.getKey(), entry.getValue()));
+        pendingRanges.put(keyspaceName, prm);
+    }
+
      /**
      * Calculate pending ranges according to bootsrapping and leaving nodes. Reasoning is:
      *
@@ -1400,6 +1408,12 @@ public class TokenMetadata
         return partitioner.decorateKey(key);
     }
 
+    public boolean isTokenInLocalPendingRange(String keyspaceName, Token token)
+    {
+        PendingRangeMaps pending = pendingRanges.get(keyspaceName);
+        return pending == null ? false : pending.isTokenInLocalPendingRange(token);
+    }
+
     /**
      * Tracks the assignment of racks and endpoints in each datacenter for all the "normal" endpoints
      * in this TokenMetadata. This allows faster calculation of endpoints in NetworkTopologyStrategy.
