@@ -21,26 +21,28 @@ package org.apache.cassandra.metrics;
 import java.io.OutputStream;
 import java.io.OutputStreamWriter;
 import java.io.PrintWriter;
-import java.nio.charset.Charset;
-import java.util.Arrays;
+import java.nio.charset.StandardCharsets;
+import java.util.Objects;
 import java.util.concurrent.atomic.AtomicBoolean;
-import java.util.concurrent.atomic.AtomicLong;
 import java.util.concurrent.atomic.AtomicLongArray;
-import java.util.concurrent.locks.ReentrantReadWriteLock;
 
 import com.google.common.annotations.VisibleForTesting;
+import com.google.common.primitives.Ints;
 
 import com.codahale.metrics.Clock;
 import com.codahale.metrics.Reservoir;
 import com.codahale.metrics.Snapshot;
 import org.apache.cassandra.utils.EstimatedHistogram;
 
+import static java.lang.Math.max;
+import static java.lang.Math.min;
+
 /**
  * A decaying histogram reservoir where values collected during each minute will be twice as significant as the values
  * collected in the previous minute. Measured values are collected in variable sized buckets, using small buckets in the
  * lower range and larger buckets in the upper range. Use this histogram when you want to know if the distribution of
  * the underlying data stream has changed recently and you want high resolution on values in the lower range.
- *
+ * <p/>
  * The histogram use forward decay [1] to make recent values more significant. The forward decay factor will be doubled
  * every minute (half-life time set to 60 seconds) [2]. The forward decay landmark is reset every 30 minutes (or at
  * first read/update after 30 minutes). During landmark reset, updates and reads in the reservoir will be blocked in a
@@ -48,42 +50,95 @@ import org.apache.cassandra.utils.EstimatedHistogram;
  * assumption that in an extreme case we would have to collect a metric 1M times for a single bucket each second. By the
  * end of the 30:th minute all collected values will roughly add up to 1.000.000 * 60 * pow(2, 30) which can be
  * represented with 56 bits giving us some head room in a signed 64 bit long.
- *
- * Internally two reservoirs are maintained, one with decay and one without decay. All public getters in a {@Snapshot}
+ * <p/>
+ * Internally two reservoirs are maintained, one with decay and one without decay. All public getters in a {@link Snapshot}
  * will expose the decay functionality with the exception of the {@link Snapshot#getValues()} which will return values
  * from the reservoir without decay. This makes it possible for the caller to maintain precise deltas in an interval of
- * its choise.
- *
+ * its choice.
+ * <p/>
  * The bucket size starts at 1 and grows by 1.2 each time (rounding and removing duplicates). It goes from 1 to around
  * 18T by default (creating 164+1 buckets), which will give a timing resolution from microseconds to roughly 210 days,
  * with less precision as the numbers get larger.
- *
+ * <p/>
  * The series of values to which the counts in `decayingBuckets` correspond:
  * 1, 2, 3, 4, 5, 6, 7, 8, 10, 12, 14, 17, 20, 24, 29, 35, 42, 50, 60, 72 etc.
  * Thus, a `decayingBuckets` of [0, 0, 1, 10] would mean we had seen 1 value of 3 and 10 values of 4.
- *
+ * <p/>
  * Each bucket represents values from (previous bucket offset, current offset].
- *
- * [1]: http://dimacs.rutgers.edu/~graham/pubs/papers/fwddecay.pdf
- * [2]: https://en.wikipedia.org/wiki/Half-life
- * [3]: https://github.com/dropwizard/metrics/blob/v3.1.2/metrics-core/src/main/java/com/codahale/metrics/ExponentiallyDecayingReservoir.java
+ * <p/>
+ * To reduce contention each logical bucket is striped accross a configurable number of stripes (default: 2). Threads are
+ * assigned to specific stripes. In addition, logical buckets are distributed across the physical storage to reduce conention
+ * when logically adjacent buckets are updated. See CASSANDRA-15213.
+ * <p/>
+ * <ul>
+ *   <li>[1]: http://dimacs.rutgers.edu/~graham/pubs/papers/fwddecay.pdf</li>
+ *   <li>[2]: https://en.wikipedia.org/wiki/Half-life</li>
+ *   <li>[3]: https://github.com/dropwizard/metrics/blob/v3.1.2/metrics-core/src/main/java/com/codahale/metrics/ExponentiallyDecayingReservoir.java</li>
+ * </ul>
  */
 public class DecayingEstimatedHistogramReservoir implements Reservoir
 {
+
     /**
      * The default number of decayingBuckets. Use this bucket count to reduce memory allocation for bucket offsets.
      */
     public static final int DEFAULT_BUCKET_COUNT = 164;
+    public static final int DEFAULT_STRIPE_COUNT = Integer.parseInt(System.getProperty("cassandra.dehr_stripe_count", "2"));
+    public static final int MAX_BUCKET_COUNT = 237;
     public static final boolean DEFAULT_ZERO_CONSIDERATION = false;
 
+    private static final int[] DISTRIBUTION_PRIMES = new int[] { 17, 19, 23, 29 };
+
     // The offsets used with a default sized bucket array without a separate bucket for zero values.
     public static final long[] DEFAULT_WITHOUT_ZERO_BUCKET_OFFSETS = EstimatedHistogram.newOffsets(DEFAULT_BUCKET_COUNT, false);
 
     // The offsets used with a default sized bucket array with a separate bucket for zero values.
     public static final long[] DEFAULT_WITH_ZERO_BUCKET_OFFSETS = EstimatedHistogram.newOffsets(DEFAULT_BUCKET_COUNT, true);
 
+    private static final int TABLE_BITS = 4;
+    private static final int TABLE_MASK = -1 >>> (32 - TABLE_BITS);
+    private static final float[] LOG2_TABLE = computeTable(TABLE_BITS);
+    private static final float log2_12_recp = (float) (1d / slowLog2(1.2d));
+
+    private static float[] computeTable(int bits)
+    {
+        float[] table = new float[1 << bits];
+        for (int i = 1 ; i < 1<<bits ; ++i)
+            table[i] = (float) slowLog2(ratio(i, bits));
+        return table;
+    }
+
+    public static float fastLog12(long v)
+    {
+        return fastLog2(v) * log2_12_recp;
+    }
+
+    // returns 0 for all inputs <= 1
+    private static float fastLog2(long v)
+    {
+        v = max(v, 1);
+        int highestBitPosition = 63 - Long.numberOfLeadingZeros(v);
+        v = Long.rotateRight(v, highestBitPosition - TABLE_BITS);
+        int index = (int) (v & TABLE_MASK);
+        float result = LOG2_TABLE[index];
+        result += highestBitPosition;
+        return result;
+    }
+
+    private static double slowLog2(double v)
+    {
+        return Math.log(v) / Math.log(2);
+    }
+
+    private static double ratio(int i, int bits)
+    {
+        return Float.intBitsToFloat((127 << 23) | (i << (23 - bits)));
+    }
+
     // Represents the bucket offset as created by {@link EstimatedHistogram#newOffsets()}
+    private final int nStripes;
     private final long[] bucketOffsets;
+    private final int distributionPrime;
 
     // decayingBuckets and buckets are one element longer than bucketOffsets -- the last element is values greater than the last offset
     private final AtomicLongArray decayingBuckets;
@@ -96,8 +151,6 @@ public class DecayingEstimatedHistogramReservoir implements Reservoir
     private final AtomicBoolean rescaling = new AtomicBoolean(false);
     private volatile long decayLandmark;
 
-    private final ReentrantReadWriteLock lock = new ReentrantReadWriteLock();
-
     // Wrapper around System.nanoTime() to simplify unit testing.
     private final Clock clock;
 
@@ -107,7 +160,7 @@ public class DecayingEstimatedHistogramReservoir implements Reservoir
      */
     public DecayingEstimatedHistogramReservoir()
     {
-        this(DEFAULT_ZERO_CONSIDERATION, DEFAULT_BUCKET_COUNT, Clock.defaultClock());
+        this(DEFAULT_ZERO_CONSIDERATION, DEFAULT_BUCKET_COUNT, DEFAULT_STRIPE_COUNT, Clock.defaultClock());
     }
 
     /**
@@ -118,7 +171,7 @@ public class DecayingEstimatedHistogramReservoir implements Reservoir
      */
     public DecayingEstimatedHistogramReservoir(boolean considerZeroes)
     {
-        this(considerZeroes, DEFAULT_BUCKET_COUNT, Clock.defaultClock());
+        this(considerZeroes, DEFAULT_BUCKET_COUNT, DEFAULT_STRIPE_COUNT, Clock.defaultClock());
     }
 
     /**
@@ -128,14 +181,22 @@ public class DecayingEstimatedHistogramReservoir implements Reservoir
      *                       same bucket as 1-value measurements
      * @param bucketCount number of buckets used to collect measured values
      */
-    public DecayingEstimatedHistogramReservoir(boolean considerZeroes, int bucketCount)
+    public DecayingEstimatedHistogramReservoir(boolean considerZeroes, int bucketCount, int stripes)
     {
-        this(considerZeroes, bucketCount, Clock.defaultClock());
+        this(considerZeroes, bucketCount, stripes, Clock.defaultClock());
     }
 
     @VisibleForTesting
-    DecayingEstimatedHistogramReservoir(boolean considerZeroes, int bucketCount, Clock clock)
+    public DecayingEstimatedHistogramReservoir(Clock clock)
     {
+        this(DEFAULT_ZERO_CONSIDERATION, DEFAULT_BUCKET_COUNT, DEFAULT_STRIPE_COUNT, clock);
+    }
+
+    @VisibleForTesting
+    DecayingEstimatedHistogramReservoir(boolean considerZeroes, int bucketCount, int stripes, Clock clock)
+    {
+        assert bucketCount <= MAX_BUCKET_COUNT : "bucket count cannot exceed: " + MAX_BUCKET_COUNT;
+
         if (bucketCount == DEFAULT_BUCKET_COUNT)
         {
             if (considerZeroes == true)
@@ -151,10 +212,22 @@ public class DecayingEstimatedHistogramReservoir implements Reservoir
         {
             bucketOffsets = EstimatedHistogram.newOffsets(bucketCount, considerZeroes);
         }
-        decayingBuckets = new AtomicLongArray(bucketOffsets.length + 1);
-        buckets = new AtomicLongArray(bucketOffsets.length + 1);
+
+        nStripes = stripes;
+        decayingBuckets = new AtomicLongArray((bucketOffsets.length + 1) * nStripes);
+        buckets = new AtomicLongArray((bucketOffsets.length + 1) * nStripes);
         this.clock = clock;
         decayLandmark = clock.getTime();
+        int distributionPrime = 1;
+        for (int prime : DISTRIBUTION_PRIMES)
+        {
+            if (buckets.length() % prime != 0)
+            {
+                distributionPrime = prime;
+                break;
+            }
+        }
+        this.distributionPrime = distributionPrime;
     }
 
     /**
@@ -167,45 +240,70 @@ public class DecayingEstimatedHistogramReservoir implements Reservoir
         long now = clock.getTime();
         rescaleIfNeeded(now);
 
-        int index = Arrays.binarySearch(bucketOffsets, value);
-        if (index < 0)
-        {
-            // inexact match, take the first bucket higher than n
-            index = -index - 1;
-        }
-        // else exact match; we're good
+        int index = findIndex(bucketOffsets, value);
 
-        lockForRegularUsage();
+        updateBucket(decayingBuckets, index, Math.round(forwardDecayWeight(now)));
+        updateBucket(buckets, index, 1);
+    }
 
-        try
-        {
-            decayingBuckets.getAndAdd(index, Math.round(forwardDecayWeight(now)));
-        }
-        finally
-        {
-            unlockForRegularUsage();
-        }
+    public void updateBucket(AtomicLongArray buckets, int index, long value)
+    {
+        int stripe = (int) (Thread.currentThread().getId() & (nStripes - 1));
+        buckets.addAndGet(stripedIndex(index, stripe), value);
+    }
 
-        buckets.getAndIncrement(index);
+    public int stripedIndex(int offsetIndex, int stripe)
+    {
+        return (((offsetIndex * nStripes + stripe) * distributionPrime) % buckets.length());
+    }
+
+    @VisibleForTesting
+    public static int findIndex(long[] bucketOffsets, long value)
+    {
+        // values below zero are nonsense, but we have never failed when presented them
+        value = max(value, 0);
+
+        // The bucket index can be estimated using the equation Math.floor(Math.log(value) / Math.log(1.2))
+
+        // By using an integer domain we effectively squeeze multiple exponents of 1.2 into the same bucket,
+        // so for values > 2, we must "subtract" these exponents from the logarithm to determine which two buckets
+        // to consult (as our approximation otherwise produces a value that is within 1 of the true value)
+        int offset = (value > 2 ? 3 : 1) + (int)bucketOffsets[0];
+
+        // See DecayingEstimatedHistogramResevoirTest#showEstimationWorks and DecayingEstimatedHistogramResevoirTest#testFindIndex()
+        // for a runnable "proof"
+        //
+        // With this assumption, the estimate is calculated and the furthest offset from the estimation is checked
+        // if this bucket does not contain the value then the next one will
+
+        int firstCandidate = max(0, min(bucketOffsets.length - 1, ((int) fastLog12(value)) - offset));
+        return value <= bucketOffsets[firstCandidate] ? firstCandidate : firstCandidate + 1;
     }
 
     private double forwardDecayWeight(long now)
     {
-        return Math.exp(((now - decayLandmark) / 1000L) / MEAN_LIFETIME_IN_S);
+        return Math.exp(((now - decayLandmark) / 1000.0) / MEAN_LIFETIME_IN_S);
     }
 
     /**
-     * Return the number of buckets where recorded values are stored.
+     * Returns the logical number of buckets where recorded values are stored. The actual number of physical buckets
+     * is size() * stripeCount()
      *
      * This method does not return the number of recorded values as suggested by the {@link Reservoir} interface.
      *
      * @return the number of buckets
+     * @see #stripeCount()
      */
     public int size()
     {
-        return decayingBuckets.length();
+        return bucketOffsets.length + 1;
     }
 
+
+    public int stripeCount()
+    {
+        return nStripes;
+    }
     /**
      * Returns a snapshot of the decaying values in this reservoir.
      *
@@ -216,17 +314,7 @@ public class DecayingEstimatedHistogramReservoir implements Reservoir
     public Snapshot getSnapshot()
     {
         rescaleIfNeeded();
-
-        lockForRegularUsage();
-
-        try
-        {
-            return new EstimatedHistogramReservoirSnapshot(this);
-        }
-        finally
-        {
-            unlockForRegularUsage();
-        }
+        return new EstimatedHistogramReservoirSnapshot(this);
     }
 
     /**
@@ -235,7 +323,23 @@ public class DecayingEstimatedHistogramReservoir implements Reservoir
     @VisibleForTesting
     boolean isOverflowed()
     {
-        return decayingBuckets.get(decayingBuckets.length() - 1) > 0;
+        return bucketValue(bucketOffsets.length, true) > 0;
+    }
+
+    private long bucketValue(int index, boolean withDecay)
+    {
+        long val = 0;
+        AtomicLongArray bs = withDecay ? decayingBuckets : buckets;
+        for (int stripe = 0; stripe < nStripes; stripe++)
+            val += bs.get(stripedIndex(index, stripe));
+
+        return val;
+    }
+
+    @VisibleForTesting
+    long stripedBucketValue(int i, boolean withDecay)
+    {
+        return withDecay ? decayingBuckets.get(i) : buckets.get(i);
     }
 
     private void rescaleIfNeeded()
@@ -255,6 +359,7 @@ public class DecayingEstimatedHistogramReservoir implements Reservoir
                 }
                 finally
                 {
+                    decayLandmark = now;
                     rescaling.set(false);
                 }
             }
@@ -263,27 +368,12 @@ public class DecayingEstimatedHistogramReservoir implements Reservoir
 
     private void rescale(long now)
     {
-        // Check again to make sure that another thread didn't complete rescale already
-        if (needRescale(now))
+        // despite striping its safe to rescale each bucket individually
+        final double rescaleFactor = forwardDecayWeight(now);
+        for (int i = 0; i < decayingBuckets.length(); i++)
         {
-            lockForRescale();
-
-            try
-            {
-                final double rescaleFactor = forwardDecayWeight(now);
-                decayLandmark = now;
-
-                final int bucketCount = decayingBuckets.length();
-                for (int i = 0; i < bucketCount; i++)
-                {
-                    long newValue = Math.round((decayingBuckets.get(i) / rescaleFactor));
-                    decayingBuckets.set(i, newValue);
-                }
-            }
-            finally
-            {
-                unlockForRescale();
-            }
+            long newValue = Math.round(decayingBuckets.get(i) / rescaleFactor);
+            decayingBuckets.set(i, newValue);
         }
     }
 
@@ -295,46 +385,50 @@ public class DecayingEstimatedHistogramReservoir implements Reservoir
     @VisibleForTesting
     public void clear()
     {
-        lockForRescale();
-
-        try
+        final int bucketCount = decayingBuckets.length();
+        for (int i = 0; i < bucketCount; i++)
         {
-            final int bucketCount = decayingBuckets.length();
-            for (int i = 0; i < bucketCount; i++)
+            decayingBuckets.set(i, 0L);
+            buckets.set(i, 0L);
+        }
+    }
+
+    /**
+     * Replaces current internal values with the given one from a Snapshot. This method is NOT thread safe, values
+     * added at the same time to this reservoir using methods such as update may lose their data
+     */
+    public void rebase(EstimatedHistogramReservoirSnapshot snapshot)
+    {
+        // Check bucket count (a snapshot always has one stripe so the logical bucket count is used
+        if (size() != snapshot.decayingBuckets.length)
+        {
+            throw new IllegalStateException("Unable to merge two DecayingEstimatedHistogramReservoirs with different bucket sizes");
+        }
+
+        // Check bucketOffsets
+        for (int i = 0; i < bucketOffsets.length; i++)
+        {
+            if (bucketOffsets[i] != snapshot.bucketOffsets[i])
             {
-                decayingBuckets.set(i, 0L);
-                buckets.set(i, 0L);
+                throw new IllegalStateException("Merge is only supported with equal bucketOffsets");
             }
         }
-        finally
+
+        this.decayLandmark = snapshot.snapshotLandmark;
+        for (int i = 0; i < size(); i++)
         {
-            unlockForRescale();
+            // set rebased values in the first stripe and clear out all other data
+            decayingBuckets.set(stripedIndex(i, 0), snapshot.decayingBuckets[i]);
+            buckets.set(stripedIndex(i, 0), snapshot.values[i]);
+            for (int stripe = 1; stripe < nStripes; stripe++)
+            {
+                decayingBuckets.set(stripedIndex(i, stripe), 0);
+                buckets.set(stripedIndex(i, stripe), 0);
+            }
         }
+
     }
 
-    private void lockForRegularUsage()
-    {
-        this.lock.readLock().lock();
-    }
-
-    private void unlockForRegularUsage()
-    {
-        this.lock.readLock().unlock();
-    }
-
-    private void lockForRescale()
-    {
-        this.lock.writeLock().lock();
-    }
-
-    private void unlockForRescale()
-    {
-        this.lock.writeLock().unlock();
-    }
-
-
-    private static final Charset UTF_8 = Charset.forName("UTF-8");
-
     /**
      * Represents a snapshot of the decaying histogram.
      *
@@ -345,19 +439,32 @@ public class DecayingEstimatedHistogramReservoir implements Reservoir
      * The decaying buckets will be used for quantile calculations and mean values, but the non decaying buckets will be
      * exposed for calls to {@link Snapshot#getValues()}.
      */
-    private class EstimatedHistogramReservoirSnapshot extends Snapshot
+    static class EstimatedHistogramReservoirSnapshot extends Snapshot
     {
         private final long[] decayingBuckets;
+        private final long[] values;
+        private long count;
+        private long snapshotLandmark;
+        private long[] bucketOffsets;
+        private DecayingEstimatedHistogramReservoir reservoir;
 
         public EstimatedHistogramReservoirSnapshot(DecayingEstimatedHistogramReservoir reservoir)
         {
-            final int length = reservoir.decayingBuckets.length();
-            final double rescaleFactor = forwardDecayWeight(clock.getTime());
+            final int length = reservoir.size();
+            final double rescaleFactor = reservoir.forwardDecayWeight(reservoir.clock.getTime());
 
             this.decayingBuckets = new long[length];
+            this.values = new long[length];
+            this.snapshotLandmark = reservoir.decayLandmark;
+            this.bucketOffsets = reservoir.bucketOffsets; // No need to copy, these are immutable
 
             for (int i = 0; i < length; i++)
-                this.decayingBuckets[i] = Math.round(reservoir.decayingBuckets.get(i) / rescaleFactor);
+            {
+                this.decayingBuckets[i] = Math.round(reservoir.bucketValue(i, true) / rescaleFactor);
+                this.values[i] = reservoir.bucketValue(i, false);
+            }
+            this.count = count();
+            this.reservoir = reservoir;
         }
 
         /**
@@ -400,26 +507,31 @@ public class DecayingEstimatedHistogramReservoir implements Reservoir
          */
         public long[] getValues()
         {
-            final int length = buckets.length();
-
-            long[] values = new long[length];
-
-            for (int i = 0; i < length; i++)
-                values[i] = buckets.get(i);
-
             return values;
         }
 
         /**
-         * Return the number of buckets where recorded values are stored.
-         *
-         * This method does not return the number of recorded values as suggested by the {@link Snapshot} interface.
-         *
-         * @return the number of buckets
+         * @see {@link Snapshot#size()}
+         * @return
          */
         public int size()
         {
-            return decayingBuckets.length;
+            return Ints.saturatedCast(count);
+        }
+
+        @VisibleForTesting
+        public long getSnapshotLandmark()
+        {
+            return snapshotLandmark;
+        }
+
+        @VisibleForTesting
+        public Range getBucketingRangeForValue(long value)
+        {
+            int index = findIndex(bucketOffsets, value);
+            long max = bucketOffsets[index];
+            long min = index == 0 ? 0 : 1 + bucketOffsets[index - 1];
+            return new Range(min, max);
         }
 
         /**
@@ -438,7 +550,7 @@ public class DecayingEstimatedHistogramReservoir implements Reservoir
         /**
          * Get the estimated max-value that could have been added to this reservoir.
          *
-         * As values are collected in variable sized buckets, the actual max value recored in the reservoir may be less
+         * As values are collected in variable sized buckets, the actual max value recorded in the reservoir may be less
          * than the value returned.
          *
          * @return the largest value that could have been added to this reservoir, or Long.MAX_VALUE if the reservoir
@@ -487,7 +599,7 @@ public class DecayingEstimatedHistogramReservoir implements Reservoir
         /**
          * Get the estimated min-value that could have been added to this reservoir.
          *
-         * As values are collected in variable sized buckets, the actual min value recored in the reservoir may be
+         * As values are collected in variable sized buckets, the actual min value recorded in the reservoir may be
          * higher than the value returned.
          *
          * @return the smallest value that could have been added to this reservoir
@@ -519,32 +631,126 @@ public class DecayingEstimatedHistogramReservoir implements Reservoir
 
             final long count = count();
 
-            if(count <= 1) {
+            if(count <= 1)
+            {
                 return 0.0D;
-            } else {
+            }
+            else
+            {
                 double mean = this.getMean();
                 double sum = 0.0D;
 
-                for(int i = 0; i < lastBucket; ++i) {
+                for(int i = 0; i < lastBucket; ++i)
+                {
                     long value = bucketOffsets[i];
-                    double diff = (double)value - mean;
+                    double diff = value - mean;
                     sum += diff * diff * decayingBuckets[i];
                 }
 
-                return Math.sqrt(sum / (double)(count - 1));
+                return Math.sqrt(sum / (count - 1));
             }
         }
 
         public void dump(OutputStream output)
         {
-            try (PrintWriter out = new PrintWriter(new OutputStreamWriter(output, UTF_8)))
+            try (PrintWriter out = new PrintWriter(new OutputStreamWriter(output, StandardCharsets.UTF_8)))
             {
                 int length = decayingBuckets.length;
 
-                for(int i = 0; i < length; ++i) {
+                for(int i = 0; i < length; ++i)
+                {
                     out.printf("%d%n", decayingBuckets[i]);
                 }
             }
         }
+
+        /**
+         * Adds another DecayingEstimatedHistogramReservoir's Snapshot to this one. Both reservoirs must have same bucket definitions. This will rescale both snapshots if needed.
+         *
+         * @param other EstimatedHistogramReservoirSnapshot with identical bucket definition (offsets and length)
+         */
+        public void add(Snapshot other)
+        {
+            if (!(other instanceof EstimatedHistogramReservoirSnapshot))
+            {
+                throw new IllegalStateException("Unable to add other types of Snapshot than another DecayingEstimatedHistogramReservoir");
+            }
+
+            EstimatedHistogramReservoirSnapshot snapshot = (EstimatedHistogramReservoirSnapshot) other;
+
+            if (decayingBuckets.length != snapshot.decayingBuckets.length)
+            {
+                throw new IllegalStateException("Unable to merge two DecayingEstimatedHistogramReservoirs with different bucket sizes");
+            }
+
+            // Check bucketOffsets
+            for (int i = 0; i < bucketOffsets.length; i++)
+            {
+                if (bucketOffsets[i] != snapshot.bucketOffsets[i])
+                {
+                    throw new IllegalStateException("Merge is only supported with equal bucketOffsets");
+                }
+            }
+
+            // We need to rescale the reservoirs to the same landmark
+            if (snapshot.snapshotLandmark < snapshotLandmark)
+            {
+                rescaleArray(snapshot.decayingBuckets, (snapshotLandmark - snapshot.snapshotLandmark));
+            }
+            else if (snapshot.snapshotLandmark > snapshotLandmark)
+            {
+                rescaleArray(decayingBuckets, (snapshot.snapshotLandmark - snapshotLandmark));
+                this.snapshotLandmark = snapshot.snapshotLandmark;
+            }
+
+            // Now merge the buckets
+            for (int i = 0; i < snapshot.decayingBuckets.length; i++)
+            {
+                decayingBuckets[i] += snapshot.decayingBuckets[i];
+                values[i] += snapshot.values[i];
+            }
+
+            this.count += snapshot.count;
+        }
+
+        private void rescaleArray(long[] decayingBuckets, long landMarkDifference)
+        {
+            final double rescaleFactor = Math.exp((landMarkDifference / 1000.0) / MEAN_LIFETIME_IN_S);
+            for (int i = 0; i < decayingBuckets.length; i++)
+            {
+                decayingBuckets[i] = Math.round(decayingBuckets[i] / rescaleFactor);
+            }
+        }
+
+        public void rebaseReservoir() 
+        {
+            this.reservoir.rebase(this);
+        }
+    }
+
+    static class Range
+    {
+        public final long min;
+        public final long max;
+
+        public Range(long min, long max)
+        {
+            this.min = min;
+            this.max = max;
+        }
+
+        public boolean equals(Object o)
+        {
+            if (this == o) return true;
+            if (o == null || getClass() != o.getClass()) return false;
+            Range that = (Range) o;
+            return min == that.min &&
+                   max == that.max;
+        }
+
+        public int hashCode()
+        {
+            return Objects.hash(min, max);
+        }
     }
 }
