@@ -17,19 +17,20 @@
  */
 package org.apache.cassandra.metrics;
 
-import java.net.InetAddress;
 import java.util.Map.Entry;
 
+import com.google.common.util.concurrent.MoreExecutors;
+
 import com.codahale.metrics.Counter;
+import com.github.benmanes.caffeine.cache.Caffeine;
+import com.github.benmanes.caffeine.cache.LoadingCache;
+
 import org.apache.cassandra.db.SystemKeyspace;
+import org.apache.cassandra.locator.InetAddressAndPort;
 import org.apache.cassandra.utils.UUIDGen;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
-import com.google.common.cache.CacheBuilder;
-import com.google.common.cache.CacheLoader;
-import com.google.common.cache.LoadingCache;
-
 import static org.apache.cassandra.metrics.CassandraMetricsRegistry.Metrics;
 
 /**
@@ -39,39 +40,39 @@ public class HintedHandoffMetrics
 {
     private static final Logger logger = LoggerFactory.getLogger(HintedHandoffMetrics.class);
 
-    private static final MetricNameFactory factory = new DefaultNameFactory("HintedHandOffManager");
+    private static final MetricNameFactory factory = new DefaultNameFactory("HintsService");
 
     /** Total number of hints which are not stored, This is not a cache. */
-    private final LoadingCache<InetAddress, DifferencingCounter> notStored = CacheBuilder.newBuilder().build(new CacheLoader<InetAddress, DifferencingCounter>()
-    {
-        public DifferencingCounter load(InetAddress address)
-        {
-            return new DifferencingCounter(address);
-        }
-    });
+    private final LoadingCache<InetAddressAndPort, DifferencingCounter> notStored = Caffeine.newBuilder()
+                                                                                            .executor(MoreExecutors.directExecutor())
+                                                                                            .build(DifferencingCounter::new);
 
     /** Total number of hints that have been created, This is not a cache. */
-    private final LoadingCache<InetAddress, Counter> createdHintCounts = CacheBuilder.newBuilder().build(new CacheLoader<InetAddress, Counter>()
-    {
-        public Counter load(InetAddress address)
-        {
-            return Metrics.counter(factory.createMetricName("Hints_created-" + address.getHostAddress().replace(':', '.')));
-        }
-    });
+    private final LoadingCache<InetAddressAndPort, Counter> createdHintCounts = Caffeine.newBuilder()
+                                                                                        .executor(MoreExecutors.directExecutor())
+                                                                                        .build(address -> Metrics.counter(factory.createMetricName("Hints_created-" + address.toString().replace(':', '.'))));
 
-    public void incrCreatedHints(InetAddress address)
+    /** Counter of hints received whose mutations are for keys outside the owned and pending ranges for this node */
+    private final Counter hintsReceivedForUnownedRanges = Metrics.counter(factory.createMetricName("Hints_for_unowned_ranges"));
+
+    public void incrCreatedHints(InetAddressAndPort address)
     {
-        createdHintCounts.getUnchecked(address).inc();
+        createdHintCounts.get(address).inc();
     }
 
-    public void incrPastWindow(InetAddress address)
+    public void incrPastWindow(InetAddressAndPort address)
     {
-        notStored.getUnchecked(address).mark();
+        notStored.get(address).mark();
+    }
+
+    public void incrHintsReceivedForUnownedRanges()
+    {
+        hintsReceivedForUnownedRanges.inc();
     }
 
     public void log()
     {
-        for (Entry<InetAddress, DifferencingCounter> entry : notStored.asMap().entrySet())
+        for (Entry<InetAddressAndPort, DifferencingCounter> entry : notStored.asMap().entrySet())
         {
             long difference = entry.getValue().difference();
             if (difference == 0)
@@ -86,9 +87,10 @@ public class HintedHandoffMetrics
         private final Counter meter;
         private long reported = 0;
 
-        public DifferencingCounter(InetAddress address)
+        public DifferencingCounter(InetAddressAndPort address)
         {
-            this.meter = Metrics.counter(factory.createMetricName("Hints_not_stored-" + address.getHostAddress().replace(':', '.')));
+            //This changes the name of the metric, people can update their monitoring when upgrading?
+            this.meter = Metrics.counter(factory.createMetricName("Hints_not_stored-" + address.toString().replace(':', '.')));
         }
 
         public long difference()
