@@ -17,11 +17,13 @@
  */
 package org.apache.cassandra.metrics;
 
-import java.net.InetAddress;
 import java.util.concurrent.ConcurrentMap;
 
 
 import com.codahale.metrics.Counter;
+import com.codahale.metrics.Timer;
+import org.apache.cassandra.locator.InetAddressAndPort;
+
 import org.cliffc.high_scale_lib.NonBlockingHashMap;
 
 import static org.apache.cassandra.metrics.CassandraMetricsRegistry.Metrics;
@@ -33,29 +35,49 @@ public class StreamingMetrics
 {
     public static final String TYPE_NAME = "Streaming";
 
-    private static final ConcurrentMap<InetAddress, StreamingMetrics> instances = new NonBlockingHashMap<InetAddress, StreamingMetrics>();
+    private static final ConcurrentMap<InetAddressAndPort, StreamingMetrics> instances = new NonBlockingHashMap<>();
 
+    @Deprecated
     public static final Counter activeStreamsOutbound = Metrics.counter(DefaultNameFactory.createMetricName(TYPE_NAME, "ActiveOutboundStreams", null));
     public static final Counter totalIncomingBytes = Metrics.counter(DefaultNameFactory.createMetricName(TYPE_NAME, "TotalIncomingBytes", null));
     public static final Counter totalOutgoingBytes = Metrics.counter(DefaultNameFactory.createMetricName(TYPE_NAME, "TotalOutgoingBytes", null));
+    public static final Counter totalOutgoingRepairBytes = Metrics.counter(DefaultNameFactory.createMetricName(TYPE_NAME, "TotalOutgoingRepairBytes", null));
+    public static final Counter totalOutgoingRepairSSTables = Metrics.counter(DefaultNameFactory.createMetricName(TYPE_NAME, "TotalOutgoingRepairSSTables", null));
     public final Counter incomingBytes;
     public final Counter outgoingBytes;
+    /* Measures the time taken for processing the incoming stream message after being deserialized, including the time to flush to disk. */
+    public final Timer incomingProcessTime;
 
-    public static StreamingMetrics get(InetAddress ip)
+    public static StreamingMetrics get(InetAddressAndPort ip)
     {
+       /*
+         computeIfAbsent doesn't work for this situation. Since JMX metrics register themselves in their ctor, we need
+         to create the metric exactly once, otherwise we'll get duplicate name exceptions. Although computeIfAbsent is
+         thread safe in the context of the map, it uses compare and swap to add the computed value to the map. This
+         means it eagerly allocates new metric instances, which can cause the jmx name collision we're trying to avoid
+         if multiple calls interleave. So here we use synchronized to ensure we only instantiate metrics exactly once.
+        */
        StreamingMetrics metrics = instances.get(ip);
        if (metrics == null)
        {
-           metrics = new StreamingMetrics(ip);
-           instances.put(ip, metrics);
+           synchronized (instances)
+           {
+               metrics = instances.get(ip);
+               if (metrics == null)
+               {
+                   metrics = new StreamingMetrics(ip);
+                   instances.put(ip, metrics);
+               }
+           }
        }
        return metrics;
     }
 
-    public StreamingMetrics(final InetAddress peer)
+    public StreamingMetrics(final InetAddressAndPort peer)
     {
-        MetricNameFactory factory = new DefaultNameFactory("Streaming", peer.getHostAddress().replace(':', '.'));
+        MetricNameFactory factory = new DefaultNameFactory("Streaming", peer.toString().replace(':', '.'));
         incomingBytes = Metrics.counter(factory.createMetricName("IncomingBytes"));
         outgoingBytes= Metrics.counter(factory.createMetricName("OutgoingBytes"));
+        incomingProcessTime = Metrics.timer(factory.createMetricName("IncomingProcessTime"));
     }
 }
