@@ -17,15 +17,17 @@
  */
 package org.apache.cassandra.repair;
 
-import java.net.InetAddress;
-
 import com.google.common.util.concurrent.AbstractFuture;
 
 import org.apache.cassandra.exceptions.RepairException;
-import org.apache.cassandra.net.MessagingService;
+import org.apache.cassandra.locator.InetAddressAndPort;
+import org.apache.cassandra.repair.messages.RepairMessage;
 import org.apache.cassandra.repair.messages.ValidationRequest;
+import org.apache.cassandra.streaming.PreviewKind;
 import org.apache.cassandra.utils.MerkleTrees;
 
+import static org.apache.cassandra.net.Verb.VALIDATION_REQ;
+
 /**
  * ValidationTask sends {@link ValidationRequest} to a replica.
  * When a replica sends back message, task completes.
@@ -33,14 +35,16 @@ import org.apache.cassandra.utils.MerkleTrees;
 public class ValidationTask extends AbstractFuture<TreeResponse> implements Runnable
 {
     private final RepairJobDesc desc;
-    private final InetAddress endpoint;
-    private final int gcBefore;
+    private final InetAddressAndPort endpoint;
+    private final int nowInSec;
+    private final PreviewKind previewKind;
 
-    public ValidationTask(RepairJobDesc desc, InetAddress endpoint, int gcBefore)
+    public ValidationTask(RepairJobDesc desc, InetAddressAndPort endpoint, int nowInSec, PreviewKind previewKind)
     {
         this.desc = desc;
         this.endpoint = endpoint;
-        this.gcBefore = gcBefore;
+        this.nowInSec = nowInSec;
+        this.previewKind = previewKind;
     }
 
     /**
@@ -48,8 +52,10 @@ public class ValidationTask extends AbstractFuture<TreeResponse> implements Runn
      */
     public void run()
     {
-        ValidationRequest request = new ValidationRequest(desc, gcBefore);
-        MessagingService.instance().sendOneWay(request.createMessage(), endpoint);
+        RepairMessage.sendMessageWithFailureCB(new ValidationRequest(desc, nowInSec),
+                                               VALIDATION_REQ,
+                                               endpoint,
+                                               this::setException);
     }
 
     /**
@@ -61,7 +67,7 @@ public class ValidationTask extends AbstractFuture<TreeResponse> implements Runn
     {
         if (trees == null)
         {
-            setException(new RepairException(desc, "Validation failed in " + endpoint));
+            setException(new RepairException(desc, previewKind, "Validation failed in " + endpoint));
         }
         else
         {
