@@ -20,13 +20,18 @@ package org.apache.cassandra.schema;
 import java.io.IOException;
 import java.lang.reflect.InvocationTargetException;
 import java.lang.reflect.Method;
+import java.util.Collection;
 import java.util.Collections;
 import java.util.HashMap;
 import java.util.Map;
+import java.util.concurrent.ThreadLocalRandom;
 
+import com.google.common.annotations.VisibleForTesting;
+import com.google.common.base.Objects;
 import com.google.common.collect.ImmutableMap;
-import org.apache.commons.lang3.builder.EqualsBuilder;
+
 import org.apache.commons.lang3.builder.HashCodeBuilder;
+
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
@@ -37,6 +42,7 @@ import org.apache.cassandra.io.IVersionedSerializer;
 import org.apache.cassandra.io.compress.*;
 import org.apache.cassandra.io.util.DataInputPlus;
 import org.apache.cassandra.io.util.DataOutputPlus;
+import org.apache.cassandra.net.MessagingService;
 
 import static java.lang.String.format;
 
@@ -49,17 +55,30 @@ public final class CompressionParams
     private static volatile boolean hasLoggedChunkLengthWarning;
     private static volatile boolean hasLoggedCrcCheckChanceWarning;
 
-    public static final int DEFAULT_CHUNK_LENGTH = 65536;
+    public static final int DEFAULT_CHUNK_LENGTH = 1024 * 16;
+    public static final double DEFAULT_MIN_COMPRESS_RATIO = 0.0;        // Since pre-4.0 versions do not understand the
+                                                                        // new compression parameter we can't use a
+                                                                        // different default value.
     public static final IVersionedSerializer<CompressionParams> serializer = new Serializer();
 
     public static final String CLASS = "class";
     public static final String CHUNK_LENGTH_IN_KB = "chunk_length_in_kb";
     public static final String ENABLED = "enabled";
+    public static final String MIN_COMPRESS_RATIO = "min_compress_ratio";
 
     public static final CompressionParams DEFAULT = new CompressionParams(LZ4Compressor.create(Collections.<String, String>emptyMap()),
                                                                           DEFAULT_CHUNK_LENGTH,
+                                                                          calcMaxCompressedLength(DEFAULT_CHUNK_LENGTH, DEFAULT_MIN_COMPRESS_RATIO),
+                                                                          DEFAULT_MIN_COMPRESS_RATIO,
                                                                           Collections.emptyMap());
 
+    public static final CompressionParams NOOP = new CompressionParams(NoopCompressor.create(Collections.emptyMap()),
+                                                                       // 4 KiB is often the underlying disk block size
+                                                                       1024 * 4,
+                                                                       Integer.MAX_VALUE,
+                                                                       DEFAULT_MIN_COMPRESS_RATIO,
+                                                                       Collections.emptyMap());
+
     private static final String CRC_CHECK_CHANCE_WARNING = "The option crc_check_chance was deprecated as a compression option. " +
                                                            "You should specify it as a top-level table option instead";
 
@@ -68,9 +87,12 @@ public final class CompressionParams
     @Deprecated public static final String CRC_CHECK_CHANCE = "crc_check_chance";
 
     private final ICompressor sstableCompressor;
-    private final Integer chunkLength;
+    private final int chunkLength;
+    private final int maxCompressedLength;  // In content we store max length to avoid rounding errors causing compress/decompress mismatch.
+    private final double minCompressRatio;  // In configuration we store min ratio, the input parameter.
     private final ImmutableMap<String, String> otherOptions; // Unrecognized options, can be used by the compressor
 
+    // TODO: deprecated, should now be carefully removed. Doesn't affect schema code as it isn't included in equals() and hashCode()
     private volatile double crcCheckChance = 1.0;
 
     public static CompressionParams fromMap(Map<String, String> opts)
@@ -94,9 +116,10 @@ public final class CompressionParams
             sstableCompressionClass = removeSstableCompressionClass(options);
         }
 
-        Integer chunkLength = removeChunkLength(options);
+        int chunkLength = removeChunkLength(options);
+        double minCompressRatio = removeMinCompressRatio(options);
 
-        CompressionParams cp = new CompressionParams(sstableCompressionClass, chunkLength, options);
+        CompressionParams cp = new CompressionParams(sstableCompressionClass, options, chunkLength, minCompressRatio);
         cp.validate();
 
         return cp;
@@ -109,54 +132,112 @@ public final class CompressionParams
 
     public static CompressionParams noCompression()
     {
-        return new CompressionParams((ICompressor) null, DEFAULT_CHUNK_LENGTH, Collections.emptyMap());
+        return new CompressionParams((ICompressor) null, DEFAULT_CHUNK_LENGTH, Integer.MAX_VALUE, 0.0, Collections.emptyMap());
     }
 
+    // The shorthand methods below are only used for tests. They are a little inconsistent in their choice of
+    // parameters -- this is done on purpose to test out various compression parameter combinations.
+
+    @VisibleForTesting
     public static CompressionParams snappy()
     {
-        return snappy(null);
+        return snappy(DEFAULT_CHUNK_LENGTH);
     }
 
-    public static CompressionParams snappy(Integer chunkLength)
+    @VisibleForTesting
+    public static CompressionParams snappy(int chunkLength)
     {
-        return new CompressionParams(SnappyCompressor.instance, chunkLength, Collections.emptyMap());
+        return snappy(chunkLength, 1.1);
     }
 
+    @VisibleForTesting
+    public static CompressionParams snappy(int chunkLength, double minCompressRatio)
+    {
+        return new CompressionParams(SnappyCompressor.instance, chunkLength, calcMaxCompressedLength(chunkLength, minCompressRatio), minCompressRatio, Collections.emptyMap());
+    }
+
+    @VisibleForTesting
     public static CompressionParams deflate()
     {
-        return deflate(null);
+        return deflate(DEFAULT_CHUNK_LENGTH);
     }
 
-    public static CompressionParams deflate(Integer chunkLength)
+    @VisibleForTesting
+    public static CompressionParams deflate(int chunkLength)
     {
-        return new CompressionParams(DeflateCompressor.instance, chunkLength, Collections.emptyMap());
+        return new CompressionParams(DeflateCompressor.instance, chunkLength, Integer.MAX_VALUE, 0.0, Collections.emptyMap());
     }
 
+    @VisibleForTesting
     public static CompressionParams lz4()
     {
-        return lz4(null);
+        return lz4(DEFAULT_CHUNK_LENGTH);
     }
 
-    public static CompressionParams lz4(Integer chunkLength)
+    @VisibleForTesting
+    public static CompressionParams lz4(int chunkLength)
     {
-        return new CompressionParams(LZ4Compressor.create(Collections.emptyMap()), chunkLength, Collections.emptyMap());
+        return lz4(chunkLength, chunkLength);
     }
 
-    public CompressionParams(String sstableCompressorClass, Integer chunkLength, Map<String, String> otherOptions) throws ConfigurationException
+    @VisibleForTesting
+    public static CompressionParams lz4(int chunkLength, int maxCompressedLength)
     {
-        this(createCompressor(parseCompressorClass(sstableCompressorClass), otherOptions), chunkLength, otherOptions);
+        return new CompressionParams(LZ4Compressor.create(Collections.emptyMap()), chunkLength, maxCompressedLength, calcMinCompressRatio(chunkLength, maxCompressedLength), Collections.emptyMap());
     }
 
-    private CompressionParams(ICompressor sstableCompressor, Integer chunkLength, Map<String, String> otherOptions) throws ConfigurationException
+    public static CompressionParams zstd()
+    {
+        return zstd(DEFAULT_CHUNK_LENGTH);
+    }
+
+    public static CompressionParams zstd(Integer chunkLength)
+    {
+        ZstdCompressor compressor = ZstdCompressor.create(Collections.emptyMap());
+        return new CompressionParams(compressor, chunkLength, Integer.MAX_VALUE, DEFAULT_MIN_COMPRESS_RATIO, Collections.emptyMap());
+    }
+
+    @VisibleForTesting
+    public static CompressionParams noop()
+    {
+        NoopCompressor compressor = NoopCompressor.create(Collections.emptyMap());
+        return new CompressionParams(compressor, DEFAULT_CHUNK_LENGTH, Integer.MAX_VALUE, DEFAULT_MIN_COMPRESS_RATIO, Collections.emptyMap());
+    }
+
+    public CompressionParams(String sstableCompressorClass, Map<String, String> otherOptions, int chunkLength, double minCompressRatio) throws ConfigurationException
+    {
+        this(createCompressor(parseCompressorClass(sstableCompressorClass), otherOptions), chunkLength, calcMaxCompressedLength(chunkLength, minCompressRatio), minCompressRatio, otherOptions);
+    }
+
+    static int calcMaxCompressedLength(int chunkLength, double minCompressRatio)
+    {
+        return (int) Math.ceil(Math.min(chunkLength / minCompressRatio, Integer.MAX_VALUE));
+    }
+
+    public CompressionParams(String sstableCompressorClass, int chunkLength, int maxCompressedLength, Map<String, String> otherOptions) throws ConfigurationException
+    {
+        this(createCompressor(parseCompressorClass(sstableCompressorClass), otherOptions), chunkLength, maxCompressedLength, calcMinCompressRatio(chunkLength, maxCompressedLength), otherOptions);
+    }
+
+    static double calcMinCompressRatio(int chunkLength, int maxCompressedLength)
+    {
+        if (maxCompressedLength == Integer.MAX_VALUE)
+            return 0;
+        return chunkLength * 1.0 / maxCompressedLength;
+    }
+
+    private CompressionParams(ICompressor sstableCompressor, int chunkLength, int maxCompressedLength, double minCompressRatio, Map<String, String> otherOptions) throws ConfigurationException
     {
         this.sstableCompressor = sstableCompressor;
         this.chunkLength = chunkLength;
         this.otherOptions = ImmutableMap.copyOf(otherOptions);
+        this.minCompressRatio = minCompressRatio;
+        this.maxCompressedLength = maxCompressedLength;
     }
 
     public CompressionParams copy()
     {
-        return new CompressionParams(sstableCompressor, chunkLength, otherOptions);
+        return new CompressionParams(sstableCompressor, chunkLength, maxCompressedLength, minCompressRatio, otherOptions);
     }
 
     /**
@@ -184,7 +265,12 @@ public final class CompressionParams
 
     public int chunkLength()
     {
-        return chunkLength == null ? DEFAULT_CHUNK_LENGTH : chunkLength;
+        return chunkLength;
+    }
+
+    public int maxCompressedLength()
+    {
+        return maxCompressedLength;
     }
 
     private static Class<?> parseCompressorClass(String className) throws ConfigurationException
@@ -312,7 +398,7 @@ public final class CompressionParams
      * @param options the options
      * @return the chunk length value
      */
-    private static Integer removeChunkLength(Map<String, String> options)
+    private static int removeChunkLength(Map<String, String> options)
     {
         if (options.containsKey(CHUNK_LENGTH_IN_KB))
         {
@@ -339,7 +425,23 @@ public final class CompressionParams
             return parseChunkLength(options.remove(CHUNK_LENGTH_KB));
         }
 
-        return null;
+        return DEFAULT_CHUNK_LENGTH;
+    }
+
+    /**
+     * Removes the min compress ratio option from the specified set of option.
+     *
+     * @param options the options
+     * @return the min compress ratio, used to calculate max chunk size to write compressed
+     */
+    private static double removeMinCompressRatio(Map<String, String> options)
+    {
+        String ratio = options.remove(MIN_COMPRESS_RATIO);
+        if (ratio != null)
+        {
+            return Double.parseDouble(ratio);
+        }
+        return DEFAULT_MIN_COMPRESS_RATIO;
     }
 
     /**
@@ -420,25 +522,17 @@ public final class CompressionParams
     public void validate() throws ConfigurationException
     {
         // if chunk length was not set (chunkLength == null), this is fine, default will be used
-        if (chunkLength != null)
-        {
-            if (chunkLength <= 0)
-                throw new ConfigurationException("Invalid negative or null " + CHUNK_LENGTH_IN_KB);
+        if (chunkLength <= 0)
+            throw new ConfigurationException("Invalid negative or null " + CHUNK_LENGTH_IN_KB);
 
-            int c = chunkLength;
-            boolean found = false;
-            while (c != 0)
-            {
-                if ((c & 0x01) != 0)
-                {
-                    if (found)
-                        throw new ConfigurationException(CHUNK_LENGTH_IN_KB + " must be a power of 2");
-                    else
-                        found = true;
-                }
-                c >>= 1;
-            }
-        }
+        if ((chunkLength & (chunkLength - 1)) != 0)
+            throw new ConfigurationException(CHUNK_LENGTH_IN_KB + " must be a power of 2");
+
+        if (maxCompressedLength < 0)
+            throw new ConfigurationException("Invalid negative " + MIN_COMPRESS_RATIO);
+
+        if (maxCompressedLength > chunkLength && maxCompressedLength < Integer.MAX_VALUE)
+            throw new ConfigurationException(MIN_COMPRESS_RATIO + " can either be 0 or greater than or equal to 1");
     }
 
     public Map<String, String> asMap()
@@ -449,6 +543,8 @@ public final class CompressionParams
         Map<String, String> options = new HashMap<>(otherOptions);
         options.put(CLASS, sstableCompressor.getClass().getName());
         options.put(CHUNK_LENGTH_IN_KB, chunkLengthInKB());
+        if (minCompressRatio != DEFAULT_MIN_COMPRESS_RATIO)
+            options.put(MIN_COMPRESS_RATIO, String.valueOf(minCompressRatio));
 
         return options;
     }
@@ -468,24 +564,28 @@ public final class CompressionParams
         return crcCheckChance;
     }
 
+    public boolean shouldCheckCrc()
+    {
+        double checkChance = getCrcCheckChance();
+        return checkChance >= 1d ||
+               (checkChance > 0d && checkChance > ThreadLocalRandom.current().nextDouble());
+    }
+
     @Override
     public boolean equals(Object obj)
     {
         if (obj == this)
-        {
             return true;
-        }
-        else if (obj == null || obj.getClass() != getClass())
-        {
+
+        if (!(obj instanceof CompressionParams))
             return false;
-        }
 
         CompressionParams cp = (CompressionParams) obj;
-        return new EqualsBuilder()
-            .append(sstableCompressor, cp.sstableCompressor)
-            .append(chunkLength(), cp.chunkLength())
-            .append(otherOptions, cp.otherOptions)
-            .isEquals();
+
+        return Objects.equal(sstableCompressor, cp.sstableCompressor)
+            && chunkLength == cp.chunkLength
+            && otherOptions.equals(cp.otherOptions)
+            && minCompressRatio == cp.minCompressRatio;
     }
 
     @Override
@@ -493,8 +593,9 @@ public final class CompressionParams
     {
         return new HashCodeBuilder(29, 1597)
             .append(sstableCompressor)
-            .append(chunkLength())
+            .append(chunkLength)
             .append(otherOptions)
+            .append(minCompressRatio)
             .toHashCode();
     }
 
@@ -510,6 +611,11 @@ public final class CompressionParams
                 out.writeUTF(entry.getValue());
             }
             out.writeInt(parameters.chunkLength());
+            if (version >= MessagingService.VERSION_40)
+                out.writeInt(parameters.maxCompressedLength);
+            else
+                if (parameters.maxCompressedLength != Integer.MAX_VALUE)
+                    throw new UnsupportedOperationException("Cannot stream SSTables with uncompressed chunks to pre-4.0 nodes.");
         }
 
         public CompressionParams deserialize(DataInputPlus in, int version) throws IOException
@@ -524,10 +630,14 @@ public final class CompressionParams
                 options.put(key, value);
             }
             int chunkLength = in.readInt();
+            int minCompressRatio = Integer.MAX_VALUE;   // Earlier Cassandra cannot use uncompressed chunks.
+            if (version >= MessagingService.VERSION_40)
+                minCompressRatio = in.readInt();
+
             CompressionParams parameters;
             try
             {
-                parameters = new CompressionParams(compressorName, chunkLength, options);
+                parameters = new CompressionParams(compressorName, chunkLength, minCompressRatio, options);
             }
             catch (ConfigurationException e)
             {
@@ -546,6 +656,8 @@ public final class CompressionParams
                 size += TypeSizes.sizeof(entry.getValue());
             }
             size += TypeSizes.sizeof(parameters.chunkLength());
+            if (version >= MessagingService.VERSION_40)
+                size += TypeSizes.sizeof(parameters.maxCompressedLength());
             return size;
         }
     }
