@@ -15,14 +15,15 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-
 package org.apache.cassandra.schema;
 
 import java.util.*;
+import java.util.stream.Stream;
 
 import com.google.common.collect.ImmutableMap;
+import com.google.common.collect.Sets;
 
-import org.apache.cassandra.config.Schema;
+import static java.lang.String.format;
 
 import static com.google.common.collect.Iterables.filter;
 
@@ -35,7 +36,7 @@ import static com.google.common.collect.Iterables.filter;
  * support is added for multiple target columns per-index and for indexes with
  * TargetType.ROW
  */
-public class Indexes implements Iterable<IndexMetadata>
+public final class Indexes implements Iterable<IndexMetadata>
 {
     private final ImmutableMap<String, IndexMetadata> indexesByName;
     private final ImmutableMap<UUID, IndexMetadata> indexesById;
@@ -56,11 +57,26 @@ public class Indexes implements Iterable<IndexMetadata>
         return builder().build();
     }
 
+    public static Indexes of(IndexMetadata... indexes)
+    {
+        return builder().add(indexes).build();
+    }
+
+    public static Indexes of(Iterable<IndexMetadata> indexes)
+    {
+        return builder().add(indexes).build();
+    }
+
     public Iterator<IndexMetadata> iterator()
     {
         return indexesByName.values().iterator();
     }
 
+    public Stream<IndexMetadata> stream()
+    {
+        return indexesById.values().stream();
+    }
+
     public int size()
     {
         return indexesByName.size();
@@ -121,7 +137,7 @@ public class Indexes implements Iterable<IndexMetadata>
     public Indexes with(IndexMetadata index)
     {
         if (get(index.name).isPresent())
-            throw new IllegalStateException(String.format("Index %s already exists", index.name));
+            throw new IllegalStateException(format("Index %s already exists", index.name));
 
         return builder().add(this).add(index).build();
     }
@@ -131,7 +147,7 @@ public class Indexes implements Iterable<IndexMetadata>
      */
     public Indexes without(String name)
     {
-        IndexMetadata index = get(name).orElseThrow(() -> new IllegalStateException(String.format("Index %s doesn't exist", name)));
+        IndexMetadata index = get(name).orElseThrow(() -> new IllegalStateException(format("Index %s doesn't exist", name)));
         return builder().add(filter(this, v -> v != index)).build();
     }
 
@@ -149,6 +165,11 @@ public class Indexes implements Iterable<IndexMetadata>
         return this == o || (o instanceof Indexes && indexesByName.equals(((Indexes) o).indexesByName));
     }
 
+    public void validate(TableMetadata table)
+    {
+        indexesByName.values().forEach(i -> i.validate(table));
+    }
+
     @Override
     public int hashCode()
     {
@@ -161,20 +182,6 @@ public class Indexes implements Iterable<IndexMetadata>
         return indexesByName.values().toString();
     }
 
-    public static String getAvailableIndexName(String ksName, String cfName, String indexNameRoot)
-    {
-
-        KeyspaceMetadata ksm = Schema.instance.getKSMetaData(ksName);
-        Set<String> existingNames = ksm == null ? new HashSet<>() : ksm.existingIndexNames(null);
-        String baseName = IndexMetadata.getDefaultIndexName(cfName, indexNameRoot);
-        String acceptedName = baseName;
-        int i = 0;
-        while (existingNames.contains(acceptedName))
-            acceptedName = baseName + '_' + (++i);
-
-        return acceptedName;
-    }
-
     public static final class Builder
     {
         final ImmutableMap.Builder<String, IndexMetadata> indexesByName = new ImmutableMap.Builder<>();
@@ -196,6 +203,13 @@ public class Indexes implements Iterable<IndexMetadata>
             return this;
         }
 
+        public Builder add(IndexMetadata... indexes)
+        {
+            for (IndexMetadata index : indexes)
+                add(index);
+            return this;
+        }
+
         public Builder add(Iterable<IndexMetadata> indexes)
         {
             indexes.forEach(this::add);
