@@ -18,24 +18,20 @@
 
 package org.apache.cassandra.serializers;
 
-import java.nio.ByteBuffer;
+import org.apache.cassandra.db.marshal.ValueAccessor;
 
 public class TimeUUIDSerializer extends UUIDSerializer
 {
     public static final TimeUUIDSerializer instance = new TimeUUIDSerializer();
 
-    @Override
-    public void validate(ByteBuffer bytes) throws MarshalException
+    public <V> void validate(V value, ValueAccessor<V> accessor) throws MarshalException
     {
-        super.validate(bytes);
-
+        super.validate(value, accessor);
         // Super class only validates the Time UUID
-        ByteBuffer slice = bytes.slice();
         // version is bits 4-7 of byte 6.
-        if (bytes.remaining() > 0)
+        if (!accessor.isEmpty(value))
         {
-            slice.position(6);
-            if ((slice.get() & 0xf0) != 0x10)
+            if ((accessor.getByte(value, 6) & 0xf0) != 0x10)
                 throw new MarshalException("Invalid version for TimeUUID type.");
         }
     }
