@@ -73,6 +73,7 @@ import org.apache.cassandra.repair.CommonRange;
 import org.apache.cassandra.repair.RepairJobDesc;
 import org.apache.cassandra.repair.RepairParallelism;
 import org.apache.cassandra.repair.RepairSession;
+import org.apache.cassandra.repair.Scheduler;
 import org.apache.cassandra.repair.consistent.CoordinatorSessions;
 import org.apache.cassandra.repair.consistent.LocalSessions;
 import org.apache.cassandra.repair.consistent.admin.CleanupSummary;
@@ -198,10 +199,10 @@ public class ActiveRepairService implements IEndpointStateChangeSubscriber, IFai
     private final Gossiper gossiper;
     private final Cache<Integer, Pair<ParentRepairStatus, List<String>>> repairStatusByCmd;
 
-    private final DebuggableThreadPoolExecutor clearSnapshotExecutor = DebuggableThreadPoolExecutor.createWithMaximumPoolSize("RepairClearSnapshot",
-                                                                                                                              1,
-                                                                                                                              1,
-                                                                                                                              TimeUnit.HOURS);
+    public final DebuggableThreadPoolExecutor snapshotExecutor = DebuggableThreadPoolExecutor.createWithMaximumPoolSize("RepairSnapshotExecutor",
+                                                                                                                        1,
+                                                                                                                        1,
+                                                                                                                        TimeUnit.HOURS);
 
     public ActiveRepairService(IFailureDetector failureDetector, Gossiper gossiper)
     {
@@ -338,6 +339,7 @@ public class ActiveRepairService implements IEndpointStateChangeSubscriber, IFai
                                              PreviewKind previewKind,
                                              boolean optimiseStreams,
                                              ListeningExecutorService executor,
+                                             Scheduler validationScheduler,
                                              String... cfnames)
     {
         if (range.endpoints.isEmpty())
@@ -346,7 +348,8 @@ public class ActiveRepairService implements IEndpointStateChangeSubscriber, IFai
         if (cfnames.length == 0)
             return null;
 
-        final RepairSession session = new RepairSession(parentRepairSession, UUIDGen.getTimeUUID(), range, keyspace,
+        final RepairSession session = new RepairSession(parentRepairSession, UUIDGen.getTimeUUID(),
+                                                        validationScheduler, range, keyspace,
                                                         parallelismDegree, isIncremental, pullRepair,
                                                         previewKind, optimiseStreams, cfnames);
 
@@ -697,8 +700,9 @@ public class ActiveRepairService implements IEndpointStateChangeSubscriber, IFai
      *
      * clears out any snapshots created by this repair
      *
-     * @param parentSessionId
-     * @return
+     * @param parentSessionId id of parent session
+     * @return parent session of given id or null if there is not such
+     * @see org.apache.cassandra.db.repair.CassandraTableRepairManager#snapshot(String, Collection, boolean) 
      */
     public synchronized ParentRepairSession removeParentRepairSession(UUID parentSessionId)
     {
@@ -709,7 +713,7 @@ public class ActiveRepairService implements IEndpointStateChangeSubscriber, IFai
 
         if (session.hasSnapshots)
         {
-            clearSnapshotExecutor.submit(() -> {
+            snapshotExecutor.submit(() -> {
                 logger.info("[repair #{}] Clearing snapshots for {}", parentSessionId,
                             session.columnFamilyStores.values()
                                                       .stream()
@@ -897,4 +901,10 @@ public class ActiveRepairService implements IEndpointStateChangeSubscriber, IFai
     {
         return parentRepairSessions.size();
     }
+
+    @VisibleForTesting
+    public int sessionCount()
+    {
+        return sessions.size();
+    }
 }
