@@ -17,49 +17,48 @@
  */
 package org.apache.cassandra.service;
 
-import java.net.InetAddress;
-import java.util.Collection;
-
-import org.apache.cassandra.db.Keyspace;
-import org.apache.cassandra.net.MessageIn;
-import org.apache.cassandra.db.ConsistencyLevel;
 import org.apache.cassandra.db.WriteType;
+import org.apache.cassandra.locator.InOurDcTester;
+import org.apache.cassandra.locator.InetAddressAndPort;
+import org.apache.cassandra.locator.ReplicaPlan;
+import org.apache.cassandra.net.Message;
+
+import java.util.function.Predicate;
 
 /**
  * This class blocks for a quorum of responses _in the local datacenter only_ (CL.LOCAL_QUORUM).
  */
 public class DatacenterWriteResponseHandler<T> extends WriteResponseHandler<T>
 {
-    public DatacenterWriteResponseHandler(Collection<InetAddress> naturalEndpoints,
-                                          Collection<InetAddress> pendingEndpoints,
-                                          ConsistencyLevel consistencyLevel,
-                                          Keyspace keyspace,
+    private final Predicate<InetAddressAndPort> waitingFor = InOurDcTester.endpoints();
+
+    public DatacenterWriteResponseHandler(ReplicaPlan.ForTokenWrite replicaPlan,
                                           Runnable callback,
                                           WriteType writeType,
                                           long queryStartNanoTime)
     {
-        super(naturalEndpoints, pendingEndpoints, consistencyLevel, keyspace, callback, writeType, queryStartNanoTime);
-        assert consistencyLevel.isDatacenterLocal();
+        super(replicaPlan, callback, writeType, queryStartNanoTime);
+        assert replicaPlan.consistencyLevel().isDatacenterLocal();
     }
 
     @Override
-    public void response(MessageIn<T> message)
+    public void onResponse(Message<T> message)
     {
-        if (message == null || waitingFor(message.from))
-            super.response(message);
+        if (message == null || waitingFor(message.from()))
+        {
+            super.onResponse(message);
+        }
+        else
+        {
+            //WriteResponseHandler.response will call logResonseToIdealCLDelegate so only do it if not calling WriteResponseHandler.response.
+            //Must be last after all subclass processing
+            logResponseToIdealCLDelegate(message);
+        }
     }
 
     @Override
-    protected int totalBlockFor()
+    protected boolean waitingFor(InetAddressAndPort from)
     {
-        // during bootstrap, include pending endpoints (only local here) in the count
-        // or we may fail the consistency level guarantees (see #833, #8058)
-        return consistencyLevel.blockFor(keyspace) + consistencyLevel.countLocalEndpoints(pendingEndpoints);
-    }
-
-    @Override
-    protected boolean waitingFor(InetAddress from)
-    {
-        return consistencyLevel.isLocal(from);
+        return waitingFor.test(from);
     }
 }
