@@ -24,6 +24,7 @@ import java.util.Collections;
 import java.util.concurrent.TimeUnit;
 
 import com.google.common.annotations.VisibleForTesting;
+
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
@@ -31,13 +32,13 @@ import io.netty.channel.EventLoopGroup;
 import io.netty.channel.epoll.Epoll;
 import io.netty.channel.epoll.EpollEventLoopGroup;
 import io.netty.channel.nio.NioEventLoopGroup;
-import io.netty.util.concurrent.EventExecutor;
+import io.netty.util.Version;
 import org.apache.cassandra.config.DatabaseDescriptor;
-import org.apache.cassandra.metrics.AuthMetrics;
+import org.apache.cassandra.config.EncryptionOptions;
 import org.apache.cassandra.metrics.ClientMetrics;
-import org.apache.cassandra.transport.ConfiguredLimit;
-import org.apache.cassandra.transport.Message;
+import org.apache.cassandra.transport.Dispatcher;
 import org.apache.cassandra.transport.Server;
+import org.apache.cassandra.utils.NativeLibrary;
 
 /**
  * Handles native transport server lifecycle and associated resources. Lazily initialized.
@@ -51,7 +52,6 @@ public class NativeTransportService
 
     private boolean initialized = false;
     private EventLoopGroup workerGroup;
-    private ConfiguredLimit protocolVersionLimit;
 
     /**
      * Creates netty thread pools and event loops.
@@ -73,45 +73,52 @@ public class NativeTransportService
             logger.info("Netty using Java NIO event loop");
         }
 
-        protocolVersionLimit = ConfiguredLimit.newLimit();
-
         int nativePort = DatabaseDescriptor.getNativeTransportPort();
         int nativePortSSL = DatabaseDescriptor.getNativeTransportPortSSL();
         InetAddress nativeAddr = DatabaseDescriptor.getRpcAddress();
 
         org.apache.cassandra.transport.Server.Builder builder = new org.apache.cassandra.transport.Server.Builder()
                                                                 .withEventLoopGroup(workerGroup)
-                                                                .withProtocolVersionLimit(protocolVersionLimit)
                                                                 .withHost(nativeAddr);
 
-        if (!DatabaseDescriptor.getClientEncryptionOptions().enabled)
+        EncryptionOptions.TlsEncryptionPolicy encryptionPolicy = DatabaseDescriptor.getNativeProtocolEncryptionOptions().tlsEncryptionPolicy();
+        Server regularPortServer;
+        Server tlsPortServer = null;
+
+        // If an SSL port is separately supplied for the native transport, listen for unencrypted connections on the
+        // regular port, and encryption / optionally encrypted connections on the ssl port.
+        if (nativePort != nativePortSSL)
         {
-            servers = Collections.singleton(builder.withSSL(false).withPort(nativePort).build());
+            regularPortServer = builder.withTlsEncryptionPolicy(EncryptionOptions.TlsEncryptionPolicy.UNENCRYPTED).withPort(nativePort).build();
+            switch(encryptionPolicy)
+            {
+                case OPTIONAL: // FALLTHRU - encryption is optional on the regular port, but encrypted on the tls port.
+                case ENCRYPTED:
+                    tlsPortServer = builder.withTlsEncryptionPolicy(encryptionPolicy).withPort(nativePortSSL).build();
+                    break;
+                case UNENCRYPTED: // Should have been caught by DatabaseDescriptor.applySimpleConfig
+                    throw new IllegalStateException("Encryption must be enabled in client_encryption_options for native_transport_port_ssl");
+                default:
+                    throw new IllegalStateException("Unrecognized TLS encryption policy: " + encryptionPolicy);
+            }
+        }
+        // Otherwise, if only the regular port is supplied, listen as the encryption policy specifies
+        else
+        {
+            regularPortServer = builder.withTlsEncryptionPolicy(encryptionPolicy).withPort(nativePort).build();
+        }
+
+        if (tlsPortServer == null)
+        {
+            servers = Collections.singleton(regularPortServer);
         }
         else
         {
-            if (nativePort != nativePortSSL)
-            {
-                // user asked for dedicated ssl port for supporting both non-ssl and ssl connections
-                servers = Collections.unmodifiableList(
-                                                      Arrays.asList(
-                                                                   builder.withSSL(false).withPort(nativePort).build(),
-                                                                   builder.withSSL(true).withPort(nativePortSSL).build()
-                                                      )
-                );
-            }
-            else
-            {
-                // ssl only mode using configured native port
-                servers = Collections.singleton(builder.withSSL(true).withPort(nativePort).build());
-            }
+            servers = Collections.unmodifiableList(Arrays.asList(regularPortServer, tlsPortServer));
         }
 
-        // register metrics
         ClientMetrics.instance.init(servers);
 
-        AuthMetrics.init();
-
         initialized = true;
     }
 
@@ -120,6 +127,7 @@ public class NativeTransportService
      */
     public void start()
     {
+        logger.info("Using Netty Version: {}", Version.identify().entrySet());
         initialize();
         servers.forEach(Server::start);
     }
@@ -143,29 +151,19 @@ public class NativeTransportService
         // shutdown executors used by netty for native transport server
         workerGroup.shutdownGracefully(3, 5, TimeUnit.SECONDS).awaitUninterruptibly();
 
-        Message.Dispatcher.shutdown();
-    }
-
-    public int getMaxProtocolVersion()
-    {
-        return protocolVersionLimit.getMaxVersion().asInt();
-    }
-
-    public void refreshMaxNegotiableProtocolVersion()
-    {
-        // lowering the max negotiable protocol version is only safe if we haven't already
-        // allowed clients to connect with a higher version. This still allows the max
-        // version to be raised, as that is safe.
-        if (initialized)
-            protocolVersionLimit.updateMaxSupportedVersion();
+        Dispatcher.shutdown();
     }
 
     /**
-     * @return intend to use epoll bassed event looping
+     * @return intend to use epoll based event looping
      */
     public static boolean useEpoll()
     {
         final boolean enableEpoll = Boolean.parseBoolean(System.getProperty("cassandra.native.epoll.enabled", "true"));
+
+        if (enableEpoll && !Epoll.isAvailable() && NativeLibrary.osType == NativeLibrary.OSType.LINUX)
+            logger.warn("epoll not available", Epoll.unavailabilityCause());
+
         return enableEpoll && Epoll.isAvailable();
     }
 
@@ -190,4 +188,10 @@ public class NativeTransportService
     {
         return servers;
     }
+
+    public void clearConnectionHistory()
+    {
+        for (Server server : servers)
+            server.clearConnectionHistory();
+    }
 }
