@@ -536,6 +536,13 @@ public interface StorageServiceMBean extends NotificationEmitter
      */
     public int getDynamicUpdateInterval();
 
+    public String getBatchlogEndpointStrategy();
+
+    /**
+     * See {@link org.apache.cassandra.config.Config.BatchlogEndpointStrategy} for valid values.
+     */
+    public void setBatchlogEndpointStrategy(String batchlogEndpointStrategy);
+
     // allows a user to forcibly 'kill' a sick node
     public void stopGossiping();
 
@@ -770,6 +777,16 @@ public interface StorageServiceMBean extends NotificationEmitter
     public boolean getTransferHintsOnDecommission();
     public void setTransferHintsOnDecommission(boolean enabled);
 
+    /** Returns whether we are using the creation time of the mutation for determining hint ttl **/
+    public boolean isHintTtlUseMutationCreationTime();
+    /** Sets whether we are using the creation time of the mutation for determining hint ttl **/
+    public void setUseCreationTimeForHintTtl(boolean enabled);
+
+    /** Returns upper bound on the hint ttl **/
+    public int getMaxHintTTL();
+    /** Sets the upper bound on the hint ttl **/
+    public void setMaxHintTTL(int maxHintTTL);
+
     /**
      * Resume bootstrap streaming when there is failed data streaming.
      *
