@@ -1,4 +1,3 @@
-package org.apache.cassandra.service.paxos;
 /*
  * 
  * Licensed to the Apache Software Foundation (ASF) under one
@@ -19,19 +18,26 @@ package org.apache.cassandra.service.paxos;
  * under the License.
  * 
  */
-
+package org.apache.cassandra.service.paxos;
 
 import java.util.concurrent.CountDownLatch;
-import java.util.concurrent.TimeUnit;
+
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 
 import org.apache.cassandra.config.DatabaseDescriptor;
 import org.apache.cassandra.db.ConsistencyLevel;
 import org.apache.cassandra.db.WriteType;
+import org.apache.cassandra.exceptions.RequestFailureReason;
 import org.apache.cassandra.exceptions.WriteTimeoutException;
-import org.apache.cassandra.net.IAsyncCallback;
+import org.apache.cassandra.locator.InetAddressAndPort;
+import org.apache.cassandra.net.RequestCallback;
 
-public abstract class AbstractPaxosCallback<T> implements IAsyncCallback<T>
+import static java.util.concurrent.TimeUnit.NANOSECONDS;
+
+public abstract class AbstractPaxosCallback<T> implements RequestCallback<T>
 {
+    private static final Logger logger = LoggerFactory.getLogger(AbstractPaxosCallback.class);
     protected final CountDownLatch latch;
     protected final int targets;
     private final ConsistencyLevel consistency;
@@ -45,11 +51,6 @@ public abstract class AbstractPaxosCallback<T> implements IAsyncCallback<T>
         this.queryStartNanoTime = queryStartNanoTime;
     }
 
-    public boolean isLatencyForSnitch()
-    {
-        return false;
-    }
-
     public int getResponseCount()
     {
         return (int) (targets - latch.getCount());
@@ -59,8 +60,8 @@ public abstract class AbstractPaxosCallback<T> implements IAsyncCallback<T>
     {
         try
         {
-            long timeout = TimeUnit.MILLISECONDS.toNanos(DatabaseDescriptor.getWriteRpcTimeout()) - (System.nanoTime() - queryStartNanoTime);
-            if (!latch.await(timeout, TimeUnit.NANOSECONDS))
+            long timeout = DatabaseDescriptor.getWriteRpcTimeout(NANOSECONDS) - (System.nanoTime() - queryStartNanoTime);
+            if (!latch.await(timeout, NANOSECONDS))
                 throw new WriteTimeoutException(WriteType.CAS, consistency, getResponseCount(), targets);
         }
         catch (InterruptedException ex)
@@ -68,4 +69,16 @@ public abstract class AbstractPaxosCallback<T> implements IAsyncCallback<T>
             throw new AssertionError("This latch shouldn't have been interrupted.");
         }
     }
+
+    @Override
+    public void onFailure(InetAddressAndPort from, RequestFailureReason failureReason)
+    {
+        logger.debug("Received paxos propose/prepare failure response from {} reason {}", from, failureReason);
+    }
+
+    @Override
+    public boolean invokeOnFailure()
+    {
+        return true;
+    }
 }
