@@ -20,19 +20,20 @@
  */
 package org.apache.cassandra.service.paxos;
 
-import org.apache.cassandra.db.WriteResponse;
-import org.apache.cassandra.net.IVerbHandler;
-import org.apache.cassandra.net.MessageIn;
+import org.apache.cassandra.net.Message;
 import org.apache.cassandra.net.MessagingService;
 import org.apache.cassandra.tracing.Tracing;
 
-public class CommitVerbHandler implements IVerbHandler<Commit>
+public class CommitVerbHandler extends AbstractPaxosVerbHandler
 {
-    public void doVerb(MessageIn<Commit> message, int id)
+    public static final CommitVerbHandler instance = new CommitVerbHandler();
+
+    @Override
+    void processMessage(Message<Commit> message)
     {
         PaxosState.commit(message.payload);
 
-        Tracing.trace("Enqueuing acknowledge to {}", message.from);
-        MessagingService.instance().sendReply(WriteResponse.createMessage(), id, message.from);
+        Tracing.trace("Enqueuing acknowledge to {}", message.from());
+        MessagingService.instance().send(message.emptyResponse(), message.from());
     }
 }
