@@ -355,21 +355,9 @@ public class SSTableMetadataViewer
             if (validation != null && header != null)
                 printMinMaxToken(descriptor, FBUtilities.newPartitioner(descriptor), header.getKeyType());
 
-            if (header != null && header.getClusteringTypes().size() == stats.minClusteringValues.size())
-            {
-                List<AbstractType<?>> clusteringTypes = header.getClusteringTypes();
-                List<ByteBuffer> minClusteringValues = stats.minClusteringValues;
-                List<ByteBuffer> maxClusteringValues = stats.maxClusteringValues;
-                String[] minValues = new String[clusteringTypes.size()];
-                String[] maxValues = new String[clusteringTypes.size()];
-                for (int i = 0; i < clusteringTypes.size(); i++)
-                {
-                    minValues[i] = clusteringTypes.get(i).getString(minClusteringValues.get(i));
-                    maxValues[i] = clusteringTypes.get(i).getString(maxClusteringValues.get(i));
-                }
-                field("minClusteringValues", Arrays.toString(minValues));
-                field("maxClusteringValues", Arrays.toString(maxValues));
-            }
+            printClusteringValues(header, "minClusteringValues", stats.minClusteringValues);
+            printClusteringValues(header, "maxClusteringValues", stats.maxClusteringValues);
+
             field("Estimated droppable tombstones",
                   stats.getEstimatedDroppableTombstoneRatio((int) (System.currentTimeMillis() / 1000) - this.gc));
             field("SSTable Level", stats.sstableLevel);
@@ -436,6 +424,21 @@ public class SSTableMetadataViewer
         }
     }
 
+    private void printClusteringValues(SerializationHeader.Component header, String name, List<ByteBuffer> clusteringValues)
+    {
+        if (header != null && header.getClusteringTypes().size() >= clusteringValues.size())
+        {
+            List<AbstractType<?>> clusteringTypes = header.getClusteringTypes();
+            int size = Math.min(clusteringTypes.size(), clusteringValues.size());
+            String[] values = new String[size];
+            for (int i = 0; i < size; i++)
+            {
+                values[i] = clusteringTypes.get(i).getString(clusteringValues.get(i));
+            }
+            field(name, Arrays.toString(values));
+        }
+    }
+
     private void field(String field, Object value)
     {
         field(field, value, null);
