@@ -21,17 +21,22 @@ import java.net.InetAddress;
 import java.nio.ByteBuffer;
 import java.util.*;
 
-import org.apache.cassandra.config.CFMetaData;
-import org.apache.cassandra.config.SchemaConstants;
+import org.apache.cassandra.cql3.statements.schema.CreateTableStatement;
 import org.apache.cassandra.db.Mutation;
-import org.apache.cassandra.db.rows.Row;
 import org.apache.cassandra.db.partitions.PartitionUpdate;
+import org.apache.cassandra.db.rows.Row;
+import org.apache.cassandra.gms.Gossiper;
 import org.apache.cassandra.schema.KeyspaceMetadata;
 import org.apache.cassandra.schema.KeyspaceParams;
+import org.apache.cassandra.schema.SchemaConstants;
+import org.apache.cassandra.schema.TableId;
+import org.apache.cassandra.schema.TableMetadata;
 import org.apache.cassandra.schema.Tables;
 import org.apache.cassandra.utils.FBUtilities;
 import org.apache.cassandra.utils.UUIDGen;
 
+import static java.lang.String.format;
+
 public final class TraceKeyspace
 {
     private TraceKeyspace()
@@ -47,18 +52,19 @@ public final class TraceKeyspace
      *                       will ever start; see the note below for why this is necessary; actual change in 3.0:
      *                       removed default ttl, reduced bloom filter fp chance from 0.1 to 0.01.
      * gen 1577836800000001: (pre-)adds coordinator_port column to sessions and source_port column to events in 3.0, 3.11, 4.0
+     * gen 1577836800000002: compression chunk length reduced to 16KiB, memtable_flush_period_in_ms now unset on all tables in 4.0
      *
      * * Until CASSANDRA-6016 (Oct 13, 2.0.2) and in all of 1.2, we used to create system_traces keyspace and
      *   tables in the same way that we created the purely local 'system' keyspace - using current time on node bounce
      *   (+1). For new definitions to take, we need to bump the generation further than that.
      */
-    public static final long GENERATION = 1577836800000001L;
+    public static final long GENERATION = 1577836800000002L;
 
     public static final String SESSIONS = "sessions";
     public static final String EVENTS = "events";
 
-    private static final CFMetaData Sessions =
-        compile(SESSIONS,
+    private static final TableMetadata Sessions =
+        parse(SESSIONS,
                 "tracing sessions",
                 "CREATE TABLE %s ("
                 + "session_id uuid,"
@@ -72,8 +78,8 @@ public final class TraceKeyspace
                 + "started_at timestamp,"
                 + "PRIMARY KEY ((session_id)))");
 
-    private static final CFMetaData Events =
-        compile(EVENTS,
+    private static final TableMetadata Events =
+        parse(EVENTS,
                 "tracing events",
                 "CREATE TABLE %s ("
                 + "session_id uuid,"
@@ -85,10 +91,13 @@ public final class TraceKeyspace
                 + "thread text,"
                 + "PRIMARY KEY ((session_id), event_id))");
 
-    private static CFMetaData compile(String name, String description, String schema)
+    private static TableMetadata parse(String table, String description, String cql)
     {
-        return CFMetaData.compile(String.format(schema, name), SchemaConstants.TRACE_KEYSPACE_NAME)
-                         .comment(description);
+        return CreateTableStatement.parse(format(cql, table), SchemaConstants.TRACE_KEYSPACE_NAME)
+                                   .id(TableId.forSystemTable(SchemaConstants.TRACE_KEYSPACE_NAME, table))
+                                   .gcGraceSeconds(0)
+                                   .comment(description)
+                                   .build();
     }
 
     public static KeyspaceMetadata metadata()
@@ -105,14 +114,16 @@ public final class TraceKeyspace
                                              int ttl)
     {
         PartitionUpdate.SimpleBuilder builder = PartitionUpdate.simpleBuilder(Sessions, sessionId);
-        builder.row()
-               .ttl(ttl)
-               .add("client", client)
-               .add("coordinator", FBUtilities.getBroadcastAddress())
-               .add("request", request)
-               .add("started_at", new Date(startedAt))
-               .add("command", command)
-               .appendAll("parameters", parameters);
+        Row.SimpleBuilder rb = builder.row();
+        rb.ttl(ttl)
+          .add("client", client)
+          .add("coordinator", FBUtilities.getBroadcastAddressAndPort().address);
+        if (!Gossiper.instance.hasMajorVersion3OrUnknownNodes())
+            rb.add("coordinator_port", FBUtilities.getBroadcastAddressAndPort().port);
+        rb.add("request", request)
+          .add("started_at", new Date(startedAt))
+          .add("command", command)
+          .appendAll("parameters", parameters);
 
         return builder.buildAsMutation();
     }
@@ -133,8 +144,10 @@ public final class TraceKeyspace
                                               .ttl(ttl);
 
         rowBuilder.add("activity", message)
-                  .add("source", FBUtilities.getBroadcastAddress())
-                  .add("thread", threadName);
+                  .add("source", FBUtilities.getBroadcastAddressAndPort().address);
+        if (!Gossiper.instance.hasMajorVersion3OrUnknownNodes())
+            rowBuilder.add("source_port", FBUtilities.getBroadcastAddressAndPort().port);
+        rowBuilder.add("thread", threadName);
 
         if (elapsed >= 0)
             rowBuilder.add("source_elapsed", elapsed);
