@@ -17,12 +17,16 @@
  */
 package org.apache.cassandra.transport.messages;
 
+import java.net.InetAddress;
+import java.util.Collections;
 import java.util.List;
+import java.util.Map;
 
 import io.netty.buffer.ByteBuf;
 import io.netty.handler.codec.CodecException;
 import com.google.common.annotations.VisibleForTesting;
 import com.google.common.base.Predicate;
+import com.google.common.collect.ImmutableMap;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
@@ -30,6 +34,7 @@ import org.apache.cassandra.cql3.functions.FunctionName;
 import org.apache.cassandra.db.ConsistencyLevel;
 import org.apache.cassandra.db.WriteType;
 import org.apache.cassandra.exceptions.*;
+import org.apache.cassandra.locator.InetAddressAndPort;
 import org.apache.cassandra.transport.*;
 import org.apache.cassandra.utils.MD5Digest;
 
@@ -42,7 +47,7 @@ public class ErrorMessage extends Message.Response
 
     public static final Message.Codec<ErrorMessage> codec = new Message.Codec<ErrorMessage>()
     {
-        public ErrorMessage decode(ByteBuf body, int version)
+        public ErrorMessage decode(ByteBuf body, ProtocolVersion version)
         {
             ExceptionCode code = ExceptionCode.fromValue(body.readInt());
             String msg = CBUtil.readString(body);
@@ -64,7 +69,7 @@ public class ErrorMessage extends Message.Response
                         ConsistencyLevel cl = CBUtil.readConsistencyLevel(body);
                         int required = body.readInt();
                         int alive = body.readInt();
-                        te = new UnavailableException(cl, required, alive);
+                        te = UnavailableException.create(cl, required, alive);
                     }
                     break;
                 case OVERLOADED:
@@ -76,46 +81,75 @@ public class ErrorMessage extends Message.Response
                 case TRUNCATE_ERROR:
                     te = new TruncateException(msg);
                     break;
-                case WRITE_FAILURE: 
+                case WRITE_FAILURE:
                 case READ_FAILURE:
                     {
                         ConsistencyLevel cl = CBUtil.readConsistencyLevel(body);
                         int received = body.readInt();
                         int blockFor = body.readInt();
+                        // The number of failures is also present in protocol v5, but used instead to specify the size of the failure map
                         int failure = body.readInt();
+
+                        Map<InetAddressAndPort, RequestFailureReason> failureReasonByEndpoint;
+                        if (version.isGreaterOrEqualTo(ProtocolVersion.V5))
+                        {
+                            ImmutableMap.Builder<InetAddressAndPort, RequestFailureReason> builder = ImmutableMap.builderWithExpectedSize(failure);
+                            for (int i = 0; i < failure; i++)
+                            {
+                                InetAddress endpoint = CBUtil.readInetAddr(body);
+                                RequestFailureReason failureReason = RequestFailureReason.fromCode(body.readUnsignedShort());
+                                builder.put(InetAddressAndPort.getByAddress(endpoint), failureReason);
+                            }
+                            failureReasonByEndpoint = builder.build();
+                        }
+                        else
+                        {
+                            failureReasonByEndpoint = Collections.emptyMap();
+                        }
+
                         if (code == ExceptionCode.WRITE_FAILURE)
                         {
                             WriteType writeType = Enum.valueOf(WriteType.class, CBUtil.readString(body));
-                            te = new WriteFailureException(cl, received, failure, blockFor, writeType);
+                            te = new WriteFailureException(cl, received, blockFor, writeType, failureReasonByEndpoint);
                         }
                         else
                         {
                             byte dataPresent = body.readByte();
-                            te = new ReadFailureException(cl, received, failure, blockFor, dataPresent != 0);   
+                            te = new ReadFailureException(cl, received, blockFor, dataPresent != 0, failureReasonByEndpoint);
                         }
                     }
                     break;
                 case WRITE_TIMEOUT:
                 case READ_TIMEOUT:
-                    ConsistencyLevel cl = CBUtil.readConsistencyLevel(body);
-                    int received = body.readInt();
-                    int blockFor = body.readInt();
-                    if (code == ExceptionCode.WRITE_TIMEOUT)
                     {
-                        WriteType writeType = Enum.valueOf(WriteType.class, CBUtil.readString(body));
-                        te = new WriteTimeoutException(writeType, cl, received, blockFor);
+                        ConsistencyLevel cl = CBUtil.readConsistencyLevel(body);
+                        int received = body.readInt();
+                        int blockFor = body.readInt();
+                        if (code == ExceptionCode.WRITE_TIMEOUT)
+                        {
+                            WriteType writeType = Enum.valueOf(WriteType.class, CBUtil.readString(body));
+                            if (version.isGreaterOrEqualTo(ProtocolVersion.V5) && writeType == WriteType.CAS)
+                            {
+                                int contentions = body.readShort();
+                                te = new CasWriteTimeoutException(writeType, cl, received, blockFor, contentions);
+                            }
+                            else
+                            {
+                                te = new WriteTimeoutException(writeType, cl, received, blockFor);
+                            }
+                        }
+                        else
+                        {
+                            byte dataPresent = body.readByte();
+                            te = new ReadTimeoutException(cl, received, blockFor, dataPresent != 0);
+                        }
+                        break;
                     }
-                    else
-                    {
-                        byte dataPresent = body.readByte();
-                        te = new ReadTimeoutException(cl, received, blockFor, dataPresent != 0);
-                    }
-                    break;
                 case FUNCTION_FAILURE:
                     String fKeyspace = CBUtil.readString(body);
                     String fName = CBUtil.readString(body);
                     List<String> argTypes = CBUtil.readStringList(body);
-                    te = new FunctionExecutionException(new FunctionName(fKeyspace, fName), argTypes, msg);
+                    te = FunctionExecutionException.create(new FunctionName(fKeyspace, fName), argTypes, msg);
                     break;
                 case UNPREPARED:
                     {
@@ -135,6 +169,9 @@ public class ErrorMessage extends Message.Response
                 case CONFIG_ERROR:
                     te = new ConfigurationException(msg);
                     break;
+                case CDC_WRITE_FAILURE:
+                    te = new CDCWriteException(msg);
+                    break;
                 case ALREADY_EXISTS:
                     String ksName = CBUtil.readString(body);
                     String cfName = CBUtil.readString(body);
@@ -143,11 +180,19 @@ public class ErrorMessage extends Message.Response
                     else
                         te = new AlreadyExistsException(ksName, cfName);
                     break;
+                case CAS_WRITE_UNKNOWN:
+                    assert version.isGreaterOrEqualTo(ProtocolVersion.V5);
+
+                    ConsistencyLevel cl = CBUtil.readConsistencyLevel(body);
+                    int received = body.readInt();
+                    int blockFor = body.readInt();
+                    te = new CasWriteUnknownResultException(cl, received, blockFor);
+                    break;
             }
             return new ErrorMessage(te);
         }
 
-        public void encode(ErrorMessage msg, ByteBuf dest, int version)
+        public void encode(ErrorMessage msg, ByteBuf dest, ProtocolVersion version)
         {
             final TransportException err = getBackwardsCompatibleException(msg, version);
             dest.writeInt(err.code().value);
@@ -165,37 +210,52 @@ public class ErrorMessage extends Message.Response
                 case WRITE_FAILURE:
                 case READ_FAILURE:
                     {
-                        RequestFailureException rfe = (RequestFailureException)err;
-                        boolean isWrite = err.code() == ExceptionCode.WRITE_FAILURE;
+                        RequestFailureException rfe = (RequestFailureException) err;
 
                         CBUtil.writeConsistencyLevel(rfe.consistency, dest);
                         dest.writeInt(rfe.received);
                         dest.writeInt(rfe.blockFor);
-                        dest.writeInt(rfe.failures);
+                        // The number of failures is also present in protocol v5, but used instead to specify the size of the failure map
+                        dest.writeInt(rfe.failureReasonByEndpoint.size());
 
-                        if (isWrite)
-                            CBUtil.writeString(((WriteFailureException)rfe).writeType.toString(), dest);
+                        if (version.isGreaterOrEqualTo(ProtocolVersion.V5))
+                        {
+                            for (Map.Entry<InetAddressAndPort, RequestFailureReason> entry : rfe.failureReasonByEndpoint.entrySet())
+                            {
+                                CBUtil.writeInetAddr(entry.getKey().address, dest);
+                                dest.writeShort(entry.getValue().code);
+                            }
+                        }
+
+                        if (err.code() == ExceptionCode.WRITE_FAILURE)
+                            CBUtil.writeAsciiString(((WriteFailureException) rfe).writeType.toString(), dest);
                         else
-                            dest.writeByte((byte)(((ReadFailureException)rfe).dataPresent ? 1 : 0));
+                            dest.writeByte((byte) (((ReadFailureException) rfe).dataPresent ? 1 : 0));
+                        break;
                     }
-                    break;
                 case WRITE_TIMEOUT:
                 case READ_TIMEOUT:
                     RequestTimeoutException rte = (RequestTimeoutException)err;
-                    boolean isWrite = err.code() == ExceptionCode.WRITE_TIMEOUT;
 
                     CBUtil.writeConsistencyLevel(rte.consistency, dest);
                     dest.writeInt(rte.received);
                     dest.writeInt(rte.blockFor);
-                    if (isWrite)
-                        CBUtil.writeString(((WriteTimeoutException)rte).writeType.toString(), dest);
+                    if (err.code() == ExceptionCode.WRITE_TIMEOUT)
+                    {
+                        CBUtil.writeAsciiString(((WriteTimeoutException)rte).writeType.toString(), dest);
+                        // CasWriteTimeoutException already implies protocol V5, but double check to be safe.
+                        if (version.isGreaterOrEqualTo(ProtocolVersion.V5) && rte instanceof CasWriteTimeoutException)
+                            dest.writeShort(((CasWriteTimeoutException)rte).contentions);
+                    }
                     else
+                    {
                         dest.writeByte((byte)(((ReadTimeoutException)rte).dataPresent ? 1 : 0));
+                    }
                     break;
                 case FUNCTION_FAILURE:
                     FunctionExecutionException fee = (FunctionExecutionException)msg.error;
-                    CBUtil.writeString(fee.functionName.keyspace, dest);
-                    CBUtil.writeString(fee.functionName.name, dest);
+                    CBUtil.writeAsciiString(fee.functionName.keyspace, dest);
+                    CBUtil.writeAsciiString(fee.functionName.name, dest);
                     CBUtil.writeStringList(fee.argTypes, dest);
                     break;
                 case UNPREPARED:
@@ -204,15 +264,21 @@ public class ErrorMessage extends Message.Response
                     break;
                 case ALREADY_EXISTS:
                     AlreadyExistsException aee = (AlreadyExistsException)err;
-                    CBUtil.writeString(aee.ksName, dest);
-                    CBUtil.writeString(aee.cfName, dest);
+                    CBUtil.writeAsciiString(aee.ksName, dest);
+                    CBUtil.writeAsciiString(aee.cfName, dest);
                     break;
+                case CAS_WRITE_UNKNOWN:
+                    assert version.isGreaterOrEqualTo(ProtocolVersion.V5);
+                    CasWriteUnknownResultException cwue = (CasWriteUnknownResultException)err;
+                    CBUtil.writeConsistencyLevel(cwue.consistency, dest);
+                    dest.writeInt(cwue.received);
+                    dest.writeInt(cwue.blockFor);
             }
         }
 
-        public int encodedSize(ErrorMessage msg, int version)
+        public int encodedSize(ErrorMessage msg, ProtocolVersion version)
         {
-            final TransportException err = getBackwardsCompatibleException(msg, version);
+            TransportException err = getBackwardsCompatibleException(msg, version);
             String errorString = err.getMessage() == null ? "" : err.getMessage();
             int size = 4 + CBUtil.sizeOfString(errorString);
             switch (err.code())
@@ -225,9 +291,21 @@ public class ErrorMessage extends Message.Response
                 case READ_FAILURE:
                     {
                         RequestFailureException rfe = (RequestFailureException)err;
-                        boolean isWrite = err.code() == ExceptionCode.WRITE_FAILURE;
+
                         size += CBUtil.sizeOfConsistencyLevel(rfe.consistency) + 4 + 4 + 4;
-                        size += isWrite ? CBUtil.sizeOfString(((WriteFailureException)rfe).writeType.toString()) : 1;
+                        if (err.code() == ExceptionCode.WRITE_FAILURE)
+                            size += CBUtil.sizeOfAsciiString(((WriteFailureException)rfe).writeType.toString());
+                        else
+                            size += 1;
+
+                        if (version.isGreaterOrEqualTo(ProtocolVersion.V5))
+                        {
+                            for (Map.Entry<InetAddressAndPort, RequestFailureReason> entry : rfe.failureReasonByEndpoint.entrySet())
+                            {
+                                size += CBUtil.sizeOfInetAddr(entry.getKey().address);
+                                size += 2; // RequestFailureReason code
+                            }
+                        }
                     }
                     break;
                 case WRITE_TIMEOUT:
@@ -235,12 +313,19 @@ public class ErrorMessage extends Message.Response
                     RequestTimeoutException rte = (RequestTimeoutException)err;
                     boolean isWrite = err.code() == ExceptionCode.WRITE_TIMEOUT;
                     size += CBUtil.sizeOfConsistencyLevel(rte.consistency) + 8;
-                    size += isWrite ? CBUtil.sizeOfString(((WriteTimeoutException)rte).writeType.toString()) : 1;
+                    if (isWrite)
+                        size += CBUtil.sizeOfAsciiString(((WriteTimeoutException)rte).writeType.toString());
+                    else
+                        size += 1;
+
+                    // CasWriteTimeoutException already implies protocol V5, but double check to be safe.
+                    if (isWrite && version.isGreaterOrEqualTo(ProtocolVersion.V5) && rte instanceof CasWriteTimeoutException)
+                        size += 2; // CasWriteTimeoutException appends a short for contentions occured.
                     break;
                 case FUNCTION_FAILURE:
                     FunctionExecutionException fee = (FunctionExecutionException)msg.error;
-                    size += CBUtil.sizeOfString(fee.functionName.keyspace);
-                    size += CBUtil.sizeOfString(fee.functionName.name);
+                    size += CBUtil.sizeOfAsciiString(fee.functionName.keyspace);
+                    size += CBUtil.sizeOfAsciiString(fee.functionName.name);
                     size += CBUtil.sizeOfStringList(fee.argTypes);
                     break;
                 case UNPREPARED:
@@ -249,17 +334,22 @@ public class ErrorMessage extends Message.Response
                     break;
                 case ALREADY_EXISTS:
                     AlreadyExistsException aee = (AlreadyExistsException)err;
-                    size += CBUtil.sizeOfString(aee.ksName);
-                    size += CBUtil.sizeOfString(aee.cfName);
+                    size += CBUtil.sizeOfAsciiString(aee.ksName);
+                    size += CBUtil.sizeOfAsciiString(aee.cfName);
+                    break;
+                case CAS_WRITE_UNKNOWN:
+                    assert version.isGreaterOrEqualTo(ProtocolVersion.V5);
+                    CasWriteUnknownResultException cwue = (CasWriteUnknownResultException)err;
+                    size += CBUtil.sizeOfConsistencyLevel(cwue.consistency) + 4 + 4; // receivedFor: 4, blockFor: 4
                     break;
             }
             return size;
         }
     };
 
-    private static TransportException getBackwardsCompatibleException(ErrorMessage msg, int version)
+    private static TransportException getBackwardsCompatibleException(ErrorMessage msg, ProtocolVersion version)
     {
-        if (version < Server.VERSION_4)
+        if (version.isSmallerThan(ProtocolVersion.V4))
         {
             switch (msg.error.code())
             {
@@ -270,10 +360,28 @@ public class ErrorMessage extends Message.Response
                     WriteFailureException wfe = (WriteFailureException) msg.error;
                     return new WriteTimeoutException(wfe.writeType, wfe.consistency, wfe.received, wfe.blockFor);
                 case FUNCTION_FAILURE:
+                case CDC_WRITE_FAILURE:
                     return new InvalidRequestException(msg.toString());
             }
         }
 
+        if (version.isSmallerThan(ProtocolVersion.V5))
+        {
+            switch (msg.error.code())
+            {
+                case WRITE_TIMEOUT:
+                    if (msg.error instanceof CasWriteTimeoutException)
+                    {
+                        CasWriteTimeoutException cwte = (CasWriteTimeoutException) msg.error;
+                        return new WriteTimeoutException(WriteType.CAS, cwte.consistency, cwte.received, cwte.blockFor);
+                    }
+                    break;
+                case CAS_WRITE_UNKNOWN:
+                    CasWriteUnknownResultException cwue = (CasWriteUnknownResultException) msg.error;
+                    return new WriteTimeoutException(WriteType.CAS, cwue.consistency, cwue.received, cwue.blockFor);
+            }
+        }
+
         return msg.error;
     }
 
@@ -335,11 +443,11 @@ public class ErrorMessage extends Message.Response
             ErrorMessage message = new ErrorMessage((TransportException) e, streamId);
             if (e instanceof ProtocolException)
             {
-                // if the driver attempted to connect with a protocol version lower than the minimum supported
-                // version, respond with a protocol error message with the correct frame header for that version
-                Integer attemptedLowProtocolVersion = ((ProtocolException) e).getAttemptedLowProtocolVersion();
-                if (attemptedLowProtocolVersion != null)
-                    message.forcedProtocolVersion = attemptedLowProtocolVersion;
+                // if the driver attempted to connect with a protocol version not supported then
+                // respond with the appropiate version, see ProtocolVersion.decode()
+                ProtocolVersion forcedProtocolVersion = ((ProtocolException) e).getForcedProtocolVersion();
+                if (forcedProtocolVersion != null)
+                    message.forcedProtocolVersion = forcedProtocolVersion;
             }
             return message;
         }
