@@ -17,16 +17,15 @@
  */
 package org.apache.cassandra.utils;
 
-import java.io.DataInput;
+import java.io.DataInputStream;
 import java.io.IOException;
 
 import org.apache.cassandra.db.TypeSizes;
 import org.apache.cassandra.io.util.DataOutputPlus;
 import org.apache.cassandra.utils.obs.IBitSet;
 import org.apache.cassandra.utils.obs.OffHeapBitSet;
-import org.apache.cassandra.utils.obs.OpenBitSet;
 
-final class BloomFilterSerializer
+public final class BloomFilterSerializer
 {
     private BloomFilterSerializer()
     {
@@ -38,18 +37,13 @@ final class BloomFilterSerializer
         bf.bitset.serialize(out);
     }
 
-    public static BloomFilter deserialize(DataInput in, boolean oldBfHashOrder) throws IOException
-    {
-        return deserialize(in, false, oldBfHashOrder);
-    }
-
     @SuppressWarnings("resource")
-    public static BloomFilter deserialize(DataInput in, boolean offheap, boolean oldBfHashOrder) throws IOException
+    public static BloomFilter deserialize(DataInputStream in, boolean oldBfFormat) throws IOException
     {
         int hashes = in.readInt();
-        IBitSet bs = offheap ? OffHeapBitSet.deserialize(in) : OpenBitSet.deserialize(in);
+        IBitSet bs = OffHeapBitSet.deserialize(in, oldBfFormat);
 
-        return new BloomFilter(hashes, bs, oldBfHashOrder);
+        return new BloomFilter(hashes, bs);
     }
 
     /**
