@@ -20,7 +20,6 @@ package org.apache.cassandra.utils.btree;
 
 import java.util.*;
 
-import com.google.common.collect.ImmutableList;
 import com.google.common.collect.Ordering;
 
 import org.apache.cassandra.utils.btree.BTree.Dir;
@@ -39,11 +38,6 @@ public class BTreeSet<V> implements NavigableSet<V>, List<V>
         this.comparator = comparator;
     }
 
-    public BTreeSet<V> update(Collection<V> updateWith)
-    {
-        return new BTreeSet<>(BTree.update(tree, comparator, updateWith, UpdateFunction.<V>noOp()), comparator);
-    }
-
     @Override
     public Comparator<? super V> comparator()
     {
@@ -582,31 +576,38 @@ public class BTreeSet<V> implements NavigableSet<V>, List<V>
 
     public static class Builder<V>
     {
-        final BTree.Builder<V> builder;
+        final BTree.Builder<V> wrapped;
+
         protected Builder(Comparator<? super V> comparator)
         {
-            builder= BTree.builder(comparator);
+            wrapped = BTree.builder(comparator);
+        }
+
+        protected Builder(Comparator<? super V> comparator, int size)
+        {
+            wrapped = BTree.builder(comparator, size);
         }
 
         public Builder<V> add(V v)
         {
-            builder.add(v);
+            wrapped .add(v);
             return this;
         }
 
         public Builder<V> addAll(Collection<V> iter)
         {
-            builder.addAll(iter);
+            wrapped .addAll(iter);
             return this;
         }
 
         public boolean isEmpty()
         {
-            return builder.isEmpty();
+            return wrapped .isEmpty();
         }
+
         public BTreeSet<V> build()
         {
-            return new BTreeSet<>(builder.build(), builder.comparator);
+            return new BTreeSet<>(wrapped .build(), wrapped .comparator);
         }
     }
 
@@ -615,19 +616,25 @@ public class BTreeSet<V> implements NavigableSet<V>, List<V>
         return new Builder<>(comparator);
     }
 
-    public static <V> BTreeSet<V> wrap(Object[] btree, Comparator<V> comparator)
+    /** if you know the precise size of the resultant set use {@code perfectBuilder} instead. */
+    public static <V> Builder<V> builder(Comparator<? super V> comparator, int initialCapacity)
+    {
+        return new Builder<>(comparator, initialCapacity);
+    }
+
+    public static <V> BTreeSet<V> wrap(Object[] btree, Comparator<? super V> comparator)
     {
         return new BTreeSet<>(btree, comparator);
     }
 
     public static <V extends Comparable<V>> BTreeSet<V> of(Collection<V> sortedValues)
     {
-        return new BTreeSet<>(BTree.build(sortedValues, UpdateFunction.<V>noOp()), Ordering.<V>natural());
+        return new BTreeSet<>(BTree.build(sortedValues), Ordering.<V>natural());
     }
 
     public static <V extends Comparable<V>> BTreeSet<V> of(V value)
     {
-        return new BTreeSet<>(BTree.build(ImmutableList.of(value), UpdateFunction.<V>noOp()), Ordering.<V>natural());
+        return new BTreeSet<>(BTree.singleton(value), Ordering.<V>natural());
     }
 
     public static <V> BTreeSet<V> empty(Comparator<? super V> comparator)
@@ -639,4 +646,13 @@ public class BTreeSet<V> implements NavigableSet<V>, List<V>
     {
         return new BTreeSet<>(BTree.singleton(value), comparator);
     }
+
+    public static <V> BTreeSet<V> copy(SortedSet<? extends V> copy, Comparator<? super V> comparator)
+    {
+        try (BTree.FastBuilder<V> builder = BTree.fastBuilder())
+        {
+            copy.forEach(builder::add);
+            return wrap(builder.build(), comparator);
+        }
+    }
 }
