@@ -263,6 +263,15 @@ public final class WaitQueue
          * @throws InterruptedException
          */
         public boolean awaitUntil(long nanos) throws InterruptedException;
+
+        /**
+         * Wait until signalled, or the provided time is reached, or the thread is interrupted. If signalled,
+         * isSignalled() will be true on exit, and the method will return true; if timedout, the method will return
+         * false and isCancelled() will be true
+         * @param nanos System.nanoTime() to wait until
+         * @return true if signalled, false if timed out
+         */
+        public boolean awaitUntilUninterruptibly(long nanos);
     }
 
     /**
@@ -306,6 +315,17 @@ public final class WaitQueue
             return checkAndClear();
         }
 
+        public boolean awaitUntilUninterruptibly(long until)
+        {
+            long now;
+            while (until > (now = System.nanoTime()) && !isSignalled())
+            {
+                long delta = until - now;
+                LockSupport.parkNanos(delta);
+            }
+            return checkAndClear();
+        }
+
         private void checkInterrupted() throws InterruptedException
         {
             if (Thread.interrupted())
