@@ -22,6 +22,8 @@ import java.nio.ByteBuffer;
 
 import org.apache.cassandra.utils.concurrent.OpOrder;
 
+import com.google.common.annotations.VisibleForTesting;
+
 public class HeapPool extends MemtablePool
 {
     public HeapPool(long maxOnHeapMemory, float cleanupThreshold, MemtableCleaner cleaner)
@@ -29,19 +31,18 @@ public class HeapPool extends MemtablePool
         super(maxOnHeapMemory, 0, cleanupThreshold, cleaner);
     }
 
-    public boolean needToCopyOnHeap()
-    {
-        return false;
-    }
-
     public MemtableAllocator newAllocator()
     {
         return new Allocator(this);
     }
 
-    private static class Allocator extends MemtableBufferAllocator
+    @VisibleForTesting
+    public static class Allocator extends MemtableBufferAllocator
     {
-        Allocator(HeapPool pool)
+        private static final EnsureOnHeap ENSURE_NOOP = new EnsureOnHeap.NoOp();
+
+        @VisibleForTesting
+        public Allocator(HeapPool pool)
         {
             super(pool.onHeap.newAllocator(), pool.offHeap.newAllocator());
         }
@@ -51,5 +52,15 @@ public class HeapPool extends MemtablePool
             super.onHeap().allocate(size, opGroup);
             return ByteBuffer.allocate(size);
         }
+
+        public EnsureOnHeap ensureOnHeap()
+        {
+            return ENSURE_NOOP;
+        }
+
+        public Cloner cloner(OpOrder.Group opGroup)
+        {
+            return allocator(opGroup);
+        }
     }
 }
