@@ -27,13 +27,16 @@ import java.util.concurrent.*;
 import java.util.concurrent.atomic.AtomicLong;
 
 import com.google.common.util.concurrent.Uninterruptibles;
+import org.junit.AfterClass;
 import org.junit.BeforeClass;
 import org.junit.Test;
 
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
+import org.apache.cassandra.concurrent.NamedThreadFactory;
 import org.apache.cassandra.config.DatabaseDescriptor;
+import org.apache.cassandra.io.compress.BufferType;
 import org.apache.cassandra.utils.DynamicList;
 
 import static org.junit.Assert.*;
@@ -62,22 +65,98 @@ import static org.junit.Assert.*;
  */
 public class LongBufferPoolTest
 {
-    @BeforeClass
-    public static void setup() throws Exception
-    {
-        DatabaseDescriptor.daemonInitialization();
-    }
-
     private static final Logger logger = LoggerFactory.getLogger(LongBufferPoolTest.class);
 
     private static final int AVG_BUFFER_SIZE = 16 << 10;
     private static final int STDEV_BUFFER_SIZE = 10 << 10; // picked to ensure exceeding buffer size is rare, but occurs
     private static final DateFormat DATE_FORMAT = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss");
 
-    @Test
-    public void testAllocate() throws InterruptedException, ExecutionException, BrokenBarrierException, TimeoutException
+    static final class Debug implements BufferPool.Debug
     {
-        testAllocate(Runtime.getRuntime().availableProcessors() * 2, TimeUnit.MINUTES.toNanos(2L), 16 << 20);
+        static class DebugChunk
+        {
+            volatile long lastAcquired = 0; // the number of last round when the chunk was acquired from the global pool
+            volatile long lastRecycled = 0; // the number of last round when the chunk was recycled back to the global pool
+            static DebugChunk get(BufferPool.Chunk chunk)
+            {
+                if (chunk.debugAttachment == null)
+                    chunk.debugAttachment = new DebugChunk();
+                return (DebugChunk) chunk.debugAttachment;
+            }
+        }
+        AtomicLong recycleRound = new AtomicLong(1);
+        final List<BufferPool.Chunk> normalChunks = new ArrayList<>();
+
+        public synchronized void registerNormal(BufferPool.Chunk chunk)
+        {
+            chunk.debugAttachment = new DebugChunk();
+            normalChunks.add(chunk);
+        }
+
+        @Override
+        public void acquire(BufferPool.Chunk chunk)
+        {
+            DebugChunk.get(chunk).lastAcquired = recycleRound.get();
+        }
+
+        @Override
+        public void recycleNormal(BufferPool.Chunk oldVersion, BufferPool.Chunk newVersion)
+        {
+            newVersion.debugAttachment = oldVersion.debugAttachment;
+            DebugChunk.get(oldVersion).lastRecycled = recycleRound.get();
+        }
+
+        @Override
+        public void recyclePartial(BufferPool.Chunk chunk)
+        {
+            DebugChunk.get(chunk).lastRecycled = recycleRound.get();
+        }
+
+        public synchronized void check()
+        {
+            long lastRecycledMax = 0;
+            long currentRound = recycleRound.get();
+            for (BufferPool.Chunk chunk : normalChunks)
+            {
+                DebugChunk dc = DebugChunk.get(chunk);
+                assert dc.lastRecycled >= dc.lastAcquired: "Last recycled " + dc.lastRecycled + " < last acquired " + dc.lastAcquired;
+                lastRecycledMax = Math.max(lastRecycledMax, dc.lastRecycled);
+            }
+            assert lastRecycledMax == currentRound : "No chunk recycled in round " + currentRound + ". " +
+                                                     "Last chunk recycled in round " + lastRecycledMax + '.';
+            recycleRound.incrementAndGet();
+        }
+    }
+
+    @BeforeClass
+    public static void setup() throws Exception
+    {
+        DatabaseDescriptor.daemonInitialization();
+    }
+
+    @AfterClass
+    public static void teardown()
+    {
+        BufferPools.forChunkCache().unsafeReset();
+        BufferPools.forNetworking().unsafeReset();
+    }
+
+    @Test
+    public void testPoolAllocateWithRecyclePartially() throws InterruptedException, ExecutionException, BrokenBarrierException, TimeoutException
+    {
+        testPoolAllocate(true);
+    }
+
+    @Test
+    public void testPoolAllocateWithoutRecyclePartially() throws InterruptedException, ExecutionException, BrokenBarrierException, TimeoutException
+    {
+        testPoolAllocate(false);
+    }
+
+    private void testPoolAllocate(boolean recyclePartially) throws InterruptedException, ExecutionException, BrokenBarrierException, TimeoutException
+    {
+        BufferPool pool = new BufferPool("test_pool", 16 << 20, recyclePartially);
+        testAllocate(pool, Runtime.getRuntime().availableProcessors() * 2, TimeUnit.MINUTES.toNanos(2L));
     }
 
     private static final class BufferCheck
@@ -126,11 +205,11 @@ public class LongBufferPoolTest
         final List<Future<Boolean>> threadResultFuture;
         final int targetSizeQuanta;
 
-        TestEnvironment(int threadCount, long duration, int poolSize)
+        TestEnvironment(int threadCount, long duration, long poolSize)
         {
             this.threadCount = threadCount;
             this.duration = duration;
-            this.poolSize = poolSize;
+            this.poolSize = Math.toIntExact(poolSize);
             until = System.nanoTime() + duration;
             latch = new CountDownLatch(threadCount);
             sharedRecycle = new SPSCQueue[threadCount];
@@ -140,7 +219,7 @@ public class LongBufferPoolTest
             freedAllMemoryBarrier = new CyclicBarrier(threadCount + 2);
             resumeAllocationsBarrier = new CyclicBarrier(threadCount + 2);
 
-            executorService = Executors.newFixedThreadPool(threadCount + 2);
+            executorService = Executors.newFixedThreadPool(threadCount + 2, new NamedThreadFactory("test"));
             threadResultFuture = new ArrayList<>(threadCount);
 
             for (int i = 0; i < sharedRecycle.length; i++)
@@ -154,8 +233,8 @@ public class LongBufferPoolTest
             // using their own algorithm the targetSize should be poolSize / targetSizeQuanta.
             //
             // This should divide double the poolSize across the working threads,
-            // plus CHUNK_SIZE for thread0 and 1/10 poolSize for the burn producer/consumer pair.
-            targetSizeQuanta = 2 * poolSize / sum1toN(threadCount - 1);
+            // plus NORMAL_CHUNK_SIZE for thread0 and 1/10 poolSize for the burn producer/consumer pair.
+            targetSizeQuanta = 2 * this.poolSize / sum1toN(threadCount - 1);
         }
 
         void addCheckedFuture(Future<Boolean> future)
@@ -245,23 +324,19 @@ public class LongBufferPoolTest
         }
     }
 
-    public void testAllocate(int threadCount, long duration, int poolSize) throws InterruptedException, ExecutionException, BrokenBarrierException, TimeoutException
+    public void testAllocate(BufferPool bufferPool, int threadCount, long duration) throws InterruptedException, ExecutionException, BrokenBarrierException, TimeoutException
     {
-        System.out.println(String.format("%s - testing %d threads for %dm",
-                                         DATE_FORMAT.format(new Date()),
-                                         threadCount,
-                                         TimeUnit.NANOSECONDS.toMinutes(duration)));
-        long prevPoolSize = BufferPool.MEMORY_USAGE_THRESHOLD;
-        logger.info("Overriding configured BufferPool.MEMORY_USAGE_THRESHOLD={} and enabling BufferPool.DEBUG", poolSize);
-        BufferPool.MEMORY_USAGE_THRESHOLD = poolSize;
-        BufferPool.DEBUG = true;
+        logger.info("{} - testing {} threads for {}m", DATE_FORMAT.format(new Date()), threadCount, TimeUnit.NANOSECONDS.toMinutes(duration));
+        logger.info("Testing BufferPool with memoryUsageThreshold={} and enabling BufferPool.DEBUG", bufferPool.memoryUsageThreshold());
+        Debug debug = new Debug();
+        bufferPool.debug(debug);
 
-        TestEnvironment testEnv = new TestEnvironment(threadCount, duration, poolSize);
+        TestEnvironment testEnv = new TestEnvironment(threadCount, duration, bufferPool.memoryUsageThreshold());
 
-        startBurnerThreads(testEnv);
+        startBurnerThreads(bufferPool, testEnv);
 
         for (int threadIdx = 0; threadIdx < threadCount; threadIdx++)
-            testEnv.addCheckedFuture(startWorkerThread(testEnv, threadIdx));
+            testEnv.addCheckedFuture(startWorkerThread(bufferPool, testEnv, threadIdx));
 
         while (!testEnv.latch.await(1L, TimeUnit.SECONDS))
         {
@@ -273,7 +348,9 @@ public class LongBufferPoolTest
                 // wait until all memory released
                 testEnv.freedAllMemoryBarrier.await(10, TimeUnit.SECONDS);
                 // now all buffers should be back in the pool, and no more allocations happening
-                BufferPool.assertAllRecycled();
+                assert bufferPool.usedSizeInBytes() == 0 : "Some buffers haven't been freed. Memory in use = "
+                                                           + bufferPool.usedSizeInBytes() + " (expected 0)";
+                debug.check();
                 // resume threads only after debug.cycleRound has been increased
                 testEnv.shouldFreeMemoryAndSuspend = false;
                 testEnv.resumeAllocationsBarrier.await(10, TimeUnit.SECONDS);
@@ -296,27 +373,25 @@ public class LongBufferPoolTest
             while ( null != (check = queue.poll()) )
             {
                 check.validate();
-                BufferPool.put(check.buffer);
+                bufferPool.put(check.buffer);
             }
         }
 
         assertEquals(0, testEnv.executorService.shutdownNow().size());
 
-        logger.info("Reverting BufferPool.MEMORY_USAGE_THRESHOLD={}", prevPoolSize);
-        BufferPool.MEMORY_USAGE_THRESHOLD = prevPoolSize;
-        BufferPool.DEBUG = false;
+        logger.info("Reverting BufferPool DEBUG config");
+        bufferPool.debug(BufferPool.Debug.NO_OP);
 
         testEnv.assertCheckedThreadsSucceeded();
 
-        System.out.println(String.format("%s - finished.",
-                                         DATE_FORMAT.format(new Date())));
+        logger.info("{} - finished.", DATE_FORMAT.format(new Date()));
     }
 
-    private Future<Boolean> startWorkerThread(TestEnvironment testEnv, final int threadIdx)
+    private Future<Boolean> startWorkerThread(BufferPool bufferPool, TestEnvironment testEnv, final int threadIdx)
     {
-        return testEnv.executorService.submit(new TestUntil(testEnv.until)
+        return testEnv.executorService.submit(new TestUntil(bufferPool, testEnv.until)
         {
-            final int targetSize = threadIdx == 0 ? BufferPool.CHUNK_SIZE : testEnv.targetSizeQuanta * threadIdx;
+            final int targetSize = threadIdx == 0 ? BufferPool.NORMAL_CHUNK_SIZE : testEnv.targetSizeQuanta * threadIdx;
             final SPSCQueue<BufferCheck> shareFrom = testEnv.sharedRecycle[threadIdx];
             final DynamicList<BufferCheck> checks = new DynamicList<>((int) Math.max(1, targetSize / (1 << 10)));
             final SPSCQueue<BufferCheck> shareTo = testEnv.sharedRecycle[(threadIdx + 1) % testEnv.threadCount];
@@ -367,8 +442,8 @@ public class LongBufferPoolTest
                     checks.remove(check.listnode);
                     check.validate();
 
-                    size = BufferPool.roundUpNormal(check.buffer.capacity());
-                    if (size > BufferPool.CHUNK_SIZE)
+                    size = BufferPool.roundUp(check.buffer.capacity());
+                    if (size > BufferPool.NORMAL_CHUNK_SIZE)
                         size = 0;
 
                     // either share to free, or free immediately
@@ -382,16 +457,16 @@ public class LongBufferPoolTest
                     else
                     {
                         check.validate();
-                        BufferPool.put(check.buffer);
+                        bufferPool.put(check.buffer);
                         totalSize -= size;
                     }
                 }
 
                 // allocate a new buffer
                 size = (int) Math.max(1, AVG_BUFFER_SIZE + (STDEV_BUFFER_SIZE * rand.nextGaussian()));
-                if (size <= BufferPool.CHUNK_SIZE)
+                if (size <= BufferPool.NORMAL_CHUNK_SIZE)
                 {
-                    totalSize += BufferPool.roundUpNormal(size);
+                    totalSize += BufferPool.roundUp(size);
                     allocate(size);
                 }
                 else if (rand.nextBoolean())
@@ -404,10 +479,10 @@ public class LongBufferPoolTest
                     while (totalSize < testEnv.poolSize)
                     {
                         size = (int) Math.max(1, AVG_BUFFER_SIZE + (STDEV_BUFFER_SIZE * rand.nextGaussian()));
-                        if (size <= BufferPool.CHUNK_SIZE)
+                        if (size <= BufferPool.NORMAL_CHUNK_SIZE)
                         {
                             allocate(size);
-                            totalSize += BufferPool.roundUpNormal(size);
+                            totalSize += BufferPool.roundUp(size);
                         }
                     }
                 }
@@ -429,17 +504,17 @@ public class LongBufferPoolTest
                     BufferCheck check = sample();
                     checks.remove(check.listnode);
                     check.validate();
-                    BufferPool.put(check.buffer);
+                    bufferPool.put(check.buffer);
                 }
 
                 BufferCheck check;
                 while ((check = shareFrom.poll()) != null)
                 {
                     check.validate();
-                    BufferPool.put(check.buffer);
+                    bufferPool.put(check.buffer);
                 }
 
-                BufferPool.releaseLocal();
+                bufferPool.releaseLocal();
             }
 
             void cleanup()
@@ -447,7 +522,7 @@ public class LongBufferPoolTest
                 while (checks.size() > 0)
                 {
                     BufferCheck check = checks.get(0);
-                    BufferPool.put(check.buffer);
+                    bufferPool.put(check.buffer);
                     checks.remove(check.listnode);
                 }
                 testEnv.latch.countDown();
@@ -459,13 +534,13 @@ public class LongBufferPoolTest
                 if (check == null)
                     return false;
                 check.validate();
-                BufferPool.put(check.buffer);
+                bufferPool.put(check.buffer);
                 return true;
             }
 
             BufferCheck allocate(int size)
             {
-                ByteBuffer buffer = BufferPool.get(size);
+                ByteBuffer buffer = bufferPool.get(size, BufferType.OFF_HEAP);
                 assertNotNull(buffer);
                 BufferCheck check = new BufferCheck(buffer, rand.nextLong());
                 assertEquals(size, buffer.capacity());
@@ -510,34 +585,35 @@ public class LongBufferPoolTest
         });
     }
 
-    private void startBurnerThreads(TestEnvironment testEnv)
+    private void startBurnerThreads(BufferPool bufferPool, TestEnvironment testEnv)
     {
         // setup some high churn allocate/deallocate, without any checking
 
         final AtomicLong pendingBuffersCount = new AtomicLong(0);
         final SPSCQueue<ByteBuffer> burn = new SPSCQueue<>();
         final CountDownLatch doneAdd = new CountDownLatch(1);
-        testEnv.addCheckedFuture(testEnv.executorService.submit(new TestUntil(testEnv.until)
+        testEnv.addCheckedFuture(testEnv.executorService.submit(new TestUntil(bufferPool, testEnv.until)
         {
             int count = 0;
             final ThreadLocalRandom rand = ThreadLocalRandom.current();
             void testOne() throws Exception
             {
-                if (count * BufferPool.CHUNK_SIZE >= testEnv.poolSize / 10)
+                if (count * BufferPool.NORMAL_CHUNK_SIZE >= testEnv.poolSize / 10)
                 {
                     if (pendingBuffersCount.get() == 0)
                     {
                         count = 0;
-                        testEnv.maybeSuspendAndFreeMemory(BufferPool::releaseLocal);
-                    }
-                    else
+                        testEnv.maybeSuspendAndFreeMemory(bufferPool::releaseLocal);
+                    } else
                     {
                         Thread.yield();
                     }
                     return;
                 }
 
-                ByteBuffer buffer = BufferPool.tryGet(BufferPool.CHUNK_SIZE);
+                ByteBuffer buffer = rand.nextInt(4) < 1
+                        ? bufferPool.tryGet(BufferPool.NORMAL_CHUNK_SIZE)
+                        : bufferPool.tryGet(BufferPool.TINY_ALLOCATION_LIMIT);
                 if (buffer == null)
                 {
                     Thread.yield();
@@ -547,7 +623,7 @@ public class LongBufferPoolTest
                 // 50/50 chance of returning the buffer from the producer thread, or
                 // pass it on to the consumer.
                 if (rand.nextBoolean())
-                    BufferPool.put(buffer);
+                    bufferPool.put(buffer);
                 else
                 {
                     pendingBuffersCount.incrementAndGet();
@@ -560,7 +636,7 @@ public class LongBufferPoolTest
                 doneAdd.countDown();
             }
         }));
-        testEnv.threadResultFuture.add(testEnv.executorService.submit(new TestUntil(testEnv.until)
+        testEnv.threadResultFuture.add(testEnv.executorService.submit(new TestUntil(bufferPool, testEnv.until)
         {
             void testOne() throws Exception
             {
@@ -570,7 +646,7 @@ public class LongBufferPoolTest
                     Thread.yield();
                     return;
                 }
-                BufferPool.put(buffer);
+                bufferPool.put(buffer);
                 pendingBuffersCount.decrementAndGet();
             }
             void cleanup()
@@ -582,9 +658,11 @@ public class LongBufferPoolTest
 
     static abstract class TestUntil implements Callable<Boolean>
     {
+        final BufferPool bufferPool;
         final long until;
-        protected TestUntil(long until)
+        protected TestUntil(BufferPool bufferPool, long until)
         {
+            this.bufferPool = bufferPool;
             this.until = until;
         }
 
@@ -607,7 +685,7 @@ public class LongBufferPoolTest
             {
                 logger.error("Got exception {}, current chunk {}",
                              ex.getMessage(),
-                             BufferPool.currentChunk());
+                             bufferPool.unsafeCurrentChunk());
                 ex.printStackTrace();
                 return false;
             }
@@ -615,7 +693,7 @@ public class LongBufferPoolTest
             {
                 logger.error("Got throwable {}, current chunk {}",
                              tr.getMessage(),
-                             BufferPool.currentChunk());
+                             bufferPool.unsafeCurrentChunk());
                 tr.printStackTrace();
                 return false;
             }
@@ -631,13 +709,15 @@ public class LongBufferPoolTest
     {
         try
         {
-            new LongBufferPoolTest().testAllocate(Runtime.getRuntime().availableProcessors(),
-                                                  TimeUnit.HOURS.toNanos(2L), 16 << 20);
+            LongBufferPoolTest.setup();
+            new LongBufferPoolTest().testAllocate(new BufferPool("test_pool", 16 << 20, true),
+                                                  Runtime.getRuntime().availableProcessors(),
+                                                  TimeUnit.HOURS.toNanos(2L));
             System.exit(0);
         }
         catch (Throwable tr)
         {
-            System.out.println(String.format("Test failed - %s", tr.getMessage()));
+            logger.error("Test failed - {}", tr.getMessage(), tr);
             System.exit(1); // Force exit so that non-daemon threads like REQUEST-SCHEDULER do not hang the process on failure
         }
     }
