@@ -43,7 +43,7 @@ public class RowUtil
         if (res != null && res.kind == ResultMessage.Kind.ROWS)
         {
             ResultMessage.Rows rows = (ResultMessage.Rows) res;
-            String[] names = getColumnNames(rows.result.metadata.names);
+            String[] names = getColumnNames(rows.result.metadata.requestNames());
             Object[][] results = toObjects(rows);
             
             // Warnings may be null here, due to ClientWarn#getWarnings() handling of empty warning lists.
@@ -70,7 +70,7 @@ public class RowUtil
 
     public static Object[][] toObjects(ResultMessage.Rows rows)
     {
-        return toObjects(rows.result.metadata.names, rows.result.rows);
+        return toObjects(rows.result.metadata.requestNames(), rows.result.rows);
     }
 
     public static Object[][] toObjects(List<ColumnSpecification> specs, List<List<ByteBuffer>> rows)
@@ -79,8 +79,8 @@ public class RowUtil
         for (int i = 0; i < rows.size(); i++)
         {
             List<ByteBuffer> row = rows.get(i);
-            result[i] = new Object[row.size()];
-            for (int j = 0; j < row.size(); j++)
+            result[i] = new Object[specs.size()];
+            for (int j = 0; j < specs.size(); j++)
             {
                 ByteBuffer bb = row.get(j);
 
