@@ -219,14 +219,16 @@ public class JVMStabilityInspectorThrowableTest extends TestBaseImpl
             this.shouldThrowCorrupted = shouldThrowCorrupted;
         }
 
-        public UnfilteredRowIterator iterator(DecoratedKey key, Slices slices, ColumnFilter selectedColumns, boolean reversed, boolean isForThrift, SSTableReadsListener listener)
+        @Override
+        public UnfilteredRowIterator iterator(DecoratedKey key, Slices slices, ColumnFilter selectedColumns, boolean reversed, SSTableReadsListener listener)
         {
             if (shouldThrowCorrupted)
                 throw throwCorrupted();
             throw throwFSError();
         }
 
-        public UnfilteredRowIterator iterator(FileDataInput file, DecoratedKey key, RowIndexEntry indexEntry, Slices slices, ColumnFilter selectedColumns, boolean reversed, boolean isForThrift)
+        @Override
+        public UnfilteredRowIterator iterator(FileDataInput file, DecoratedKey key, RowIndexEntry indexEntry, Slices slices, ColumnFilter selectedColumns, boolean reversed)
         {
             if (shouldThrowCorrupted)
                 throw throwCorrupted();
@@ -243,4 +245,4 @@ public class JVMStabilityInspectorThrowableTest extends TestBaseImpl
             throw new FSReadError(new IOException("failed to get position"), descriptor.baseFilename());
         }
     }
-}
\ No newline at end of file
+}
