@@ -29,6 +29,7 @@ import java.util.concurrent.ExecutorService;
 import java.util.concurrent.Executors;
 import java.util.concurrent.Future;
 import java.util.concurrent.TimeUnit;
+import java.util.concurrent.TimeoutException;
 import java.util.concurrent.atomic.AtomicBoolean;
 import java.util.concurrent.atomic.AtomicInteger;
 
@@ -106,6 +107,7 @@ public class PreviewRepairTest extends TestBaseImpl
                 FBUtilities.waitOnFutures(CompactionManager.instance.submitBackground(cfs));
                 cfs.disableAutoCompaction();
             }));
+            long[] marks = logMark(cluster);
             cluster.get(1).callOnInstance(repair(options(false, false)));
             // now re-enable autocompaction on node1, this moves the sstables for the new repair to repaired
             cluster.get(1).runOnInstance(() -> {
@@ -114,8 +116,7 @@ public class PreviewRepairTest extends TestBaseImpl
                 FBUtilities.waitOnFutures(CompactionManager.instance.submitBackground(cfs));
             });
 
-            //IR and Preview repair can't run concurrently. In case the test is flaky, please check CASSANDRA-15685
-            Thread.sleep(1000);
+            waitLogsRepairFullyFinished(cluster, marks);
 
             RepairResult rs = cluster.get(1).callOnInstance(repair(options(true, false)));
             assertTrue(rs.success); // preview repair should succeed
@@ -123,6 +124,22 @@ public class PreviewRepairTest extends TestBaseImpl
         }
     }
 
+    public static void waitLogsRepairFullyFinished(Cluster cluster, long[] marks) throws TimeoutException
+    {
+        for (int i = 0; i < cluster.size(); i++)
+            cluster.get(i + 1).logs().watchFor(marks[i], "Finalized local repair session");
+    }
+
+    public static long[] logMark(Cluster cluster)
+    {
+        long [] marks = new long[cluster.size()];
+        for (int i = 0; i < cluster.size(); i++)
+        {
+            marks[i] = cluster.get(i + 1).logs().mark();
+        }
+        return marks;
+    }
+
     /**
      * another case where the repaired datasets could mismatch is if an incremental repair finishes just as the preview
      * repair is starting up.
