@@ -22,6 +22,8 @@ package org.apache.cassandra.concurrent;
 
 
 import java.util.UUID;
+import java.util.concurrent.Callable;
+import java.util.concurrent.ConcurrentLinkedQueue;
 import java.util.concurrent.CountDownLatch;
 import java.util.concurrent.LinkedBlockingQueue;
 import java.util.concurrent.RunnableFuture;
@@ -31,16 +33,22 @@ import java.util.concurrent.atomic.AtomicReference;
 import com.google.common.base.Throwables;
 import com.google.common.net.InetAddresses;
 import com.google.common.util.concurrent.ListenableFutureTask;
+import com.google.common.util.concurrent.Uninterruptibles;
 import org.junit.Assert;
 import org.junit.BeforeClass;
 import org.junit.Test;
 
 import org.apache.cassandra.config.DatabaseDescriptor;
 import org.apache.cassandra.locator.InetAddressAndPort;
+import org.apache.cassandra.service.ClientWarn;
 import org.apache.cassandra.tracing.TraceState;
 import org.apache.cassandra.tracing.TraceStateImpl;
 import org.apache.cassandra.tracing.Tracing;
+import org.apache.cassandra.utils.FBUtilities;
 import org.apache.cassandra.utils.WrappedRunnable;
+import org.assertj.core.api.Assertions;
+
+import static java.util.concurrent.TimeUnit.MILLISECONDS;
 
 public class DebuggableThreadPoolExecutorTest
 {
@@ -78,6 +86,74 @@ public class DebuggableThreadPoolExecutorTest
         assert delta >= 9 * 50 : delta;
     }
 
+    @Test
+    public void testLocalStatePropagation()
+    {
+        DebuggableThreadPoolExecutor executor = DebuggableThreadPoolExecutor.createWithFixedPoolSize("TEST", 1);
+        try
+        {
+            checkLocalStateIsPropagated(executor);
+        }
+        finally
+        {
+            executor.shutdown();
+        }
+    }
+
+    public static void checkLocalStateIsPropagated(LocalAwareExecutorService executor)
+    {
+        checkClientWarningsArePropagated(executor, () -> executor.execute(() -> ClientWarn.instance.warn("msg")));
+        checkClientWarningsArePropagated(executor, () -> executor.submit(() -> ClientWarn.instance.warn("msg")));
+        checkClientWarningsArePropagated(executor, () -> executor.submit(() -> ClientWarn.instance.warn("msg"), null));
+        checkClientWarningsArePropagated(executor, () -> executor.submit((Callable<Void>) () -> {
+            ClientWarn.instance.warn("msg");
+            return null;
+        }));
+
+        checkTracingIsPropagated(executor, () -> executor.execute(() -> Tracing.trace("msg")));
+        checkTracingIsPropagated(executor, () -> executor.submit(() -> Tracing.trace("msg")));
+        checkTracingIsPropagated(executor, () -> executor.submit(() -> Tracing.trace("msg"), null));
+        checkTracingIsPropagated(executor, () -> executor.submit((Callable<Void>) () -> {
+            Tracing.trace("msg");
+            return null;
+        }));
+    }
+
+    public static void checkClientWarningsArePropagated(LocalAwareExecutorService executor, Runnable schedulingTask) {
+        ClientWarn.instance.captureWarnings();
+        Assertions.assertThat(ClientWarn.instance.getWarnings()).isNullOrEmpty();
+
+        ClientWarn.instance.warn("msg0");
+        long initCompletedTasks = executor.getCompletedTaskCount();
+        schedulingTask.run();
+        while (executor.getCompletedTaskCount() == initCompletedTasks) Uninterruptibles.sleepUninterruptibly(10, MILLISECONDS);
+        ClientWarn.instance.warn("msg1");
+
+        Assertions.assertThat(ClientWarn.instance.getWarnings()).containsExactlyInAnyOrder("msg0", "msg", "msg1");
+    }
+
+    public static void checkTracingIsPropagated(LocalAwareExecutorService executor, Runnable schedulingTask) {
+        ClientWarn.instance.captureWarnings();
+        Assertions.assertThat(ClientWarn.instance.getWarnings()).isNullOrEmpty();
+
+        ConcurrentLinkedQueue<String> q = new ConcurrentLinkedQueue<>();
+        Tracing.instance.set(new TraceState(FBUtilities.getLocalAddressAndPort(), UUID.randomUUID(), Tracing.TraceType.NONE)
+        {
+            @Override
+            protected void traceImpl(String message)
+            {
+                q.add(message);
+            }
+        });
+        Tracing.trace("msg0");
+        long initCompletedTasks = executor.getCompletedTaskCount();
+        schedulingTask.run();
+        while (executor.getCompletedTaskCount() == initCompletedTasks) Uninterruptibles.sleepUninterruptibly(10, MILLISECONDS);
+        Tracing.trace("msg1");
+
+        Assertions.assertThat(q.toArray()).containsExactlyInAnyOrder("msg0", "msg", "msg1");
+    }
+
     @Test
     public void testExecuteFutureTaskWhileTracing()
     {
