@@ -19,7 +19,7 @@ package org.apache.cassandra.cql3.validation.entities;
 
 import org.junit.Test;
 
-import junit.framework.Assert;
+import org.junit.Assert;
 import org.apache.cassandra.cql3.CQLTester;
 
 import static junit.framework.Assert.assertNull;
@@ -181,4 +181,42 @@ public class TimestampTest extends CQLTester
                    row(2, 2, null, null),
                    row(3, 3, 3, 1L));
     }
+
+    @Test
+    public void testTimestampAndTTLPrepared() throws Throwable
+    {
+
+        createTable("CREATE TABLE %s (k int , c int, i int, PRIMARY KEY (k, c))");
+        execute("INSERT INTO %s (k, c, i) VALUES (1, 1, 1) USING TIMESTAMP ? AND TTL ?;", 1L,5);
+        execute("INSERT INTO %s (k, c) VALUES (1, 2) USING TIMESTAMP ? AND TTL ? ;", 1L, 5);
+        execute("INSERT INTO %s (k, c, i) VALUES (1, 3, 1) USING TIMESTAMP ? AND TTL ?;", 1L, 5);
+        execute("INSERT INTO %s (k, c) VALUES (2, 2) USING TIMESTAMP ? AND TTL ?;", 2L, 5);
+        execute("INSERT INTO %s (k, c, i) VALUES (3, 3, 3) USING TIMESTAMP ? AND TTL ?;", 1L, 5);
+        assertRows(execute("SELECT k, c, i, writetime(i) FROM %s "),
+                row(1, 1, 1, 1L),
+                row(1, 2, null, null),
+                row(1, 3, 1, 1L),
+                row(2, 2, null, null),
+                row(3, 3, 3, 1L));
+        Thread.sleep(6*1000);
+        assertEmpty(execute("SELECT k, c, i, writetime(i) FROM %s "));
+    }
+
+    @Test
+    public void testTimestampAndTTLUpdatePrepared() throws Throwable
+    {
+
+        createTable("CREATE TABLE %s (k int , c int, i int, PRIMARY KEY (k, c))");
+        execute("UPDATE %s USING TIMESTAMP ? AND TTL ? SET i=1 WHERE k=1 AND c = 1 ;", 1L, 5);
+        execute("UPDATE %s USING TIMESTAMP ? AND TTL ? SET i=1 WHERE k=1 AND c = 3 ;", 1L, 5);
+        execute("UPDATE %s USING TIMESTAMP ? AND TTL ? SET i=1 WHERE k=2 AND c = 2 ;", 2L, 5);
+        execute("UPDATE %s USING TIMESTAMP ? AND TTL ? SET i=3 WHERE k=3 AND c = 3 ;", 1L, 5);
+        assertRows(execute("SELECT k, c, i, writetime(i) FROM %s "),
+                row(1, 1, 1, 1L),
+                row(1, 3, 1, 1L),
+                row(2, 2, 1, 2L),
+                row(3, 3, 3, 1L));
+        Thread.sleep(6*1000);
+        assertEmpty(execute("SELECT k, c, i, writetime(i) FROM %s "));
+    }
 }
