@@ -21,7 +21,7 @@ package org.apache.cassandra.cql3.validation.entities.udfverify;
 import java.nio.ByteBuffer;
 import java.util.List;
 
-import com.datastax.driver.core.TypeCodec;
+import org.apache.cassandra.cql3.functions.types.TypeCodec;
 import org.apache.cassandra.cql3.functions.JavaUDF;
 import org.apache.cassandra.cql3.functions.UDFContext;
 import org.apache.cassandra.transport.ProtocolVersion;
