@@ -24,10 +24,14 @@ import com.google.common.base.Throwables;
 import org.junit.AfterClass;
 import org.junit.BeforeClass;
 import org.junit.Test;
+
 import org.apache.cassandra.config.DatabaseDescriptor;
 import org.apache.cassandra.cql3.CQLTester;
+import org.apache.cassandra.db.ColumnFamilyStore;
+import org.apache.cassandra.db.Keyspace;
 import org.apache.cassandra.db.filter.TombstoneOverwhelmingException;
 
+import static junit.framework.Assert.assertEquals;
 import static junit.framework.Assert.assertTrue;
 import static junit.framework.Assert.fail;
 
@@ -36,33 +40,45 @@ import static junit.framework.Assert.fail;
  */
 public class TombstonesTest extends CQLTester
 {
-    static final int ORIGINAL_THRESHOLD = DatabaseDescriptor.getTombstoneFailureThreshold();
-    static final int THRESHOLD = 100;
+    static final int ORIGINAL_FAILURE_THRESHOLD = DatabaseDescriptor.getTombstoneFailureThreshold();
+    static final int FAILURE_THRESHOLD = 100;
+
+    static final int ORIGINAL_WARN_THRESHOLD = DatabaseDescriptor.getTombstoneFailureThreshold();
+    static final int WARN_THRESHOLD = 50;
 
     @BeforeClass
     public static void setUp() throws Throwable
     {
-        DatabaseDescriptor.setTombstoneFailureThreshold(THRESHOLD);
+        DatabaseDescriptor.daemonInitialization();
+        DatabaseDescriptor.setTombstoneFailureThreshold(FAILURE_THRESHOLD);
+        DatabaseDescriptor.setTombstoneWarnThreshold(WARN_THRESHOLD);
     }
 
     @AfterClass
     public static void tearDown()
     {
-        DatabaseDescriptor.setTombstoneFailureThreshold(ORIGINAL_THRESHOLD);
+        DatabaseDescriptor.setTombstoneFailureThreshold(ORIGINAL_FAILURE_THRESHOLD);
+        DatabaseDescriptor.setTombstoneWarnThreshold(ORIGINAL_WARN_THRESHOLD);
     }
 
     @Test
     public void testBelowThresholdSelect() throws Throwable
     {
-        createTable("CREATE TABLE %s (a text, b text, c text, PRIMARY KEY (a, b));");
+
+        String tableName = createTable("CREATE TABLE %s (a text, b text, c text, PRIMARY KEY (a, b));");
+        ColumnFamilyStore cfs = Keyspace.open(KEYSPACE).getColumnFamilyStore(tableName);
+        long oldFailures = cfs.metric.tombstoneFailures.getCount();
+        long oldWarnings = cfs.metric.tombstoneWarnings.getCount();
 
         // insert exactly the amount of tombstones that shouldn't trigger an exception
-        for (int i = 0; i < THRESHOLD; i++)
+        for (int i = 0; i < FAILURE_THRESHOLD; i++)
             execute("INSERT INTO %s (a, b, c) VALUES ('key', 'column" + i + "', null);");
 
         try
         {
             execute("SELECT * FROM %s WHERE a = 'key';");
+            assertEquals(oldFailures, cfs.metric.tombstoneFailures.getCount());
+            assertEquals(oldWarnings, cfs.metric.tombstoneWarnings.getCount());
         }
         catch (Throwable e)
         {
@@ -73,10 +89,13 @@ public class TombstonesTest extends CQLTester
     @Test
     public void testBeyondThresholdSelect() throws Throwable
     {
-        createTable("CREATE TABLE %s (a text, b text, c text, PRIMARY KEY (a, b));");
+        String tableName = createTable("CREATE TABLE %s (a text, b text, c text, PRIMARY KEY (a, b));");
+        ColumnFamilyStore cfs = Keyspace.open(KEYSPACE).getColumnFamilyStore(tableName);
+        long oldFailures = cfs.metric.tombstoneFailures.getCount();
+        long oldWarnings = cfs.metric.tombstoneWarnings.getCount();
 
         // insert exactly the amount of tombstones that *SHOULD* trigger an exception
-        for (int i = 0; i < THRESHOLD + 1; i++)
+        for (int i = 0; i < FAILURE_THRESHOLD + 1; i++)
             execute("INSERT INTO %s (a, b, c) VALUES ('key', 'column" + i + "', null);");
 
         try
@@ -87,19 +106,24 @@ public class TombstonesTest extends CQLTester
         catch (Throwable e)
         {
             String error = "Expected exception instanceof TombstoneOverwhelmingException instead got "
-                          + System.lineSeparator()
-                          + Throwables.getStackTraceAsString(e);
+                           + System.lineSeparator()
+                           + Throwables.getStackTraceAsString(e);
             assertTrue(error, e instanceof TombstoneOverwhelmingException);
+            assertEquals(oldWarnings, cfs.metric.tombstoneWarnings.getCount());
+            assertEquals(oldFailures + 1, cfs.metric.tombstoneFailures.getCount());
         }
     }
 
     @Test
     public void testAllShadowedSelect() throws Throwable
     {
-        createTable("CREATE TABLE %s (a text, b text, c text, PRIMARY KEY (a, b));");
+        String tableName = createTable("CREATE TABLE %s (a text, b text, c text, PRIMARY KEY (a, b));");
+        ColumnFamilyStore cfs = Keyspace.open(KEYSPACE).getColumnFamilyStore(tableName);
+        long oldFailures = cfs.metric.tombstoneFailures.getCount();
+        long oldWarnings = cfs.metric.tombstoneWarnings.getCount();
 
         // insert exactly the amount of tombstones that *SHOULD* normally trigger an exception
-        for (int i = 0; i < THRESHOLD + 1; i++)
+        for (int i = 0; i < FAILURE_THRESHOLD + 1; i++)
             execute("INSERT INTO %s (a, b, c) VALUES ('key', 'column" + i + "', null);");
 
         // delete all with a partition level tombstone
@@ -108,6 +132,8 @@ public class TombstonesTest extends CQLTester
         try
         {
             execute("SELECT * FROM %s WHERE a = 'key';");
+            assertEquals(oldFailures, cfs.metric.tombstoneFailures.getCount());
+            assertEquals(oldWarnings, cfs.metric.tombstoneWarnings.getCount());
         }
         catch (Throwable e)
         {
@@ -118,9 +144,12 @@ public class TombstonesTest extends CQLTester
     @Test
     public void testLiveShadowedCellsSelect() throws Throwable
     {
-        createTable("CREATE TABLE %s (a text, b text, c text, PRIMARY KEY (a, b));");
+        String tableName = createTable("CREATE TABLE %s (a text, b text, c text, PRIMARY KEY (a, b));");
+        ColumnFamilyStore cfs = Keyspace.open(KEYSPACE).getColumnFamilyStore(tableName);
+        long oldFailures = cfs.metric.tombstoneFailures.getCount();
+        long oldWarnings = cfs.metric.tombstoneWarnings.getCount();
 
-        for (int i = 0; i < THRESHOLD + 1; i++)
+        for (int i = 0; i < FAILURE_THRESHOLD + 1; i++)
             execute("INSERT INTO %s (a, b, c) VALUES ('key', 'column" + i + "', 'column');");
 
         // delete all with a partition level tombstone
@@ -129,6 +158,8 @@ public class TombstonesTest extends CQLTester
         try
         {
             execute("SELECT * FROM %s WHERE a = 'key';");
+            assertEquals(oldFailures, cfs.metric.tombstoneFailures.getCount());
+            assertEquals(oldWarnings, cfs.metric.tombstoneWarnings.getCount());
         }
         catch (Throwable e)
         {
@@ -139,9 +170,12 @@ public class TombstonesTest extends CQLTester
     @Test
     public void testExpiredTombstones() throws Throwable
     {
-        createTable("CREATE TABLE %s (a text, b text, c text, PRIMARY KEY (a, b)) WITH gc_grace_seconds = 1;");
+        String tableName = createTable("CREATE TABLE %s (a text, b text, c text, PRIMARY KEY (a, b)) WITH gc_grace_seconds = 1;");
+        ColumnFamilyStore cfs = Keyspace.open(KEYSPACE).getColumnFamilyStore(tableName);
+        long oldFailures = cfs.metric.tombstoneFailures.getCount();
+        long oldWarnings = cfs.metric.tombstoneWarnings.getCount();
 
-        for (int i = 0; i < THRESHOLD + 1; i++)
+        for (int i = 0; i < FAILURE_THRESHOLD + 1; i++)
             execute("INSERT INTO %s (a, b, c) VALUES ('key', 'column" + i + "', null);");
 
         // not yet past gc grace - must throw a TOE
@@ -153,6 +187,9 @@ public class TombstonesTest extends CQLTester
         catch (Throwable e)
         {
             assertTrue(e instanceof TombstoneOverwhelmingException);
+
+            assertEquals(++oldFailures, cfs.metric.tombstoneFailures.getCount());
+            assertEquals(oldWarnings, cfs.metric.tombstoneWarnings.getCount());
         }
 
         // sleep past gc grace
@@ -162,10 +199,37 @@ public class TombstonesTest extends CQLTester
         try
         {
             execute("SELECT * FROM %s WHERE a = 'key';");
+            assertEquals(oldFailures, cfs.metric.tombstoneFailures.getCount());
+            assertEquals(oldWarnings, cfs.metric.tombstoneWarnings.getCount());
         }
         catch (Throwable e)
         {
             fail("SELECT with expired tombstones beyond the threshold should not have failed, but has: " + e);
         }
     }
+
+    @Test
+    public void testBeyondWarnThresholdSelect() throws Throwable
+    {
+        String tableName = createTable("CREATE TABLE %s (a text, b text, c text, PRIMARY KEY (a,b));");
+        ColumnFamilyStore cfs = Keyspace.open(KEYSPACE).getColumnFamilyStore(tableName);
+        long oldFailures = cfs.metric.tombstoneFailures.getCount();
+        long oldWarnings = cfs.metric.tombstoneWarnings.getCount();
+
+        // insert the number of tombstones that *SHOULD* trigger an Warning
+        for (int i = 0; i < WARN_THRESHOLD + 1; i++)
+            execute("INSERT INTO %s (a, b, c ) VALUES ('key', 'cc" + i + "',  null);");
+        try
+        {
+            execute("SELECT * FROM %s WHERE a = 'key';");
+            assertEquals(oldWarnings + 1, cfs.metric.tombstoneWarnings.getCount());
+            assertEquals(oldFailures, cfs.metric.tombstoneFailures.getCount());
+        }
+        catch (Throwable e)
+        {
+            fail("SELECT with tombstones below the failure threshold and above warning threashhold should not have failed, but has: " + e);
+        }
+    }
+
+
 }
