@@ -29,72 +29,66 @@ public class SelectOrderByTest extends CQLTester
     @Test
     public void testNormalSelectionOrderSingleClustering() throws Throwable
     {
-        for (String descOption : new String[]{"", " WITH CLUSTERING ORDER BY (b DESC)"})
-        {
-            createTable("CREATE TABLE %s (a int, b int, c int, PRIMARY KEY (a, b))" + descOption);
-            execute("INSERT INTO %s (a, b, c) VALUES (?, ?, ?)", 0, 0, 0);
-            execute("INSERT INTO %s (a, b, c) VALUES (?, ?, ?)", 0, 1, 1);
-            execute("INSERT INTO %s (a, b, c) VALUES (?, ?, ?)", 0, 2, 2);
+        createTable("CREATE TABLE %s (a int, b int, c int, PRIMARY KEY (a, b))");
+        execute("INSERT INTO %s (a, b, c) VALUES (?, ?, ?)", 0, 0, 0);
+        execute("INSERT INTO %s (a, b, c) VALUES (?, ?, ?)", 0, 1, 1);
+        execute("INSERT INTO %s (a, b, c) VALUES (?, ?, ?)", 0, 2, 2);
 
-            beforeAndAfterFlush(() -> {
-                assertRows(execute("SELECT * FROM %s WHERE a=? ORDER BY b ASC", 0),
-                           row(0, 0, 0),
-                           row(0, 1, 1),
-                           row(0, 2, 2)
-                        );
+        beforeAndAfterFlush(() -> {
+            assertRows(execute("SELECT * FROM %s WHERE a=? ORDER BY b ASC", 0),
+                       row(0, 0, 0),
+                       row(0, 1, 1),
+                       row(0, 2, 2)
+            );
 
-                assertRows(execute("SELECT * FROM %s WHERE a=? ORDER BY b DESC", 0),
-                           row(0, 2, 2),
-                           row(0, 1, 1),
-                           row(0, 0, 0)
-                        );
+            assertRows(execute("SELECT * FROM %s WHERE a=? ORDER BY b DESC", 0),
+                       row(0, 2, 2),
+                       row(0, 1, 1),
+                       row(0, 0, 0)
+            );
 
-                // order by the only column in the selection
-                assertRows(execute("SELECT b FROM %s WHERE a=? ORDER BY b ASC", 0),
-                           row(0), row(1), row(2));
+            // order by the only column in the selection
+            assertRows(execute("SELECT b FROM %s WHERE a=? ORDER BY b ASC", 0),
+                       row(0), row(1), row(2));
 
-                assertRows(execute("SELECT b FROM %s WHERE a=? ORDER BY b DESC", 0),
-                           row(2), row(1), row(0));
+            assertRows(execute("SELECT b FROM %s WHERE a=? ORDER BY b DESC", 0),
+                       row(2), row(1), row(0));
 
-                // order by a column not in the selection
-                assertRows(execute("SELECT c FROM %s WHERE a=? ORDER BY b ASC", 0),
-                           row(0), row(1), row(2));
+            // order by a column not in the selection
+            assertRows(execute("SELECT c FROM %s WHERE a=? ORDER BY b ASC", 0),
+                       row(0), row(1), row(2));
 
-                assertRows(execute("SELECT c FROM %s WHERE a=? ORDER BY b DESC", 0),
-                           row(2), row(1), row(0));
-            });
-        }
+            assertRows(execute("SELECT c FROM %s WHERE a=? ORDER BY b DESC", 0),
+                       row(2), row(1), row(0));
+        });
     }
 
     @Test
     public void testFunctionSelectionOrderSingleClustering() throws Throwable
     {
-        for (String descOption : new String[]{"", " WITH CLUSTERING ORDER BY (b DESC)"})
-        {
-            createTable("CREATE TABLE %s (a int, b int, c int, PRIMARY KEY (a, b))" + descOption);
-            execute("INSERT INTO %s (a, b, c) VALUES (?, ?, ?)", 0, 0, 0);
-            execute("INSERT INTO %s (a, b, c) VALUES (?, ?, ?)", 0, 1, 1);
-            execute("INSERT INTO %s (a, b, c) VALUES (?, ?, ?)", 0, 2, 2);
+        createTable("CREATE TABLE %s (a int, b int, c int, PRIMARY KEY (a, b))");
+        execute("INSERT INTO %s (a, b, c) VALUES (?, ?, ?)", 0, 0, 0);
+        execute("INSERT INTO %s (a, b, c) VALUES (?, ?, ?)", 0, 1, 1);
+        execute("INSERT INTO %s (a, b, c) VALUES (?, ?, ?)", 0, 2, 2);
 
-            beforeAndAfterFlush(() -> {
-                // order by the only column in the selection
-                assertRows(execute("SELECT blobAsInt(intAsBlob(b)) FROM %s WHERE a=? ORDER BY b ASC", 0),
-                           row(0), row(1), row(2));
+        beforeAndAfterFlush(() -> {
+            // order by the only column in the selection
+            assertRows(execute("SELECT blobAsInt(intAsBlob(b)) FROM %s WHERE a=? ORDER BY b ASC", 0),
+                       row(0), row(1), row(2));
 
-                assertRows(execute("SELECT blobAsInt(intAsBlob(b)) FROM %s WHERE a=? ORDER BY b DESC", 0),
-                           row(2), row(1), row(0));
+            assertRows(execute("SELECT blobAsInt(intAsBlob(b)) FROM %s WHERE a=? ORDER BY b DESC", 0),
+                       row(2), row(1), row(0));
 
-                // order by a column not in the selection
-                assertRows(execute("SELECT blobAsInt(intAsBlob(c)) FROM %s WHERE a=? ORDER BY b ASC", 0),
-                           row(0), row(1), row(2));
+            // order by a column not in the selection
+            assertRows(execute("SELECT blobAsInt(intAsBlob(c)) FROM %s WHERE a=? ORDER BY b ASC", 0),
+                       row(0), row(1), row(2));
 
-                assertRows(execute("SELECT blobAsInt(intAsBlob(c)) FROM %s WHERE a=? ORDER BY b DESC", 0),
-                           row(2), row(1), row(0));
+            assertRows(execute("SELECT blobAsInt(intAsBlob(c)) FROM %s WHERE a=? ORDER BY b DESC", 0),
+                       row(2), row(1), row(0));
 
-                assertInvalid("SELECT * FROM %s WHERE a=? ORDER BY c ASC", 0);
-                assertInvalid("SELECT * FROM %s WHERE a=? ORDER BY c DESC", 0);
-            });
-        }
+            assertInvalid("SELECT * FROM %s WHERE a=? ORDER BY c ASC", 0);
+            assertInvalid("SELECT * FROM %s WHERE a=? ORDER BY c DESC", 0);
+        });
     }
 
     @Test
@@ -102,26 +96,23 @@ public class SelectOrderByTest extends CQLTester
     {
         String type = createType("CREATE TYPE %s (a int)");
 
-        for (String descOption : new String[]{"", " WITH CLUSTERING ORDER BY (b DESC)"})
-        {
-            createTable("CREATE TABLE %s (a int, b int, c frozen<" + type + "   >, PRIMARY KEY (a, b))" + descOption);
-            execute("INSERT INTO %s (a, b, c) VALUES (?, ?, {a: ?})", 0, 0, 0);
-            execute("INSERT INTO %s (a, b, c) VALUES (?, ?, {a: ?})", 0, 1, 1);
-            execute("INSERT INTO %s (a, b, c) VALUES (?, ?, {a: ?})", 0, 2, 2);
+        createTable("CREATE TABLE %s (a int, b int, c frozen<" + type + "   >, PRIMARY KEY (a, b))");
+        execute("INSERT INTO %s (a, b, c) VALUES (?, ?, {a: ?})", 0, 0, 0);
+        execute("INSERT INTO %s (a, b, c) VALUES (?, ?, {a: ?})", 0, 1, 1);
+        execute("INSERT INTO %s (a, b, c) VALUES (?, ?, {a: ?})", 0, 2, 2);
 
-            beforeAndAfterFlush(() -> {
-                // order by a column not in the selection
-                assertRows(execute("SELECT c.a FROM %s WHERE a=? ORDER BY b ASC", 0),
-                           row(0), row(1), row(2));
+        beforeAndAfterFlush(() -> {
+            // order by a column not in the selection
+            assertRows(execute("SELECT c.a FROM %s WHERE a=? ORDER BY b ASC", 0),
+                       row(0), row(1), row(2));
 
-                assertRows(execute("SELECT c.a FROM %s WHERE a=? ORDER BY b DESC", 0),
-                           row(2), row(1), row(0));
+            assertRows(execute("SELECT c.a FROM %s WHERE a=? ORDER BY b DESC", 0),
+                       row(2), row(1), row(0));
 
-                assertRows(execute("SELECT blobAsInt(intAsBlob(c.a)) FROM %s WHERE a=? ORDER BY b DESC", 0),
-                           row(2), row(1), row(0));
-            });
-            dropTable("DROP TABLE %s");
-        }
+            assertRows(execute("SELECT blobAsInt(intAsBlob(c.a)) FROM %s WHERE a=? ORDER BY b DESC", 0),
+                       row(2), row(1), row(0));
+        });
+        dropTable("DROP TABLE %s");
     }
 
     @Test
@@ -244,77 +235,6 @@ public class SelectOrderByTest extends CQLTester
         });
     }
 
-    /**
-     * Check ORDER BY support in SELECT statement
-     * migrated from cql_tests.py:TestCQL.order_by_test()
-     */
-    @Test
-    public void testSimpleOrderBy() throws Throwable
-    {
-        createTable("CREATE TABLE %s (k int, c int, v int, PRIMARY KEY (k, c)) WITH COMPACT STORAGE");
-
-        for (int i = 0; i < 10; i++)
-            execute("INSERT INTO %s (k, c, v) VALUES (0, ?, ?)", i, i);
-
-        beforeAndAfterFlush(() -> {
-            assertRows(execute("SELECT v FROM %s WHERE k = 0 ORDER BY c DESC"),
-                       row(9), row(8), row(7), row(6), row(5), row(4), row(3), row(2), row(1), row(0));
-        });
-
-        createTable("CREATE TABLE %s (k int, c1 int, c2 int, v int, PRIMARY KEY (k, c1, c2)) WITH COMPACT STORAGE");
-
-        for (int i = 0; i < 4; i++)
-            for (int j = 0; j < 2; j++)
-                execute("INSERT INTO %s (k, c1, c2, v) VALUES (0, ?, ?, ?)", i, j, i * 2 + j);
-
-        beforeAndAfterFlush(() -> {
-            assertInvalid("SELECT v FROM %s WHERE k = 0 ORDER BY c DESC");
-            assertInvalid("SELECT v FROM %s WHERE k = 0 ORDER BY c2 DESC");
-            assertInvalid("SELECT v FROM %s WHERE k = 0 ORDER BY k DESC");
-
-            assertRows(execute("SELECT v FROM %s WHERE k = 0 ORDER BY c1 DESC"),
-                       row(7), row(6), row(5), row(4), row(3), row(2), row(1), row(0));
-
-            assertRows(execute("SELECT v FROM %s WHERE k = 0 ORDER BY c1"),
-                       row(0), row(1), row(2), row(3), row(4), row(5), row(6), row(7));
-        });
-    }
-
-    /**
-     * More ORDER BY checks (#4160)
-     * migrated from cql_tests.py:TestCQL.more_order_by_test()
-     */
-    @Test
-    public void testMoreOrderBy() throws Throwable
-    {
-        createTable("CREATE TABLE %s (row text, number int, string text, PRIMARY KEY(row, number)) WITH COMPACT STORAGE ");
-
-        execute("INSERT INTO %s (row, number, string) VALUES ('row', 1, 'one')");
-        execute("INSERT INTO %s (row, number, string) VALUES ('row', 2, 'two')");
-        execute("INSERT INTO %s (row, number, string) VALUES ('row', 3, 'three')");
-        execute("INSERT INTO %s (row, number, string) VALUES ('row', 4, 'four')");
-
-        beforeAndAfterFlush(() -> {
-            assertRows(execute("SELECT number FROM %s WHERE row='row' AND number < 3 ORDER BY number ASC"),
-                       row(1), row(2));
-
-            assertRows(execute("SELECT number FROM %s WHERE row='row' AND number >= 3 ORDER BY number ASC"),
-                       row(3), row(4));
-
-            assertRows(execute("SELECT number FROM %s WHERE row='row' AND number < 3 ORDER BY number DESC"),
-                       row(2), row(1));
-
-            assertRows(execute("SELECT number FROM %s WHERE row='row' AND number >= 3 ORDER BY number DESC"),
-                       row(4), row(3));
-
-            assertRows(execute("SELECT number FROM %s WHERE row='row' AND number > 3 ORDER BY number DESC"),
-                       row(4));
-
-            assertRows(execute("SELECT number FROM %s WHERE row='row' AND number <= 3 ORDER BY number DESC"),
-                       row(3), row(2), row(1));
-        });
-    }
-
     /**
      * Check we don't allow order by on row key (#4246)
      * migrated from cql_tests.py:TestCQL.order_by_validation_test()
@@ -454,6 +374,23 @@ public class SelectOrderByTest extends CQLTester
         });
     }
 
+    @Test
+    public void testOrderByForInClauseWithCollectionElementSelection() throws Throwable
+    {
+        createTable("CREATE TABLE %s (pk int, c frozen<set<int>>, v int, PRIMARY KEY (pk, c))");
+
+        execute("INSERT INTO %s (pk, c, v) VALUES (0, {1, 2}, 0)");
+        execute("INSERT INTO %s (pk, c, v) VALUES (0, {1, 2, 3}, 1)");
+        execute("INSERT INTO %s (pk, c, v) VALUES (1, {2, 3}, 2)");
+
+        beforeAndAfterFlush(() -> {
+            assertRows(execute("SELECT c[2], v FROM %s WHERE pk = 0 ORDER BY c"),
+                       row(2, 0), row(2, 1));
+            assertRows(execute("SELECT c[2], v FROM %s WHERE pk IN (0, 1) ORDER BY c"),
+                       row(2, 0), row(2, 1), row(2, 2));
+        });
+    }
+
     @Test
     public void testOrderByForInClauseWithNullValue() throws Throwable
     {
@@ -702,6 +639,132 @@ public class SelectOrderByTest extends CQLTester
         }
     }
 
+    /**
+     * Test that ORDER BY columns allow skipping equality-restricted clustering columns, see CASSANDRA-10271.
+     */
+    @Test
+    public void testAllowSkippingEqualityAndSingleValueInRestrictedClusteringColumns() throws Throwable
+    {
+        createTable("CREATE TABLE %s (a int, b int, c int, d int, PRIMARY KEY (a, b, c))");
+        execute("INSERT INTO %s (a, b, c, d) VALUES (?, ?, ?, ?)", 0, 0, 0, 0);
+        execute("INSERT INTO %s (a, b, c, d) VALUES (?, ?, ?, ?)", 0, 0, 1, 1);
+        execute("INSERT INTO %s (a, b, c, d) VALUES (?, ?, ?, ?)", 0, 0, 2, 2);
+        execute("INSERT INTO %s (a, b, c, d) VALUES (?, ?, ?, ?)", 0, 1, 0, 3);
+        execute("INSERT INTO %s (a, b, c, d) VALUES (?, ?, ?, ?)", 0, 1, 1, 4);
+        execute("INSERT INTO %s (a, b, c, d) VALUES (?, ?, ?, ?)", 0, 1, 2, 5);
+
+        assertInvalidMessage("Order by is currently only supported on the clustered columns of the PRIMARY KEY, got d",
+                             "SELECT * FROM %s WHERE a=? ORDER BY d DESC", 0);
+
+        assertInvalidMessage("Order by is currently only supported on the clustered columns of the PRIMARY KEY, got d",
+                             "SELECT * FROM %s WHERE a=? ORDER BY b ASC, c ASC, d ASC", 0);
+
+        String errorMsg = "Order by currently only supports the ordering of columns following their declared order in the PRIMARY KEY";
+
+        assertRows(execute("SELECT * FROM %s WHERE a=? AND b=? ORDER BY c", 0, 0),
+                   row(0, 0, 0, 0),
+                   row(0, 0, 1, 1),
+                   row(0, 0, 2, 2)
+        );
+
+        assertRows(execute("SELECT * FROM %s WHERE a=? AND b=? ORDER BY c ASC", 0, 0),
+                   row(0, 0, 0, 0),
+                   row(0, 0, 1, 1),
+                   row(0, 0, 2, 2)
+        );
+
+        assertRows(execute("SELECT * FROM %s WHERE a=? AND b=? ORDER BY c DESC", 0, 0),
+                   row(0, 0, 2, 2),
+                   row(0, 0, 1, 1),
+                   row(0, 0, 0, 0)
+        );
+
+        assertRows(execute("SELECT * FROM %s WHERE a=? AND b=? AND c>=? ORDER BY c ASC", 0, 0, 1),
+                   row(0, 0, 1, 1),
+                   row(0, 0, 2, 2));
+
+        assertRows(execute("SELECT * FROM %s WHERE a=? AND b=? AND c>=? ORDER BY c DESC", 0, 0, 1),
+                   row(0, 0, 2, 2),
+                   row(0, 0, 1, 1));
+
+        assertRows(execute("SELECT * FROM %s WHERE a=? AND b=? AND c IN (?, ?) ORDER BY c ASC", 0, 0, 1, 2),
+                   row(0, 0, 1, 1),
+                   row(0, 0, 2, 2));
+
+        assertRows(execute("SELECT * FROM %s WHERE a=? AND b=? AND c IN (?, ?) ORDER BY c DESC", 0, 0, 1, 2),
+                   row(0, 0, 2, 2),
+                   row(0, 0, 1, 1));
+
+        assertInvalidMessage(errorMsg, "SELECT * FROM %s WHERE a=? AND b<? ORDER BY c DESC", 0, 1);
+
+        assertInvalidMessage(errorMsg, "SELECT * FROM %s WHERE a=? AND (b, c) > (?, ?) ORDER BY c", 0, 0, 0);
+        assertInvalidMessage(errorMsg, "SELECT * FROM %s WHERE a=? AND (b, c) >= (?, ?) ORDER BY c", 0, 0, 0);
+        assertInvalidMessage(errorMsg, "SELECT * FROM %s WHERE a=? AND (b, c) < (?, ?) ORDER BY c", 0, 0, 0);
+        assertInvalidMessage(errorMsg, "SELECT * FROM %s WHERE a=? AND (b, c) <= (?, ?) ORDER BY c", 0, 0, 0);
+        assertRows(execute("SELECT * FROM %s WHERE a=? AND (b, c) = (?, ?) ORDER BY c ASC", 0, 0, 0),
+                   row(0, 0, 0, 0));
+        assertRows(execute("SELECT * FROM %s WHERE a=? AND (b, c) = (?, ?) ORDER BY c DESC", 0, 0, 0),
+                   row(0, 0, 0, 0));
+
+        assertInvalidMessage(errorMsg, "SELECT * FROM %s WHERE a=? AND (b, c) > ? ORDER BY c", 0, tuple(0, 0));
+        assertInvalidMessage(errorMsg, "SELECT * FROM %s WHERE a=? AND (b, c) >= ? ORDER BY c", 0, tuple(0, 0));
+        assertInvalidMessage(errorMsg, "SELECT * FROM %s WHERE a=? AND (b, c) < ? ORDER BY c", 0, tuple(0, 0));
+        assertInvalidMessage(errorMsg, "SELECT * FROM %s WHERE a=? AND (b, c) <= ? ORDER BY c", 0, tuple(0, 0));
+        assertRows(execute("SELECT * FROM %s WHERE a=? AND (b, c) = ? ORDER BY c ASC", 0, tuple(0, 0)),
+                   row(0, 0, 0, 0));
+        assertRows(execute("SELECT * FROM %s WHERE a=? AND (b, c) = ? ORDER BY c DESC", 0, tuple(0, 0)),
+                   row(0, 0, 0, 0));
+
+        assertRows(execute("SELECT * FROM %s WHERE a IN (?, ?) AND b=? AND c>=? ORDER BY c ASC", 0, 1, 0, 0),
+                   row(0, 0, 0, 0),
+                   row(0, 0, 1, 1),
+                   row(0, 0, 2, 2));
+
+        assertRows(execute("SELECT * FROM %s WHERE a IN (?, ?) AND b=? AND c>=? ORDER BY c DESC", 0, 1, 0, 0),
+                   row(0, 0, 2, 2),
+                   row(0, 0, 1, 1),
+                   row(0, 0, 0, 0));
+
+        assertRows(execute("SELECT * FROM %s WHERE a IN (?, ?) AND b=? ORDER BY c ASC", 0, 1, 0),
+                   row(0, 0, 0, 0),
+                   row(0, 0, 1, 1),
+                   row(0, 0, 2, 2));
+
+        assertRows(execute("SELECT * FROM %s WHERE a IN (?, ?) AND b=? ORDER BY c DESC", 0, 1, 0),
+                   row(0, 0, 2, 2),
+                   row(0, 0, 1, 1),
+                   row(0, 0, 0, 0));
+
+        assertRows(execute("SELECT * FROM %s WHERE a=? AND b IN (?) ORDER BY c ASC", 0, 1),
+                   row(0, 1, 0, 3),
+                   row(0, 1, 1, 4),
+                   row(0, 1, 2, 5));
+
+        assertRows(execute("SELECT * FROM %s WHERE a=? AND b IN (?) ORDER BY c DESC", 0, 1),
+                   row(0, 1, 2, 5),
+                   row(0, 1, 1, 4),
+                   row(0, 1, 0, 3));
+
+        assertRows(execute("SELECT * FROM %s WHERE a=? AND (b, c) IN ((?, ?)) ORDER BY c ASC", 0, 1, 1),
+                   row(0, 1, 1, 4));
+
+        assertRows(execute("SELECT * FROM %s WHERE a=? AND (b, c) IN ((?, ?)) ORDER BY c DESC", 0, 1, 1),
+                   row(0, 1, 1, 4));
+
+        assertRows(execute("SELECT * FROM %s WHERE a=? AND b IN (?, ?) AND c=? ORDER BY b ASC", 0, 0, 1, 2),
+                   row(0, 0, 2, 2),
+                   row(0, 1, 2, 5));
+
+        assertRows(execute("SELECT * FROM %s WHERE a=? AND b IN (?, ?) AND c=? ORDER BY b DESC", 0, 0, 1, 2),
+                   row(0, 1, 2, 5),
+                   row(0, 0, 2, 2));
+
+        assertInvalidMessage(errorMsg, "SELECT * FROM %s WHERE a=? AND b IN ? ORDER BY c", 0, list(0));
+        assertInvalidMessage(errorMsg, "SELECT * FROM %s WHERE a=? AND b IN (?,?) ORDER BY c", 0, 1, 3);
+        assertInvalidMessage(errorMsg, "SELECT * FROM %s WHERE a=? AND (b,c) IN ? ORDER BY c", 0, list(tuple(0, 0)));
+        assertInvalidMessage(errorMsg, "SELECT * FROM %s WHERE a=? AND (b,c) IN ((?,?), (?,?)) ORDER BY c", 0, 0, 0, 0, 1);
+    }
+
     @Test
     public void testSelectWithReversedTypeInReverseOrderWithStaticColumnsWithoutStaticRow() throws Throwable
     {
