@@ -30,7 +30,6 @@ import static org.junit.Assert.assertEquals;
 
 import org.apache.cassandra.config.DatabaseDescriptor;
 import org.apache.cassandra.cql3.CQLTester;
-import org.apache.cassandra.cql3.UntypedResultSet;
 import org.apache.cassandra.cql3.restrictions.StatementRestrictions;
 import org.apache.cassandra.dht.ByteOrderedPartitioner;
 
@@ -386,19 +385,6 @@ public class SelectOrderedPartitionerTest extends CQLTester
     {
         createTable("CREATE TABLE %s (k int, c int, PRIMARY KEY (k, c))");
 
-        for (int k = 0; k < 2; k++)
-            for (int c = 0; c < 2; c++)
-                execute("INSERT INTO %s (k, c) VALUES (?, ?)", k, c);
-
-        assertRows(execute("SELECT * FROM %s"),
-                   row(0, 0),
-                   row(0, 1),
-                   row(1, 0),
-                   row(1, 1));
-
-        // Check for dense tables too
-        createTable(" CREATE TABLE %s (k int, c int, PRIMARY KEY (k, c)) WITH COMPACT STORAGE");
-
         for (int k = 0; k < 2; k++)
             for (int c = 0; c < 2; c++)
                 execute("INSERT INTO %s (k, c) VALUES (?, ?)", k, c);
