@@ -23,10 +23,12 @@ import java.io.FileInputStream;
 import java.io.FileOutputStream;
 import java.io.IOException;
 
-import static org.junit.Assert.assertEquals;
-import static org.junit.Assert.assertTrue;
-import static org.junit.Assert.fail;
+import org.junit.After;
+import org.junit.Before;
+import org.junit.Test;
 
+import org.apache.cassandra.config.Config;
+import org.apache.cassandra.config.DatabaseDescriptor;
 import org.apache.cassandra.cql3.Attributes;
 import org.apache.cassandra.cql3.CQLTester;
 import org.apache.cassandra.cql3.UntypedResultSet;
@@ -35,9 +37,15 @@ import org.apache.cassandra.db.ExpirationDateOverflowHandling;
 import org.apache.cassandra.db.Keyspace;
 import org.apache.cassandra.db.rows.AbstractCell;
 import org.apache.cassandra.exceptions.InvalidRequestException;
+import org.apache.cassandra.tools.StandaloneScrubber;
+import org.apache.cassandra.tools.ToolRunner;
+import org.apache.cassandra.tools.ToolRunner.ToolResult;
 import org.apache.cassandra.utils.FBUtilities;
+import org.assertj.core.api.Assertions;
 
-import org.junit.Test;
+import static org.junit.Assert.assertEquals;
+import static org.junit.Assert.assertTrue;
+import static org.junit.Assert.fail;
 
 public class TTLTest extends CQLTester
 {
@@ -49,6 +57,19 @@ public class TTLTest extends CQLTester
     public static final String SIMPLE_CLUSTERING = "table2";
     public static final String COMPLEX_NOCLUSTERING = "table3";
     public static final String COMPLEX_CLUSTERING = "table4";
+    private Config.CorruptedTombstoneStrategy corruptTombstoneStrategy;
+
+    @Before
+    public void before()
+    {
+        corruptTombstoneStrategy = DatabaseDescriptor.getCorruptedTombstoneStrategy();
+    }
+
+    @After
+    public void after()
+    {
+        DatabaseDescriptor.setCorruptedTombstoneStrategy(corruptTombstoneStrategy);
+    }
 
     @Test
     public void testTTLPerRequestLimit() throws Throwable
@@ -185,9 +206,15 @@ public class TTLTest extends CQLTester
     @Test
     public void testRecoverOverflowedExpirationWithScrub() throws Throwable
     {
-        baseTestRecoverOverflowedExpiration(false, false);
-        baseTestRecoverOverflowedExpiration(true, false);
-        baseTestRecoverOverflowedExpiration(true, true);
+        // this tests writes corrupt tombstones on purpose, disable the strategy:
+        DatabaseDescriptor.setCorruptedTombstoneStrategy(Config.CorruptedTombstoneStrategy.disabled);
+        baseTestRecoverOverflowedExpiration(false, false, false);
+        baseTestRecoverOverflowedExpiration(true, false, false);
+        baseTestRecoverOverflowedExpiration(true, false, true);
+
+        baseTestRecoverOverflowedExpiration(false, true, false);
+        baseTestRecoverOverflowedExpiration(false, true, true);
+        // we reset the corrupted ts strategy after each test in @After above
     }
 
     public void testCapExpirationDateOverflowPolicy(ExpirationDateOverflowHandling.ExpirationDateOverflowPolicy policy) throws Throwable
@@ -257,16 +284,16 @@ public class TTLTest extends CQLTester
         }
     }
 
-    public void baseTestRecoverOverflowedExpiration(boolean runScrub, boolean reinsertOverflowedTTL) throws Throwable
+    public void baseTestRecoverOverflowedExpiration(boolean runScrub, boolean runSStableScrub, boolean reinsertOverflowedTTL) throws Throwable
     {
         // simple column, clustering
-        testRecoverOverflowedExpirationWithScrub(true, true, runScrub, reinsertOverflowedTTL);
+        testRecoverOverflowedExpirationWithScrub(true, true, runScrub, runSStableScrub, reinsertOverflowedTTL);
         // simple column, noclustering
-        testRecoverOverflowedExpirationWithScrub(true, false, runScrub, reinsertOverflowedTTL);
+        testRecoverOverflowedExpirationWithScrub(true, false, runScrub, runSStableScrub, reinsertOverflowedTTL);
         // complex column, clustering
-        testRecoverOverflowedExpirationWithScrub(false, true, runScrub, reinsertOverflowedTTL);
+        testRecoverOverflowedExpirationWithScrub(false, true, runScrub, runSStableScrub, reinsertOverflowedTTL);
         // complex column, noclustering
-        testRecoverOverflowedExpirationWithScrub(false, false, runScrub, reinsertOverflowedTTL);
+        testRecoverOverflowedExpirationWithScrub(false, false, runScrub, runSStableScrub, reinsertOverflowedTTL);
     }
 
     private void createTable(boolean simple, boolean clustering)
@@ -312,12 +339,12 @@ public class TTLTest extends CQLTester
         // TTL is computed dynamically from row expiration time, so if it is
         // equal or higher to the minimum max TTL we compute before the query
         // we are fine.
-        int minMaxTTL = computeMaxTTL();
         UntypedResultSet execute = execute("SELECT ttl(" + field + ") FROM %s WHERE k = 1");
+        int minMaxTTL = computeMaxTTL();
         for (UntypedResultSet.Row row : execute)
         {
             int ttl = row.getInt("ttl(" + field + ")");
-            assertTrue(ttl >= minMaxTTL);
+            assert (ttl >= minMaxTTL) : "ttl must be greater than or equal to minMaxTTL, but " + ttl + " is less than " + minMaxTTL;
         }
     }
 
@@ -331,11 +358,11 @@ public class TTLTest extends CQLTester
         return AbstractCell.MAX_DELETION_TIME - nowInSecs;
     }
 
-    public void testRecoverOverflowedExpirationWithScrub(boolean simple, boolean clustering, boolean runScrub, boolean reinsertOverflowedTTL) throws Throwable
+    public void testRecoverOverflowedExpirationWithScrub(boolean simple, boolean clustering, boolean runScrub, boolean runSStableScrub,  boolean reinsertOverflowedTTL) throws Throwable
     {
         if (reinsertOverflowedTTL)
         {
-            assert runScrub;
+            assert runScrub || runSStableScrub;
         }
 
         createTable(simple, clustering);
@@ -352,25 +379,61 @@ public class TTLTest extends CQLTester
         if (runScrub)
         {
             cfs.scrub(true, false, true, reinsertOverflowedTTL, 1);
+
+            if (reinsertOverflowedTTL)
+            {
+                if (simple)
+                    assertRows(execute("SELECT * from %s"), row(1, 1, 1), row(2, 2, null));
+                else
+                    assertRows(execute("SELECT * from %s"), row(1, 1, set("v11", "v12", "v13", "v14")), row(2, 2, set("v21", "v22", "v23", "v24")));
+
+                cfs.forceMajorCompaction();
+
+                if (simple)
+                    assertRows(execute("SELECT * from %s"), row(1, 1, 1), row(2, 2, null));
+                else
+                    assertRows(execute("SELECT * from %s"), row(1, 1, set("v11", "v12", "v13", "v14")), row(2, 2, set("v21", "v22", "v23", "v24")));
+            }
+            else
+            {
+                assertEmpty(execute("SELECT * from %s"));
+            }
+        }
+        if (runSStableScrub)
+        {
+            System.setProperty(org.apache.cassandra.tools.Util.ALLOW_TOOL_REINIT_FOR_TEST, "true"); // Necessary for testing
+
+            try
+            {
+                ToolResult tool;
+                if (reinsertOverflowedTTL)
+                    tool = ToolRunner.invokeClass(StandaloneScrubber.class, "-r", KEYSPACE, cfs.name);
+                else
+                    tool = ToolRunner.invokeClass(StandaloneScrubber.class, KEYSPACE, cfs.name);
+
+                tool.assertOnCleanExit();
+                Assertions.assertThat(tool.getStdout()).contains("Pre-scrub sstables snapshotted into");
+                if (reinsertOverflowedTTL)
+                    Assertions.assertThat(tool.getStdout()).contains("Fixed 2 rows with overflowed local deletion time.");
+                else
+                    Assertions.assertThat(tool.getStdout()).contains("No valid partitions found while scrubbing");
+            }
+            finally
+            {
+                System.clearProperty(org.apache.cassandra.tools.Util.ALLOW_TOOL_REINIT_FOR_TEST);
+            }
         }
 
-        if (reinsertOverflowedTTL)
+        try
         {
-            if (simple)
-                assertRows(execute("SELECT * from %s"), row(1, 1, 1), row(2, 2, null));
-            else
-                assertRows(execute("SELECT * from %s"), row(1, 1, set("v11", "v12", "v13", "v14")), row(2, 2, set("v21", "v22", "v23", "v24")));
-
-            cfs.forceMajorCompaction();
-
-            if (simple)
-                assertRows(execute("SELECT * from %s"), row(1, 1, 1), row(2, 2, null));
-            else
-                assertRows(execute("SELECT * from %s"), row(1, 1, set("v11", "v12", "v13", "v14")), row(2, 2, set("v21", "v22", "v23", "v24")));
+            cfs.truncateBlocking();
+            dropTable("DROP TABLE %s");
         }
-        else
+        catch (Throwable e)
         {
-            assertEmpty(execute("SELECT * from %s"));
+            // StandaloneScrubber.class should be ran as a tool with a stable env. In a test env there are things moving
+            // under its feet such as the async CQLTester.afterTest() operations. We try to sync cleanup of tables here
+            // but we need to catch any exceptions we might run into bc of the hack. See CASSANDRA-16546
         }
     }
 
