@@ -18,6 +18,7 @@
 package org.apache.cassandra.db;
 
 import java.io.IOException;
+import java.net.UnknownHostException;
 import java.util.ArrayList;
 import java.util.Collection;
 import java.util.Collections;
@@ -40,8 +41,8 @@ import org.apache.cassandra.cache.AutoSavingCache;
 import org.apache.cassandra.cache.ICache;
 import org.apache.cassandra.cache.KeyCacheKey;
 import org.apache.cassandra.config.DatabaseDescriptor;
-import org.apache.cassandra.db.compaction.OperationType;
 import org.apache.cassandra.db.compaction.CompactionManager;
+import org.apache.cassandra.db.compaction.OperationType;
 import org.apache.cassandra.db.lifecycle.LifecycleTransaction;
 import org.apache.cassandra.exceptions.ConfigurationException;
 import org.apache.cassandra.io.sstable.format.SSTableReader;
@@ -79,7 +80,7 @@ public class KeyCacheTest
 
 
     @BeforeClass
-    public static void defineSchema() throws ConfigurationException
+    public static void defineSchema() throws ConfigurationException, UnknownHostException
     {
         SchemaLoader.prepareServer();
         SchemaLoader.createKeyspace(KEYSPACE1,
@@ -97,7 +98,6 @@ public class KeyCacheTest
         SchemaLoader.createKeyspace(KEYSPACE2,
                                     KeyspaceParams.simple(1),
                                     SchemaLoader.standardCFMD(KEYSPACE2, COLUMN_FAMILY_K2_1));
-
     }
 
     @AfterClass
@@ -299,8 +299,8 @@ public class KeyCacheTest
         Mutation rm;
 
         // inserts
-        new RowUpdateBuilder(cfs.metadata, 0, "key1").clustering("1").build().applyUnsafe();
-        new RowUpdateBuilder(cfs.metadata, 0, "key2").clustering("2").build().applyUnsafe();
+        new RowUpdateBuilder(cfs.metadata(), 0, "key1").clustering("1").build().applyUnsafe();
+        new RowUpdateBuilder(cfs.metadata(), 0, "key2").clustering("2").build().applyUnsafe();
 
         // to make sure we have SSTable
         cfs.forceBlockingFlush();
@@ -317,7 +317,7 @@ public class KeyCacheTest
             throw new IllegalStateException();
 
         Util.compactAll(cfs, Integer.MAX_VALUE).get();
-        boolean noEarlyOpen = DatabaseDescriptor.getSSTablePreempiveOpenIntervalInMB() < 0;
+        boolean noEarlyOpen = DatabaseDescriptor.getSSTablePreemptiveOpenIntervalInMB() < 0;
 
         // after compaction cache should have entries for new SSTables,
         // but since we have kept a reference to the old sstables,
@@ -351,7 +351,6 @@ public class KeyCacheTest
         // Here max time to load cache is negative which means no time left to load cache. So the keyCache size should
         // be zero after loadSaved().
         assertKeyCacheSize(0, KEYSPACE1, cf);
-        assertEquals(0, CacheService.instance.keyCache.size());
     }
 
     @Test
@@ -373,7 +372,6 @@ public class KeyCacheTest
         // be zero after load.
         assertKeyCacheSize(numberOfRows, KEYSPACE1, columnFamily1);
         assertKeyCacheSize(numberOfRows, KEYSPACE2, columnFamily2);
-        assertEquals(numberOfRows * tables.size(), CacheService.instance.keyCache.size());
     }
 
     @SuppressWarnings({ "unchecked", "rawtypes" })
@@ -420,8 +418,8 @@ public class KeyCacheTest
 
         for(Pair<String, String> entry : tables)
         {
-            String keyspace = entry.left;
-            String cf = entry.right;
+            String keyspace = entry.left();
+            String cf = entry.right();
             ColumnFamilyStore store = Keyspace.open(keyspace).getColumnFamilyStore(cf);
 
             // insert data and force to disk
@@ -430,8 +428,8 @@ public class KeyCacheTest
         }
         for(Pair<String, String> entry : tables)
         {
-            String keyspace = entry.left;
-            String cf = entry.right;
+            String keyspace = entry.left();
+            String cf = entry.right();
             // populate the cache
             readData(keyspace, cf, 0, numberOfRows);
             assertKeyCacheSize(numberOfRows, keyspace, cf);
