@@ -18,7 +18,6 @@
 
 package org.apache.cassandra.db;
 
-import java.net.InetAddress;
 import java.nio.ByteBuffer;
 import java.util.ArrayList;
 import java.util.Arrays;
@@ -32,8 +31,9 @@ import org.junit.Test;
 import org.apache.cassandra.SchemaLoader;
 import org.apache.cassandra.Util;
 import org.apache.cassandra.cache.RowCacheKey;
-import org.apache.cassandra.config.CFMetaData;
-import org.apache.cassandra.config.Schema;
+import org.apache.cassandra.db.marshal.ValueAccessors;
+import org.apache.cassandra.locator.InetAddressAndPort;
+import org.apache.cassandra.schema.Schema;
 import org.apache.cassandra.db.marshal.AsciiType;
 import org.apache.cassandra.db.rows.*;
 import org.apache.cassandra.db.compaction.CompactionManager;
@@ -99,9 +99,9 @@ public class RowCacheTest
 
         ByteBuffer key = ByteBufferUtil.bytes("rowcachekey");
         DecoratedKey dk = cachedStore.decorateKey(key);
-        RowCacheKey rck = new RowCacheKey(cachedStore.metadata.ksAndCFName, dk);
+        RowCacheKey rck = new RowCacheKey(cachedStore.metadata(), dk);
 
-        RowUpdateBuilder rub = new RowUpdateBuilder(cachedStore.metadata, System.currentTimeMillis(), key);
+        RowUpdateBuilder rub = new RowUpdateBuilder(cachedStore.metadata(), System.currentTimeMillis(), key);
         rub.clustering(String.valueOf(0));
         rub.add("val", ByteBufferUtil.bytes("val" + 0));
         rub.build().applyUnsafe();
@@ -122,7 +122,7 @@ public class RowCacheTest
             Row r = (Row) unfiltered;
             for (ColumnData c : r)
             {
-                assertEquals(((Cell)c).value(), ByteBufferUtil.bytes("val" + 0));
+                assertEquals(((Cell<?>)c).buffer(), ByteBufferUtil.bytes("val" + 0));
             }
         }
         cachedStore.truncateBlocking();
@@ -162,12 +162,12 @@ public class RowCacheTest
                 Row r = (Row)ai.next();
                 assertFalse(ai.hasNext());
 
-                Iterator<Cell> ci = r.cells().iterator();
+                Iterator<Cell<?>> ci = r.cells().iterator();
                 assert(ci.hasNext());
-                Cell cell = ci.next();
+                Cell<?> cell = ci.next();
 
                 assert cell.column().name.bytes.equals(ByteBufferUtil.bytes("val"));
-                assert cell.value().equals(ByteBufferUtil.bytes("val" + i));
+                assert cell.buffer().equals(ByteBufferUtil.bytes("val" + i));
             }
         }
 
@@ -189,12 +189,12 @@ public class RowCacheTest
                 Row r = (Row)ai.next();
                 assertFalse(ai.hasNext());
 
-                Iterator<Cell> ci = r.cells().iterator();
+                Iterator<Cell<?>> ci = r.cells().iterator();
                 assert(ci.hasNext());
-                Cell cell = ci.next();
+                Cell<?> cell = ci.next();
 
                 assert cell.column().name.bytes.equals(ByteBufferUtil.bytes("val"));
-                assert cell.value().equals(ByteBufferUtil.bytes("val" + i));
+                assert cell.buffer().equals(ByteBufferUtil.bytes("val" + i));
             }
         }
 
@@ -256,12 +256,12 @@ public class RowCacheTest
                 Row r = (Row)ai.next();
                 assertFalse(ai.hasNext());
 
-                Iterator<Cell> ci = r.cells().iterator();
+                Iterator<Cell<?>> ci = r.cells().iterator();
                 assert(ci.hasNext());
-                Cell cell = ci.next();
+                Cell<?> cell = ci.next();
 
                 assert cell.column().name.bytes.equals(ByteBufferUtil.bytes("val"));
-                assert cell.value().equals(ByteBufferUtil.bytes("val" + i));
+                assert cell.buffer().equals(ByteBufferUtil.bytes("val" + i));
             }
         }
 
@@ -301,8 +301,8 @@ public class RowCacheTest
         byte[] tk1, tk2;
         tk1 = "key1000".getBytes();
         tk2 = "key1050".getBytes();
-        tmd.updateNormalToken(new BytesToken(tk1), InetAddress.getByName("127.0.0.1"));
-        tmd.updateNormalToken(new BytesToken(tk2), InetAddress.getByName("127.0.0.2"));
+        tmd.updateNormalToken(new BytesToken(tk1), InetAddressAndPort.getByName("127.0.0.1"));
+        tmd.updateNormalToken(new BytesToken(tk2), InetAddressAndPort.getByName("127.0.0.2"));
         store.cleanupCache();
         assertEquals(50, CacheService.instance.rowCache.size());
         CacheService.instance.setRowCacheCapacityInMB(0);
@@ -414,11 +414,11 @@ public class RowCacheTest
 
         ByteBuffer key = ByteBufferUtil.bytes("rowcachekey");
         DecoratedKey dk = cachedStore.decorateKey(key);
-        RowCacheKey rck = new RowCacheKey(cachedStore.metadata.ksAndCFName, dk);
+        RowCacheKey rck = new RowCacheKey(cachedStore.metadata(), dk);
         String values[] = new String[200];
         for (int i = 0; i < 200; i++)
         {
-            RowUpdateBuilder rub = new RowUpdateBuilder(cachedStore.metadata, System.currentTimeMillis(), key);
+            RowUpdateBuilder rub = new RowUpdateBuilder(cachedStore.metadata(), System.currentTimeMillis(), key);
             rub.clustering(String.valueOf(i));
             values[i] = "val" + i;
             rub.add("val", ByteBufferUtil.bytes(values[i]));
@@ -461,11 +461,11 @@ public class RowCacheTest
         {
             Row r = (Row) unfiltered;
 
-            assertEquals(r.clustering().get(0), ByteBufferUtil.bytes(values[i].substring(3)));
+            ValueAccessors.assertDataEquals(r.clustering().get(0), ByteBufferUtil.bytes(values[i].substring(3)));
 
             for (ColumnData c : r)
             {
-                assertEquals(((Cell)c).value(), ByteBufferUtil.bytes(values[i]));
+                assertEquals(((Cell<?>)c).buffer(), ByteBufferUtil.bytes(values[i]));
             }
             i++;
         }
@@ -548,12 +548,10 @@ public class RowCacheTest
     private static void readData(String keyspace, String columnFamily, int offset, int numberOfRows)
     {
         ColumnFamilyStore store = Keyspace.open(keyspace).getColumnFamilyStore(columnFamily);
-        CFMetaData cfm = Schema.instance.getCFMetaData(keyspace, columnFamily);
 
         for (int i = offset; i < offset + numberOfRows; i++)
         {
             DecoratedKey key = Util.dk("key" + i);
-            Clustering cl = Clustering.make(ByteBufferUtil.bytes("col" + i));
             Util.getAll(Util.cmd(store, key).build());
         }
     }
