@@ -26,6 +26,7 @@ import java.nio.ByteBuffer;
 import java.nio.file.Files;
 import java.nio.file.Path;
 import java.nio.file.Paths;
+import java.nio.file.StandardOpenOption;
 import java.util.Arrays;
 import java.util.Collections;
 import java.util.HashSet;
@@ -38,13 +39,12 @@ import java.util.concurrent.atomic.AtomicInteger;
 
 import org.apache.commons.lang3.StringUtils;
 
+import org.junit.After;
 import org.junit.AfterClass;
 import org.junit.Before;
 import org.junit.BeforeClass;
 import org.junit.Test;
-import org.junit.runner.RunWith;
 
-import org.apache.cassandra.OrderedJUnit4ClassRunner;
 import org.apache.cassandra.SchemaLoader;
 import org.apache.cassandra.UpdateBuilder;
 import org.apache.cassandra.Util;
@@ -84,24 +84,20 @@ import org.apache.cassandra.io.sstable.metadata.MetadataCollector;
 import org.apache.cassandra.io.util.FileUtils;
 import org.apache.cassandra.schema.KeyspaceParams;
 import org.apache.cassandra.schema.TableMetadataRef;
-import org.apache.cassandra.tools.StandaloneScrubber;
-import org.apache.cassandra.tools.ToolRunner;
-import org.apache.cassandra.tools.ToolRunner.ToolResult;
 import org.apache.cassandra.utils.ByteBufferUtil;
-import org.assertj.core.api.Assertions;
 
 import static org.apache.cassandra.SchemaLoader.counterCFMD;
 import static org.apache.cassandra.SchemaLoader.createKeyspace;
 import static org.apache.cassandra.SchemaLoader.getCompressionParameters;
 import static org.apache.cassandra.SchemaLoader.loadSchema;
 import static org.apache.cassandra.SchemaLoader.standardCFMD;
+import static org.assertj.core.api.Assertions.assertThat;
 import static org.junit.Assert.assertEquals;
 import static org.junit.Assert.assertNotNull;
 import static org.junit.Assert.assertTrue;
 import static org.junit.Assert.fail;
 import static org.junit.Assume.assumeTrue;
 
-@RunWith(OrderedJUnit4ClassRunner.class)
 public class ScrubTest
 {
     public static final String INVALID_LEGACY_SSTABLE_ROOT_PROP = "invalid-legacy-sstable-root";
@@ -120,8 +116,8 @@ public class ScrubTest
 
     private static final AtomicInteger seq = new AtomicInteger();
 
-    private String ksName;
-    private Keyspace keyspace;
+    String ksName;
+    Keyspace keyspace;
 
     @BeforeClass
     public static void defineSchema() throws ConfigurationException
@@ -170,6 +166,28 @@ public class ScrubTest
         assertOrderedAll(cfs, 1);
     }
 
+    @Test
+    public void testScrubLastBrokenPartition() throws ExecutionException, InterruptedException, IOException
+    {
+        CompactionManager.instance.disableAutoCompaction();
+        ColumnFamilyStore cfs = ColumnFamilyStore.getIfExists(ksName, CF);
+
+        // insert data and verify we get it back w/ range query
+        fillCF(cfs, 1);
+        assertOrderedAll(cfs, 1);
+
+        Set<SSTableReader> liveSSTables = cfs.getLiveSSTables();
+        assertThat(liveSSTables).hasSize(1);
+        String fileName = liveSSTables.iterator().next().getFilename();
+        Files.write(Paths.get(fileName), new byte[10], StandardOpenOption.CREATE, StandardOpenOption.TRUNCATE_EXISTING);
+        ChunkCache.instance.invalidateFile(fileName);
+
+        CompactionManager.instance.performScrub(cfs, true, true, false, 2);
+
+        // check data is still there
+        assertOrderedAll(cfs, 0);
+    }
+
     @Test
     public void testScrubCorruptedCounterPartition() throws IOException, WriteTimeoutException
     {
@@ -179,7 +197,7 @@ public class ScrubTest
 
         CompactionManager.instance.disableAutoCompaction();
         ColumnFamilyStore cfs = keyspace.getColumnFamilyStore(COUNTER_CF);
-
+        cfs.truncateBlocking();
         fillCounterCF(cfs, numPartitions);
 
         assertOrderedAll(cfs, numPartitions);
@@ -418,7 +436,7 @@ public class ScrubTest
         }
     }
 
-    private void overrideWithGarbage(SSTableReader sstable, ByteBuffer key1, ByteBuffer key2) throws IOException
+    static void overrideWithGarbage(SSTableReader sstable, ByteBuffer key1, ByteBuffer key2) throws IOException
     {
         boolean compression = Boolean.parseBoolean(System.getProperty("cassandra.test.compression", "false"));
         long startPosition, endPosition;
@@ -448,7 +466,7 @@ public class ScrubTest
         overrideWithGarbage(sstable, startPosition, endPosition);
     }
 
-    private void overrideWithGarbage(SSTableReader sstable, long startPosition, long endPosition) throws IOException
+    private static void overrideWithGarbage(SSTableReader sstable, long startPosition, long endPosition) throws IOException
     {
         try (RandomAccessFile file = new RandomAccessFile(sstable.getFilename(), "rw"))
         {
@@ -459,7 +477,7 @@ public class ScrubTest
             ChunkCache.instance.invalidateFile(sstable.getFilename());
     }
 
-    private static void assertOrderedAll(ColumnFamilyStore cfs, int expectedSize)
+    static void assertOrderedAll(ColumnFamilyStore cfs, int expectedSize)
     {
         assertOrdered(Util.cmd(cfs).build(), expectedSize);
     }
@@ -478,7 +496,7 @@ public class ScrubTest
         assertEquals(expectedSize, size);
     }
 
-    protected void fillCF(ColumnFamilyStore cfs, int partitionsPerSSTable)
+    protected static void fillCF(ColumnFamilyStore cfs, int partitionsPerSSTable)
     {
         for (int i = 0; i < partitionsPerSSTable; i++)
         {
@@ -517,7 +535,7 @@ public class ScrubTest
         cfs.forceBlockingFlush();
     }
 
-    protected void fillCounterCF(ColumnFamilyStore cfs, int partitionsPerSSTable) throws WriteTimeoutException
+    protected static void fillCounterCF(ColumnFamilyStore cfs, int partitionsPerSSTable) throws WriteTimeoutException
     {
         for (int i = 0; i < partitionsPerSSTable; i++)
         {
@@ -709,193 +727,46 @@ public class ScrubTest
     @Test
     public void testFilterOutDuplicates() throws Exception
     {
-        DatabaseDescriptor.setPartitionerUnsafe(Murmur3Partitioner.instance);
-        QueryProcessor.process(String.format("CREATE TABLE \"%s\".cf_with_duplicates_3_0 (a int, b int, c int, PRIMARY KEY (a, b))", ksName), ConsistencyLevel.ONE);
-
-        ColumnFamilyStore cfs = keyspace.getColumnFamilyStore("cf_with_duplicates_3_0");
-
-        Path legacySSTableRoot = Paths.get(System.getProperty(INVALID_LEGACY_SSTABLE_ROOT_PROP),
-                                           "Keyspace1",
-                                           "cf_with_duplicates_3_0");
-
-        for (String filename : new String[]{ "mb-3-big-CompressionInfo.db",
-                                             "mb-3-big-Digest.crc32",
-                                             "mb-3-big-Index.db",
-                                             "mb-3-big-Summary.db",
-                                             "mb-3-big-Data.db",
-                                             "mb-3-big-Filter.db",
-                                             "mb-3-big-Statistics.db",
-                                             "mb-3-big-TOC.txt" })
-        {
-            Files.copy(Paths.get(legacySSTableRoot.toString(), filename), cfs.getDirectories().getDirectoryForNewSSTables().toPath().resolve(filename));
-        }
-
-        cfs.loadNewSSTables();
-
-        cfs.scrub(true, true, false, false, false, 1);
-
-        UntypedResultSet rs = QueryProcessor.executeInternal(String.format("SELECT * FROM \"%s\".cf_with_duplicates_3_0", ksName));
-        assertNotNull(rs);
-        assertEquals(1, rs.size());
-
-        QueryProcessor.executeInternal(String.format("DELETE FROM \"%s\".cf_with_duplicates_3_0 WHERE a=1 AND b =2", ksName));
-        rs = QueryProcessor.executeInternal(String.format("SELECT * FROM \"%s\".cf_with_duplicates_3_0", ksName));
-        assertNotNull(rs);
-        assertEquals(0, rs.size());
-    }
-
-    @Test
-    public void testScrubOnePartitionWithTool()
-    {
-        ColumnFamilyStore cfs = keyspace.getColumnFamilyStore(CF);
-
-        // insert data and verify we get it back w/ range query
-        fillCF(cfs, 1);
-        assertOrderedAll(cfs, 1);
-
-        ToolResult tool = ToolRunner.invokeClass(StandaloneScrubber.class, ksName, CF);
-        Assertions.assertThat(tool.getStdout()).contains("Pre-scrub sstables snapshotted into");
-        Assertions.assertThat(tool.getStdout()).contains("1 partitions in new sstable and 0 empty");
-        tool.assertOnCleanExit();
-
-        // check data is still there
-        assertOrderedAll(cfs, 1);
-    }
-
-    @Test
-    public void testSkipScrubCorruptedCounterPartitionWithTool() throws IOException, WriteTimeoutException
-    {
-        ColumnFamilyStore cfs = keyspace.getColumnFamilyStore(COUNTER_CF);
-        int numPartitions = 1000;
-
-        fillCounterCF(cfs, numPartitions);
-        assertOrderedAll(cfs, numPartitions);
-        assertEquals(1, cfs.getLiveSSTables().size());
-        SSTableReader sstable = cfs.getLiveSSTables().iterator().next();
-
-        overrideWithGarbage(sstable, ByteBufferUtil.bytes("0"), ByteBufferUtil.bytes("1"));
-
-        // with skipCorrupted == true, the corrupt rows will be skipped
-        ToolResult tool = ToolRunner.invokeClass(StandaloneScrubber.class, "-s", ksName, COUNTER_CF);
-        Assertions.assertThat(tool.getStdout()).contains("0 empty");
-        Assertions.assertThat(tool.getStdout()).contains("partitions that were skipped");
-        tool.assertOnCleanExit();
-
-        assertEquals(1, cfs.getLiveSSTables().size());
-    }
-
-    @Test
-    public void testNoSkipScrubCorruptedCounterPartitionWithTool() throws IOException, WriteTimeoutException
-    {
-        ColumnFamilyStore cfs = keyspace.getColumnFamilyStore(COUNTER_CF);
-        int numPartitions = 1000;
-
-        fillCounterCF(cfs, numPartitions);
-        assertOrderedAll(cfs, numPartitions);
-        assertEquals(1, cfs.getLiveSSTables().size());
-        SSTableReader sstable = cfs.getLiveSSTables().iterator().next();
-
-        overrideWithGarbage(sstable, ByteBufferUtil.bytes("0"), ByteBufferUtil.bytes("1"));
-
-        // with skipCorrupted == false, the scrub is expected to fail
+        IPartitioner oldPart = DatabaseDescriptor.getPartitioner();
         try
         {
-            ToolRunner.invokeClass(StandaloneScrubber.class, ksName, COUNTER_CF);
-            fail("Expected a CorruptSSTableException to be thrown");
+            DatabaseDescriptor.setPartitionerUnsafe(Murmur3Partitioner.instance);
+            QueryProcessor.process(String.format("CREATE TABLE \"%s\".cf_with_duplicates_3_0 (a int, b int, c int, PRIMARY KEY (a, b))", ksName), ConsistencyLevel.ONE);
+
+            ColumnFamilyStore cfs = keyspace.getColumnFamilyStore("cf_with_duplicates_3_0");
+
+            Path legacySSTableRoot = Paths.get(System.getProperty(INVALID_LEGACY_SSTABLE_ROOT_PROP),
+                                               "Keyspace1",
+                                               "cf_with_duplicates_3_0");
+
+            for (String filename : new String[]{ "mb-3-big-CompressionInfo.db",
+                                                 "mb-3-big-Digest.crc32",
+                                                 "mb-3-big-Index.db",
+                                                 "mb-3-big-Summary.db",
+                                                 "mb-3-big-Data.db",
+                                                 "mb-3-big-Filter.db",
+                                                 "mb-3-big-Statistics.db",
+                                                 "mb-3-big-TOC.txt" })
+            {
+                Files.copy(Paths.get(legacySSTableRoot.toString(), filename), cfs.getDirectories().getDirectoryForNewSSTables().toPath().resolve(filename));
+            }
+
+            cfs.loadNewSSTables();
+
+            cfs.scrub(true, true, false, false, false, 1);
+
+            UntypedResultSet rs = QueryProcessor.executeInternal(String.format("SELECT * FROM \"%s\".cf_with_duplicates_3_0", ksName));
+            assertNotNull(rs);
+            assertEquals(1, rs.size());
+
+            QueryProcessor.executeInternal(String.format("DELETE FROM \"%s\".cf_with_duplicates_3_0 WHERE a=1 AND b =2", ksName));
+            rs = QueryProcessor.executeInternal(String.format("SELECT * FROM \"%s\".cf_with_duplicates_3_0", ksName));
+            assertNotNull(rs);
+            assertEquals(0, rs.size());
         }
-        catch (IOError err) {
-            assertTrue(err.getCause() instanceof CorruptSSTableException);
+        finally
+        {
+            DatabaseDescriptor.setPartitionerUnsafe(oldPart);
         }
     }
-
-    @Test
-    public void testNoCheckScrubMultiPartitionWithTool()
-    {
-        ColumnFamilyStore cfs = keyspace.getColumnFamilyStore(CF);
-
-        // insert data and verify we get it back w/ range query
-        fillCF(cfs, 10);
-        assertOrderedAll(cfs, 10);
-
-        ToolResult tool = ToolRunner.invokeClass(StandaloneScrubber.class, "-n", ksName, CF);
-        Assertions.assertThat(tool.getStdout()).contains("Pre-scrub sstables snapshotted into");
-        Assertions.assertThat(tool.getStdout()).contains("10 partitions in new sstable and 0 empty");
-        tool.assertOnCleanExit();
-
-        // check data is still there
-        assertOrderedAll(cfs, 10);
-    }
-
-    @Test
-    public void testHeaderFixValidateOnlyWithTool()
-    {
-        ColumnFamilyStore cfs = keyspace.getColumnFamilyStore(CF);
-
-        fillCF(cfs, 1);
-        assertOrderedAll(cfs, 1);
-
-        ToolResult tool = ToolRunner.invokeClass(StandaloneScrubber.class, "-e", "validate_only", ksName, CF);
-        Assertions.assertThat(tool.getStdout()).contains("Not continuing with scrub, since '--header-fix validate-only' was specified.");
-        tool.assertOnCleanExit();
-        assertOrderedAll(cfs, 1);
-    }
-
-    @Test
-    public void testHeaderFixValidateWithTool()
-    {
-        ColumnFamilyStore cfs = keyspace.getColumnFamilyStore(CF);
-
-        fillCF(cfs, 1);
-        assertOrderedAll(cfs, 1);
-
-        ToolResult tool = ToolRunner.invokeClass(StandaloneScrubber.class, "-e", "validate", ksName, CF);
-        Assertions.assertThat(tool.getStdout()).contains("Pre-scrub sstables snapshotted into");
-        Assertions.assertThat(tool.getStdout()).contains("1 partitions in new sstable and 0 empty");
-        tool.assertOnCleanExit();
-        assertOrderedAll(cfs, 1);
-    }
-
-    @Test
-    public void testHeaderFixFixOnlyWithTool()
-    {
-        ColumnFamilyStore cfs = keyspace.getColumnFamilyStore(CF);
-
-        fillCF(cfs, 1);
-        assertOrderedAll(cfs, 1);
-
-        ToolResult tool = ToolRunner.invokeClass(StandaloneScrubber.class, "-e", "fix-only", ksName, CF);
-        Assertions.assertThat(tool.getStdout()).contains("Not continuing with scrub, since '--header-fix fix-only' was specified.");
-        tool.assertOnCleanExit();
-        assertOrderedAll(cfs, 1);
-    }
-
-    @Test
-    public void testHeaderFixWithTool()
-    {
-        ColumnFamilyStore cfs = keyspace.getColumnFamilyStore(CF);
-
-        fillCF(cfs, 1);
-        assertOrderedAll(cfs, 1);
-
-        ToolResult tool = ToolRunner.invokeClass(StandaloneScrubber.class, "-e", "fix", ksName, CF);
-        Assertions.assertThat(tool.getStdout()).contains("Pre-scrub sstables snapshotted into");
-        Assertions.assertThat(tool.getStdout()).contains("1 partitions in new sstable and 0 empty");
-        tool.assertOnCleanExit();
-        assertOrderedAll(cfs, 1);
-    }
-
-    @Test
-    public void testHeaderFixNoChecksWithTool()
-    {
-        ColumnFamilyStore cfs = keyspace.getColumnFamilyStore(CF);
-
-        fillCF(cfs, 1);
-        assertOrderedAll(cfs, 1);
-
-        ToolResult tool = ToolRunner.invokeClass(StandaloneScrubber.class, "-e", "off", ksName, CF);
-        Assertions.assertThat(tool.getStdout()).contains("Pre-scrub sstables snapshotted into");
-        Assertions.assertThat(tool.getStdout()).contains("1 partitions in new sstable and 0 empty");
-        tool.assertOnCleanExit();
-        assertOrderedAll(cfs, 1);
-    }
 }
