@@ -36,6 +36,7 @@ import static org.junit.Assert.assertFalse;
 import static org.junit.Assert.assertNotNull;
 import static org.junit.Assert.assertNull;
 import static org.junit.Assert.assertTrue;
+import static org.junit.Assert.assertFalse;
 import static org.junit.Assert.fail;
 
 import org.apache.cassandra.SchemaLoader;
@@ -47,7 +48,7 @@ import org.apache.cassandra.db.RowUpdateBuilder;
 import org.apache.cassandra.exceptions.ConfigurationException;
 import org.apache.cassandra.io.sstable.format.SSTableReader;
 import org.apache.cassandra.schema.KeyspaceParams;
-import org.apache.cassandra.MockSchema;
+import org.apache.cassandra.schema.MockSchema;
 import org.apache.cassandra.utils.Pair;
 
 import static org.apache.cassandra.db.compaction.TimeWindowCompactionStrategy.getWindowBoundsInMillis;
@@ -179,7 +180,7 @@ public class TimeWindowCompactionStrategyTest extends SchemaLoader
         for (int r = 0; r < 3; r++)
         {
             DecoratedKey key = Util.dk(String.valueOf(r));
-            new RowUpdateBuilder(cfs.metadata, r, key.getKey())
+            new RowUpdateBuilder(cfs.metadata(), r, key.getKey())
                 .clustering("column")
                 .add("val", value).build().applyUnsafe();
 
@@ -190,7 +191,7 @@ public class TimeWindowCompactionStrategyTest extends SchemaLoader
         {
             // And add progressively more cells into each sstable
             DecoratedKey key = Util.dk(String.valueOf(r));
-            new RowUpdateBuilder(cfs.metadata, r, key.getKey())
+            new RowUpdateBuilder(cfs.metadata(), r, key.getKey())
                 .clustering("column")
                 .add("val", value).build().applyUnsafe();
             cfs.forceBlockingFlush();
@@ -233,9 +234,9 @@ public class TimeWindowCompactionStrategyTest extends SchemaLoader
         for (int r = 5; r < numSSTables; r++)
         {
             DecoratedKey key = Util.dk(String.valueOf(r));
-            for(int i = 0; i < r; i++)
+            for (int i = 0; i < r; i++)
             {
-                new RowUpdateBuilder(cfs.metadata, tstamp + r, key.getKey())
+                new RowUpdateBuilder(cfs.metadata(), tstamp + r, key.getKey())
                     .clustering("column")
                     .add("val", value).build().applyUnsafe();
             }
@@ -270,7 +271,7 @@ public class TimeWindowCompactionStrategyTest extends SchemaLoader
 
         // Create a expiring sstable with a TTL
         DecoratedKey key = Util.dk("expired");
-        new RowUpdateBuilder(cfs.metadata, System.currentTimeMillis(), TTL_SECONDS, key.getKey())
+        new RowUpdateBuilder(cfs.metadata(), System.currentTimeMillis(), TTL_SECONDS, key.getKey())
             .clustering("column")
             .add("val", value).build().applyUnsafe();
 
@@ -280,7 +281,7 @@ public class TimeWindowCompactionStrategyTest extends SchemaLoader
 
         // Create a second sstable without TTL
         key = Util.dk("nonexpired");
-        new RowUpdateBuilder(cfs.metadata, System.currentTimeMillis(), key.getKey())
+        new RowUpdateBuilder(cfs.metadata(), System.currentTimeMillis(), key.getKey())
             .clustering("column")
             .add("val", value).build().applyUnsafe();
 
@@ -322,7 +323,7 @@ public class TimeWindowCompactionStrategyTest extends SchemaLoader
 
         // Create a expiring sstable with a TTL
         DecoratedKey key = Util.dk("expired");
-        new RowUpdateBuilder(cfs.metadata, timestamp, TTL_SECONDS, key.getKey())
+        new RowUpdateBuilder(cfs.metadata(), timestamp, TTL_SECONDS, key.getKey())
             .clustering("column")
             .add("val", value).build().applyUnsafe();
 
@@ -331,11 +332,11 @@ public class TimeWindowCompactionStrategyTest extends SchemaLoader
         Thread.sleep(10);
 
         // Create a second sstable without TTL and with a row superceded by the expiring row
-        new RowUpdateBuilder(cfs.metadata, timestamp - 1000, key.getKey())
+        new RowUpdateBuilder(cfs.metadata(), timestamp - 1000, key.getKey())
             .clustering("column")
             .add("val", value).build().applyUnsafe();
         key = Util.dk("nonexpired");
-        new RowUpdateBuilder(cfs.metadata, timestamp, key.getKey())
+        new RowUpdateBuilder(cfs.metadata(), timestamp, key.getKey())
             .clustering("column")
             .add("val", value).build().applyUnsafe();
 
