@@ -25,7 +25,6 @@ import org.junit.Test;
 
 import com.google.common.collect.Iterators;
 
-import org.apache.cassandra.config.CFMetaData;
 import org.apache.cassandra.db.*;
 import org.apache.cassandra.db.ClusteringPrefix.Kind;
 import org.apache.cassandra.db.marshal.AbstractType;
@@ -35,6 +34,7 @@ import org.apache.cassandra.db.partitions.UnfilteredPartitionIterator;
 import org.apache.cassandra.db.rows.*;
 import org.apache.cassandra.db.transform.RTBoundValidator.Stage;
 import org.apache.cassandra.dht.Murmur3Partitioner;
+import org.apache.cassandra.schema.TableMetadata;
 import org.apache.cassandra.utils.FBUtilities;
 
 import static org.apache.cassandra.db.transform.RTBoundCloser.close;
@@ -52,21 +52,20 @@ public final class RTTransformationsTest
 
     private final int nowInSec = FBUtilities.nowInSeconds();
 
-    private CFMetaData metadata;
+    private TableMetadata metadata;
     private DecoratedKey key;
 
     @Before
     public void setUp()
     {
         metadata =
-            CFMetaData.Builder
-                      .create(KEYSPACE, TABLE)
-                      .addPartitionKey("pk", UTF8Type.instance)
-                      .addClusteringColumn("ck0", UTF8Type.instance)
-                      .addClusteringColumn("ck1", UTF8Type.instance)
-                      .addClusteringColumn("ck2", UTF8Type.instance)
-                      .withPartitioner(Murmur3Partitioner.instance)
-                      .build();
+            TableMetadata.builder(KEYSPACE, TABLE)
+                         .addPartitionKeyColumn("pk", UTF8Type.instance)
+                         .addClusteringColumn("ck0", UTF8Type.instance)
+                         .addClusteringColumn("ck1", UTF8Type.instance)
+                         .addClusteringColumn("ck2", UTF8Type.instance)
+                         .partitioner(Murmur3Partitioner.instance)
+                         .build();
         key = Murmur3Partitioner.instance.decorateKey(bytes("key"));
     }
 
@@ -376,7 +375,7 @@ public final class RTTransformationsTest
         for (int i = 0; i < clusteringValues.length; i++)
             clusteringByteBuffers[i] = decompose(metadata.clusteringColumns().get(i).type, clusteringValues[i]);
 
-        return new RangeTombstoneBoundMarker(ClusteringBound.create(kind, clusteringByteBuffers), new DeletionTime(timestamp, nowInSec));
+        return new RangeTombstoneBoundMarker(BufferClusteringBound.create(kind, clusteringByteBuffers), new DeletionTime(timestamp, nowInSec));
     }
 
     private RangeTombstoneBoundaryMarker boundary(ClusteringPrefix.Kind kind, long closeTimestamp, long openTimestamp, Object... clusteringValues)
@@ -385,7 +384,7 @@ public final class RTTransformationsTest
         for (int i = 0; i < clusteringValues.length; i++)
             clusteringByteBuffers[i] = decompose(metadata.clusteringColumns().get(i).type, clusteringValues[i]);
 
-        return new RangeTombstoneBoundaryMarker(ClusteringBoundary.create(kind, clusteringByteBuffers),
+        return new RangeTombstoneBoundaryMarker(BufferClusteringBoundary.create(kind, clusteringByteBuffers),
                                                 new DeletionTime(closeTimestamp, nowInSec),
                                                 new DeletionTime(openTimestamp, nowInSec));
     }
@@ -413,7 +412,7 @@ public final class RTTransformationsTest
             new AbstractUnfilteredRowIterator(metadata,
                                               key,
                                               DeletionTime.LIVE,
-                                              metadata.partitionColumns(),
+                                              metadata.regularAndStaticColumns(),
                                               Rows.EMPTY_STATIC_ROW,
                                               isReversedOrder,
                                               EncodingStats.NO_STATS)
@@ -424,7 +423,7 @@ public final class RTTransformationsTest
             }
         };
 
-        return new SingletonUnfilteredPartitionIterator(rowIter, false);
+        return new SingletonUnfilteredPartitionIterator(rowIter);
     }
 
     private void assertIteratorsEqual(UnfilteredPartitionIterator iter1, UnfilteredPartitionIterator iter2)
