@@ -25,10 +25,9 @@ import java.util.*;
 
 import com.google.common.collect.ImmutableMap;
 import org.apache.cassandra.SchemaLoader;
-import org.apache.cassandra.config.Schema;
 import org.apache.cassandra.db.Mutation;
-import org.apache.cassandra.db.RowUpdateBuilder;
 import org.apache.cassandra.schema.KeyspaceParams;
+import org.apache.cassandra.schema.Schema;
 import org.apache.cassandra.utils.FBUtilities;
 import org.junit.BeforeClass;
 import org.junit.Test;
@@ -69,20 +68,6 @@ public class HintsCatalogTest
         }
     }
 
-    @Test
-    public void exciseHintFiles() throws IOException
-    {
-        File directory = Files.createTempDirectory(null).toFile();
-        try
-        {
-            exciseHintFiles(directory);
-        }
-        finally
-        {
-            directory.deleteOnExit();
-        }
-    }
-
     private void loadCompletenessAndOrderTest(File directory) throws IOException
     {
         UUID hostId1 = UUID.randomUUID();
@@ -119,6 +104,51 @@ public class HintsCatalogTest
         assertNull(store2.poll());
     }
 
+    @Test
+    public void deleteHintsTest() throws IOException
+    {
+        File directory = Files.createTempDirectory(null).toFile();
+        UUID hostId1 = UUID.randomUUID();
+        UUID hostId2 = UUID.randomUUID();
+        long now = System.currentTimeMillis();
+        writeDescriptor(directory, new HintsDescriptor(hostId1, now));
+        writeDescriptor(directory, new HintsDescriptor(hostId1, now + 1));
+        writeDescriptor(directory, new HintsDescriptor(hostId2, now + 2));
+        writeDescriptor(directory, new HintsDescriptor(hostId2, now + 3));
+
+        // load catalog containing two stores (one for each host)
+        HintsCatalog catalog = HintsCatalog.load(directory, ImmutableMap.of());
+        assertEquals(2, catalog.stores().count());
+        assertTrue(catalog.hasFiles());
+
+        // delete all hints from store 1
+        assertTrue(catalog.get(hostId1).hasFiles());
+        catalog.deleteAllHints(hostId1);
+        assertFalse(catalog.get(hostId1).hasFiles());
+        // stores are still keepts for each host, even after deleting hints
+        assertEquals(2, catalog.stores().count());
+        assertTrue(catalog.hasFiles());
+
+        // delete all hints from all stores
+        catalog.deleteAllHints();
+        assertEquals(2, catalog.stores().count());
+        assertFalse(catalog.hasFiles());
+    }
+
+    @Test
+    public void exciseHintFiles() throws IOException
+    {
+        File directory = Files.createTempDirectory(null).toFile();
+        try
+        {
+            exciseHintFiles(directory);
+        }
+        finally
+        {
+            directory.deleteOnExit();
+        }
+    }
+
     private static void exciseHintFiles(File directory) throws IOException
     {
         UUID hostId = UUID.randomUUID();
@@ -152,37 +182,6 @@ public class HintsCatalogTest
         assertEquals(0, store.getDispatchQueueSize());
     }
 
-    @Test
-    public void deleteHintsTest() throws IOException
-    {
-        File directory = Files.createTempDirectory(null).toFile();
-        UUID hostId1 = UUID.randomUUID();
-        UUID hostId2 = UUID.randomUUID();
-        long now = System.currentTimeMillis();
-        writeDescriptor(directory, new HintsDescriptor(hostId1, now));
-        writeDescriptor(directory, new HintsDescriptor(hostId1, now+1));
-        writeDescriptor(directory, new HintsDescriptor(hostId2, now+2));
-        writeDescriptor(directory, new HintsDescriptor(hostId2, now+3));
-
-        // load catalog containing two stores (one for each host)
-        HintsCatalog catalog = HintsCatalog.load(directory, ImmutableMap.of());
-        assertEquals(2, catalog.stores().count());
-        assertTrue(catalog.hasFiles());
-
-        // delete all hints from store 1
-        assertTrue(catalog.get(hostId1).hasFiles());
-        catalog.deleteAllHints(hostId1);
-        assertFalse(catalog.get(hostId1).hasFiles());
-        // stores are still keepts for each host, even after deleting hints
-        assertEquals(2, catalog.stores().count());
-        assertTrue(catalog.hasFiles());
-
-        // delete all hints from all stores
-        catalog.deleteAllHints();
-        assertEquals(2, catalog.stores().count());
-        assertFalse(catalog.hasFiles());
-    }
-
     @SuppressWarnings("EmptyTryBlock")
     private static void writeDescriptor(File directory, HintsDescriptor descriptor) throws IOException
     {
@@ -193,24 +192,24 @@ public class HintsCatalogTest
 
     private static Mutation createMutation(String key, long now)
     {
-        Mutation mutation = new Mutation(KEYSPACE, dk(key));
+        Mutation.SimpleBuilder builder = Mutation.simpleBuilder(KEYSPACE, dk(key));
 
-        new RowUpdateBuilder(Schema.instance.getCFMetaData(KEYSPACE, TABLE0), now, mutation)
-                .clustering("column0")
-                .add("val", "value0")
-                .build();
+        builder.update(Schema.instance.getTableMetadata(KEYSPACE, TABLE0))
+               .timestamp(now)
+               .row("column0")
+               .add("val", "value0");
 
-        new RowUpdateBuilder(Schema.instance.getCFMetaData(KEYSPACE, TABLE1), now + 1, mutation)
-                .clustering("column1")
-                .add("val", "value1")
-                .build();
+        builder.update(Schema.instance.getTableMetadata(KEYSPACE, TABLE1))
+               .timestamp(now + 1)
+               .row("column1")
+               .add("val", "value1");
 
-        new RowUpdateBuilder(Schema.instance.getCFMetaData(KEYSPACE, TABLE2), now + 2, mutation)
-                .clustering("column2")
-                .add("val", "value2")
-                .build();
+        builder.update(Schema.instance.getTableMetadata(KEYSPACE, TABLE2))
+               .timestamp(now + 2)
+               .row("column2")
+               .add("val", "value2");
 
-        return mutation;
+        return builder.build();
     }
 
     @SuppressWarnings("EmptyTryBlock")
