@@ -17,8 +17,6 @@
  */
 package org.apache.cassandra.hints;
 
-import java.net.InetAddress;
-import java.util.Collections;
 import java.util.UUID;
 import java.util.concurrent.ExecutionException;
 import java.util.concurrent.TimeUnit;
@@ -27,6 +25,7 @@ import javax.annotation.Nullable;
 
 import com.google.common.util.concurrent.Futures;
 import com.google.common.util.concurrent.ListenableFuture;
+import com.google.common.util.concurrent.MoreExecutors;
 import org.junit.After;
 import org.junit.Before;
 import org.junit.BeforeClass;
@@ -34,22 +33,25 @@ import org.junit.Test;
 
 import com.datastax.driver.core.utils.MoreFutures;
 import org.apache.cassandra.SchemaLoader;
-import org.apache.cassandra.config.CFMetaData;
-import org.apache.cassandra.config.Schema;
+import org.apache.cassandra.locator.InetAddressAndPort;
+import org.apache.cassandra.net.NoPayload;
+import org.apache.cassandra.schema.TableMetadata;
+import org.apache.cassandra.schema.Schema;
 import org.apache.cassandra.db.DecoratedKey;
 import org.apache.cassandra.db.partitions.PartitionUpdate;
 import org.apache.cassandra.gms.IFailureDetectionEventListener;
 import org.apache.cassandra.gms.IFailureDetector;
 import org.apache.cassandra.metrics.StorageMetrics;
-import org.apache.cassandra.net.MessageIn;
+import org.apache.cassandra.net.Message;
 import org.apache.cassandra.net.MessagingService;
 import org.apache.cassandra.net.MockMessagingService;
 import org.apache.cassandra.net.MockMessagingSpy;
 import org.apache.cassandra.schema.KeyspaceParams;
 import org.apache.cassandra.service.StorageService;
-import org.apache.cassandra.utils.FBUtilities;
 
 import static org.apache.cassandra.Util.dk;
+import static org.apache.cassandra.net.Verb.HINT_REQ;
+import static org.apache.cassandra.net.Verb.HINT_RSP;
 import static org.apache.cassandra.net.MockMessagingService.verb;
 import static org.junit.Assert.assertEquals;
 import static org.junit.Assert.assertTrue;
@@ -78,9 +80,10 @@ public class HintsServiceTest
     }
 
     @Before
-    public void reinstanciateService() throws ExecutionException, InterruptedException
+    public void reinstanciateService() throws Throwable
     {
-        MessagingService.instance().clearMessageSinks();
+        MessagingService.instance().inboundSink.clear();
+        MessagingService.instance().outboundSink.clear();
 
         if (!HintsService.instance.isShutDown())
         {
@@ -89,7 +92,9 @@ public class HintsServiceTest
         }
 
         failureDetector.isAlive = true;
+
         HintsService.instance = new HintsService(failureDetector);
+
         HintsService.instance.startDispatch();
     }
 
@@ -125,7 +130,7 @@ public class HintsServiceTest
             {
                 HintsService.instance.resumeDispatch();
             }
-        });
+        }, MoreExecutors.directExecutor());
 
         Futures.allAsList(
                 noMessagesWhilePaused,
@@ -180,20 +185,16 @@ public class HintsServiceTest
     private MockMessagingSpy sendHintsAndResponses(int noOfHints, int noOfResponses)
     {
         // create spy for hint messages, but only create responses for noOfResponses hints
-        MessageIn<HintResponse> messageIn = MessageIn.create(FBUtilities.getBroadcastAddress(),
-                HintResponse.instance,
-                Collections.emptyMap(),
-                MessagingService.Verb.REQUEST_RESPONSE,
-                MessagingService.current_version);
+        Message<NoPayload> message = Message.internalResponse(HINT_RSP, NoPayload.noPayload);
 
         MockMessagingSpy spy;
         if (noOfResponses != -1)
         {
-            spy = MockMessagingService.when(verb(MessagingService.Verb.HINT)).respondN(messageIn, noOfResponses);
+            spy = MockMessagingService.when(verb(HINT_REQ)).respondN(message, noOfResponses);
         }
         else
         {
-            spy = MockMessagingService.when(verb(MessagingService.Verb.HINT)).respond(messageIn);
+            spy = MockMessagingService.when(verb(HINT_REQ)).respond(message);
         }
 
         // create and write noOfHints using service
@@ -202,8 +203,8 @@ public class HintsServiceTest
         {
             long now = System.currentTimeMillis();
             DecoratedKey dkey = dk(String.valueOf(i));
-            CFMetaData cfMetaData = Schema.instance.getCFMetaData(KEYSPACE, TABLE);
-            PartitionUpdate.SimpleBuilder builder = PartitionUpdate.simpleBuilder(cfMetaData, dkey).timestamp(now);
+            TableMetadata metadata = Schema.instance.getTableMetadata(KEYSPACE, TABLE);
+            PartitionUpdate.SimpleBuilder builder = PartitionUpdate.simpleBuilder(metadata, dkey).timestamp(now);
             builder.row("column0").add("val", "value0");
             Hint hint = Hint.create(builder.buildAsMutation(), now);
             HintsService.instance.write(hostId, hint);
@@ -215,17 +216,17 @@ public class HintsServiceTest
     {
         private boolean isAlive = true;
 
-        public boolean isAlive(InetAddress ep)
+        public boolean isAlive(InetAddressAndPort ep)
         {
             return isAlive;
         }
 
-        public void interpret(InetAddress ep)
+        public void interpret(InetAddressAndPort ep)
         {
             throw new UnsupportedOperationException();
         }
 
-        public void report(InetAddress ep)
+        public void report(InetAddressAndPort ep)
         {
             throw new UnsupportedOperationException();
         }
@@ -240,12 +241,12 @@ public class HintsServiceTest
             throw new UnsupportedOperationException();
         }
 
-        public void remove(InetAddress ep)
+        public void remove(InetAddressAndPort ep)
         {
             throw new UnsupportedOperationException();
         }
 
-        public void forceConviction(InetAddress ep)
+        public void forceConviction(InetAddressAndPort ep)
         {
             throw new UnsupportedOperationException();
         }
