@@ -25,13 +25,12 @@ import java.util.*;
 import java.util.concurrent.ExecutionException;
 import java.util.concurrent.atomic.AtomicInteger;
 import java.util.stream.Collectors;
-import java.util.stream.Stream;
 import java.util.stream.StreamSupport;
 
 import com.google.common.collect.ImmutableList;
 import com.google.common.collect.ImmutableMap;
-import com.google.common.io.Files;
 
+import org.apache.cassandra.io.sstable.format.SSTableReader;
 import org.junit.Before;
 import org.junit.BeforeClass;
 import org.junit.Rule;
@@ -919,6 +918,92 @@ public class CQLSSTableWriterTest
         }
     }
 
+    @Test
+    public void testNotifySSTableFinishedForSorted() throws Exception
+    {
+        testNotifySSTableFinished(true, false);
+    }
+
+    @Test
+    public void testNotifySSTableFinishedForUnsorted() throws Exception
+    {
+        testNotifySSTableFinished(false, false);
+    }
+
+    @Test
+    public void testCloseSortedWriterOnFirstPorducedShouldStillResultInTwoSSTables() throws Exception
+    {
+        // Writing a new partition (and exceeding the size limit) leads to closing the current writer and buffering the last partition update.
+        // Since there is a last partition buffered, closing the sstable writer flushes to a new sstable.
+        // Therefore, even though the test closes the writer immediately, there are still 2 sstables produced.
+        testNotifySSTableFinished(true, true);
+    }
+
+    private void testNotifySSTableFinished(boolean sorted, boolean closeWriterOnFirstProduced) throws Exception
+    {
+        List<SSTableReader> produced = new ArrayList<>();
+        String schema = "CREATE TABLE " + qualifiedTable + " ("
+                + "  k int PRIMARY KEY,"
+                + "  v text )";
+        CQLSSTableWriter.Builder builder = CQLSSTableWriter
+                .builder()
+                .inDirectory(dataDir)
+                .forTable(schema)
+                .using("INSERT INTO " + qualifiedTable +
+                        " (k, v) VALUES (?, ?)")
+                .withMaxSSTableSizeInMiB(1)
+                .openSSTableOnProduced()
+                .withSSTableProducedListener(produced::addAll);
+        if (sorted)
+        {
+            builder.sorted();
+        }
+        CQLSSTableWriter writer = builder.build();
+
+        int rowCount = 30_000;
+        // Max SSTable size is 1 MiB
+        // 30_000 rows should take 30_000 * (4 + 37) = 1.17 MiB > 1 MiB, i.e. producing 2 sstables
+        for (int i = 0; i < rowCount; i++)
+        {
+            writer.addRow(i, UUID.randomUUID().toString());
+            if (closeWriterOnFirstProduced && !produced.isEmpty())
+            {
+                // on closing writer, it flushes the last update to the new sstable
+                writer.close();
+                break;
+            }
+        }
+        // the assertion is only performed for sorted because unsorted writer writes asynchrously; avoid flakiness
+        if (!closeWriterOnFirstProduced && sorted)
+        {
+            // while writing, one sstable should be finished
+            assertEquals(1, produced.size());
+        }
+
+        if (!closeWriterOnFirstProduced)
+            writer.close();
+        // another sstable is finished on closing the writer
+        assertEquals(2, produced.size());
+
+        File[] dataFiles = dataDir.listFiles((file, name) -> name.endsWith(Component.DATA.name()));
+        assertNotNull(dataFiles);
+        assertEquals("The sorted writer should produce 2 sstables when max sstable size is configured",
+                2, dataFiles.length);
+        Set<File> notifiedDataFileSet = produced.stream()
+                .map(sstable ->  new File(sstable.descriptor.filenameFor(Component.DATA)))
+                .collect(Collectors.toSet());
+        Set<File> listedDataFileSet = Arrays.stream(dataFiles)
+                .map(f -> {
+                    try {
+                        return f.getCanonicalFile();
+                    } catch (IOException e) {
+                        throw new RuntimeException(e);
+                    }
+                })
+                .collect(Collectors.toSet());
+        assertEquals(notifiedDataFileSet, listedDataFileSet);
+    }
+
     private static void loadSSTables(File dataDir, String ks) throws ExecutionException, InterruptedException
     {
         SSTableLoader loader = new SSTableLoader(dataDir, new SSTableLoader.Client()
