@@ -23,7 +23,6 @@ import java.util.*;
 import java.util.concurrent.*;
 import java.util.concurrent.atomic.AtomicReference;
 import java.util.function.Consumer;
-import java.util.stream.Collectors;
 
 import com.google.common.base.Joiner;
 import com.google.common.collect.Sets;
@@ -31,18 +30,16 @@ import org.junit.After;
 import org.junit.Before;
 import org.junit.BeforeClass;
 import org.junit.Test;
-import org.junit.runner.RunWith;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
-import org.apache.cassandra.OrderedJUnit4ClassRunner;
 import org.apache.cassandra.SchemaLoader;
 import org.apache.cassandra.Util;
 import org.apache.cassandra.concurrent.NamedThreadFactory;
+import org.apache.cassandra.config.DatabaseDescriptor;
 import org.apache.cassandra.db.ColumnFamilyStore;
 import org.apache.cassandra.db.Keyspace;
 import org.apache.cassandra.db.RowUpdateBuilder;
-import org.apache.cassandra.db.compaction.AntiCompactionTest;
 import org.apache.cassandra.db.compaction.CompactionInfo;
 import org.apache.cassandra.db.compaction.CompactionInterruptedException;
 import org.apache.cassandra.db.compaction.CompactionManager;
@@ -51,7 +48,6 @@ import org.apache.cassandra.db.compaction.OperationType;
 import org.apache.cassandra.db.lifecycle.LifecycleTransaction;
 import org.apache.cassandra.exceptions.ConfigurationException;
 import org.apache.cassandra.io.sstable.format.SSTableReader;
-import org.apache.cassandra.metrics.CompactionMetrics;
 import org.apache.cassandra.metrics.RestorableMeter;
 import org.apache.cassandra.schema.CachingParams;
 import org.apache.cassandra.schema.KeyspaceParams;
@@ -71,7 +67,6 @@ import static org.junit.Assert.assertNull;
 import static org.junit.Assert.assertTrue;
 import static org.junit.Assert.fail;
 
-@RunWith(OrderedJUnit4ClassRunner.class)
 public class IndexSummaryManagerTest
 {
     private static final Logger logger = LoggerFactory.getLogger(IndexSummaryManagerTest.class);
@@ -550,13 +545,16 @@ public class IndexSummaryManagerTest
 
         // resize interval
         manager.setResizeIntervalInMinutes(-1);
+        assertEquals(-1, DatabaseDescriptor.getIndexSummaryResizeIntervalInMinutes());
         assertNull(manager.getTimeToNextResize(TimeUnit.MINUTES));
 
         manager.setResizeIntervalInMinutes(10);
         assertEquals(10, manager.getResizeIntervalInMinutes());
+        assertEquals(10, DatabaseDescriptor.getIndexSummaryResizeIntervalInMinutes());
         assertEquals(10, manager.getTimeToNextResize(TimeUnit.MINUTES), 1);
         manager.setResizeIntervalInMinutes(15);
         assertEquals(15, manager.getResizeIntervalInMinutes());
+        assertEquals(15, DatabaseDescriptor.getIndexSummaryResizeIntervalInMinutes());
         assertEquals(15, manager.getTimeToNextResize(TimeUnit.MINUTES), 2);
 
         // memory pool capacity
