@@ -24,14 +24,13 @@ import java.util.Collection;
 import java.util.List;
 
 import com.google.common.collect.Iterables;
-import com.google.common.util.concurrent.RateLimiter;
 
 import org.junit.BeforeClass;
 import org.junit.Test;
 
 import org.apache.cassandra.SchemaLoader;
 import org.apache.cassandra.Util;
-import org.apache.cassandra.config.CFMetaData;
+import org.apache.cassandra.schema.TableMetadata;
 import org.apache.cassandra.db.ColumnFamilyStore;
 import org.apache.cassandra.db.DataRange;
 import org.apache.cassandra.db.DecoratedKey;
@@ -75,7 +74,7 @@ public class SSTableScannerTest
     }
 
     // we produce all DataRange variations that produce an inclusive start and exclusive end range
-    private static Iterable<DataRange> dataRanges(CFMetaData metadata, int start, int end)
+    private static Iterable<DataRange> dataRanges(TableMetadata metadata, int start, int end)
     {
         if (end < start)
             return dataRanges(metadata, start, end, false, true);
@@ -86,7 +85,7 @@ public class SSTableScannerTest
         );
     }
 
-    private static Iterable<DataRange> dataRanges(CFMetaData metadata, int start, int end, boolean inclusiveStart, boolean inclusiveEnd)
+    private static Iterable<DataRange> dataRanges(TableMetadata metadata, int start, int end, boolean inclusiveStart, boolean inclusiveEnd)
     {
         List<DataRange> ranges = new ArrayList<>();
         if (start == end + 1)
@@ -144,7 +143,7 @@ public class SSTableScannerTest
         return token(key).maxKeyBound();
     }
 
-    private static DataRange dataRange(CFMetaData metadata, PartitionPosition start, boolean startInclusive, PartitionPosition end, boolean endInclusive)
+    private static DataRange dataRange(TableMetadata metadata, PartitionPosition start, boolean startInclusive, PartitionPosition end, boolean endInclusive)
     {
         Slices.Builder sb = new Slices.Builder(metadata.comparator);
         ClusteringIndexSliceFilter filter = new ClusteringIndexSliceFilter(sb.build(), false);
@@ -166,7 +165,7 @@ public class SSTableScannerTest
         return ranges;
     }
 
-    private static void insertRowWithKey(CFMetaData metadata, int key)
+    private static void insertRowWithKey(TableMetadata metadata, int key)
     {
         long timestamp = System.currentTimeMillis();
 
@@ -181,9 +180,11 @@ public class SSTableScannerTest
     private static void assertScanMatches(SSTableReader sstable, int scanStart, int scanEnd, int ... boundaries)
     {
         assert boundaries.length % 2 == 0;
-        for (DataRange range : dataRanges(sstable.metadata, scanStart, scanEnd))
+        for (DataRange range : dataRanges(sstable.metadata(), scanStart, scanEnd))
         {
-            try(ISSTableScanner scanner = sstable.getScanner(ColumnFilter.all(sstable.metadata), range, false, SSTableReadsListener.NOOP_LISTENER))
+            try(ISSTableScanner scanner = sstable.getScanner(ColumnFilter.all(sstable.metadata()),
+                                                             range,
+                                                             SSTableReadsListener.NOOP_LISTENER))
             {
                 for (int b = 0; b < boundaries.length; b += 2)
                     for (int i = boundaries[b]; i <= boundaries[b + 1]; i++)
@@ -213,14 +214,14 @@ public class SSTableScannerTest
         store.disableAutoCompaction();
 
         for (int i = 2; i < 10; i++)
-            insertRowWithKey(store.metadata, i);
+            insertRowWithKey(store.metadata(), i);
         store.forceBlockingFlush();
 
         assertEquals(1, store.getLiveSSTables().size());
         SSTableReader sstable = store.getLiveSSTables().iterator().next();
 
         // full range scan
-        ISSTableScanner scanner = sstable.getScanner(RateLimiter.create(Double.MAX_VALUE));
+        ISSTableScanner scanner = sstable.getScanner();
         for (int i = 2; i < 10; i++)
             assertEquals(toKey(i), new String(scanner.next().partitionKey().getKey().array()));
 
@@ -319,14 +320,14 @@ public class SSTableScannerTest
 
         for (int i = 0; i < 3; i++)
             for (int j = 2; j < 10; j++)
-                insertRowWithKey(store.metadata, i * 100 + j);
+                insertRowWithKey(store.metadata(), i * 100 + j);
         store.forceBlockingFlush();
 
         assertEquals(1, store.getLiveSSTables().size());
         SSTableReader sstable = store.getLiveSSTables().iterator().next();
 
         // full range scan
-        ISSTableScanner fullScanner = sstable.getScanner(RateLimiter.create(Double.MAX_VALUE));
+        ISSTableScanner fullScanner = sstable.getScanner();
         assertScanContainsRanges(fullScanner,
                                  2, 9,
                                  102, 109,
@@ -336,8 +337,7 @@ public class SSTableScannerTest
         // scan all three ranges separately
         ISSTableScanner scanner = sstable.getScanner(makeRanges(1, 9,
                                                                    101, 109,
-                                                                   201, 209),
-                                                        null);
+                                                                   201, 209));
         assertScanContainsRanges(scanner,
                                  2, 9,
                                  102, 109,
@@ -345,16 +345,14 @@ public class SSTableScannerTest
 
         // skip the first range
         scanner = sstable.getScanner(makeRanges(101, 109,
-                                                201, 209),
-                                     null);
+                                                201, 209));
         assertScanContainsRanges(scanner,
                                  102, 109,
                                  202, 209);
 
         // skip the second range
         scanner = sstable.getScanner(makeRanges(1, 9,
-                                                201, 209),
-                                     null);
+                                                201, 209));
         assertScanContainsRanges(scanner,
                                  2, 9,
                                  202, 209);
@@ -362,8 +360,7 @@ public class SSTableScannerTest
 
         // skip the last range
         scanner = sstable.getScanner(makeRanges(1, 9,
-                                                101, 109),
-                                     null);
+                                                101, 109));
         assertScanContainsRanges(scanner,
                                  2, 9,
                                  102, 109);
@@ -371,8 +368,7 @@ public class SSTableScannerTest
         // the first scanned range stops short of the actual data in the first range
         scanner = sstable.getScanner(makeRanges(1, 5,
                                                 101, 109,
-                                                201, 209),
-                                     null);
+                                                201, 209));
         assertScanContainsRanges(scanner,
                                  2, 5,
                                  102, 109,
@@ -381,8 +377,7 @@ public class SSTableScannerTest
         // the first scanned range requests data beyond actual data in the first range
         scanner = sstable.getScanner(makeRanges(1, 20,
                                                 101, 109,
-                                                201, 209),
-                                     null);
+                                                201, 209));
         assertScanContainsRanges(scanner,
                                  2, 9,
                                  102, 109,
@@ -392,8 +387,7 @@ public class SSTableScannerTest
         // the middle scan range splits the outside two data ranges
         scanner = sstable.getScanner(makeRanges(1, 5,
                                                 6, 205,
-                                                206, 209),
-                                     null);
+                                                206, 209));
         assertScanContainsRanges(scanner,
                                  2, 5,
                                  7, 9,
@@ -407,8 +401,7 @@ public class SSTableScannerTest
                                                 101, 109,
                                                 150, 159,
                                                 201, 209,
-                                                1000, 1001),
-                                     null);
+                                                1000, 1001));
         assertScanContainsRanges(scanner,
                                  3, 9,
                                  102, 109,
@@ -420,8 +413,7 @@ public class SSTableScannerTest
                                                 201, 209,
                                                 101, 109,
                                                 1000, 1001,
-                                                150, 159),
-                                     null);
+                                                150, 159));
         assertScanContainsRanges(scanner,
                                  2, 9,
                                  102, 109,
@@ -430,12 +422,11 @@ public class SSTableScannerTest
         // only empty ranges
         scanner = sstable.getScanner(makeRanges(0, 1,
                                                 150, 159,
-                                                250, 259),
-                                     null);
+                                                250, 259));
         assertFalse(scanner.hasNext());
 
         // no ranges is equivalent to a full scan
-        scanner = sstable.getScanner(new ArrayList<Range<Token>>(), null);
+        scanner = sstable.getScanner(new ArrayList<Range<Token>>());
         assertFalse(scanner.hasNext());
     }
 
@@ -449,19 +440,19 @@ public class SSTableScannerTest
         // disable compaction while flushing
         store.disableAutoCompaction();
 
-        insertRowWithKey(store.metadata, 205);
+        insertRowWithKey(store.metadata(), 205);
         store.forceBlockingFlush();
 
         assertEquals(1, store.getLiveSSTables().size());
         SSTableReader sstable = store.getLiveSSTables().iterator().next();
 
         // full range scan
-        ISSTableScanner fullScanner = sstable.getScanner(RateLimiter.create(Double.MAX_VALUE));
+        ISSTableScanner fullScanner = sstable.getScanner();
         assertScanContainsRanges(fullScanner, 205, 205);
 
         // scan three ranges separately
         ISSTableScanner scanner = sstable.getScanner(makeRanges(101, 109,
-                                                                   201, 209), null);
+                                                                   201, 209));
 
         // this will currently fail
         assertScanContainsRanges(scanner, 205, 205);
