@@ -143,7 +143,7 @@ public class BufferedRandomAccessFileTest
     @Test
     public void testReadAndWriteOnCapacity() throws IOException
     {
-        File tmpFile = File.createTempFile("readtest", "bin");
+        File tmpFile = FileUtils.createTempFile("readtest", "bin");
         try (SequentialWriter w = new SequentialWriter(tmpFile))
         {
             // Fully write the file and sync..
@@ -170,7 +170,7 @@ public class BufferedRandomAccessFileTest
     @Test
     public void testLength() throws IOException
     {
-        File tmpFile = File.createTempFile("lengthtest", "bin");
+        File tmpFile = FileUtils.createTempFile("lengthtest", "bin");
         try (SequentialWriter w = new SequentialWriter(tmpFile))
         {
             assertEquals(0, w.length());
@@ -423,7 +423,7 @@ public class BufferedRandomAccessFileTest
     @Test
     public void testBytesPastMark() throws IOException
     {
-        File tmpFile = File.createTempFile("overflowtest", "bin");
+        File tmpFile = FileUtils.createTempFile("overflowtest", "bin");
         tmpFile.deleteOnExit();
 
         // Create the BRAF by filename instead of by file.
@@ -588,7 +588,7 @@ public class BufferedRandomAccessFileTest
     @Test (expected=IllegalArgumentException.class)
     public void testSetNegativeLength() throws IOException, IllegalArgumentException
     {
-        File tmpFile = File.createTempFile("set_negative_length", "bin");
+        File tmpFile = FileUtils.createTempFile("set_negative_length", "bin");
         try (SequentialWriter file = new SequentialWriter(tmpFile))
         {
             file.truncate(-8L);
@@ -597,7 +597,7 @@ public class BufferedRandomAccessFileTest
 
     private SequentialWriter createTempFile(String name) throws IOException
     {
-        File tempFile = File.createTempFile(name, null);
+        File tempFile = FileUtils.createTempFile(name, null);
         tempFile.deleteOnExit();
 
         return new SequentialWriter(tempFile);
@@ -605,7 +605,7 @@ public class BufferedRandomAccessFileTest
 
     private File writeTemporaryFile(byte[] data) throws IOException
     {
-        File f = File.createTempFile("BRAFTestFile", null);
+        File f = FileUtils.createTempFile("BRAFTestFile", null);
         f.deleteOnExit();
         FileOutputStream fout = new FileOutputStream(f);
         fout.write(data);
