@@ -26,7 +26,7 @@ import java.util.List;
 import org.junit.After;
 import org.junit.BeforeClass;
 
-import junit.framework.Assert;
+import org.junit.Assert;
 import org.apache.cassandra.config.DatabaseDescriptor;
 
 public class ChecksummedSequentialWriterTest extends SequentialWriterTest
