@@ -29,13 +29,14 @@ import java.nio.channels.FileChannel;
 import java.nio.channels.FileLock;
 import java.nio.channels.ReadableByteChannel;
 import java.nio.channels.WritableByteChannel;
-import java.nio.charset.Charset;
+import java.nio.charset.StandardCharsets;
 import java.util.Arrays;
 import java.util.Random;
 import java.util.concurrent.ExecutorService;
 import java.util.concurrent.Executors;
 import java.util.concurrent.TimeUnit;
 
+import org.junit.Assert;
 import org.junit.BeforeClass;
 import org.junit.Test;
 
@@ -266,7 +267,7 @@ public class RandomAccessReaderTest
 
     private static File writeFile(Parameters params) throws IOException
     {
-        final File f = File.createTempFile("testReadFully", "1");
+        final File f = FileUtils.createTempFile("testReadFully", "1");
         f.deleteOnExit();
 
         try(SequentialWriter writer = new SequentialWriter(f))
@@ -319,7 +320,7 @@ public class RandomAccessReaderTest
     @Test
     public void testReadBytes() throws IOException
     {
-        File f = File.createTempFile("testReadBytes", "1");
+        File f = FileUtils.createTempFile("testReadBytes", "1");
         final String expected = "The quick brown fox jumps over the lazy dog";
 
         try(SequentialWriter writer = new SequentialWriter(f))
@@ -338,7 +339,7 @@ public class RandomAccessReaderTest
             assertEquals(expected.length(), reader.length());
 
             ByteBuffer b = ByteBufferUtil.read(reader, expected.length());
-            assertEquals(expected, new String(b.array(), Charset.forName("UTF-8")));
+            assertEquals(expected, new String(b.array(), StandardCharsets.UTF_8));
 
             assertTrue(reader.isEOF());
             assertEquals(0, reader.bytesRemaining());
@@ -348,7 +349,7 @@ public class RandomAccessReaderTest
     @Test
     public void testReset() throws IOException
     {
-        File f = File.createTempFile("testMark", "1");
+        File f = FileUtils.createTempFile("testMark", "1");
         final String expected = "The quick brown fox jumps over the lazy dog";
         final int numIterations = 10;
 
@@ -368,7 +369,7 @@ public class RandomAccessReaderTest
             assertEquals(expected.length() * numIterations, reader.length());
 
             ByteBuffer b = ByteBufferUtil.read(reader, expected.length());
-            assertEquals(expected, new String(b.array(), Charset.forName("UTF-8")));
+            assertEquals(expected, new String(b.array(), StandardCharsets.UTF_8));
 
             assertFalse(reader.isEOF());
             assertEquals((numIterations - 1) * expected.length(), reader.bytesRemaining());
@@ -380,7 +381,7 @@ public class RandomAccessReaderTest
             for (int i = 0; i < (numIterations - 1); i++)
             {
                 b = ByteBufferUtil.read(reader, expected.length());
-                assertEquals(expected, new String(b.array(), Charset.forName("UTF-8")));
+                assertEquals(expected, new String(b.array(), StandardCharsets.UTF_8));
             }
             assertTrue(reader.isEOF());
             assertEquals(expected.length() * (numIterations - 1), reader.bytesPastMark());
@@ -393,7 +394,7 @@ public class RandomAccessReaderTest
             for (int i = 0; i < (numIterations - 1); i++)
             {
                 b = ByteBufferUtil.read(reader, expected.length());
-                assertEquals(expected, new String(b.array(), Charset.forName("UTF-8")));
+                assertEquals(expected, new String(b.array(), StandardCharsets.UTF_8));
             }
 
             reader.reset();
@@ -403,7 +404,7 @@ public class RandomAccessReaderTest
             for (int i = 0; i < (numIterations - 1); i++)
             {
                 b = ByteBufferUtil.read(reader, expected.length());
-                assertEquals(expected, new String(b.array(), Charset.forName("UTF-8")));
+                assertEquals(expected, new String(b.array(), StandardCharsets.UTF_8));
             }
 
             assertTrue(reader.isEOF());
@@ -424,7 +425,7 @@ public class RandomAccessReaderTest
 
     private static void testSeek(int numThreads) throws IOException, InterruptedException
     {
-        final File f = File.createTempFile("testMark", "1");
+        final File f = FileUtils.createTempFile("testMark", "1");
         final byte[] expected = new byte[1 << 16];
 
         long seed = System.nanoTime();
@@ -494,7 +495,7 @@ public class RandomAccessReaderTest
                     executor.submit(worker);
 
                 executor.shutdown();
-                executor.awaitTermination(1, TimeUnit.MINUTES);
+                Assert.assertTrue(executor.awaitTermination(1, TimeUnit.MINUTES));
             }
         }
     }
@@ -531,7 +532,7 @@ public class RandomAccessReaderTest
         }
     }
 
-    @Test(expected = NullPointerException.class)
+    @Test(expected = IOException.class)
     public void testSkipBytesClosed() throws IOException
     {
         Parameters params = new Parameters(8192, 4096);
