@@ -21,22 +21,26 @@ import java.io.*;
 import java.util.*;
 
 import org.junit.BeforeClass;
+import org.junit.Rule;
 import org.junit.Test;
+import org.junit.rules.TemporaryFolder;
 
 import org.apache.cassandra.config.DatabaseDescriptor;
 import org.apache.cassandra.db.ClusteringComparator;
 import org.apache.cassandra.db.marshal.BytesType;
 import org.apache.cassandra.io.compress.CompressedSequentialWriter;
 import org.apache.cassandra.io.compress.CompressionMetadata;
+import org.apache.cassandra.io.sstable.format.SSTableReader;
+import org.apache.cassandra.io.util.DataInputPlus.DataInputStreamPlus;
+import org.apache.cassandra.io.util.FileUtils;
 import org.apache.cassandra.io.util.SequentialWriterOption;
 import org.apache.cassandra.schema.CompressionParams;
 import org.apache.cassandra.io.sstable.Component;
 import org.apache.cassandra.io.sstable.Descriptor;
 import org.apache.cassandra.io.sstable.metadata.MetadataCollector;
-import org.apache.cassandra.streaming.compress.CompressedInputStream;
-import org.apache.cassandra.streaming.compress.CompressionInfo;
+import org.apache.cassandra.db.streaming.CompressedInputStream;
+import org.apache.cassandra.db.streaming.CompressionInfo;
 import org.apache.cassandra.utils.ChecksumType;
-import org.apache.cassandra.utils.Pair;
 
 import static org.junit.Assert.assertEquals;
 import static org.junit.Assert.fail;
@@ -45,6 +49,9 @@ import static org.junit.Assert.fail;
  */
 public class CompressedInputStreamTest
 {
+    @Rule
+    public TemporaryFolder tempFolder = new TemporaryFolder();
+
     @BeforeClass
     public static void setupDD()
     {
@@ -54,17 +61,17 @@ public class CompressedInputStreamTest
     @Test
     public void testCompressedRead() throws Exception
     {
-        testCompressedReadWith(new long[]{0L}, false, false);
-        testCompressedReadWith(new long[]{1L}, false, false);
-        testCompressedReadWith(new long[]{100L}, false, false);
+        testCompressedReadWith(new long[]{0L}, false, false, 0);
+        testCompressedReadWith(new long[]{1L}, false, false, 0);
+        testCompressedReadWith(new long[]{100L}, false, false, 0);
 
-        testCompressedReadWith(new long[]{1L, 122L, 123L, 124L, 456L}, false, false);
+        testCompressedReadWith(new long[]{1L, 122L, 123L, 124L, 456L}, false, false, 0);
     }
 
     @Test(expected = EOFException.class)
     public void testTruncatedRead() throws Exception
     {
-        testCompressedReadWith(new long[]{1L, 122L, 123L, 124L, 456L}, true, false);
+        testCompressedReadWith(new long[]{1L, 122L, 123L, 124L, 456L}, true, false, 0);
     }
 
     /**
@@ -73,22 +80,45 @@ public class CompressedInputStreamTest
     @Test(timeout = 30000)
     public void testException() throws Exception
     {
-        testCompressedReadWith(new long[]{1L, 122L, 123L, 124L, 456L}, false, true);
+        testCompressedReadWith(new long[]{1L, 122L, 123L, 124L, 456L}, false, true, 0);
+    }
+
+    @Test
+    public void testCompressedReadUncompressedChunks() throws Exception
+    {
+        testCompressedReadWith(new long[]{0L}, false, false, 3);
+        testCompressedReadWith(new long[]{1L}, false, false, 3);
+        testCompressedReadWith(new long[]{100L}, false, false, 3);
+
+        testCompressedReadWith(new long[]{1L, 122L, 123L, 124L, 456L}, false, false, 3);
+    }
+
+    @Test(expected = EOFException.class)
+    public void testTruncatedReadUncompressedChunks() throws Exception
+    {
+        testCompressedReadWith(new long[]{1L, 122L, 123L, 124L, 456L}, true, false, 3);
+    }
+
+    @Test(timeout = 30000)
+    public void testCorruptedReadUncompressedChunks() throws Exception
+    {
+        testCompressedReadWith(new long[]{1L, 122L, 123L, 124L, 456L}, false, true, 3);
     }
 
     /**
      * @param valuesToCheck array of longs of range(0-999)
      * @throws Exception
      */
-    private void testCompressedReadWith(long[] valuesToCheck, boolean testTruncate, boolean testException) throws Exception
+    private void testCompressedReadWith(long[] valuesToCheck, boolean testTruncate, boolean testException, double minCompressRatio) throws Exception
     {
         assert valuesToCheck != null && valuesToCheck.length > 0;
 
         // write compressed data file of longs
-        File tmp = new File(File.createTempFile("cassandra", "unittest").getParent(), "ks-cf-ib-1-Data.db");
-        Descriptor desc = Descriptor.fromFilename(tmp.getAbsolutePath());
+        File parentDir = tempFolder.newFolder();
+        Descriptor desc = new Descriptor(parentDir, "ks", "cf", 1);
+        File tmp = new File(desc.filenameFor(Component.DATA));
         MetadataCollector collector = new MetadataCollector(new ClusteringComparator(BytesType.instance));
-        CompressionParams param = CompressionParams.snappy(32);
+        CompressionParams param = CompressionParams.snappy(32, minCompressRatio);
         Map<Long, Long> index = new HashMap<Long, Long>();
         try (CompressedSequentialWriter writer = new CompressedSequentialWriter(tmp,
                                                                                 desc.filenameFor(Component.COMPRESSION_INFO),
@@ -105,11 +135,11 @@ public class CompressedInputStreamTest
         }
 
         CompressionMetadata comp = CompressionMetadata.create(tmp.getAbsolutePath());
-        List<Pair<Long, Long>> sections = new ArrayList<>();
+        List<SSTableReader.PartitionPositionBounds> sections = new ArrayList<>();
         for (long l : valuesToCheck)
         {
             long position = index.get(l);
-            sections.add(Pair.create(position, position + 8));
+            sections.add(new SSTableReader.PartitionPositionBounds(position, position + 8));
         }
         CompressionMetadata.Chunk[] chunks = comp.getChunksForSections(sections);
         long totalSize = comp.getTotalSizeForSections(sections);
@@ -142,36 +172,36 @@ public class CompressedInputStreamTest
         }
 
         // read buffer using CompressedInputStream
-        CompressionInfo info = new CompressionInfo(chunks, param);
+        CompressionInfo info = CompressionInfo.newInstance(chunks, param);
 
         if (testException)
         {
             testException(sections, info);
             return;
         }
-        CompressedInputStream input = new CompressedInputStream(new ByteArrayInputStream(toRead), info, ChecksumType.CRC32, () -> 1.0);
+        CompressedInputStream input = new CompressedInputStream(new DataInputStreamPlus(new ByteArrayInputStream(toRead)), info, ChecksumType.CRC32, () -> 1.0);
 
         try (DataInputStream in = new DataInputStream(input))
         {
             for (int i = 0; i < sections.size(); i++)
             {
-                input.position(sections.get(i).left);
+                input.position(sections.get(i).lowerPosition);
                 long readValue = in.readLong();
                 assertEquals("expected " + valuesToCheck[i] + " but was " + readValue, valuesToCheck[i], readValue);
             }
         }
     }
 
-    private static void testException(List<Pair<Long, Long>> sections, CompressionInfo info) throws IOException
+    private static void testException(List<SSTableReader.PartitionPositionBounds> sections, CompressionInfo info) throws IOException
     {
-        CompressedInputStream input = new CompressedInputStream(new ByteArrayInputStream(new byte[0]), info, ChecksumType.CRC32, () -> 1.0);
+        CompressedInputStream input = new CompressedInputStream(new DataInputStreamPlus(new ByteArrayInputStream(new byte[0])), info, ChecksumType.CRC32, () -> 1.0);
 
         try (DataInputStream in = new DataInputStream(input))
         {
             for (int i = 0; i < sections.size(); i++)
             {
-                input.position(sections.get(i).left);
                 try {
+                    input.position(sections.get(i).lowerPosition);
                     in.readLong();
                     fail("Should have thrown IOException");
                 }
@@ -183,3 +213,4 @@ public class CompressedInputStreamTest
         }
     }
 }
+
