@@ -20,6 +20,8 @@ package org.apache.cassandra.transport;
 import java.nio.ByteBuffer;
 import java.util.*;
 
+import org.apache.commons.lang3.RandomStringUtils;
+
 import io.netty.buffer.Unpooled;
 import io.netty.buffer.ByteBuf;
 
@@ -32,7 +34,9 @@ import org.apache.cassandra.cql3.*;
 import org.apache.cassandra.db.ConsistencyLevel;
 import org.apache.cassandra.db.marshal.*;
 import org.apache.cassandra.serializers.CollectionSerializer;
-import org.apache.cassandra.service.pager.PagingState;
+import org.apache.cassandra.serializers.MarshalException;
+import org.apache.cassandra.service.ClientState;
+import org.apache.cassandra.service.QueryState;
 import org.apache.cassandra.transport.Event.TopologyChange;
 import org.apache.cassandra.transport.Event.SchemaChange;
 import org.apache.cassandra.transport.Event.StatusChange;
@@ -49,6 +53,7 @@ import static org.junit.Assert.assertNotSame;
  */
 public class SerDeserTest
 {
+
     @BeforeClass
     public static void setupDD()
     {
@@ -103,6 +108,41 @@ public class SerDeserTest
         assertEquals(m, mt.getSerializer().deserializeForNativeProtocol(CollectionSerializer.pack(mb, m.size(), version), version));
     }
 
+    @Test(expected = MarshalException.class)
+    public void setsMayNotContainNullsTest()
+    {
+        ProtocolVersion version = ProtocolVersion.MIN_SUPPORTED_VERSION;
+        SetType<?> st = SetType.getInstance(UTF8Type.instance, true);
+        List<ByteBuffer> sb = new ArrayList<>(1);
+        sb.add(null);
+
+        st.getSerializer().deserializeForNativeProtocol(CollectionSerializer.pack(sb, sb.size(), version), version);
+    }
+
+    @Test(expected = MarshalException.class)
+    public void mapKeysMayNotContainNullsTest()
+    {
+        ProtocolVersion version = ProtocolVersion.MIN_SUPPORTED_VERSION;
+        MapType<?, ?> mt = MapType.getInstance(UTF8Type.instance, LongType.instance, true);
+        List<ByteBuffer> mb = new ArrayList<>(2);
+        mb.add(null);
+        mb.add(LongType.instance.decompose(999L));
+
+        mt.getSerializer().deserializeForNativeProtocol(CollectionSerializer.pack(mb, mb.size(), version), version);
+    }
+
+    @Test(expected = MarshalException.class)
+    public void mapValueMayNotContainNullsTest()
+    {
+        ProtocolVersion version = ProtocolVersion.MIN_SUPPORTED_VERSION;
+        MapType<?, ?> mt = MapType.getInstance(UTF8Type.instance, LongType.instance, true);
+        List<ByteBuffer> mb = new ArrayList<>(2);
+        mb.add(UTF8Type.instance.decompose("danger"));
+        mb.add(null);
+
+        mt.getSerializer().deserializeForNativeProtocol(CollectionSerializer.pack(mb, mb.size(), version), version);
+    }
+
     @Test
     public void eventSerDeserTest() throws Exception
     {
@@ -114,12 +154,12 @@ public class SerDeserTest
     {
         List<Event> events = new ArrayList<>();
 
-        events.add(TopologyChange.newNode(FBUtilities.getBroadcastAddress(), 42));
-        events.add(TopologyChange.removedNode(FBUtilities.getBroadcastAddress(), 42));
-        events.add(TopologyChange.movedNode(FBUtilities.getBroadcastAddress(), 42));
+        events.add(TopologyChange.newNode(FBUtilities.getBroadcastAddressAndPort()));
+        events.add(TopologyChange.removedNode(FBUtilities.getBroadcastAddressAndPort()));
+        events.add(TopologyChange.movedNode(FBUtilities.getBroadcastAddressAndPort()));
 
-        events.add(StatusChange.nodeUp(FBUtilities.getBroadcastAddress(), 42));
-        events.add(StatusChange.nodeDown(FBUtilities.getBroadcastAddress(), 42));
+        events.add(StatusChange.nodeUp(FBUtilities.getBroadcastAddressAndPort()));
+        events.add(StatusChange.nodeDown(FBUtilities.getBroadcastAddressAndPort()));
 
         events.add(new SchemaChange(SchemaChange.Change.CREATED, "ks"));
         events.add(new SchemaChange(SchemaChange.Change.UPDATED, "ks"));
@@ -201,20 +241,32 @@ public class SerDeserTest
         SetType<?> st = SetType.getInstance(UTF8Type.instance, true);
         MapType<?, ?> mt = MapType.getInstance(UTF8Type.instance, LongType.instance, true);
 
+        String typeName = "myType" + randomUTF8(3);
+        String f1 = 'f' + randomUTF8(3);
+
         UserType udt = new UserType("ks",
-                                    bb("myType"),
-                                    Arrays.asList(field("f1"), field("f2"), field("f3"), field("f4")),
+                                    bb(typeName),
+                                    Arrays.asList(field(f1), field("f2"), field("f3"), field("f4")),
                                     Arrays.asList(LongType.instance, lt, st, mt),
                                     true);
 
         Map<FieldIdentifier, Term.Raw> value = new HashMap<>();
-        value.put(field("f1"), lit(42));
+        value.put(field(f1), lit(42));
         value.put(field("f2"), new Lists.Literal(Arrays.<Term.Raw>asList(lit(3), lit(1))));
         value.put(field("f3"), new Sets.Literal(Arrays.<Term.Raw>asList(lit("foo"), lit("bar"))));
         value.put(field("f4"), new Maps.Literal(Arrays.<Pair<Term.Raw, Term.Raw>>asList(
                                    Pair.<Term.Raw, Term.Raw>create(lit("foo"), lit(24)),
                                    Pair.<Term.Raw, Term.Raw>create(lit("bar"), lit(12)))));
 
+        ByteBuf buf = Unpooled.buffer(DataType.UDT.serializedValueSize(udt, version));
+        DataType.UDT.writeValue(udt, buf, version);
+        UserType decoded = (UserType) DataType.UDT.readValue(buf, version);
+        assertNotNull(decoded);
+        assertEquals("User type name mismatches: " + typeName,
+                     udt.name, decoded.name);
+        assertEquals("Decoded field name mismatches: " + f1,
+                     udt.fieldNameAsString(0), decoded.fieldNameAsString(0));
+
         UserTypes.Literal u = new UserTypes.Literal(value);
         Term t = u.prepare("ks", columnSpec("myValue", udt));
 
@@ -256,6 +308,9 @@ public class SerDeserTest
         List<ColumnSpecification> columnNames = new ArrayList<>();
         for (int i = 0; i < 3; i++)
             columnNames.add(new ColumnSpecification("ks", "cf", new ColumnIdentifier("col" + i, false), Int32Type.instance));
+        // add a column name that contains UTF-8 string (that is valid to cassandra)
+        String utf8ColName = "col" + randomUTF8(3);
+        columnNames.add(new ColumnSpecification("ks", "cf", new ColumnIdentifier(utf8ColName, false), Int32Type.instance));
 
         if (version == ProtocolVersion.V3)
         {
@@ -307,27 +362,66 @@ public class SerDeserTest
     }
 
     @Test
-    public void queryOptionsSerDeserTest() throws Exception
+    public void queryOptionsSerDeserTest()
     {
         for (ProtocolVersion version : ProtocolVersion.SUPPORTED)
-            queryOptionsSerDeserTest(version);
+        {
+            queryOptionsSerDeserTest(
+                version,
+                QueryOptions.create(ConsistencyLevel.ALL,
+                                    Collections.singletonList(ByteBuffer.wrap(new byte[] { 0x00, 0x01, 0x02 })),
+                                    false,
+                                    5000,
+                                    Util.makeSomePagingState(version),
+                                    ConsistencyLevel.SERIAL,
+                                    version,
+                                    null)
+            );
+        }
+
+        for (ProtocolVersion version : ProtocolVersion.supportedVersionsStartingWith(ProtocolVersion.V5))
+        {
+            queryOptionsSerDeserTest(
+                version,
+                QueryOptions.create(ConsistencyLevel.LOCAL_ONE,
+                                    Arrays.asList(ByteBuffer.wrap(new byte[] { 0x00, 0x01, 0x02 }),
+                                                  ByteBuffer.wrap(new byte[] { 0x03, 0x04, 0x05, 0x03, 0x04, 0x05 })),
+                                    true,
+                                    10,
+                                    Util.makeSomePagingState(version),
+                                    ConsistencyLevel.SERIAL,
+                                    version,
+                                    "some_keyspace")
+            );
+        }
+
+        for (ProtocolVersion version : ProtocolVersion.supportedVersionsStartingWith(ProtocolVersion.V5))
+        {
+            queryOptionsSerDeserTest(
+                version,
+                QueryOptions.create(ConsistencyLevel.LOCAL_ONE,
+                                    Arrays.asList(ByteBuffer.wrap(new byte[] { 0x00, 0x01, 0x02 }),
+                                                  ByteBuffer.wrap(new byte[] { 0x03, 0x04, 0x05, 0x03, 0x04, 0x05 })),
+                                    true,
+                                    10,
+                                    Util.makeSomePagingState(version),
+                                    ConsistencyLevel.SERIAL,
+                                    version,
+                                    "some_keyspace",
+                                    FBUtilities.timestampMicros(),
+                                    FBUtilities.nowInSeconds())
+            );
+        }
     }
 
-    private void queryOptionsSerDeserTest(ProtocolVersion version) throws Exception
+    private void queryOptionsSerDeserTest(ProtocolVersion version, QueryOptions options)
     {
-        QueryOptions options = QueryOptions.create(ConsistencyLevel.ALL,
-                                                   Collections.singletonList(ByteBuffer.wrap(new byte[] { 0x00, 0x01, 0x02 })),
-                                                   false,
-                                                   5000,
-                                                   Util.makeSomePagingState(version),
-                                                   ConsistencyLevel.SERIAL,
-                                                   version
-                                                   );
-
         ByteBuf buf = Unpooled.buffer(QueryOptions.codec.encodedSize(options, version));
         QueryOptions.codec.encode(options, buf, version);
         QueryOptions decodedOptions = QueryOptions.codec.decode(buf, version);
 
+        QueryState state = new QueryState(ClientState.forInternalCalls());
+
         assertNotNull(decodedOptions);
         assertEquals(options.getConsistency(), decodedOptions.getConsistency());
         assertEquals(options.getSerialConsistency(), decodedOptions.getSerialConsistency());
@@ -336,5 +430,15 @@ public class SerDeserTest
         assertEquals(options.getValues(), decodedOptions.getValues());
         assertEquals(options.getPagingState(), decodedOptions.getPagingState());
         assertEquals(options.skipMetadata(), decodedOptions.skipMetadata());
+        assertEquals(options.getKeyspace(), decodedOptions.getKeyspace());
+        assertEquals(options.getTimestamp(state), decodedOptions.getTimestamp(state));
+        assertEquals(options.getNowInSeconds(state), decodedOptions.getNowInSeconds(state));
+    }
+
+    // return utf8 string that contains no ascii chars
+    public static String randomUTF8(int count)
+    {
+        // valid for cassandra
+        return RandomStringUtils.random(count, 129, 0xD800, false, false);
     }
 }
