@@ -17,32 +17,26 @@
  */
 package org.apache.cassandra.triggers;
 
-import java.nio.ByteBuffer;
 import java.util.Collection;
 import java.util.Collections;
 
-import org.junit.AfterClass;
 import org.junit.Before;
 import org.junit.BeforeClass;
 import org.junit.Test;
 
 import org.apache.cassandra.SchemaLoader;
-import org.apache.cassandra.config.Schema;
+import org.apache.cassandra.schema.Schema;
 import org.apache.cassandra.cql3.QueryProcessor;
 import org.apache.cassandra.cql3.UntypedResultSet;
 import org.apache.cassandra.db.*;
 import org.apache.cassandra.db.ConsistencyLevel;
 import org.apache.cassandra.db.Mutation;
 import org.apache.cassandra.db.partitions.Partition;
-import org.apache.cassandra.db.partitions.PartitionUpdate;
 import org.apache.cassandra.exceptions.ConfigurationException;
 import org.apache.cassandra.exceptions.RequestExecutionException;
 import org.apache.cassandra.service.StorageService;
-import org.apache.cassandra.thrift.*;
 import org.apache.cassandra.utils.FBUtilities;
-import org.apache.thrift.protocol.TBinaryProtocol;
 
-import static org.apache.cassandra.utils.ByteBufferUtil.bytes;
 import static org.apache.cassandra.utils.ByteBufferUtil.toInt;
 import static org.junit.Assert.assertEquals;
 import static org.junit.Assert.assertTrue;
@@ -50,7 +44,6 @@ import static org.junit.Assert.assertTrue;
 public class TriggersTest
 {
     private static boolean triggerCreated = false;
-    private static ThriftServer thriftServer;
 
     private static String ksName = "triggers_test_ks";
     private static String cfName = "test_table";
@@ -66,11 +59,6 @@ public class TriggersTest
     public void setup() throws Exception
     {
         StorageService.instance.initServer(0);
-        if (thriftServer == null || ! thriftServer.isRunning())
-        {
-            thriftServer = new ThriftServer(FBUtilities.getLocalAddress(), 9170, 50);
-            thriftServer.start();
-        }
 
         String cql = String.format("CREATE KEYSPACE IF NOT EXISTS %s " +
                                    "WITH REPLICATION = {'class': 'SimpleStrategy', 'replication_factor': 1}",
@@ -93,15 +81,6 @@ public class TriggersTest
         }
     }
 
-    @AfterClass
-    public static void teardown()
-    {
-        if (thriftServer != null && thriftServer.isRunning())
-        {
-            thriftServer.stop();
-        }
-    }
-
     @Test
     public void executeTriggerOnCqlInsert() throws Exception
     {
@@ -121,43 +100,6 @@ public class TriggersTest
         assertUpdateIsAugmented(1, "v1", 1);
     }
 
-    @Test
-    public void executeTriggerOnThriftInsert() throws Exception
-    {
-        Cassandra.Client client = new Cassandra.Client(
-                                        new TBinaryProtocol(
-                                            new TFramedTransportFactory().openTransport(
-                                                FBUtilities.getLocalAddress().getHostName(), 9170)));
-        client.set_keyspace(ksName);
-        client.insert(bytes(2),
-                      new ColumnParent(cfName),
-                      getColumnForInsert("v1", 2),
-                      org.apache.cassandra.thrift.ConsistencyLevel.ONE);
-
-        assertUpdateIsAugmented(2, "v1", 2);
-    }
-
-    @Test
-    public void executeTriggerOnThriftBatchUpdate() throws Exception
-    {
-        Cassandra.Client client = new Cassandra.Client(
-                                    new TBinaryProtocol(
-                                        new TFramedTransportFactory().openTransport(
-                                            FBUtilities.getLocalAddress().getHostName(), 9170)));
-        client.set_keyspace(ksName);
-        org.apache.cassandra.thrift.Mutation mutation = new org.apache.cassandra.thrift.Mutation();
-        ColumnOrSuperColumn cosc = new ColumnOrSuperColumn();
-        cosc.setColumn(getColumnForInsert("v1", 3));
-        mutation.setColumn_or_supercolumn(cosc);
-        client.batch_mutate(
-            Collections.singletonMap(bytes(3),
-                                     Collections.singletonMap(cfName,
-                                                              Collections.singletonList(mutation))),
-            org.apache.cassandra.thrift.ConsistencyLevel.ONE);
-
-        assertUpdateIsAugmented(3, "v1", 3);
-    }
-
     @Test
     public void executeTriggerOnCqlInsertWithConditions() throws Exception
     {
@@ -178,27 +120,7 @@ public class TriggersTest
         assertUpdateIsAugmented(5, "v1", 5);
     }
 
-    @Test
-    public void executeTriggerOnThriftCASOperation() throws Exception
-    {
-        Cassandra.Client client = new Cassandra.Client(
-                new TBinaryProtocol(
-                        new TFramedTransportFactory().openTransport(
-                                FBUtilities.getLocalAddress().getHostName(), 9170)));
-        client.set_keyspace(ksName);
-        client.cas(bytes(6),
-                   cfName,
-                   Collections.<Column>emptyList(),
-                   Collections.singletonList(getColumnForInsert("v1", 6)),
-                   org.apache.cassandra.thrift.ConsistencyLevel.LOCAL_SERIAL,
-                   org.apache.cassandra.thrift.ConsistencyLevel.ONE);
-
-        assertUpdateIsAugmented(6, "v1", 6);
-    }
-
-    // Unfortunately, an IRE thrown from StorageProxy.cas
-    // results in a RuntimeException from QueryProcessor.process
-    @Test(expected=RuntimeException.class)
+    @Test(expected=org.apache.cassandra.exceptions.InvalidRequestException.class)
     public void onCqlUpdateWithConditionsRejectGeneratedUpdatesForDifferentPartition() throws Exception
     {
         String cf = "cf" + System.nanoTime();
@@ -214,9 +136,7 @@ public class TriggersTest
         }
     }
 
-    // Unfortunately, an IRE thrown from StorageProxy.cas
-    // results in a RuntimeException from QueryProcessor.process
-    @Test(expected=RuntimeException.class)
+    @Test(expected=org.apache.cassandra.exceptions.InvalidRequestException.class)
     public void onCqlUpdateWithConditionsRejectGeneratedUpdatesForDifferentTable() throws Exception
     {
         String cf = "cf" + System.nanoTime();
@@ -232,53 +152,24 @@ public class TriggersTest
         }
     }
 
-    @Test(expected=InvalidRequestException.class)
-    public void onThriftCASRejectGeneratedUpdatesForDifferentPartition() throws Exception
+    @Test(expected=org.apache.cassandra.exceptions.InvalidRequestException.class)
+    public void ifTriggerThrowsErrorNoMutationsAreApplied() throws Exception
     {
         String cf = "cf" + System.nanoTime();
         try
         {
-            setupTableWithTrigger(cf, CrossPartitionTrigger.class);
-            Cassandra.Client client = new Cassandra.Client(
-                    new TBinaryProtocol(
-                            new TFramedTransportFactory().openTransport(
-                                    FBUtilities.getLocalAddress().getHostName(), 9170)));
-            client.set_keyspace(ksName);
-            client.cas(bytes(9),
-                       cf,
-                       Collections.<Column>emptyList(),
-                       Collections.singletonList(getColumnForInsert("v1", 9)),
-                       org.apache.cassandra.thrift.ConsistencyLevel.LOCAL_SERIAL,
-                       org.apache.cassandra.thrift.ConsistencyLevel.ONE);
+            setupTableWithTrigger(cf, ErrorTrigger.class);
+            String cql = String.format("INSERT INTO %s.%s (k, v1) VALUES (11, 11)", ksName, cf);
+            QueryProcessor.process(cql, ConsistencyLevel.ONE);
+        }
+        catch (Exception e)
+        {
+            assertTrue(e.getMessage().equals(ErrorTrigger.MESSAGE));
+            throw e;
         }
         finally
         {
-            assertUpdateNotExecuted(cf, 9);
-        }
-    }
-
-    @Test(expected=InvalidRequestException.class)
-    public void onThriftCASRejectGeneratedUpdatesForDifferentCF() throws Exception
-    {
-        String cf = "cf" + System.nanoTime();
-        try
-        {
-            setupTableWithTrigger(cf, CrossTableTrigger.class);
-            Cassandra.Client client = new Cassandra.Client(
-                    new TBinaryProtocol(
-                            new TFramedTransportFactory().openTransport(
-                                    FBUtilities.getLocalAddress().getHostName(), 9170)));
-            client.set_keyspace(ksName);
-            client.cas(bytes(10),
-                       cf,
-                       Collections.<Column>emptyList(),
-                       Collections.singletonList(getColumnForInsert("v1", 10)),
-                       org.apache.cassandra.thrift.ConsistencyLevel.LOCAL_SERIAL,
-                       org.apache.cassandra.thrift.ConsistencyLevel.ONE);
-        }
-        finally
-        {
-            assertUpdateNotExecuted(cf, 10);
+            assertUpdateNotExecuted(cf, 11);
         }
     }
 
@@ -315,15 +206,6 @@ public class TriggersTest
         assertTrue(rs.isEmpty());
     }
 
-    private org.apache.cassandra.thrift.Column getColumnForInsert(String columnName, int value)
-    {
-        org.apache.cassandra.thrift.Column column = new org.apache.cassandra.thrift.Column();
-        column.setName(LegacyLayout.makeLegacyComparator(Schema.instance.getCFMetaData(ksName, cfName)).fromString(columnName));
-        column.setValue(bytes(value));
-        column.setTimestamp(System.currentTimeMillis());
-        return column;
-    }
-
     public static class TestTrigger implements ITrigger
     {
         public Collection<Mutation> augment(Partition partition)
@@ -351,10 +233,19 @@ public class TriggersTest
         public Collection<Mutation> augment(Partition partition)
         {
 
-            RowUpdateBuilder update = new RowUpdateBuilder(Schema.instance.getCFMetaData(ksName, otherCf), FBUtilities.timestampMicros(), partition.partitionKey().getKey());
+            RowUpdateBuilder update = new RowUpdateBuilder(Schema.instance.getTableMetadata(ksName, otherCf), FBUtilities.timestampMicros(), partition.partitionKey().getKey());
             update.add("v2", 999);
 
             return Collections.singletonList(update.build());
         }
     }
+
+    public static class ErrorTrigger implements ITrigger
+    {
+        public static final String MESSAGE = "Thrown by ErrorTrigger";
+        public Collection<Mutation> augment(Partition partition)
+        {
+            throw new org.apache.cassandra.exceptions.InvalidRequestException(MESSAGE);
+        }
+    }
 }
