@@ -26,6 +26,7 @@ import org.apache.cassandra.stress.settings.StressSettings;
 import org.apache.cassandra.stress.util.MultiResultLogger;
 import org.apache.cassandra.utils.FBUtilities;
 import org.apache.cassandra.utils.WindowsTimer;
+import org.apache.commons.lang3.exception.ExceptionUtils;
 
 public final class Stress
 {
@@ -87,6 +88,17 @@ public final class Stress
                 printHelpMessage();
                 return 1;
             }
+            catch (Throwable e)
+            {
+            	Throwable rc = ExceptionUtils.getRootCause(e);
+            	if (rc instanceof FileNotFoundException)
+            	{
+                    System.out.printf("File '%s' doesn't exist!%n", rc.getMessage());
+                    printHelpMessage();
+                    return 1;
+            	}
+            	throw e;
+            }
 
             MultiResultLogger logout = settings.log.getOutput();
 
