@@ -33,7 +33,6 @@ import org.apache.cassandra.stress.settings.SettingsCommand;
 import org.apache.cassandra.stress.settings.StressSettings;
 import org.apache.cassandra.stress.util.JavaDriverClient;
 import org.apache.cassandra.stress.util.ResultLogger;
-import org.apache.cassandra.stress.util.ThriftClient;
 import org.apache.cassandra.transport.SimpleClient;
 import org.jctools.queues.SpscArrayQueue;
 import org.jctools.queues.SpscUnboundedArrayQueue;
@@ -427,7 +426,6 @@ public class StressAction implements Runnable
             try
             {
                 SimpleClient sclient = null;
-                ThriftClient tclient = null;
                 JavaDriverClient jclient = null;
                 final ConnectionAPI clientType = settings.mode.api;
 
@@ -441,10 +439,6 @@ public class StressAction implements Runnable
                         case SIMPLE_NATIVE:
                             sclient = settings.getSimpleNativeClient();
                             break;
-                        case THRIFT:
-                        case THRIFT_SMART:
-                            tclient = settings.getThriftClient();
-                            break;
                         default:
                             throw new IllegalStateException();
                     }
@@ -474,10 +468,8 @@ public class StressAction implements Runnable
                             case SIMPLE_NATIVE:
                                 op.run(sclient);
                                 break;
-                            case THRIFT:
-                            case THRIFT_SMART:
                             default:
-                                op.run(tclient);
+                                throw new IllegalStateException();
                         }
                     }
                     catch (Exception e)
