@@ -23,6 +23,6 @@ package org.apache.cassandra.stress.settings;
 
 public enum ConnectionAPI
 {
-    THRIFT, THRIFT_SMART, SIMPLE_NATIVE, JAVA_DRIVER_NATIVE
+    SIMPLE_NATIVE, JAVA_DRIVER_NATIVE
 }
 
