@@ -32,7 +32,7 @@ import com.google.common.util.concurrent.Uninterruptibles;
 import org.apache.cassandra.stress.operations.OpDistributionFactory;
 import org.apache.cassandra.stress.util.JavaDriverClient;
 import org.apache.cassandra.stress.util.ResultLogger;
-import org.apache.cassandra.thrift.ConsistencyLevel;
+import org.apache.cassandra.db.ConsistencyLevel;
 
 // Generic command settings - common to read/write/etc
 public abstract class SettingsCommand implements Serializable
@@ -96,6 +96,9 @@ public abstract class SettingsCommand implements Serializable
                 case 'h':
                     this.durationUnits = TimeUnit.HOURS;
                     break;
+                case 'd':
+                    this.durationUnits = TimeUnit.DAYS;
+                    break;
                 default:
                     throw new IllegalStateException();
             }
@@ -120,7 +123,7 @@ public abstract class SettingsCommand implements Serializable
     {
         final OptionSimple noWarmup = new OptionSimple("no-warmup", "", null, "Do not warmup the process", false);
         final OptionSimple truncate = new OptionSimple("truncate=", "never|once|always", "never", "Truncate the table: never, before performing any work, or before each iteration", false);
-        final OptionSimple consistencyLevel = new OptionSimple("cl=", "ONE|QUORUM|LOCAL_QUORUM|EACH_QUORUM|ALL|ANY|TWO|THREE|LOCAL_ONE", "LOCAL_ONE", "Consistency level to use", false);
+        final OptionSimple consistencyLevel = new OptionSimple("cl=", "ONE|QUORUM|LOCAL_QUORUM|EACH_QUORUM|ALL|ANY|TWO|THREE|LOCAL_ONE|SERIAL|LOCAL_SERIAL", "LOCAL_ONE", "Consistency level to use", false);
     }
 
     static class Count extends Options
@@ -135,7 +138,7 @@ public abstract class SettingsCommand implements Serializable
 
     static class Duration extends Options
     {
-        final OptionSimple duration = new OptionSimple("duration=", "[0-9]+[smh]", null, "Time to run in (in seconds, minutes or hours)", true);
+        final OptionSimple duration = new OptionSimple("duration=", "[0-9]+[smhd]", null, "Time to run in (in seconds, minutes, hours or days)", true);
         @Override
         public List<? extends Option> options()
         {
