@@ -29,6 +29,7 @@ import java.util.Map;
 import com.datastax.driver.core.BatchStatement;
 import org.apache.cassandra.stress.generate.DistributionFactory;
 import org.apache.cassandra.stress.generate.RatioDistributionFactory;
+import org.apache.cassandra.stress.util.ResultLogger;
 
 public class SettingsInsert implements Serializable
 {
@@ -37,6 +38,7 @@ public class SettingsInsert implements Serializable
     public final DistributionFactory visits;
     public final DistributionFactory batchsize;
     public final RatioDistributionFactory selectRatio;
+    public final RatioDistributionFactory rowPopulationRatio;
     public final BatchStatement.Type batchType;
 
     private SettingsInsert(InsertOptions options)
@@ -45,6 +47,10 @@ public class SettingsInsert implements Serializable
         this.revisit = options.revisit.get();
         this.batchsize = options.partitions.get();
         this.selectRatio = options.selectRatio.get();
+        this.rowPopulationRatio = options.rowPopulationRatio.get();
+
+
+
         this.batchType = !options.batchType.setByUser() ? null : BatchStatement.Type.valueOf(options.batchType.value());
     }
 
@@ -57,15 +63,47 @@ public class SettingsInsert implements Serializable
         final OptionDistribution partitions = new OptionDistribution("partitions=", null, "The number of partitions to update in a single batch", false);
         final OptionSimple batchType = new OptionSimple("batchtype=", "unlogged|logged|counter", null, "Specify the type of batch statement (LOGGED, UNLOGGED or COUNTER)", false);
         final OptionRatioDistribution selectRatio = new OptionRatioDistribution("select-ratio=", null, "The uniform probability of visiting any CQL row in the generated partition", false);
+        final OptionRatioDistribution rowPopulationRatio = new OptionRatioDistribution("row-population-ratio=", "fixed(1)/1", "The percent of a given rows columns to populate", false);
 
         @Override
         public List<? extends Option> options()
         {
-            return Arrays.asList(revisit, visits, partitions, batchType, selectRatio);
+            return Arrays.asList(revisit, visits, partitions, batchType, selectRatio, rowPopulationRatio);
         }
     }
 
     // CLI Utility Methods
+    public void printSettings(ResultLogger out)
+    {
+
+        if (revisit != null)
+        {
+            out.println("  Revisits: " +revisit.getConfigAsString());
+        }
+        if (visits != null)
+        {
+            out.println("  Visits: " + visits.getConfigAsString());
+        }
+        if (batchsize != null)
+        {
+            out.println("  Batchsize: " +batchsize.getConfigAsString());
+        }
+        if (batchsize != null)
+        {
+            out.println("  Select Ratio: " +selectRatio.getConfigAsString());
+        }
+        if (rowPopulationRatio != null)
+        {
+            out.println("  Row Population Ratio: " +rowPopulationRatio.getConfigAsString());
+        }
+        if (batchType != null)
+        {
+            out.printf("  Batch Type: %s%n", batchType);
+        } else {
+            out.println("  Batch Type: not batching");
+        }
+    }
+
 
     public static SettingsInsert get(Map<String, String[]> clArgs)
     {
