@@ -128,12 +128,11 @@ public class StressSettings implements Serializable
 
             try
             {
-                String currentNode = node.randomNode();
                 if (client != null)
                     return client;
 
                 EncryptionOptions encOptions = transport.getEncryptionOptions();
-                JavaDriverClient c = new JavaDriverClient(this, currentNode, port.nativePort, encOptions);
+                JavaDriverClient c = new JavaDriverClient(this, node.nodes, port.nativePort, encOptions);
                 c.connect(mode.compression());
                 if (keyspace != null)
                     c.execute("USE \"" + keyspace + "\";", org.apache.cassandra.db.ConsistencyLevel.ONE);
